import { promises as fs } from 'fs'
import { pascalCase } from 'pascal-case'
import { promisify } from 'util'
import _rimraf from 'rimraf'
import mkdirp from 'mkdirp'
const rimraf = promisify(_rimraf)

const DIR_NAME = __dirname+'/..'
const DIR = 'raw'

const componentTemplate = (name: string, svg: string) => `
import Vue from 'vue'

export default Vue.extend({
  name: '${name}',

  props: {
    size: {
      type: [String, Number],
      default: 24,
    }
  },

  functional: true,

  render(h, ctx) {
    const size = \`\${ctx.props.size}px\`

    const attrs = ctx.data.attrs || {}
    attrs.width = attrs.width || size
    attrs.height = attrs.height || size
    ctx.data.attrs = attrs

    // @ts-ignore
    return (${svg.replace(/<svg([^>]+)>/, '<svg$1 {...ctx.data}>').replace(/\n/g, '')})
  }
})
`.trim()

async function run() {
  await rimraf(DIR_NAME + `/${DIR}/`)
  await mkdirp(DIR_NAME + `/${DIR}/`)
  await mkdirp(DIR_NAME + `/${DIR}/components`)
  const svgFileNames = await fs.readdir('./node_modules/bootstrap-icons/icons')
  await Promise.all([
    svgFileNames.map(async (fileName) => {
      const kebabName = `${fileName.split('.svg')[0]}-icon`
      const pascalName = pascalCase(kebabName)
      const svgData = await fs.readFile(DIR_NAME + '/node_modules/bootstrap-icons/icons/' + fileName, { encoding: 'utf8' })
      const dist = componentTemplate(pascalName, svgData)
      await fs.writeFile(DIR_NAME+'/'+DIR+'/components/'+fileName.replace('.svg', '.js'), dist, { encoding: 'utf8' })
    })
  ])
  const dist = svgFileNames.map((fileName) => {
    const kebabName = `${fileName.split('.svg')[0]}-icon`
    const pascalName = pascalCase(kebabName)
    return `export { default as ${pascalName} } from './components/${kebabName.replace('-icon', '').replace('.svg', '')}'`
  }).join('\n')
  await fs.writeFile(DIR_NAME+'/'+DIR+'/index.js', dist, { encoding: 'utf8'})
}

run()
