(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (factory((global.vueBootstrapIcons = {})));
}(this, (function (exports) { 'use strict';

  var nestRE = /^(attrs|props|on|nativeOn|class|style|hook)$/;

  var babelHelperVueJsxMergeProps = function mergeJSXProps (objs) {
    return objs.reduce(function (a, b) {
      var aa, bb, key, nestedKey, temp;
      for (key in b) {
        aa = a[key];
        bb = b[key];
        if (aa && nestRE.test(key)) {
          // normalize class
          if (key === 'class') {
            if (typeof aa === 'string') {
              temp = aa;
              a[key] = aa = {};
              aa[temp] = true;
            }
            if (typeof bb === 'string') {
              temp = bb;
              b[key] = bb = {};
              bb[temp] = true;
            }
          }
          if (key === 'on' || key === 'nativeOn' || key === 'hook') {
            // merge functions
            for (nestedKey in bb) {
              aa[nestedKey] = mergeFn(aa[nestedKey], bb[nestedKey]);
            }
          } else if (Array.isArray(aa)) {
            a[key] = aa.concat(bb);
          } else if (Array.isArray(bb)) {
            a[key] = [aa].concat(bb);
          } else {
            for (nestedKey in bb) {
              aa[nestedKey] = bb[nestedKey];
            }
          }
        } else {
          a[key] = b[key];
        }
      }
      return a
    }, {})
  };

  function mergeFn (a, b) {
    return function () {
      a && a.apply(this, arguments);
      b && b.apply(this, arguments);
    }
  }

  /*!
   * Vue.js v2.6.11
   * (c) 2014-2019 Evan You
   * Released under the MIT License.
   */
  /*  */

  var emptyObject = Object.freeze({});

  // These helpers produce better VM code in JS engines due to their
  // explicitness and function inlining.
  function isUndef (v) {
    return v === undefined || v === null
  }

  function isDef (v) {
    return v !== undefined && v !== null
  }

  function isTrue (v) {
    return v === true
  }

  function isFalse (v) {
    return v === false
  }

  /**
   * Check if value is primitive.
   */
  function isPrimitive (value) {
    return (
      typeof value === 'string' ||
      typeof value === 'number' ||
      // $flow-disable-line
      typeof value === 'symbol' ||
      typeof value === 'boolean'
    )
  }

  /**
   * Quick object check - this is primarily used to tell
   * Objects from primitive values when we know the value
   * is a JSON-compliant type.
   */
  function isObject (obj) {
    return obj !== null && typeof obj === 'object'
  }

  /**
   * Get the raw type string of a value, e.g., [object Object].
   */
  var _toString = Object.prototype.toString;

  function toRawType (value) {
    return _toString.call(value).slice(8, -1)
  }

  /**
   * Strict object type check. Only returns true
   * for plain JavaScript objects.
   */
  function isPlainObject (obj) {
    return _toString.call(obj) === '[object Object]'
  }

  function isRegExp (v) {
    return _toString.call(v) === '[object RegExp]'
  }

  /**
   * Check if val is a valid array index.
   */
  function isValidArrayIndex (val) {
    var n = parseFloat(String(val));
    return n >= 0 && Math.floor(n) === n && isFinite(val)
  }

  function isPromise (val) {
    return (
      isDef(val) &&
      typeof val.then === 'function' &&
      typeof val.catch === 'function'
    )
  }

  /**
   * Convert a value to a string that is actually rendered.
   */
  function toString (val) {
    return val == null
      ? ''
      : Array.isArray(val) || (isPlainObject(val) && val.toString === _toString)
        ? JSON.stringify(val, null, 2)
        : String(val)
  }

  /**
   * Convert an input value to a number for persistence.
   * If the conversion fails, return original string.
   */
  function toNumber (val) {
    var n = parseFloat(val);
    return isNaN(n) ? val : n
  }

  /**
   * Make a map and return a function for checking if a key
   * is in that map.
   */
  function makeMap (
    str,
    expectsLowerCase
  ) {
    var map = Object.create(null);
    var list = str.split(',');
    for (var i = 0; i < list.length; i++) {
      map[list[i]] = true;
    }
    return expectsLowerCase
      ? function (val) { return map[val.toLowerCase()]; }
      : function (val) { return map[val]; }
  }

  /**
   * Check if a tag is a built-in tag.
   */
  var isBuiltInTag = makeMap('slot,component', true);

  /**
   * Check if an attribute is a reserved attribute.
   */
  var isReservedAttribute = makeMap('key,ref,slot,slot-scope,is');

  /**
   * Remove an item from an array.
   */
  function remove (arr, item) {
    if (arr.length) {
      var index = arr.indexOf(item);
      if (index > -1) {
        return arr.splice(index, 1)
      }
    }
  }

  /**
   * Check whether an object has the property.
   */
  var hasOwnProperty = Object.prototype.hasOwnProperty;
  function hasOwn (obj, key) {
    return hasOwnProperty.call(obj, key)
  }

  /**
   * Create a cached version of a pure function.
   */
  function cached (fn) {
    var cache = Object.create(null);
    return (function cachedFn (str) {
      var hit = cache[str];
      return hit || (cache[str] = fn(str))
    })
  }

  /**
   * Camelize a hyphen-delimited string.
   */
  var camelizeRE = /-(\w)/g;
  var camelize = cached(function (str) {
    return str.replace(camelizeRE, function (_, c) { return c ? c.toUpperCase() : ''; })
  });

  /**
   * Capitalize a string.
   */
  var capitalize = cached(function (str) {
    return str.charAt(0).toUpperCase() + str.slice(1)
  });

  /**
   * Hyphenate a camelCase string.
   */
  var hyphenateRE = /\B([A-Z])/g;
  var hyphenate = cached(function (str) {
    return str.replace(hyphenateRE, '-$1').toLowerCase()
  });

  /**
   * Simple bind polyfill for environments that do not support it,
   * e.g., PhantomJS 1.x. Technically, we don't need this anymore
   * since native bind is now performant enough in most browsers.
   * But removing it would mean breaking code that was able to run in
   * PhantomJS 1.x, so this must be kept for backward compatibility.
   */

  /* istanbul ignore next */
  function polyfillBind (fn, ctx) {
    function boundFn (a) {
      var l = arguments.length;
      return l
        ? l > 1
          ? fn.apply(ctx, arguments)
          : fn.call(ctx, a)
        : fn.call(ctx)
    }

    boundFn._length = fn.length;
    return boundFn
  }

  function nativeBind (fn, ctx) {
    return fn.bind(ctx)
  }

  var bind = Function.prototype.bind
    ? nativeBind
    : polyfillBind;

  /**
   * Convert an Array-like object to a real Array.
   */
  function toArray (list, start) {
    start = start || 0;
    var i = list.length - start;
    var ret = new Array(i);
    while (i--) {
      ret[i] = list[i + start];
    }
    return ret
  }

  /**
   * Mix properties into target object.
   */
  function extend (to, _from) {
    for (var key in _from) {
      to[key] = _from[key];
    }
    return to
  }

  /**
   * Merge an Array of Objects into a single Object.
   */
  function toObject (arr) {
    var res = {};
    for (var i = 0; i < arr.length; i++) {
      if (arr[i]) {
        extend(res, arr[i]);
      }
    }
    return res
  }

  /* eslint-disable no-unused-vars */

  /**
   * Perform no operation.
   * Stubbing args to make Flow happy without leaving useless transpiled code
   * with ...rest (https://flow.org/blog/2017/05/07/Strict-Function-Call-Arity/).
   */
  function noop (a, b, c) {}

  /**
   * Always return false.
   */
  var no = function (a, b, c) { return false; };

  /* eslint-enable no-unused-vars */

  /**
   * Return the same value.
   */
  var identity = function (_) { return _; };

  /**
   * Check if two values are loosely equal - that is,
   * if they are plain objects, do they have the same shape?
   */
  function looseEqual (a, b) {
    if (a === b) { return true }
    var isObjectA = isObject(a);
    var isObjectB = isObject(b);
    if (isObjectA && isObjectB) {
      try {
        var isArrayA = Array.isArray(a);
        var isArrayB = Array.isArray(b);
        if (isArrayA && isArrayB) {
          return a.length === b.length && a.every(function (e, i) {
            return looseEqual(e, b[i])
          })
        } else if (a instanceof Date && b instanceof Date) {
          return a.getTime() === b.getTime()
        } else if (!isArrayA && !isArrayB) {
          var keysA = Object.keys(a);
          var keysB = Object.keys(b);
          return keysA.length === keysB.length && keysA.every(function (key) {
            return looseEqual(a[key], b[key])
          })
        } else {
          /* istanbul ignore next */
          return false
        }
      } catch (e) {
        /* istanbul ignore next */
        return false
      }
    } else if (!isObjectA && !isObjectB) {
      return String(a) === String(b)
    } else {
      return false
    }
  }

  /**
   * Return the first index at which a loosely equal value can be
   * found in the array (if value is a plain object, the array must
   * contain an object of the same shape), or -1 if it is not present.
   */
  function looseIndexOf (arr, val) {
    for (var i = 0; i < arr.length; i++) {
      if (looseEqual(arr[i], val)) { return i }
    }
    return -1
  }

  /**
   * Ensure a function is called only once.
   */
  function once (fn) {
    var called = false;
    return function () {
      if (!called) {
        called = true;
        fn.apply(this, arguments);
      }
    }
  }

  var SSR_ATTR = 'data-server-rendered';

  var ASSET_TYPES = [
    'component',
    'directive',
    'filter'
  ];

  var LIFECYCLE_HOOKS = [
    'beforeCreate',
    'created',
    'beforeMount',
    'mounted',
    'beforeUpdate',
    'updated',
    'beforeDestroy',
    'destroyed',
    'activated',
    'deactivated',
    'errorCaptured',
    'serverPrefetch'
  ];

  /*  */



  var config = ({
    /**
     * Option merge strategies (used in core/util/options)
     */
    // $flow-disable-line
    optionMergeStrategies: Object.create(null),

    /**
     * Whether to suppress warnings.
     */
    silent: false,

    /**
     * Show production mode tip message on boot?
     */
    productionTip: "development" !== 'production',

    /**
     * Whether to enable devtools
     */
    devtools: "development" !== 'production',

    /**
     * Whether to record perf
     */
    performance: false,

    /**
     * Error handler for watcher errors
     */
    errorHandler: null,

    /**
     * Warn handler for watcher warns
     */
    warnHandler: null,

    /**
     * Ignore certain custom elements
     */
    ignoredElements: [],

    /**
     * Custom user key aliases for v-on
     */
    // $flow-disable-line
    keyCodes: Object.create(null),

    /**
     * Check if a tag is reserved so that it cannot be registered as a
     * component. This is platform-dependent and may be overwritten.
     */
    isReservedTag: no,

    /**
     * Check if an attribute is reserved so that it cannot be used as a component
     * prop. This is platform-dependent and may be overwritten.
     */
    isReservedAttr: no,

    /**
     * Check if a tag is an unknown element.
     * Platform-dependent.
     */
    isUnknownElement: no,

    /**
     * Get the namespace of an element
     */
    getTagNamespace: noop,

    /**
     * Parse the real tag name for the specific platform.
     */
    parsePlatformTagName: identity,

    /**
     * Check if an attribute must be bound using property, e.g. value
     * Platform-dependent.
     */
    mustUseProp: no,

    /**
     * Perform updates asynchronously. Intended to be used by Vue Test Utils
     * This will significantly reduce performance if set to false.
     */
    async: true,

    /**
     * Exposed for legacy reasons
     */
    _lifecycleHooks: LIFECYCLE_HOOKS
  });

  /*  */

  /**
   * unicode letters used for parsing html tags, component names and property paths.
   * using https://www.w3.org/TR/html53/semantics-scripting.html#potentialcustomelementname
   * skipping \u10000-\uEFFFF due to it freezing up PhantomJS
   */
  var unicodeRegExp = /a-zA-Z\u00B7\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u037D\u037F-\u1FFF\u200C-\u200D\u203F-\u2040\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD/;

  /**
   * Check if a string starts with $ or _
   */
  function isReserved (str) {
    var c = (str + '').charCodeAt(0);
    return c === 0x24 || c === 0x5F
  }

  /**
   * Define a property.
   */
  function def (obj, key, val, enumerable) {
    Object.defineProperty(obj, key, {
      value: val,
      enumerable: !!enumerable,
      writable: true,
      configurable: true
    });
  }

  /**
   * Parse simple path.
   */
  var bailRE = new RegExp(("[^" + (unicodeRegExp.source) + ".$_\\d]"));
  function parsePath (path) {
    if (bailRE.test(path)) {
      return
    }
    var segments = path.split('.');
    return function (obj) {
      for (var i = 0; i < segments.length; i++) {
        if (!obj) { return }
        obj = obj[segments[i]];
      }
      return obj
    }
  }

  /*  */

  // can we use __proto__?
  var hasProto = '__proto__' in {};

  // Browser environment sniffing
  var inBrowser = typeof window !== 'undefined';
  var inWeex = typeof WXEnvironment !== 'undefined' && !!WXEnvironment.platform;
  var weexPlatform = inWeex && WXEnvironment.platform.toLowerCase();
  var UA = inBrowser && window.navigator.userAgent.toLowerCase();
  var isIE = UA && /msie|trident/.test(UA);
  var isIE9 = UA && UA.indexOf('msie 9.0') > 0;
  var isEdge = UA && UA.indexOf('edge/') > 0;
  var isAndroid = (UA && UA.indexOf('android') > 0) || (weexPlatform === 'android');
  var isIOS = (UA && /iphone|ipad|ipod|ios/.test(UA)) || (weexPlatform === 'ios');
  var isChrome = UA && /chrome\/\d+/.test(UA) && !isEdge;
  var isPhantomJS = UA && /phantomjs/.test(UA);
  var isFF = UA && UA.match(/firefox\/(\d+)/);

  // Firefox has a "watch" function on Object.prototype...
  var nativeWatch = ({}).watch;

  var supportsPassive = false;
  if (inBrowser) {
    try {
      var opts = {};
      Object.defineProperty(opts, 'passive', ({
        get: function get () {
          /* istanbul ignore next */
          supportsPassive = true;
        }
      })); // https://github.com/facebook/flow/issues/285
      window.addEventListener('test-passive', null, opts);
    } catch (e) {}
  }

  // this needs to be lazy-evaled because vue may be required before
  // vue-server-renderer can set VUE_ENV
  var _isServer;
  var isServerRendering = function () {
    if (_isServer === undefined) {
      /* istanbul ignore if */
      if (!inBrowser && !inWeex && typeof global !== 'undefined') {
        // detect presence of vue-server-renderer and avoid
        // Webpack shimming the process
        _isServer = global['process'] && global['process'].env.VUE_ENV === 'server';
      } else {
        _isServer = false;
      }
    }
    return _isServer
  };

  // detect devtools
  var devtools = inBrowser && window.__VUE_DEVTOOLS_GLOBAL_HOOK__;

  /* istanbul ignore next */
  function isNative (Ctor) {
    return typeof Ctor === 'function' && /native code/.test(Ctor.toString())
  }

  var hasSymbol =
    typeof Symbol !== 'undefined' && isNative(Symbol) &&
    typeof Reflect !== 'undefined' && isNative(Reflect.ownKeys);

  var _Set;
  /* istanbul ignore if */ // $flow-disable-line
  if (typeof Set !== 'undefined' && isNative(Set)) {
    // use native Set when available.
    _Set = Set;
  } else {
    // a non-standard Set polyfill that only works with primitive keys.
    _Set = /*@__PURE__*/(function () {
      function Set () {
        this.set = Object.create(null);
      }
      Set.prototype.has = function has (key) {
        return this.set[key] === true
      };
      Set.prototype.add = function add (key) {
        this.set[key] = true;
      };
      Set.prototype.clear = function clear () {
        this.set = Object.create(null);
      };

      return Set;
    }());
  }

  /*  */

  var warn = noop;
  var tip = noop;
  var generateComponentTrace = (noop); // work around flow check
  var formatComponentName = (noop);

  {
    var hasConsole = typeof console !== 'undefined';
    var classifyRE = /(?:^|[-_])(\w)/g;
    var classify = function (str) { return str
      .replace(classifyRE, function (c) { return c.toUpperCase(); })
      .replace(/[-_]/g, ''); };

    warn = function (msg, vm) {
      var trace = vm ? generateComponentTrace(vm) : '';

      if (config.warnHandler) {
        config.warnHandler.call(null, msg, vm, trace);
      } else if (hasConsole && (!config.silent)) {
        console.error(("[Vue warn]: " + msg + trace));
      }
    };

    tip = function (msg, vm) {
      if (hasConsole && (!config.silent)) {
        console.warn("[Vue tip]: " + msg + (
          vm ? generateComponentTrace(vm) : ''
        ));
      }
    };

    formatComponentName = function (vm, includeFile) {
      if (vm.$root === vm) {
        return '<Root>'
      }
      var options = typeof vm === 'function' && vm.cid != null
        ? vm.options
        : vm._isVue
          ? vm.$options || vm.constructor.options
          : vm;
      var name = options.name || options._componentTag;
      var file = options.__file;
      if (!name && file) {
        var match = file.match(/([^/\\]+)\.vue$/);
        name = match && match[1];
      }

      return (
        (name ? ("<" + (classify(name)) + ">") : "<Anonymous>") +
        (file && includeFile !== false ? (" at " + file) : '')
      )
    };

    var repeat = function (str, n) {
      var res = '';
      while (n) {
        if (n % 2 === 1) { res += str; }
        if (n > 1) { str += str; }
        n >>= 1;
      }
      return res
    };

    generateComponentTrace = function (vm) {
      if (vm._isVue && vm.$parent) {
        var tree = [];
        var currentRecursiveSequence = 0;
        while (vm) {
          if (tree.length > 0) {
            var last = tree[tree.length - 1];
            if (last.constructor === vm.constructor) {
              currentRecursiveSequence++;
              vm = vm.$parent;
              continue
            } else if (currentRecursiveSequence > 0) {
              tree[tree.length - 1] = [last, currentRecursiveSequence];
              currentRecursiveSequence = 0;
            }
          }
          tree.push(vm);
          vm = vm.$parent;
        }
        return '\n\nfound in\n\n' + tree
          .map(function (vm, i) { return ("" + (i === 0 ? '---> ' : repeat(' ', 5 + i * 2)) + (Array.isArray(vm)
              ? ((formatComponentName(vm[0])) + "... (" + (vm[1]) + " recursive calls)")
              : formatComponentName(vm))); })
          .join('\n')
      } else {
        return ("\n\n(found in " + (formatComponentName(vm)) + ")")
      }
    };
  }

  /*  */

  var uid = 0;

  /**
   * A dep is an observable that can have multiple
   * directives subscribing to it.
   */
  var Dep = function Dep () {
    this.id = uid++;
    this.subs = [];
  };

  Dep.prototype.addSub = function addSub (sub) {
    this.subs.push(sub);
  };

  Dep.prototype.removeSub = function removeSub (sub) {
    remove(this.subs, sub);
  };

  Dep.prototype.depend = function depend () {
    if (Dep.target) {
      Dep.target.addDep(this);
    }
  };

  Dep.prototype.notify = function notify () {
    // stabilize the subscriber list first
    var subs = this.subs.slice();
    if (!config.async) {
      // subs aren't sorted in scheduler if not running async
      // we need to sort them now to make sure they fire in correct
      // order
      subs.sort(function (a, b) { return a.id - b.id; });
    }
    for (var i = 0, l = subs.length; i < l; i++) {
      subs[i].update();
    }
  };

  // The current target watcher being evaluated.
  // This is globally unique because only one watcher
  // can be evaluated at a time.
  Dep.target = null;
  var targetStack = [];

  function pushTarget (target) {
    targetStack.push(target);
    Dep.target = target;
  }

  function popTarget () {
    targetStack.pop();
    Dep.target = targetStack[targetStack.length - 1];
  }

  /*  */

  var VNode = function VNode (
    tag,
    data,
    children,
    text,
    elm,
    context,
    componentOptions,
    asyncFactory
  ) {
    this.tag = tag;
    this.data = data;
    this.children = children;
    this.text = text;
    this.elm = elm;
    this.ns = undefined;
    this.context = context;
    this.fnContext = undefined;
    this.fnOptions = undefined;
    this.fnScopeId = undefined;
    this.key = data && data.key;
    this.componentOptions = componentOptions;
    this.componentInstance = undefined;
    this.parent = undefined;
    this.raw = false;
    this.isStatic = false;
    this.isRootInsert = true;
    this.isComment = false;
    this.isCloned = false;
    this.isOnce = false;
    this.asyncFactory = asyncFactory;
    this.asyncMeta = undefined;
    this.isAsyncPlaceholder = false;
  };

  var prototypeAccessors = { child: { configurable: true } };

  // DEPRECATED: alias for componentInstance for backwards compat.
  /* istanbul ignore next */
  prototypeAccessors.child.get = function () {
    return this.componentInstance
  };

  Object.defineProperties( VNode.prototype, prototypeAccessors );

  var createEmptyVNode = function (text) {
    if ( text === void 0 ) text = '';

    var node = new VNode();
    node.text = text;
    node.isComment = true;
    return node
  };

  function createTextVNode (val) {
    return new VNode(undefined, undefined, undefined, String(val))
  }

  // optimized shallow clone
  // used for static nodes and slot nodes because they may be reused across
  // multiple renders, cloning them avoids errors when DOM manipulations rely
  // on their elm reference.
  function cloneVNode (vnode) {
    var cloned = new VNode(
      vnode.tag,
      vnode.data,
      // #7975
      // clone children array to avoid mutating original in case of cloning
      // a child.
      vnode.children && vnode.children.slice(),
      vnode.text,
      vnode.elm,
      vnode.context,
      vnode.componentOptions,
      vnode.asyncFactory
    );
    cloned.ns = vnode.ns;
    cloned.isStatic = vnode.isStatic;
    cloned.key = vnode.key;
    cloned.isComment = vnode.isComment;
    cloned.fnContext = vnode.fnContext;
    cloned.fnOptions = vnode.fnOptions;
    cloned.fnScopeId = vnode.fnScopeId;
    cloned.asyncMeta = vnode.asyncMeta;
    cloned.isCloned = true;
    return cloned
  }

  /*
   * not type checking this file because flow doesn't play well with
   * dynamically accessing methods on Array prototype
   */

  var arrayProto = Array.prototype;
  var arrayMethods = Object.create(arrayProto);

  var methodsToPatch = [
    'push',
    'pop',
    'shift',
    'unshift',
    'splice',
    'sort',
    'reverse'
  ];

  /**
   * Intercept mutating methods and emit events
   */
  methodsToPatch.forEach(function (method) {
    // cache original method
    var original = arrayProto[method];
    def(arrayMethods, method, function mutator () {
      var args = [], len = arguments.length;
      while ( len-- ) args[ len ] = arguments[ len ];

      var result = original.apply(this, args);
      var ob = this.__ob__;
      var inserted;
      switch (method) {
        case 'push':
        case 'unshift':
          inserted = args;
          break
        case 'splice':
          inserted = args.slice(2);
          break
      }
      if (inserted) { ob.observeArray(inserted); }
      // notify change
      ob.dep.notify();
      return result
    });
  });

  /*  */

  var arrayKeys = Object.getOwnPropertyNames(arrayMethods);

  /**
   * In some cases we may want to disable observation inside a component's
   * update computation.
   */
  var shouldObserve = true;

  function toggleObserving (value) {
    shouldObserve = value;
  }

  /**
   * Observer class that is attached to each observed
   * object. Once attached, the observer converts the target
   * object's property keys into getter/setters that
   * collect dependencies and dispatch updates.
   */
  var Observer = function Observer (value) {
    this.value = value;
    this.dep = new Dep();
    this.vmCount = 0;
    def(value, '__ob__', this);
    if (Array.isArray(value)) {
      if (hasProto) {
        protoAugment(value, arrayMethods);
      } else {
        copyAugment(value, arrayMethods, arrayKeys);
      }
      this.observeArray(value);
    } else {
      this.walk(value);
    }
  };

  /**
   * Walk through all properties and convert them into
   * getter/setters. This method should only be called when
   * value type is Object.
   */
  Observer.prototype.walk = function walk (obj) {
    var keys = Object.keys(obj);
    for (var i = 0; i < keys.length; i++) {
      defineReactive$$1(obj, keys[i]);
    }
  };

  /**
   * Observe a list of Array items.
   */
  Observer.prototype.observeArray = function observeArray (items) {
    for (var i = 0, l = items.length; i < l; i++) {
      observe(items[i]);
    }
  };

  // helpers

  /**
   * Augment a target Object or Array by intercepting
   * the prototype chain using __proto__
   */
  function protoAugment (target, src) {
    /* eslint-disable no-proto */
    target.__proto__ = src;
    /* eslint-enable no-proto */
  }

  /**
   * Augment a target Object or Array by defining
   * hidden properties.
   */
  /* istanbul ignore next */
  function copyAugment (target, src, keys) {
    for (var i = 0, l = keys.length; i < l; i++) {
      var key = keys[i];
      def(target, key, src[key]);
    }
  }

  /**
   * Attempt to create an observer instance for a value,
   * returns the new observer if successfully observed,
   * or the existing observer if the value already has one.
   */
  function observe (value, asRootData) {
    if (!isObject(value) || value instanceof VNode) {
      return
    }
    var ob;
    if (hasOwn(value, '__ob__') && value.__ob__ instanceof Observer) {
      ob = value.__ob__;
    } else if (
      shouldObserve &&
      !isServerRendering() &&
      (Array.isArray(value) || isPlainObject(value)) &&
      Object.isExtensible(value) &&
      !value._isVue
    ) {
      ob = new Observer(value);
    }
    if (asRootData && ob) {
      ob.vmCount++;
    }
    return ob
  }

  /**
   * Define a reactive property on an Object.
   */
  function defineReactive$$1 (
    obj,
    key,
    val,
    customSetter,
    shallow
  ) {
    var dep = new Dep();

    var property = Object.getOwnPropertyDescriptor(obj, key);
    if (property && property.configurable === false) {
      return
    }

    // cater for pre-defined getter/setters
    var getter = property && property.get;
    var setter = property && property.set;
    if ((!getter || setter) && arguments.length === 2) {
      val = obj[key];
    }

    var childOb = !shallow && observe(val);
    Object.defineProperty(obj, key, {
      enumerable: true,
      configurable: true,
      get: function reactiveGetter () {
        var value = getter ? getter.call(obj) : val;
        if (Dep.target) {
          dep.depend();
          if (childOb) {
            childOb.dep.depend();
            if (Array.isArray(value)) {
              dependArray(value);
            }
          }
        }
        return value
      },
      set: function reactiveSetter (newVal) {
        var value = getter ? getter.call(obj) : val;
        /* eslint-disable no-self-compare */
        if (newVal === value || (newVal !== newVal && value !== value)) {
          return
        }
        /* eslint-enable no-self-compare */
        if (customSetter) {
          customSetter();
        }
        // #7981: for accessor properties without setter
        if (getter && !setter) { return }
        if (setter) {
          setter.call(obj, newVal);
        } else {
          val = newVal;
        }
        childOb = !shallow && observe(newVal);
        dep.notify();
      }
    });
  }

  /**
   * Set a property on an object. Adds the new property and
   * triggers change notification if the property doesn't
   * already exist.
   */
  function set (target, key, val) {
    if (isUndef(target) || isPrimitive(target)
    ) {
      warn(("Cannot set reactive property on undefined, null, or primitive value: " + ((target))));
    }
    if (Array.isArray(target) && isValidArrayIndex(key)) {
      target.length = Math.max(target.length, key);
      target.splice(key, 1, val);
      return val
    }
    if (key in target && !(key in Object.prototype)) {
      target[key] = val;
      return val
    }
    var ob = (target).__ob__;
    if (target._isVue || (ob && ob.vmCount)) {
      warn(
        'Avoid adding reactive properties to a Vue instance or its root $data ' +
        'at runtime - declare it upfront in the data option.'
      );
      return val
    }
    if (!ob) {
      target[key] = val;
      return val
    }
    defineReactive$$1(ob.value, key, val);
    ob.dep.notify();
    return val
  }

  /**
   * Delete a property and trigger change if necessary.
   */
  function del (target, key) {
    if (isUndef(target) || isPrimitive(target)
    ) {
      warn(("Cannot delete reactive property on undefined, null, or primitive value: " + ((target))));
    }
    if (Array.isArray(target) && isValidArrayIndex(key)) {
      target.splice(key, 1);
      return
    }
    var ob = (target).__ob__;
    if (target._isVue || (ob && ob.vmCount)) {
      warn(
        'Avoid deleting properties on a Vue instance or its root $data ' +
        '- just set it to null.'
      );
      return
    }
    if (!hasOwn(target, key)) {
      return
    }
    delete target[key];
    if (!ob) {
      return
    }
    ob.dep.notify();
  }

  /**
   * Collect dependencies on array elements when the array is touched, since
   * we cannot intercept array element access like property getters.
   */
  function dependArray (value) {
    for (var e = (void 0), i = 0, l = value.length; i < l; i++) {
      e = value[i];
      e && e.__ob__ && e.__ob__.dep.depend();
      if (Array.isArray(e)) {
        dependArray(e);
      }
    }
  }

  /*  */

  /**
   * Option overwriting strategies are functions that handle
   * how to merge a parent option value and a child option
   * value into the final value.
   */
  var strats = config.optionMergeStrategies;

  /**
   * Options with restrictions
   */
  {
    strats.el = strats.propsData = function (parent, child, vm, key) {
      if (!vm) {
        warn(
          "option \"" + key + "\" can only be used during instance " +
          'creation with the `new` keyword.'
        );
      }
      return defaultStrat(parent, child)
    };
  }

  /**
   * Helper that recursively merges two data objects together.
   */
  function mergeData (to, from) {
    if (!from) { return to }
    var key, toVal, fromVal;

    var keys = hasSymbol
      ? Reflect.ownKeys(from)
      : Object.keys(from);

    for (var i = 0; i < keys.length; i++) {
      key = keys[i];
      // in case the object is already observed...
      if (key === '__ob__') { continue }
      toVal = to[key];
      fromVal = from[key];
      if (!hasOwn(to, key)) {
        set(to, key, fromVal);
      } else if (
        toVal !== fromVal &&
        isPlainObject(toVal) &&
        isPlainObject(fromVal)
      ) {
        mergeData(toVal, fromVal);
      }
    }
    return to
  }

  /**
   * Data
   */
  function mergeDataOrFn (
    parentVal,
    childVal,
    vm
  ) {
    if (!vm) {
      // in a Vue.extend merge, both should be functions
      if (!childVal) {
        return parentVal
      }
      if (!parentVal) {
        return childVal
      }
      // when parentVal & childVal are both present,
      // we need to return a function that returns the
      // merged result of both functions... no need to
      // check if parentVal is a function here because
      // it has to be a function to pass previous merges.
      return function mergedDataFn () {
        return mergeData(
          typeof childVal === 'function' ? childVal.call(this, this) : childVal,
          typeof parentVal === 'function' ? parentVal.call(this, this) : parentVal
        )
      }
    } else {
      return function mergedInstanceDataFn () {
        // instance merge
        var instanceData = typeof childVal === 'function'
          ? childVal.call(vm, vm)
          : childVal;
        var defaultData = typeof parentVal === 'function'
          ? parentVal.call(vm, vm)
          : parentVal;
        if (instanceData) {
          return mergeData(instanceData, defaultData)
        } else {
          return defaultData
        }
      }
    }
  }

  strats.data = function (
    parentVal,
    childVal,
    vm
  ) {
    if (!vm) {
      if (childVal && typeof childVal !== 'function') {
        warn(
          'The "data" option should be a function ' +
          'that returns a per-instance value in component ' +
          'definitions.',
          vm
        );

        return parentVal
      }
      return mergeDataOrFn(parentVal, childVal)
    }

    return mergeDataOrFn(parentVal, childVal, vm)
  };

  /**
   * Hooks and props are merged as arrays.
   */
  function mergeHook (
    parentVal,
    childVal
  ) {
    var res = childVal
      ? parentVal
        ? parentVal.concat(childVal)
        : Array.isArray(childVal)
          ? childVal
          : [childVal]
      : parentVal;
    return res
      ? dedupeHooks(res)
      : res
  }

  function dedupeHooks (hooks) {
    var res = [];
    for (var i = 0; i < hooks.length; i++) {
      if (res.indexOf(hooks[i]) === -1) {
        res.push(hooks[i]);
      }
    }
    return res
  }

  LIFECYCLE_HOOKS.forEach(function (hook) {
    strats[hook] = mergeHook;
  });

  /**
   * Assets
   *
   * When a vm is present (instance creation), we need to do
   * a three-way merge between constructor options, instance
   * options and parent options.
   */
  function mergeAssets (
    parentVal,
    childVal,
    vm,
    key
  ) {
    var res = Object.create(parentVal || null);
    if (childVal) {
      assertObjectType(key, childVal, vm);
      return extend(res, childVal)
    } else {
      return res
    }
  }

  ASSET_TYPES.forEach(function (type) {
    strats[type + 's'] = mergeAssets;
  });

  /**
   * Watchers.
   *
   * Watchers hashes should not overwrite one
   * another, so we merge them as arrays.
   */
  strats.watch = function (
    parentVal,
    childVal,
    vm,
    key
  ) {
    // work around Firefox's Object.prototype.watch...
    if (parentVal === nativeWatch) { parentVal = undefined; }
    if (childVal === nativeWatch) { childVal = undefined; }
    /* istanbul ignore if */
    if (!childVal) { return Object.create(parentVal || null) }
    {
      assertObjectType(key, childVal, vm);
    }
    if (!parentVal) { return childVal }
    var ret = {};
    extend(ret, parentVal);
    for (var key$1 in childVal) {
      var parent = ret[key$1];
      var child = childVal[key$1];
      if (parent && !Array.isArray(parent)) {
        parent = [parent];
      }
      ret[key$1] = parent
        ? parent.concat(child)
        : Array.isArray(child) ? child : [child];
    }
    return ret
  };

  /**
   * Other object hashes.
   */
  strats.props =
  strats.methods =
  strats.inject =
  strats.computed = function (
    parentVal,
    childVal,
    vm,
    key
  ) {
    if (childVal && "development" !== 'production') {
      assertObjectType(key, childVal, vm);
    }
    if (!parentVal) { return childVal }
    var ret = Object.create(null);
    extend(ret, parentVal);
    if (childVal) { extend(ret, childVal); }
    return ret
  };
  strats.provide = mergeDataOrFn;

  /**
   * Default strategy.
   */
  var defaultStrat = function (parentVal, childVal) {
    return childVal === undefined
      ? parentVal
      : childVal
  };

  /**
   * Validate component names
   */
  function checkComponents (options) {
    for (var key in options.components) {
      validateComponentName(key);
    }
  }

  function validateComponentName (name) {
    if (!new RegExp(("^[a-zA-Z][\\-\\.0-9_" + (unicodeRegExp.source) + "]*$")).test(name)) {
      warn(
        'Invalid component name: "' + name + '". Component names ' +
        'should conform to valid custom element name in html5 specification.'
      );
    }
    if (isBuiltInTag(name) || config.isReservedTag(name)) {
      warn(
        'Do not use built-in or reserved HTML elements as component ' +
        'id: ' + name
      );
    }
  }

  /**
   * Ensure all props option syntax are normalized into the
   * Object-based format.
   */
  function normalizeProps (options, vm) {
    var props = options.props;
    if (!props) { return }
    var res = {};
    var i, val, name;
    if (Array.isArray(props)) {
      i = props.length;
      while (i--) {
        val = props[i];
        if (typeof val === 'string') {
          name = camelize(val);
          res[name] = { type: null };
        } else {
          warn('props must be strings when using array syntax.');
        }
      }
    } else if (isPlainObject(props)) {
      for (var key in props) {
        val = props[key];
        name = camelize(key);
        res[name] = isPlainObject(val)
          ? val
          : { type: val };
      }
    } else {
      warn(
        "Invalid value for option \"props\": expected an Array or an Object, " +
        "but got " + (toRawType(props)) + ".",
        vm
      );
    }
    options.props = res;
  }

  /**
   * Normalize all injections into Object-based format
   */
  function normalizeInject (options, vm) {
    var inject = options.inject;
    if (!inject) { return }
    var normalized = options.inject = {};
    if (Array.isArray(inject)) {
      for (var i = 0; i < inject.length; i++) {
        normalized[inject[i]] = { from: inject[i] };
      }
    } else if (isPlainObject(inject)) {
      for (var key in inject) {
        var val = inject[key];
        normalized[key] = isPlainObject(val)
          ? extend({ from: key }, val)
          : { from: val };
      }
    } else {
      warn(
        "Invalid value for option \"inject\": expected an Array or an Object, " +
        "but got " + (toRawType(inject)) + ".",
        vm
      );
    }
  }

  /**
   * Normalize raw function directives into object format.
   */
  function normalizeDirectives (options) {
    var dirs = options.directives;
    if (dirs) {
      for (var key in dirs) {
        var def$$1 = dirs[key];
        if (typeof def$$1 === 'function') {
          dirs[key] = { bind: def$$1, update: def$$1 };
        }
      }
    }
  }

  function assertObjectType (name, value, vm) {
    if (!isPlainObject(value)) {
      warn(
        "Invalid value for option \"" + name + "\": expected an Object, " +
        "but got " + (toRawType(value)) + ".",
        vm
      );
    }
  }

  /**
   * Merge two option objects into a new one.
   * Core utility used in both instantiation and inheritance.
   */
  function mergeOptions (
    parent,
    child,
    vm
  ) {
    {
      checkComponents(child);
    }

    if (typeof child === 'function') {
      child = child.options;
    }

    normalizeProps(child, vm);
    normalizeInject(child, vm);
    normalizeDirectives(child);

    // Apply extends and mixins on the child options,
    // but only if it is a raw options object that isn't
    // the result of another mergeOptions call.
    // Only merged options has the _base property.
    if (!child._base) {
      if (child.extends) {
        parent = mergeOptions(parent, child.extends, vm);
      }
      if (child.mixins) {
        for (var i = 0, l = child.mixins.length; i < l; i++) {
          parent = mergeOptions(parent, child.mixins[i], vm);
        }
      }
    }

    var options = {};
    var key;
    for (key in parent) {
      mergeField(key);
    }
    for (key in child) {
      if (!hasOwn(parent, key)) {
        mergeField(key);
      }
    }
    function mergeField (key) {
      var strat = strats[key] || defaultStrat;
      options[key] = strat(parent[key], child[key], vm, key);
    }
    return options
  }

  /**
   * Resolve an asset.
   * This function is used because child instances need access
   * to assets defined in its ancestor chain.
   */
  function resolveAsset (
    options,
    type,
    id,
    warnMissing
  ) {
    /* istanbul ignore if */
    if (typeof id !== 'string') {
      return
    }
    var assets = options[type];
    // check local registration variations first
    if (hasOwn(assets, id)) { return assets[id] }
    var camelizedId = camelize(id);
    if (hasOwn(assets, camelizedId)) { return assets[camelizedId] }
    var PascalCaseId = capitalize(camelizedId);
    if (hasOwn(assets, PascalCaseId)) { return assets[PascalCaseId] }
    // fallback to prototype chain
    var res = assets[id] || assets[camelizedId] || assets[PascalCaseId];
    if (warnMissing && !res) {
      warn(
        'Failed to resolve ' + type.slice(0, -1) + ': ' + id,
        options
      );
    }
    return res
  }

  /*  */



  function validateProp (
    key,
    propOptions,
    propsData,
    vm
  ) {
    var prop = propOptions[key];
    var absent = !hasOwn(propsData, key);
    var value = propsData[key];
    // boolean casting
    var booleanIndex = getTypeIndex(Boolean, prop.type);
    if (booleanIndex > -1) {
      if (absent && !hasOwn(prop, 'default')) {
        value = false;
      } else if (value === '' || value === hyphenate(key)) {
        // only cast empty string / same name to boolean if
        // boolean has higher priority
        var stringIndex = getTypeIndex(String, prop.type);
        if (stringIndex < 0 || booleanIndex < stringIndex) {
          value = true;
        }
      }
    }
    // check default value
    if (value === undefined) {
      value = getPropDefaultValue(vm, prop, key);
      // since the default value is a fresh copy,
      // make sure to observe it.
      var prevShouldObserve = shouldObserve;
      toggleObserving(true);
      observe(value);
      toggleObserving(prevShouldObserve);
    }
    {
      assertProp(prop, key, value, vm, absent);
    }
    return value
  }

  /**
   * Get the default value of a prop.
   */
  function getPropDefaultValue (vm, prop, key) {
    // no default, return undefined
    if (!hasOwn(prop, 'default')) {
      return undefined
    }
    var def = prop.default;
    // warn against non-factory defaults for Object & Array
    if (isObject(def)) {
      warn(
        'Invalid default value for prop "' + key + '": ' +
        'Props with type Object/Array must use a factory function ' +
        'to return the default value.',
        vm
      );
    }
    // the raw prop value was also undefined from previous render,
    // return previous default value to avoid unnecessary watcher trigger
    if (vm && vm.$options.propsData &&
      vm.$options.propsData[key] === undefined &&
      vm._props[key] !== undefined
    ) {
      return vm._props[key]
    }
    // call factory function for non-Function types
    // a value is Function if its prototype is function even across different execution context
    return typeof def === 'function' && getType(prop.type) !== 'Function'
      ? def.call(vm)
      : def
  }

  /**
   * Assert whether a prop is valid.
   */
  function assertProp (
    prop,
    name,
    value,
    vm,
    absent
  ) {
    if (prop.required && absent) {
      warn(
        'Missing required prop: "' + name + '"',
        vm
      );
      return
    }
    if (value == null && !prop.required) {
      return
    }
    var type = prop.type;
    var valid = !type || type === true;
    var expectedTypes = [];
    if (type) {
      if (!Array.isArray(type)) {
        type = [type];
      }
      for (var i = 0; i < type.length && !valid; i++) {
        var assertedType = assertType(value, type[i]);
        expectedTypes.push(assertedType.expectedType || '');
        valid = assertedType.valid;
      }
    }

    if (!valid) {
      warn(
        getInvalidTypeMessage(name, value, expectedTypes),
        vm
      );
      return
    }
    var validator = prop.validator;
    if (validator) {
      if (!validator(value)) {
        warn(
          'Invalid prop: custom validator check failed for prop "' + name + '".',
          vm
        );
      }
    }
  }

  var simpleCheckRE = /^(String|Number|Boolean|Function|Symbol)$/;

  function assertType (value, type) {
    var valid;
    var expectedType = getType(type);
    if (simpleCheckRE.test(expectedType)) {
      var t = typeof value;
      valid = t === expectedType.toLowerCase();
      // for primitive wrapper objects
      if (!valid && t === 'object') {
        valid = value instanceof type;
      }
    } else if (expectedType === 'Object') {
      valid = isPlainObject(value);
    } else if (expectedType === 'Array') {
      valid = Array.isArray(value);
    } else {
      valid = value instanceof type;
    }
    return {
      valid: valid,
      expectedType: expectedType
    }
  }

  /**
   * Use function string name to check built-in types,
   * because a simple equality check will fail when running
   * across different vms / iframes.
   */
  function getType (fn) {
    var match = fn && fn.toString().match(/^\s*function (\w+)/);
    return match ? match[1] : ''
  }

  function isSameType (a, b) {
    return getType(a) === getType(b)
  }

  function getTypeIndex (type, expectedTypes) {
    if (!Array.isArray(expectedTypes)) {
      return isSameType(expectedTypes, type) ? 0 : -1
    }
    for (var i = 0, len = expectedTypes.length; i < len; i++) {
      if (isSameType(expectedTypes[i], type)) {
        return i
      }
    }
    return -1
  }

  function getInvalidTypeMessage (name, value, expectedTypes) {
    var message = "Invalid prop: type check failed for prop \"" + name + "\"." +
      " Expected " + (expectedTypes.map(capitalize).join(', '));
    var expectedType = expectedTypes[0];
    var receivedType = toRawType(value);
    var expectedValue = styleValue(value, expectedType);
    var receivedValue = styleValue(value, receivedType);
    // check if we need to specify expected value
    if (expectedTypes.length === 1 &&
        isExplicable(expectedType) &&
        !isBoolean(expectedType, receivedType)) {
      message += " with value " + expectedValue;
    }
    message += ", got " + receivedType + " ";
    // check if we need to specify received value
    if (isExplicable(receivedType)) {
      message += "with value " + receivedValue + ".";
    }
    return message
  }

  function styleValue (value, type) {
    if (type === 'String') {
      return ("\"" + value + "\"")
    } else if (type === 'Number') {
      return ("" + (Number(value)))
    } else {
      return ("" + value)
    }
  }

  function isExplicable (value) {
    var explicitTypes = ['string', 'number', 'boolean'];
    return explicitTypes.some(function (elem) { return value.toLowerCase() === elem; })
  }

  function isBoolean () {
    var args = [], len = arguments.length;
    while ( len-- ) args[ len ] = arguments[ len ];

    return args.some(function (elem) { return elem.toLowerCase() === 'boolean'; })
  }

  /*  */

  function handleError (err, vm, info) {
    // Deactivate deps tracking while processing error handler to avoid possible infinite rendering.
    // See: https://github.com/vuejs/vuex/issues/1505
    pushTarget();
    try {
      if (vm) {
        var cur = vm;
        while ((cur = cur.$parent)) {
          var hooks = cur.$options.errorCaptured;
          if (hooks) {
            for (var i = 0; i < hooks.length; i++) {
              try {
                var capture = hooks[i].call(cur, err, vm, info) === false;
                if (capture) { return }
              } catch (e) {
                globalHandleError(e, cur, 'errorCaptured hook');
              }
            }
          }
        }
      }
      globalHandleError(err, vm, info);
    } finally {
      popTarget();
    }
  }

  function invokeWithErrorHandling (
    handler,
    context,
    args,
    vm,
    info
  ) {
    var res;
    try {
      res = args ? handler.apply(context, args) : handler.call(context);
      if (res && !res._isVue && isPromise(res) && !res._handled) {
        res.catch(function (e) { return handleError(e, vm, info + " (Promise/async)"); });
        // issue #9511
        // avoid catch triggering multiple times when nested calls
        res._handled = true;
      }
    } catch (e) {
      handleError(e, vm, info);
    }
    return res
  }

  function globalHandleError (err, vm, info) {
    if (config.errorHandler) {
      try {
        return config.errorHandler.call(null, err, vm, info)
      } catch (e) {
        // if the user intentionally throws the original error in the handler,
        // do not log it twice
        if (e !== err) {
          logError(e, null, 'config.errorHandler');
        }
      }
    }
    logError(err, vm, info);
  }

  function logError (err, vm, info) {
    {
      warn(("Error in " + info + ": \"" + (err.toString()) + "\""), vm);
    }
    /* istanbul ignore else */
    if ((inBrowser || inWeex) && typeof console !== 'undefined') {
      console.error(err);
    } else {
      throw err
    }
  }

  /*  */

  var isUsingMicroTask = false;

  var callbacks = [];
  var pending = false;

  function flushCallbacks () {
    pending = false;
    var copies = callbacks.slice(0);
    callbacks.length = 0;
    for (var i = 0; i < copies.length; i++) {
      copies[i]();
    }
  }

  // Here we have async deferring wrappers using microtasks.
  // In 2.5 we used (macro) tasks (in combination with microtasks).
  // However, it has subtle problems when state is changed right before repaint
  // (e.g. #6813, out-in transitions).
  // Also, using (macro) tasks in event handler would cause some weird behaviors
  // that cannot be circumvented (e.g. #7109, #7153, #7546, #7834, #8109).
  // So we now use microtasks everywhere, again.
  // A major drawback of this tradeoff is that there are some scenarios
  // where microtasks have too high a priority and fire in between supposedly
  // sequential events (e.g. #4521, #6690, which have workarounds)
  // or even between bubbling of the same event (#6566).
  var timerFunc;

  // The nextTick behavior leverages the microtask queue, which can be accessed
  // via either native Promise.then or MutationObserver.
  // MutationObserver has wider support, however it is seriously bugged in
  // UIWebView in iOS >= 9.3.3 when triggered in touch event handlers. It
  // completely stops working after triggering a few times... so, if native
  // Promise is available, we will use it:
  /* istanbul ignore next, $flow-disable-line */
  if (typeof Promise !== 'undefined' && isNative(Promise)) {
    var p = Promise.resolve();
    timerFunc = function () {
      p.then(flushCallbacks);
      // In problematic UIWebViews, Promise.then doesn't completely break, but
      // it can get stuck in a weird state where callbacks are pushed into the
      // microtask queue but the queue isn't being flushed, until the browser
      // needs to do some other work, e.g. handle a timer. Therefore we can
      // "force" the microtask queue to be flushed by adding an empty timer.
      if (isIOS) { setTimeout(noop); }
    };
    isUsingMicroTask = true;
  } else if (!isIE && typeof MutationObserver !== 'undefined' && (
    isNative(MutationObserver) ||
    // PhantomJS and iOS 7.x
    MutationObserver.toString() === '[object MutationObserverConstructor]'
  )) {
    // Use MutationObserver where native Promise is not available,
    // e.g. PhantomJS, iOS7, Android 4.4
    // (#6466 MutationObserver is unreliable in IE11)
    var counter = 1;
    var observer = new MutationObserver(flushCallbacks);
    var textNode = document.createTextNode(String(counter));
    observer.observe(textNode, {
      characterData: true
    });
    timerFunc = function () {
      counter = (counter + 1) % 2;
      textNode.data = String(counter);
    };
    isUsingMicroTask = true;
  } else if (typeof setImmediate !== 'undefined' && isNative(setImmediate)) {
    // Fallback to setImmediate.
    // Technically it leverages the (macro) task queue,
    // but it is still a better choice than setTimeout.
    timerFunc = function () {
      setImmediate(flushCallbacks);
    };
  } else {
    // Fallback to setTimeout.
    timerFunc = function () {
      setTimeout(flushCallbacks, 0);
    };
  }

  function nextTick (cb, ctx) {
    var _resolve;
    callbacks.push(function () {
      if (cb) {
        try {
          cb.call(ctx);
        } catch (e) {
          handleError(e, ctx, 'nextTick');
        }
      } else if (_resolve) {
        _resolve(ctx);
      }
    });
    if (!pending) {
      pending = true;
      timerFunc();
    }
    // $flow-disable-line
    if (!cb && typeof Promise !== 'undefined') {
      return new Promise(function (resolve) {
        _resolve = resolve;
      })
    }
  }

  /*  */

  /* not type checking this file because flow doesn't play well with Proxy */

  var initProxy;

  {
    var allowedGlobals = makeMap(
      'Infinity,undefined,NaN,isFinite,isNaN,' +
      'parseFloat,parseInt,decodeURI,decodeURIComponent,encodeURI,encodeURIComponent,' +
      'Math,Number,Date,Array,Object,Boolean,String,RegExp,Map,Set,JSON,Intl,' +
      'require' // for Webpack/Browserify
    );

    var warnNonPresent = function (target, key) {
      warn(
        "Property or method \"" + key + "\" is not defined on the instance but " +
        'referenced during render. Make sure that this property is reactive, ' +
        'either in the data option, or for class-based components, by ' +
        'initializing the property. ' +
        'See: https://vuejs.org/v2/guide/reactivity.html#Declaring-Reactive-Properties.',
        target
      );
    };

    var warnReservedPrefix = function (target, key) {
      warn(
        "Property \"" + key + "\" must be accessed with \"$data." + key + "\" because " +
        'properties starting with "$" or "_" are not proxied in the Vue instance to ' +
        'prevent conflicts with Vue internals. ' +
        'See: https://vuejs.org/v2/api/#data',
        target
      );
    };

    var hasProxy =
      typeof Proxy !== 'undefined' && isNative(Proxy);

    if (hasProxy) {
      var isBuiltInModifier = makeMap('stop,prevent,self,ctrl,shift,alt,meta,exact');
      config.keyCodes = new Proxy(config.keyCodes, {
        set: function set (target, key, value) {
          if (isBuiltInModifier(key)) {
            warn(("Avoid overwriting built-in modifier in config.keyCodes: ." + key));
            return false
          } else {
            target[key] = value;
            return true
          }
        }
      });
    }

    var hasHandler = {
      has: function has (target, key) {
        var has = key in target;
        var isAllowed = allowedGlobals(key) ||
          (typeof key === 'string' && key.charAt(0) === '_' && !(key in target.$data));
        if (!has && !isAllowed) {
          if (key in target.$data) { warnReservedPrefix(target, key); }
          else { warnNonPresent(target, key); }
        }
        return has || !isAllowed
      }
    };

    var getHandler = {
      get: function get (target, key) {
        if (typeof key === 'string' && !(key in target)) {
          if (key in target.$data) { warnReservedPrefix(target, key); }
          else { warnNonPresent(target, key); }
        }
        return target[key]
      }
    };

    initProxy = function initProxy (vm) {
      if (hasProxy) {
        // determine which proxy handler to use
        var options = vm.$options;
        var handlers = options.render && options.render._withStripped
          ? getHandler
          : hasHandler;
        vm._renderProxy = new Proxy(vm, handlers);
      } else {
        vm._renderProxy = vm;
      }
    };
  }

  /*  */

  var seenObjects = new _Set();

  /**
   * Recursively traverse an object to evoke all converted
   * getters, so that every nested property inside the object
   * is collected as a "deep" dependency.
   */
  function traverse (val) {
    _traverse(val, seenObjects);
    seenObjects.clear();
  }

  function _traverse (val, seen) {
    var i, keys;
    var isA = Array.isArray(val);
    if ((!isA && !isObject(val)) || Object.isFrozen(val) || val instanceof VNode) {
      return
    }
    if (val.__ob__) {
      var depId = val.__ob__.dep.id;
      if (seen.has(depId)) {
        return
      }
      seen.add(depId);
    }
    if (isA) {
      i = val.length;
      while (i--) { _traverse(val[i], seen); }
    } else {
      keys = Object.keys(val);
      i = keys.length;
      while (i--) { _traverse(val[keys[i]], seen); }
    }
  }

  var mark;
  var measure;

  {
    var perf = inBrowser && window.performance;
    /* istanbul ignore if */
    if (
      perf &&
      perf.mark &&
      perf.measure &&
      perf.clearMarks &&
      perf.clearMeasures
    ) {
      mark = function (tag) { return perf.mark(tag); };
      measure = function (name, startTag, endTag) {
        perf.measure(name, startTag, endTag);
        perf.clearMarks(startTag);
        perf.clearMarks(endTag);
        // perf.clearMeasures(name)
      };
    }
  }

  /*  */

  var normalizeEvent = cached(function (name) {
    var passive = name.charAt(0) === '&';
    name = passive ? name.slice(1) : name;
    var once$$1 = name.charAt(0) === '~'; // Prefixed last, checked first
    name = once$$1 ? name.slice(1) : name;
    var capture = name.charAt(0) === '!';
    name = capture ? name.slice(1) : name;
    return {
      name: name,
      once: once$$1,
      capture: capture,
      passive: passive
    }
  });

  function createFnInvoker (fns, vm) {
    function invoker () {
      var arguments$1 = arguments;

      var fns = invoker.fns;
      if (Array.isArray(fns)) {
        var cloned = fns.slice();
        for (var i = 0; i < cloned.length; i++) {
          invokeWithErrorHandling(cloned[i], null, arguments$1, vm, "v-on handler");
        }
      } else {
        // return handler return value for single handlers
        return invokeWithErrorHandling(fns, null, arguments, vm, "v-on handler")
      }
    }
    invoker.fns = fns;
    return invoker
  }

  function updateListeners (
    on,
    oldOn,
    add,
    remove$$1,
    createOnceHandler,
    vm
  ) {
    var name, def$$1, cur, old, event;
    for (name in on) {
      def$$1 = cur = on[name];
      old = oldOn[name];
      event = normalizeEvent(name);
      if (isUndef(cur)) {
        warn(
          "Invalid handler for event \"" + (event.name) + "\": got " + String(cur),
          vm
        );
      } else if (isUndef(old)) {
        if (isUndef(cur.fns)) {
          cur = on[name] = createFnInvoker(cur, vm);
        }
        if (isTrue(event.once)) {
          cur = on[name] = createOnceHandler(event.name, cur, event.capture);
        }
        add(event.name, cur, event.capture, event.passive, event.params);
      } else if (cur !== old) {
        old.fns = cur;
        on[name] = old;
      }
    }
    for (name in oldOn) {
      if (isUndef(on[name])) {
        event = normalizeEvent(name);
        remove$$1(event.name, oldOn[name], event.capture);
      }
    }
  }

  /*  */

  function mergeVNodeHook (def, hookKey, hook) {
    if (def instanceof VNode) {
      def = def.data.hook || (def.data.hook = {});
    }
    var invoker;
    var oldHook = def[hookKey];

    function wrappedHook () {
      hook.apply(this, arguments);
      // important: remove merged hook to ensure it's called only once
      // and prevent memory leak
      remove(invoker.fns, wrappedHook);
    }

    if (isUndef(oldHook)) {
      // no existing hook
      invoker = createFnInvoker([wrappedHook]);
    } else {
      /* istanbul ignore if */
      if (isDef(oldHook.fns) && isTrue(oldHook.merged)) {
        // already a merged invoker
        invoker = oldHook;
        invoker.fns.push(wrappedHook);
      } else {
        // existing plain hook
        invoker = createFnInvoker([oldHook, wrappedHook]);
      }
    }

    invoker.merged = true;
    def[hookKey] = invoker;
  }

  /*  */

  function extractPropsFromVNodeData (
    data,
    Ctor,
    tag
  ) {
    // we are only extracting raw values here.
    // validation and default values are handled in the child
    // component itself.
    var propOptions = Ctor.options.props;
    if (isUndef(propOptions)) {
      return
    }
    var res = {};
    var attrs = data.attrs;
    var props = data.props;
    if (isDef(attrs) || isDef(props)) {
      for (var key in propOptions) {
        var altKey = hyphenate(key);
        {
          var keyInLowerCase = key.toLowerCase();
          if (
            key !== keyInLowerCase &&
            attrs && hasOwn(attrs, keyInLowerCase)
          ) {
            tip(
              "Prop \"" + keyInLowerCase + "\" is passed to component " +
              (formatComponentName(tag || Ctor)) + ", but the declared prop name is" +
              " \"" + key + "\". " +
              "Note that HTML attributes are case-insensitive and camelCased " +
              "props need to use their kebab-case equivalents when using in-DOM " +
              "templates. You should probably use \"" + altKey + "\" instead of \"" + key + "\"."
            );
          }
        }
        checkProp(res, props, key, altKey, true) ||
        checkProp(res, attrs, key, altKey, false);
      }
    }
    return res
  }

  function checkProp (
    res,
    hash,
    key,
    altKey,
    preserve
  ) {
    if (isDef(hash)) {
      if (hasOwn(hash, key)) {
        res[key] = hash[key];
        if (!preserve) {
          delete hash[key];
        }
        return true
      } else if (hasOwn(hash, altKey)) {
        res[key] = hash[altKey];
        if (!preserve) {
          delete hash[altKey];
        }
        return true
      }
    }
    return false
  }

  /*  */

  // The template compiler attempts to minimize the need for normalization by
  // statically analyzing the template at compile time.
  //
  // For plain HTML markup, normalization can be completely skipped because the
  // generated render function is guaranteed to return Array<VNode>. There are
  // two cases where extra normalization is needed:

  // 1. When the children contains components - because a functional component
  // may return an Array instead of a single root. In this case, just a simple
  // normalization is needed - if any child is an Array, we flatten the whole
  // thing with Array.prototype.concat. It is guaranteed to be only 1-level deep
  // because functional components already normalize their own children.
  function simpleNormalizeChildren (children) {
    for (var i = 0; i < children.length; i++) {
      if (Array.isArray(children[i])) {
        return Array.prototype.concat.apply([], children)
      }
    }
    return children
  }

  // 2. When the children contains constructs that always generated nested Arrays,
  // e.g. <template>, <slot>, v-for, or when the children is provided by user
  // with hand-written render functions / JSX. In such cases a full normalization
  // is needed to cater to all possible types of children values.
  function normalizeChildren (children) {
    return isPrimitive(children)
      ? [createTextVNode(children)]
      : Array.isArray(children)
        ? normalizeArrayChildren(children)
        : undefined
  }

  function isTextNode (node) {
    return isDef(node) && isDef(node.text) && isFalse(node.isComment)
  }

  function normalizeArrayChildren (children, nestedIndex) {
    var res = [];
    var i, c, lastIndex, last;
    for (i = 0; i < children.length; i++) {
      c = children[i];
      if (isUndef(c) || typeof c === 'boolean') { continue }
      lastIndex = res.length - 1;
      last = res[lastIndex];
      //  nested
      if (Array.isArray(c)) {
        if (c.length > 0) {
          c = normalizeArrayChildren(c, ((nestedIndex || '') + "_" + i));
          // merge adjacent text nodes
          if (isTextNode(c[0]) && isTextNode(last)) {
            res[lastIndex] = createTextVNode(last.text + (c[0]).text);
            c.shift();
          }
          res.push.apply(res, c);
        }
      } else if (isPrimitive(c)) {
        if (isTextNode(last)) {
          // merge adjacent text nodes
          // this is necessary for SSR hydration because text nodes are
          // essentially merged when rendered to HTML strings
          res[lastIndex] = createTextVNode(last.text + c);
        } else if (c !== '') {
          // convert primitive to vnode
          res.push(createTextVNode(c));
        }
      } else {
        if (isTextNode(c) && isTextNode(last)) {
          // merge adjacent text nodes
          res[lastIndex] = createTextVNode(last.text + c.text);
        } else {
          // default key for nested array children (likely generated by v-for)
          if (isTrue(children._isVList) &&
            isDef(c.tag) &&
            isUndef(c.key) &&
            isDef(nestedIndex)) {
            c.key = "__vlist" + nestedIndex + "_" + i + "__";
          }
          res.push(c);
        }
      }
    }
    return res
  }

  /*  */

  function initProvide (vm) {
    var provide = vm.$options.provide;
    if (provide) {
      vm._provided = typeof provide === 'function'
        ? provide.call(vm)
        : provide;
    }
  }

  function initInjections (vm) {
    var result = resolveInject(vm.$options.inject, vm);
    if (result) {
      toggleObserving(false);
      Object.keys(result).forEach(function (key) {
        /* istanbul ignore else */
        {
          defineReactive$$1(vm, key, result[key], function () {
            warn(
              "Avoid mutating an injected value directly since the changes will be " +
              "overwritten whenever the provided component re-renders. " +
              "injection being mutated: \"" + key + "\"",
              vm
            );
          });
        }
      });
      toggleObserving(true);
    }
  }

  function resolveInject (inject, vm) {
    if (inject) {
      // inject is :any because flow is not smart enough to figure out cached
      var result = Object.create(null);
      var keys = hasSymbol
        ? Reflect.ownKeys(inject)
        : Object.keys(inject);

      for (var i = 0; i < keys.length; i++) {
        var key = keys[i];
        // #6574 in case the inject object is observed...
        if (key === '__ob__') { continue }
        var provideKey = inject[key].from;
        var source = vm;
        while (source) {
          if (source._provided && hasOwn(source._provided, provideKey)) {
            result[key] = source._provided[provideKey];
            break
          }
          source = source.$parent;
        }
        if (!source) {
          if ('default' in inject[key]) {
            var provideDefault = inject[key].default;
            result[key] = typeof provideDefault === 'function'
              ? provideDefault.call(vm)
              : provideDefault;
          } else {
            warn(("Injection \"" + key + "\" not found"), vm);
          }
        }
      }
      return result
    }
  }

  /*  */



  /**
   * Runtime helper for resolving raw children VNodes into a slot object.
   */
  function resolveSlots (
    children,
    context
  ) {
    if (!children || !children.length) {
      return {}
    }
    var slots = {};
    for (var i = 0, l = children.length; i < l; i++) {
      var child = children[i];
      var data = child.data;
      // remove slot attribute if the node is resolved as a Vue slot node
      if (data && data.attrs && data.attrs.slot) {
        delete data.attrs.slot;
      }
      // named slots should only be respected if the vnode was rendered in the
      // same context.
      if ((child.context === context || child.fnContext === context) &&
        data && data.slot != null
      ) {
        var name = data.slot;
        var slot = (slots[name] || (slots[name] = []));
        if (child.tag === 'template') {
          slot.push.apply(slot, child.children || []);
        } else {
          slot.push(child);
        }
      } else {
        (slots.default || (slots.default = [])).push(child);
      }
    }
    // ignore slots that contains only whitespace
    for (var name$1 in slots) {
      if (slots[name$1].every(isWhitespace)) {
        delete slots[name$1];
      }
    }
    return slots
  }

  function isWhitespace (node) {
    return (node.isComment && !node.asyncFactory) || node.text === ' '
  }

  /*  */

  function normalizeScopedSlots (
    slots,
    normalSlots,
    prevSlots
  ) {
    var res;
    var hasNormalSlots = Object.keys(normalSlots).length > 0;
    var isStable = slots ? !!slots.$stable : !hasNormalSlots;
    var key = slots && slots.$key;
    if (!slots) {
      res = {};
    } else if (slots._normalized) {
      // fast path 1: child component re-render only, parent did not change
      return slots._normalized
    } else if (
      isStable &&
      prevSlots &&
      prevSlots !== emptyObject &&
      key === prevSlots.$key &&
      !hasNormalSlots &&
      !prevSlots.$hasNormal
    ) {
      // fast path 2: stable scoped slots w/ no normal slots to proxy,
      // only need to normalize once
      return prevSlots
    } else {
      res = {};
      for (var key$1 in slots) {
        if (slots[key$1] && key$1[0] !== '$') {
          res[key$1] = normalizeScopedSlot(normalSlots, key$1, slots[key$1]);
        }
      }
    }
    // expose normal slots on scopedSlots
    for (var key$2 in normalSlots) {
      if (!(key$2 in res)) {
        res[key$2] = proxyNormalSlot(normalSlots, key$2);
      }
    }
    // avoriaz seems to mock a non-extensible $scopedSlots object
    // and when that is passed down this would cause an error
    if (slots && Object.isExtensible(slots)) {
      (slots)._normalized = res;
    }
    def(res, '$stable', isStable);
    def(res, '$key', key);
    def(res, '$hasNormal', hasNormalSlots);
    return res
  }

  function normalizeScopedSlot(normalSlots, key, fn) {
    var normalized = function () {
      var res = arguments.length ? fn.apply(null, arguments) : fn({});
      res = res && typeof res === 'object' && !Array.isArray(res)
        ? [res] // single vnode
        : normalizeChildren(res);
      return res && (
        res.length === 0 ||
        (res.length === 1 && res[0].isComment) // #9658
      ) ? undefined
        : res
    };
    // this is a slot using the new v-slot syntax without scope. although it is
    // compiled as a scoped slot, render fn users would expect it to be present
    // on this.$slots because the usage is semantically a normal slot.
    if (fn.proxy) {
      Object.defineProperty(normalSlots, key, {
        get: normalized,
        enumerable: true,
        configurable: true
      });
    }
    return normalized
  }

  function proxyNormalSlot(slots, key) {
    return function () { return slots[key]; }
  }

  /*  */

  /**
   * Runtime helper for rendering v-for lists.
   */
  function renderList (
    val,
    render
  ) {
    var ret, i, l, keys, key;
    if (Array.isArray(val) || typeof val === 'string') {
      ret = new Array(val.length);
      for (i = 0, l = val.length; i < l; i++) {
        ret[i] = render(val[i], i);
      }
    } else if (typeof val === 'number') {
      ret = new Array(val);
      for (i = 0; i < val; i++) {
        ret[i] = render(i + 1, i);
      }
    } else if (isObject(val)) {
      if (hasSymbol && val[Symbol.iterator]) {
        ret = [];
        var iterator = val[Symbol.iterator]();
        var result = iterator.next();
        while (!result.done) {
          ret.push(render(result.value, ret.length));
          result = iterator.next();
        }
      } else {
        keys = Object.keys(val);
        ret = new Array(keys.length);
        for (i = 0, l = keys.length; i < l; i++) {
          key = keys[i];
          ret[i] = render(val[key], key, i);
        }
      }
    }
    if (!isDef(ret)) {
      ret = [];
    }
    (ret)._isVList = true;
    return ret
  }

  /*  */

  /**
   * Runtime helper for rendering <slot>
   */
  function renderSlot (
    name,
    fallback,
    props,
    bindObject
  ) {
    var scopedSlotFn = this.$scopedSlots[name];
    var nodes;
    if (scopedSlotFn) { // scoped slot
      props = props || {};
      if (bindObject) {
        if (!isObject(bindObject)) {
          warn(
            'slot v-bind without argument expects an Object',
            this
          );
        }
        props = extend(extend({}, bindObject), props);
      }
      nodes = scopedSlotFn(props) || fallback;
    } else {
      nodes = this.$slots[name] || fallback;
    }

    var target = props && props.slot;
    if (target) {
      return this.$createElement('template', { slot: target }, nodes)
    } else {
      return nodes
    }
  }

  /*  */

  /**
   * Runtime helper for resolving filters
   */
  function resolveFilter (id) {
    return resolveAsset(this.$options, 'filters', id, true) || identity
  }

  /*  */

  function isKeyNotMatch (expect, actual) {
    if (Array.isArray(expect)) {
      return expect.indexOf(actual) === -1
    } else {
      return expect !== actual
    }
  }

  /**
   * Runtime helper for checking keyCodes from config.
   * exposed as Vue.prototype._k
   * passing in eventKeyName as last argument separately for backwards compat
   */
  function checkKeyCodes (
    eventKeyCode,
    key,
    builtInKeyCode,
    eventKeyName,
    builtInKeyName
  ) {
    var mappedKeyCode = config.keyCodes[key] || builtInKeyCode;
    if (builtInKeyName && eventKeyName && !config.keyCodes[key]) {
      return isKeyNotMatch(builtInKeyName, eventKeyName)
    } else if (mappedKeyCode) {
      return isKeyNotMatch(mappedKeyCode, eventKeyCode)
    } else if (eventKeyName) {
      return hyphenate(eventKeyName) !== key
    }
  }

  /*  */

  /**
   * Runtime helper for merging v-bind="object" into a VNode's data.
   */
  function bindObjectProps (
    data,
    tag,
    value,
    asProp,
    isSync
  ) {
    if (value) {
      if (!isObject(value)) {
        warn(
          'v-bind without argument expects an Object or Array value',
          this
        );
      } else {
        if (Array.isArray(value)) {
          value = toObject(value);
        }
        var hash;
        var loop = function ( key ) {
          if (
            key === 'class' ||
            key === 'style' ||
            isReservedAttribute(key)
          ) {
            hash = data;
          } else {
            var type = data.attrs && data.attrs.type;
            hash = asProp || config.mustUseProp(tag, type, key)
              ? data.domProps || (data.domProps = {})
              : data.attrs || (data.attrs = {});
          }
          var camelizedKey = camelize(key);
          var hyphenatedKey = hyphenate(key);
          if (!(camelizedKey in hash) && !(hyphenatedKey in hash)) {
            hash[key] = value[key];

            if (isSync) {
              var on = data.on || (data.on = {});
              on[("update:" + key)] = function ($event) {
                value[key] = $event;
              };
            }
          }
        };

        for (var key in value) loop( key );
      }
    }
    return data
  }

  /*  */

  /**
   * Runtime helper for rendering static trees.
   */
  function renderStatic (
    index,
    isInFor
  ) {
    var cached = this._staticTrees || (this._staticTrees = []);
    var tree = cached[index];
    // if has already-rendered static tree and not inside v-for,
    // we can reuse the same tree.
    if (tree && !isInFor) {
      return tree
    }
    // otherwise, render a fresh tree.
    tree = cached[index] = this.$options.staticRenderFns[index].call(
      this._renderProxy,
      null,
      this // for render fns generated for functional component templates
    );
    markStatic(tree, ("__static__" + index), false);
    return tree
  }

  /**
   * Runtime helper for v-once.
   * Effectively it means marking the node as static with a unique key.
   */
  function markOnce (
    tree,
    index,
    key
  ) {
    markStatic(tree, ("__once__" + index + (key ? ("_" + key) : "")), true);
    return tree
  }

  function markStatic (
    tree,
    key,
    isOnce
  ) {
    if (Array.isArray(tree)) {
      for (var i = 0; i < tree.length; i++) {
        if (tree[i] && typeof tree[i] !== 'string') {
          markStaticNode(tree[i], (key + "_" + i), isOnce);
        }
      }
    } else {
      markStaticNode(tree, key, isOnce);
    }
  }

  function markStaticNode (node, key, isOnce) {
    node.isStatic = true;
    node.key = key;
    node.isOnce = isOnce;
  }

  /*  */

  function bindObjectListeners (data, value) {
    if (value) {
      if (!isPlainObject(value)) {
        warn(
          'v-on without argument expects an Object value',
          this
        );
      } else {
        var on = data.on = data.on ? extend({}, data.on) : {};
        for (var key in value) {
          var existing = on[key];
          var ours = value[key];
          on[key] = existing ? [].concat(existing, ours) : ours;
        }
      }
    }
    return data
  }

  /*  */

  function resolveScopedSlots (
    fns, // see flow/vnode
    res,
    // the following are added in 2.6
    hasDynamicKeys,
    contentHashKey
  ) {
    res = res || { $stable: !hasDynamicKeys };
    for (var i = 0; i < fns.length; i++) {
      var slot = fns[i];
      if (Array.isArray(slot)) {
        resolveScopedSlots(slot, res, hasDynamicKeys);
      } else if (slot) {
        // marker for reverse proxying v-slot without scope on this.$slots
        if (slot.proxy) {
          slot.fn.proxy = true;
        }
        res[slot.key] = slot.fn;
      }
    }
    if (contentHashKey) {
      (res).$key = contentHashKey;
    }
    return res
  }

  /*  */

  function bindDynamicKeys (baseObj, values) {
    for (var i = 0; i < values.length; i += 2) {
      var key = values[i];
      if (typeof key === 'string' && key) {
        baseObj[values[i]] = values[i + 1];
      } else if (key !== '' && key !== null) {
        // null is a special value for explicitly removing a binding
        warn(
          ("Invalid value for dynamic directive argument (expected string or null): " + key),
          this
        );
      }
    }
    return baseObj
  }

  // helper to dynamically append modifier runtime markers to event names.
  // ensure only append when value is already string, otherwise it will be cast
  // to string and cause the type check to miss.
  function prependModifier (value, symbol) {
    return typeof value === 'string' ? symbol + value : value
  }

  /*  */

  function installRenderHelpers (target) {
    target._o = markOnce;
    target._n = toNumber;
    target._s = toString;
    target._l = renderList;
    target._t = renderSlot;
    target._q = looseEqual;
    target._i = looseIndexOf;
    target._m = renderStatic;
    target._f = resolveFilter;
    target._k = checkKeyCodes;
    target._b = bindObjectProps;
    target._v = createTextVNode;
    target._e = createEmptyVNode;
    target._u = resolveScopedSlots;
    target._g = bindObjectListeners;
    target._d = bindDynamicKeys;
    target._p = prependModifier;
  }

  /*  */

  function FunctionalRenderContext (
    data,
    props,
    children,
    parent,
    Ctor
  ) {
    var this$1 = this;

    var options = Ctor.options;
    // ensure the createElement function in functional components
    // gets a unique context - this is necessary for correct named slot check
    var contextVm;
    if (hasOwn(parent, '_uid')) {
      contextVm = Object.create(parent);
      // $flow-disable-line
      contextVm._original = parent;
    } else {
      // the context vm passed in is a functional context as well.
      // in this case we want to make sure we are able to get a hold to the
      // real context instance.
      contextVm = parent;
      // $flow-disable-line
      parent = parent._original;
    }
    var isCompiled = isTrue(options._compiled);
    var needNormalization = !isCompiled;

    this.data = data;
    this.props = props;
    this.children = children;
    this.parent = parent;
    this.listeners = data.on || emptyObject;
    this.injections = resolveInject(options.inject, parent);
    this.slots = function () {
      if (!this$1.$slots) {
        normalizeScopedSlots(
          data.scopedSlots,
          this$1.$slots = resolveSlots(children, parent)
        );
      }
      return this$1.$slots
    };

    Object.defineProperty(this, 'scopedSlots', ({
      enumerable: true,
      get: function get () {
        return normalizeScopedSlots(data.scopedSlots, this.slots())
      }
    }));

    // support for compiled functional template
    if (isCompiled) {
      // exposing $options for renderStatic()
      this.$options = options;
      // pre-resolve slots for renderSlot()
      this.$slots = this.slots();
      this.$scopedSlots = normalizeScopedSlots(data.scopedSlots, this.$slots);
    }

    if (options._scopeId) {
      this._c = function (a, b, c, d) {
        var vnode = createElement(contextVm, a, b, c, d, needNormalization);
        if (vnode && !Array.isArray(vnode)) {
          vnode.fnScopeId = options._scopeId;
          vnode.fnContext = parent;
        }
        return vnode
      };
    } else {
      this._c = function (a, b, c, d) { return createElement(contextVm, a, b, c, d, needNormalization); };
    }
  }

  installRenderHelpers(FunctionalRenderContext.prototype);

  function createFunctionalComponent (
    Ctor,
    propsData,
    data,
    contextVm,
    children
  ) {
    var options = Ctor.options;
    var props = {};
    var propOptions = options.props;
    if (isDef(propOptions)) {
      for (var key in propOptions) {
        props[key] = validateProp(key, propOptions, propsData || emptyObject);
      }
    } else {
      if (isDef(data.attrs)) { mergeProps(props, data.attrs); }
      if (isDef(data.props)) { mergeProps(props, data.props); }
    }

    var renderContext = new FunctionalRenderContext(
      data,
      props,
      children,
      contextVm,
      Ctor
    );

    var vnode = options.render.call(null, renderContext._c, renderContext);

    if (vnode instanceof VNode) {
      return cloneAndMarkFunctionalResult(vnode, data, renderContext.parent, options, renderContext)
    } else if (Array.isArray(vnode)) {
      var vnodes = normalizeChildren(vnode) || [];
      var res = new Array(vnodes.length);
      for (var i = 0; i < vnodes.length; i++) {
        res[i] = cloneAndMarkFunctionalResult(vnodes[i], data, renderContext.parent, options, renderContext);
      }
      return res
    }
  }

  function cloneAndMarkFunctionalResult (vnode, data, contextVm, options, renderContext) {
    // #7817 clone node before setting fnContext, otherwise if the node is reused
    // (e.g. it was from a cached normal slot) the fnContext causes named slots
    // that should not be matched to match.
    var clone = cloneVNode(vnode);
    clone.fnContext = contextVm;
    clone.fnOptions = options;
    {
      (clone.devtoolsMeta = clone.devtoolsMeta || {}).renderContext = renderContext;
    }
    if (data.slot) {
      (clone.data || (clone.data = {})).slot = data.slot;
    }
    return clone
  }

  function mergeProps (to, from) {
    for (var key in from) {
      to[camelize(key)] = from[key];
    }
  }

  /*  */

  /*  */

  /*  */

  /*  */

  // inline hooks to be invoked on component VNodes during patch
  var componentVNodeHooks = {
    init: function init (vnode, hydrating) {
      if (
        vnode.componentInstance &&
        !vnode.componentInstance._isDestroyed &&
        vnode.data.keepAlive
      ) {
        // kept-alive components, treat as a patch
        var mountedNode = vnode; // work around flow
        componentVNodeHooks.prepatch(mountedNode, mountedNode);
      } else {
        var child = vnode.componentInstance = createComponentInstanceForVnode(
          vnode,
          activeInstance
        );
        child.$mount(hydrating ? vnode.elm : undefined, hydrating);
      }
    },

    prepatch: function prepatch (oldVnode, vnode) {
      var options = vnode.componentOptions;
      var child = vnode.componentInstance = oldVnode.componentInstance;
      updateChildComponent(
        child,
        options.propsData, // updated props
        options.listeners, // updated listeners
        vnode, // new parent vnode
        options.children // new children
      );
    },

    insert: function insert (vnode) {
      var context = vnode.context;
      var componentInstance = vnode.componentInstance;
      if (!componentInstance._isMounted) {
        componentInstance._isMounted = true;
        callHook(componentInstance, 'mounted');
      }
      if (vnode.data.keepAlive) {
        if (context._isMounted) {
          // vue-router#1212
          // During updates, a kept-alive component's child components may
          // change, so directly walking the tree here may call activated hooks
          // on incorrect children. Instead we push them into a queue which will
          // be processed after the whole patch process ended.
          queueActivatedComponent(componentInstance);
        } else {
          activateChildComponent(componentInstance, true /* direct */);
        }
      }
    },

    destroy: function destroy (vnode) {
      var componentInstance = vnode.componentInstance;
      if (!componentInstance._isDestroyed) {
        if (!vnode.data.keepAlive) {
          componentInstance.$destroy();
        } else {
          deactivateChildComponent(componentInstance, true /* direct */);
        }
      }
    }
  };

  var hooksToMerge = Object.keys(componentVNodeHooks);

  function createComponent (
    Ctor,
    data,
    context,
    children,
    tag
  ) {
    if (isUndef(Ctor)) {
      return
    }

    var baseCtor = context.$options._base;

    // plain options object: turn it into a constructor
    if (isObject(Ctor)) {
      Ctor = baseCtor.extend(Ctor);
    }

    // if at this stage it's not a constructor or an async component factory,
    // reject.
    if (typeof Ctor !== 'function') {
      {
        warn(("Invalid Component definition: " + (String(Ctor))), context);
      }
      return
    }

    // async component
    var asyncFactory;
    if (isUndef(Ctor.cid)) {
      asyncFactory = Ctor;
      Ctor = resolveAsyncComponent(asyncFactory, baseCtor);
      if (Ctor === undefined) {
        // return a placeholder node for async component, which is rendered
        // as a comment node but preserves all the raw information for the node.
        // the information will be used for async server-rendering and hydration.
        return createAsyncPlaceholder(
          asyncFactory,
          data,
          context,
          children,
          tag
        )
      }
    }

    data = data || {};

    // resolve constructor options in case global mixins are applied after
    // component constructor creation
    resolveConstructorOptions(Ctor);

    // transform component v-model data into props & events
    if (isDef(data.model)) {
      transformModel(Ctor.options, data);
    }

    // extract props
    var propsData = extractPropsFromVNodeData(data, Ctor, tag);

    // functional component
    if (isTrue(Ctor.options.functional)) {
      return createFunctionalComponent(Ctor, propsData, data, context, children)
    }

    // extract listeners, since these needs to be treated as
    // child component listeners instead of DOM listeners
    var listeners = data.on;
    // replace with listeners with .native modifier
    // so it gets processed during parent component patch.
    data.on = data.nativeOn;

    if (isTrue(Ctor.options.abstract)) {
      // abstract components do not keep anything
      // other than props & listeners & slot

      // work around flow
      var slot = data.slot;
      data = {};
      if (slot) {
        data.slot = slot;
      }
    }

    // install component management hooks onto the placeholder node
    installComponentHooks(data);

    // return a placeholder vnode
    var name = Ctor.options.name || tag;
    var vnode = new VNode(
      ("vue-component-" + (Ctor.cid) + (name ? ("-" + name) : '')),
      data, undefined, undefined, undefined, context,
      { Ctor: Ctor, propsData: propsData, listeners: listeners, tag: tag, children: children },
      asyncFactory
    );

    return vnode
  }

  function createComponentInstanceForVnode (
    vnode, // we know it's MountedComponentVNode but flow doesn't
    parent // activeInstance in lifecycle state
  ) {
    var options = {
      _isComponent: true,
      _parentVnode: vnode,
      parent: parent
    };
    // check inline-template render functions
    var inlineTemplate = vnode.data.inlineTemplate;
    if (isDef(inlineTemplate)) {
      options.render = inlineTemplate.render;
      options.staticRenderFns = inlineTemplate.staticRenderFns;
    }
    return new vnode.componentOptions.Ctor(options)
  }

  function installComponentHooks (data) {
    var hooks = data.hook || (data.hook = {});
    for (var i = 0; i < hooksToMerge.length; i++) {
      var key = hooksToMerge[i];
      var existing = hooks[key];
      var toMerge = componentVNodeHooks[key];
      if (existing !== toMerge && !(existing && existing._merged)) {
        hooks[key] = existing ? mergeHook$1(toMerge, existing) : toMerge;
      }
    }
  }

  function mergeHook$1 (f1, f2) {
    var merged = function (a, b) {
      // flow complains about extra args which is why we use any
      f1(a, b);
      f2(a, b);
    };
    merged._merged = true;
    return merged
  }

  // transform component v-model info (value and callback) into
  // prop and event handler respectively.
  function transformModel (options, data) {
    var prop = (options.model && options.model.prop) || 'value';
    var event = (options.model && options.model.event) || 'input'
    ;(data.attrs || (data.attrs = {}))[prop] = data.model.value;
    var on = data.on || (data.on = {});
    var existing = on[event];
    var callback = data.model.callback;
    if (isDef(existing)) {
      if (
        Array.isArray(existing)
          ? existing.indexOf(callback) === -1
          : existing !== callback
      ) {
        on[event] = [callback].concat(existing);
      }
    } else {
      on[event] = callback;
    }
  }

  /*  */

  var SIMPLE_NORMALIZE = 1;
  var ALWAYS_NORMALIZE = 2;

  // wrapper function for providing a more flexible interface
  // without getting yelled at by flow
  function createElement (
    context,
    tag,
    data,
    children,
    normalizationType,
    alwaysNormalize
  ) {
    if (Array.isArray(data) || isPrimitive(data)) {
      normalizationType = children;
      children = data;
      data = undefined;
    }
    if (isTrue(alwaysNormalize)) {
      normalizationType = ALWAYS_NORMALIZE;
    }
    return _createElement(context, tag, data, children, normalizationType)
  }

  function _createElement (
    context,
    tag,
    data,
    children,
    normalizationType
  ) {
    if (isDef(data) && isDef((data).__ob__)) {
      warn(
        "Avoid using observed data object as vnode data: " + (JSON.stringify(data)) + "\n" +
        'Always create fresh vnode data objects in each render!',
        context
      );
      return createEmptyVNode()
    }
    // object syntax in v-bind
    if (isDef(data) && isDef(data.is)) {
      tag = data.is;
    }
    if (!tag) {
      // in case of component :is set to falsy value
      return createEmptyVNode()
    }
    // warn against non-primitive key
    if (isDef(data) && isDef(data.key) && !isPrimitive(data.key)
    ) {
      {
        warn(
          'Avoid using non-primitive value as key, ' +
          'use string/number value instead.',
          context
        );
      }
    }
    // support single function children as default scoped slot
    if (Array.isArray(children) &&
      typeof children[0] === 'function'
    ) {
      data = data || {};
      data.scopedSlots = { default: children[0] };
      children.length = 0;
    }
    if (normalizationType === ALWAYS_NORMALIZE) {
      children = normalizeChildren(children);
    } else if (normalizationType === SIMPLE_NORMALIZE) {
      children = simpleNormalizeChildren(children);
    }
    var vnode, ns;
    if (typeof tag === 'string') {
      var Ctor;
      ns = (context.$vnode && context.$vnode.ns) || config.getTagNamespace(tag);
      if (config.isReservedTag(tag)) {
        // platform built-in elements
        if (isDef(data) && isDef(data.nativeOn)) {
          warn(
            ("The .native modifier for v-on is only valid on components but it was used on <" + tag + ">."),
            context
          );
        }
        vnode = new VNode(
          config.parsePlatformTagName(tag), data, children,
          undefined, undefined, context
        );
      } else if ((!data || !data.pre) && isDef(Ctor = resolveAsset(context.$options, 'components', tag))) {
        // component
        vnode = createComponent(Ctor, data, context, children, tag);
      } else {
        // unknown or unlisted namespaced elements
        // check at runtime because it may get assigned a namespace when its
        // parent normalizes children
        vnode = new VNode(
          tag, data, children,
          undefined, undefined, context
        );
      }
    } else {
      // direct component options / constructor
      vnode = createComponent(tag, data, context, children);
    }
    if (Array.isArray(vnode)) {
      return vnode
    } else if (isDef(vnode)) {
      if (isDef(ns)) { applyNS(vnode, ns); }
      if (isDef(data)) { registerDeepBindings(data); }
      return vnode
    } else {
      return createEmptyVNode()
    }
  }

  function applyNS (vnode, ns, force) {
    vnode.ns = ns;
    if (vnode.tag === 'foreignObject') {
      // use default namespace inside foreignObject
      ns = undefined;
      force = true;
    }
    if (isDef(vnode.children)) {
      for (var i = 0, l = vnode.children.length; i < l; i++) {
        var child = vnode.children[i];
        if (isDef(child.tag) && (
          isUndef(child.ns) || (isTrue(force) && child.tag !== 'svg'))) {
          applyNS(child, ns, force);
        }
      }
    }
  }

  // ref #5318
  // necessary to ensure parent re-render when deep bindings like :style and
  // :class are used on slot nodes
  function registerDeepBindings (data) {
    if (isObject(data.style)) {
      traverse(data.style);
    }
    if (isObject(data.class)) {
      traverse(data.class);
    }
  }

  /*  */

  function initRender (vm) {
    vm._vnode = null; // the root of the child tree
    vm._staticTrees = null; // v-once cached trees
    var options = vm.$options;
    var parentVnode = vm.$vnode = options._parentVnode; // the placeholder node in parent tree
    var renderContext = parentVnode && parentVnode.context;
    vm.$slots = resolveSlots(options._renderChildren, renderContext);
    vm.$scopedSlots = emptyObject;
    // bind the createElement fn to this instance
    // so that we get proper render context inside it.
    // args order: tag, data, children, normalizationType, alwaysNormalize
    // internal version is used by render functions compiled from templates
    vm._c = function (a, b, c, d) { return createElement(vm, a, b, c, d, false); };
    // normalization is always applied for the public version, used in
    // user-written render functions.
    vm.$createElement = function (a, b, c, d) { return createElement(vm, a, b, c, d, true); };

    // $attrs & $listeners are exposed for easier HOC creation.
    // they need to be reactive so that HOCs using them are always updated
    var parentData = parentVnode && parentVnode.data;

    /* istanbul ignore else */
    {
      defineReactive$$1(vm, '$attrs', parentData && parentData.attrs || emptyObject, function () {
        !isUpdatingChildComponent && warn("$attrs is readonly.", vm);
      }, true);
      defineReactive$$1(vm, '$listeners', options._parentListeners || emptyObject, function () {
        !isUpdatingChildComponent && warn("$listeners is readonly.", vm);
      }, true);
    }
  }

  var currentRenderingInstance = null;

  function renderMixin (Vue) {
    // install runtime convenience helpers
    installRenderHelpers(Vue.prototype);

    Vue.prototype.$nextTick = function (fn) {
      return nextTick(fn, this)
    };

    Vue.prototype._render = function () {
      var vm = this;
      var ref = vm.$options;
      var render = ref.render;
      var _parentVnode = ref._parentVnode;

      if (_parentVnode) {
        vm.$scopedSlots = normalizeScopedSlots(
          _parentVnode.data.scopedSlots,
          vm.$slots,
          vm.$scopedSlots
        );
      }

      // set parent vnode. this allows render functions to have access
      // to the data on the placeholder node.
      vm.$vnode = _parentVnode;
      // render self
      var vnode;
      try {
        // There's no need to maintain a stack because all render fns are called
        // separately from one another. Nested component's render fns are called
        // when parent component is patched.
        currentRenderingInstance = vm;
        vnode = render.call(vm._renderProxy, vm.$createElement);
      } catch (e) {
        handleError(e, vm, "render");
        // return error render result,
        // or previous vnode to prevent render error causing blank component
        /* istanbul ignore else */
        if (vm.$options.renderError) {
          try {
            vnode = vm.$options.renderError.call(vm._renderProxy, vm.$createElement, e);
          } catch (e) {
            handleError(e, vm, "renderError");
            vnode = vm._vnode;
          }
        } else {
          vnode = vm._vnode;
        }
      } finally {
        currentRenderingInstance = null;
      }
      // if the returned array contains only a single node, allow it
      if (Array.isArray(vnode) && vnode.length === 1) {
        vnode = vnode[0];
      }
      // return empty vnode in case the render function errored out
      if (!(vnode instanceof VNode)) {
        if (Array.isArray(vnode)) {
          warn(
            'Multiple root nodes returned from render function. Render function ' +
            'should return a single root node.',
            vm
          );
        }
        vnode = createEmptyVNode();
      }
      // set parent
      vnode.parent = _parentVnode;
      return vnode
    };
  }

  /*  */

  function ensureCtor (comp, base) {
    if (
      comp.__esModule ||
      (hasSymbol && comp[Symbol.toStringTag] === 'Module')
    ) {
      comp = comp.default;
    }
    return isObject(comp)
      ? base.extend(comp)
      : comp
  }

  function createAsyncPlaceholder (
    factory,
    data,
    context,
    children,
    tag
  ) {
    var node = createEmptyVNode();
    node.asyncFactory = factory;
    node.asyncMeta = { data: data, context: context, children: children, tag: tag };
    return node
  }

  function resolveAsyncComponent (
    factory,
    baseCtor
  ) {
    if (isTrue(factory.error) && isDef(factory.errorComp)) {
      return factory.errorComp
    }

    if (isDef(factory.resolved)) {
      return factory.resolved
    }

    var owner = currentRenderingInstance;
    if (owner && isDef(factory.owners) && factory.owners.indexOf(owner) === -1) {
      // already pending
      factory.owners.push(owner);
    }

    if (isTrue(factory.loading) && isDef(factory.loadingComp)) {
      return factory.loadingComp
    }

    if (owner && !isDef(factory.owners)) {
      var owners = factory.owners = [owner];
      var sync = true;
      var timerLoading = null;
      var timerTimeout = null

      ;(owner).$on('hook:destroyed', function () { return remove(owners, owner); });

      var forceRender = function (renderCompleted) {
        for (var i = 0, l = owners.length; i < l; i++) {
          (owners[i]).$forceUpdate();
        }

        if (renderCompleted) {
          owners.length = 0;
          if (timerLoading !== null) {
            clearTimeout(timerLoading);
            timerLoading = null;
          }
          if (timerTimeout !== null) {
            clearTimeout(timerTimeout);
            timerTimeout = null;
          }
        }
      };

      var resolve = once(function (res) {
        // cache resolved
        factory.resolved = ensureCtor(res, baseCtor);
        // invoke callbacks only if this is not a synchronous resolve
        // (async resolves are shimmed as synchronous during SSR)
        if (!sync) {
          forceRender(true);
        } else {
          owners.length = 0;
        }
      });

      var reject = once(function (reason) {
        warn(
          "Failed to resolve async component: " + (String(factory)) +
          (reason ? ("\nReason: " + reason) : '')
        );
        if (isDef(factory.errorComp)) {
          factory.error = true;
          forceRender(true);
        }
      });

      var res = factory(resolve, reject);

      if (isObject(res)) {
        if (isPromise(res)) {
          // () => Promise
          if (isUndef(factory.resolved)) {
            res.then(resolve, reject);
          }
        } else if (isPromise(res.component)) {
          res.component.then(resolve, reject);

          if (isDef(res.error)) {
            factory.errorComp = ensureCtor(res.error, baseCtor);
          }

          if (isDef(res.loading)) {
            factory.loadingComp = ensureCtor(res.loading, baseCtor);
            if (res.delay === 0) {
              factory.loading = true;
            } else {
              timerLoading = setTimeout(function () {
                timerLoading = null;
                if (isUndef(factory.resolved) && isUndef(factory.error)) {
                  factory.loading = true;
                  forceRender(false);
                }
              }, res.delay || 200);
            }
          }

          if (isDef(res.timeout)) {
            timerTimeout = setTimeout(function () {
              timerTimeout = null;
              if (isUndef(factory.resolved)) {
                reject(
                  "timeout (" + (res.timeout) + "ms)"
                );
              }
            }, res.timeout);
          }
        }
      }

      sync = false;
      // return in case resolved synchronously
      return factory.loading
        ? factory.loadingComp
        : factory.resolved
    }
  }

  /*  */

  function isAsyncPlaceholder (node) {
    return node.isComment && node.asyncFactory
  }

  /*  */

  function getFirstComponentChild (children) {
    if (Array.isArray(children)) {
      for (var i = 0; i < children.length; i++) {
        var c = children[i];
        if (isDef(c) && (isDef(c.componentOptions) || isAsyncPlaceholder(c))) {
          return c
        }
      }
    }
  }

  /*  */

  /*  */

  function initEvents (vm) {
    vm._events = Object.create(null);
    vm._hasHookEvent = false;
    // init parent attached events
    var listeners = vm.$options._parentListeners;
    if (listeners) {
      updateComponentListeners(vm, listeners);
    }
  }

  var target;

  function add (event, fn) {
    target.$on(event, fn);
  }

  function remove$1 (event, fn) {
    target.$off(event, fn);
  }

  function createOnceHandler (event, fn) {
    var _target = target;
    return function onceHandler () {
      var res = fn.apply(null, arguments);
      if (res !== null) {
        _target.$off(event, onceHandler);
      }
    }
  }

  function updateComponentListeners (
    vm,
    listeners,
    oldListeners
  ) {
    target = vm;
    updateListeners(listeners, oldListeners || {}, add, remove$1, createOnceHandler, vm);
    target = undefined;
  }

  function eventsMixin (Vue) {
    var hookRE = /^hook:/;
    Vue.prototype.$on = function (event, fn) {
      var vm = this;
      if (Array.isArray(event)) {
        for (var i = 0, l = event.length; i < l; i++) {
          vm.$on(event[i], fn);
        }
      } else {
        (vm._events[event] || (vm._events[event] = [])).push(fn);
        // optimize hook:event cost by using a boolean flag marked at registration
        // instead of a hash lookup
        if (hookRE.test(event)) {
          vm._hasHookEvent = true;
        }
      }
      return vm
    };

    Vue.prototype.$once = function (event, fn) {
      var vm = this;
      function on () {
        vm.$off(event, on);
        fn.apply(vm, arguments);
      }
      on.fn = fn;
      vm.$on(event, on);
      return vm
    };

    Vue.prototype.$off = function (event, fn) {
      var vm = this;
      // all
      if (!arguments.length) {
        vm._events = Object.create(null);
        return vm
      }
      // array of events
      if (Array.isArray(event)) {
        for (var i$1 = 0, l = event.length; i$1 < l; i$1++) {
          vm.$off(event[i$1], fn);
        }
        return vm
      }
      // specific event
      var cbs = vm._events[event];
      if (!cbs) {
        return vm
      }
      if (!fn) {
        vm._events[event] = null;
        return vm
      }
      // specific handler
      var cb;
      var i = cbs.length;
      while (i--) {
        cb = cbs[i];
        if (cb === fn || cb.fn === fn) {
          cbs.splice(i, 1);
          break
        }
      }
      return vm
    };

    Vue.prototype.$emit = function (event) {
      var vm = this;
      {
        var lowerCaseEvent = event.toLowerCase();
        if (lowerCaseEvent !== event && vm._events[lowerCaseEvent]) {
          tip(
            "Event \"" + lowerCaseEvent + "\" is emitted in component " +
            (formatComponentName(vm)) + " but the handler is registered for \"" + event + "\". " +
            "Note that HTML attributes are case-insensitive and you cannot use " +
            "v-on to listen to camelCase events when using in-DOM templates. " +
            "You should probably use \"" + (hyphenate(event)) + "\" instead of \"" + event + "\"."
          );
        }
      }
      var cbs = vm._events[event];
      if (cbs) {
        cbs = cbs.length > 1 ? toArray(cbs) : cbs;
        var args = toArray(arguments, 1);
        var info = "event handler for \"" + event + "\"";
        for (var i = 0, l = cbs.length; i < l; i++) {
          invokeWithErrorHandling(cbs[i], vm, args, vm, info);
        }
      }
      return vm
    };
  }

  /*  */

  var activeInstance = null;
  var isUpdatingChildComponent = false;

  function setActiveInstance(vm) {
    var prevActiveInstance = activeInstance;
    activeInstance = vm;
    return function () {
      activeInstance = prevActiveInstance;
    }
  }

  function initLifecycle (vm) {
    var options = vm.$options;

    // locate first non-abstract parent
    var parent = options.parent;
    if (parent && !options.abstract) {
      while (parent.$options.abstract && parent.$parent) {
        parent = parent.$parent;
      }
      parent.$children.push(vm);
    }

    vm.$parent = parent;
    vm.$root = parent ? parent.$root : vm;

    vm.$children = [];
    vm.$refs = {};

    vm._watcher = null;
    vm._inactive = null;
    vm._directInactive = false;
    vm._isMounted = false;
    vm._isDestroyed = false;
    vm._isBeingDestroyed = false;
  }

  function lifecycleMixin (Vue) {
    Vue.prototype._update = function (vnode, hydrating) {
      var vm = this;
      var prevEl = vm.$el;
      var prevVnode = vm._vnode;
      var restoreActiveInstance = setActiveInstance(vm);
      vm._vnode = vnode;
      // Vue.prototype.__patch__ is injected in entry points
      // based on the rendering backend used.
      if (!prevVnode) {
        // initial render
        vm.$el = vm.__patch__(vm.$el, vnode, hydrating, false /* removeOnly */);
      } else {
        // updates
        vm.$el = vm.__patch__(prevVnode, vnode);
      }
      restoreActiveInstance();
      // update __vue__ reference
      if (prevEl) {
        prevEl.__vue__ = null;
      }
      if (vm.$el) {
        vm.$el.__vue__ = vm;
      }
      // if parent is an HOC, update its $el as well
      if (vm.$vnode && vm.$parent && vm.$vnode === vm.$parent._vnode) {
        vm.$parent.$el = vm.$el;
      }
      // updated hook is called by the scheduler to ensure that children are
      // updated in a parent's updated hook.
    };

    Vue.prototype.$forceUpdate = function () {
      var vm = this;
      if (vm._watcher) {
        vm._watcher.update();
      }
    };

    Vue.prototype.$destroy = function () {
      var vm = this;
      if (vm._isBeingDestroyed) {
        return
      }
      callHook(vm, 'beforeDestroy');
      vm._isBeingDestroyed = true;
      // remove self from parent
      var parent = vm.$parent;
      if (parent && !parent._isBeingDestroyed && !vm.$options.abstract) {
        remove(parent.$children, vm);
      }
      // teardown watchers
      if (vm._watcher) {
        vm._watcher.teardown();
      }
      var i = vm._watchers.length;
      while (i--) {
        vm._watchers[i].teardown();
      }
      // remove reference from data ob
      // frozen object may not have observer.
      if (vm._data.__ob__) {
        vm._data.__ob__.vmCount--;
      }
      // call the last hook...
      vm._isDestroyed = true;
      // invoke destroy hooks on current rendered tree
      vm.__patch__(vm._vnode, null);
      // fire destroyed hook
      callHook(vm, 'destroyed');
      // turn off all instance listeners.
      vm.$off();
      // remove __vue__ reference
      if (vm.$el) {
        vm.$el.__vue__ = null;
      }
      // release circular reference (#6759)
      if (vm.$vnode) {
        vm.$vnode.parent = null;
      }
    };
  }

  function mountComponent (
    vm,
    el,
    hydrating
  ) {
    vm.$el = el;
    if (!vm.$options.render) {
      vm.$options.render = createEmptyVNode;
      {
        /* istanbul ignore if */
        if ((vm.$options.template && vm.$options.template.charAt(0) !== '#') ||
          vm.$options.el || el) {
          warn(
            'You are using the runtime-only build of Vue where the template ' +
            'compiler is not available. Either pre-compile the templates into ' +
            'render functions, or use the compiler-included build.',
            vm
          );
        } else {
          warn(
            'Failed to mount component: template or render function not defined.',
            vm
          );
        }
      }
    }
    callHook(vm, 'beforeMount');

    var updateComponent;
    /* istanbul ignore if */
    if (config.performance && mark) {
      updateComponent = function () {
        var name = vm._name;
        var id = vm._uid;
        var startTag = "vue-perf-start:" + id;
        var endTag = "vue-perf-end:" + id;

        mark(startTag);
        var vnode = vm._render();
        mark(endTag);
        measure(("vue " + name + " render"), startTag, endTag);

        mark(startTag);
        vm._update(vnode, hydrating);
        mark(endTag);
        measure(("vue " + name + " patch"), startTag, endTag);
      };
    } else {
      updateComponent = function () {
        vm._update(vm._render(), hydrating);
      };
    }

    // we set this to vm._watcher inside the watcher's constructor
    // since the watcher's initial patch may call $forceUpdate (e.g. inside child
    // component's mounted hook), which relies on vm._watcher being already defined
    new Watcher(vm, updateComponent, noop, {
      before: function before () {
        if (vm._isMounted && !vm._isDestroyed) {
          callHook(vm, 'beforeUpdate');
        }
      }
    }, true /* isRenderWatcher */);
    hydrating = false;

    // manually mounted instance, call mounted on self
    // mounted is called for render-created child components in its inserted hook
    if (vm.$vnode == null) {
      vm._isMounted = true;
      callHook(vm, 'mounted');
    }
    return vm
  }

  function updateChildComponent (
    vm,
    propsData,
    listeners,
    parentVnode,
    renderChildren
  ) {
    {
      isUpdatingChildComponent = true;
    }

    // determine whether component has slot children
    // we need to do this before overwriting $options._renderChildren.

    // check if there are dynamic scopedSlots (hand-written or compiled but with
    // dynamic slot names). Static scoped slots compiled from template has the
    // "$stable" marker.
    var newScopedSlots = parentVnode.data.scopedSlots;
    var oldScopedSlots = vm.$scopedSlots;
    var hasDynamicScopedSlot = !!(
      (newScopedSlots && !newScopedSlots.$stable) ||
      (oldScopedSlots !== emptyObject && !oldScopedSlots.$stable) ||
      (newScopedSlots && vm.$scopedSlots.$key !== newScopedSlots.$key)
    );

    // Any static slot children from the parent may have changed during parent's
    // update. Dynamic scoped slots may also have changed. In such cases, a forced
    // update is necessary to ensure correctness.
    var needsForceUpdate = !!(
      renderChildren ||               // has new static slots
      vm.$options._renderChildren ||  // has old static slots
      hasDynamicScopedSlot
    );

    vm.$options._parentVnode = parentVnode;
    vm.$vnode = parentVnode; // update vm's placeholder node without re-render

    if (vm._vnode) { // update child tree's parent
      vm._vnode.parent = parentVnode;
    }
    vm.$options._renderChildren = renderChildren;

    // update $attrs and $listeners hash
    // these are also reactive so they may trigger child update if the child
    // used them during render
    vm.$attrs = parentVnode.data.attrs || emptyObject;
    vm.$listeners = listeners || emptyObject;

    // update props
    if (propsData && vm.$options.props) {
      toggleObserving(false);
      var props = vm._props;
      var propKeys = vm.$options._propKeys || [];
      for (var i = 0; i < propKeys.length; i++) {
        var key = propKeys[i];
        var propOptions = vm.$options.props; // wtf flow?
        props[key] = validateProp(key, propOptions, propsData, vm);
      }
      toggleObserving(true);
      // keep a copy of raw propsData
      vm.$options.propsData = propsData;
    }

    // update listeners
    listeners = listeners || emptyObject;
    var oldListeners = vm.$options._parentListeners;
    vm.$options._parentListeners = listeners;
    updateComponentListeners(vm, listeners, oldListeners);

    // resolve slots + force update if has children
    if (needsForceUpdate) {
      vm.$slots = resolveSlots(renderChildren, parentVnode.context);
      vm.$forceUpdate();
    }

    {
      isUpdatingChildComponent = false;
    }
  }

  function isInInactiveTree (vm) {
    while (vm && (vm = vm.$parent)) {
      if (vm._inactive) { return true }
    }
    return false
  }

  function activateChildComponent (vm, direct) {
    if (direct) {
      vm._directInactive = false;
      if (isInInactiveTree(vm)) {
        return
      }
    } else if (vm._directInactive) {
      return
    }
    if (vm._inactive || vm._inactive === null) {
      vm._inactive = false;
      for (var i = 0; i < vm.$children.length; i++) {
        activateChildComponent(vm.$children[i]);
      }
      callHook(vm, 'activated');
    }
  }

  function deactivateChildComponent (vm, direct) {
    if (direct) {
      vm._directInactive = true;
      if (isInInactiveTree(vm)) {
        return
      }
    }
    if (!vm._inactive) {
      vm._inactive = true;
      for (var i = 0; i < vm.$children.length; i++) {
        deactivateChildComponent(vm.$children[i]);
      }
      callHook(vm, 'deactivated');
    }
  }

  function callHook (vm, hook) {
    // #7573 disable dep collection when invoking lifecycle hooks
    pushTarget();
    var handlers = vm.$options[hook];
    var info = hook + " hook";
    if (handlers) {
      for (var i = 0, j = handlers.length; i < j; i++) {
        invokeWithErrorHandling(handlers[i], vm, null, vm, info);
      }
    }
    if (vm._hasHookEvent) {
      vm.$emit('hook:' + hook);
    }
    popTarget();
  }

  /*  */

  var MAX_UPDATE_COUNT = 100;

  var queue = [];
  var activatedChildren = [];
  var has = {};
  var circular = {};
  var waiting = false;
  var flushing = false;
  var index = 0;

  /**
   * Reset the scheduler's state.
   */
  function resetSchedulerState () {
    index = queue.length = activatedChildren.length = 0;
    has = {};
    {
      circular = {};
    }
    waiting = flushing = false;
  }

  // Async edge case #6566 requires saving the timestamp when event listeners are
  // attached. However, calling performance.now() has a perf overhead especially
  // if the page has thousands of event listeners. Instead, we take a timestamp
  // every time the scheduler flushes and use that for all event listeners
  // attached during that flush.
  var currentFlushTimestamp = 0;

  // Async edge case fix requires storing an event listener's attach timestamp.
  var getNow = Date.now;

  // Determine what event timestamp the browser is using. Annoyingly, the
  // timestamp can either be hi-res (relative to page load) or low-res
  // (relative to UNIX epoch), so in order to compare time we have to use the
  // same timestamp type when saving the flush timestamp.
  // All IE versions use low-res event timestamps, and have problematic clock
  // implementations (#9632)
  if (inBrowser && !isIE) {
    var performance = window.performance;
    if (
      performance &&
      typeof performance.now === 'function' &&
      getNow() > document.createEvent('Event').timeStamp
    ) {
      // if the event timestamp, although evaluated AFTER the Date.now(), is
      // smaller than it, it means the event is using a hi-res timestamp,
      // and we need to use the hi-res version for event listener timestamps as
      // well.
      getNow = function () { return performance.now(); };
    }
  }

  /**
   * Flush both queues and run the watchers.
   */
  function flushSchedulerQueue () {
    currentFlushTimestamp = getNow();
    flushing = true;
    var watcher, id;

    // Sort queue before flush.
    // This ensures that:
    // 1. Components are updated from parent to child. (because parent is always
    //    created before the child)
    // 2. A component's user watchers are run before its render watcher (because
    //    user watchers are created before the render watcher)
    // 3. If a component is destroyed during a parent component's watcher run,
    //    its watchers can be skipped.
    queue.sort(function (a, b) { return a.id - b.id; });

    // do not cache length because more watchers might be pushed
    // as we run existing watchers
    for (index = 0; index < queue.length; index++) {
      watcher = queue[index];
      if (watcher.before) {
        watcher.before();
      }
      id = watcher.id;
      has[id] = null;
      watcher.run();
      // in dev build, check and stop circular updates.
      if (has[id] != null) {
        circular[id] = (circular[id] || 0) + 1;
        if (circular[id] > MAX_UPDATE_COUNT) {
          warn(
            'You may have an infinite update loop ' + (
              watcher.user
                ? ("in watcher with expression \"" + (watcher.expression) + "\"")
                : "in a component render function."
            ),
            watcher.vm
          );
          break
        }
      }
    }

    // keep copies of post queues before resetting state
    var activatedQueue = activatedChildren.slice();
    var updatedQueue = queue.slice();

    resetSchedulerState();

    // call component updated and activated hooks
    callActivatedHooks(activatedQueue);
    callUpdatedHooks(updatedQueue);

    // devtool hook
    /* istanbul ignore if */
    if (devtools && config.devtools) {
      devtools.emit('flush');
    }
  }

  function callUpdatedHooks (queue) {
    var i = queue.length;
    while (i--) {
      var watcher = queue[i];
      var vm = watcher.vm;
      if (vm._watcher === watcher && vm._isMounted && !vm._isDestroyed) {
        callHook(vm, 'updated');
      }
    }
  }

  /**
   * Queue a kept-alive component that was activated during patch.
   * The queue will be processed after the entire tree has been patched.
   */
  function queueActivatedComponent (vm) {
    // setting _inactive to false here so that a render function can
    // rely on checking whether it's in an inactive tree (e.g. router-view)
    vm._inactive = false;
    activatedChildren.push(vm);
  }

  function callActivatedHooks (queue) {
    for (var i = 0; i < queue.length; i++) {
      queue[i]._inactive = true;
      activateChildComponent(queue[i], true /* true */);
    }
  }

  /**
   * Push a watcher into the watcher queue.
   * Jobs with duplicate IDs will be skipped unless it's
   * pushed when the queue is being flushed.
   */
  function queueWatcher (watcher) {
    var id = watcher.id;
    if (has[id] == null) {
      has[id] = true;
      if (!flushing) {
        queue.push(watcher);
      } else {
        // if already flushing, splice the watcher based on its id
        // if already past its id, it will be run next immediately.
        var i = queue.length - 1;
        while (i > index && queue[i].id > watcher.id) {
          i--;
        }
        queue.splice(i + 1, 0, watcher);
      }
      // queue the flush
      if (!waiting) {
        waiting = true;

        if (!config.async) {
          flushSchedulerQueue();
          return
        }
        nextTick(flushSchedulerQueue);
      }
    }
  }

  /*  */



  var uid$2 = 0;

  /**
   * A watcher parses an expression, collects dependencies,
   * and fires callback when the expression value changes.
   * This is used for both the $watch() api and directives.
   */
  var Watcher = function Watcher (
    vm,
    expOrFn,
    cb,
    options,
    isRenderWatcher
  ) {
    this.vm = vm;
    if (isRenderWatcher) {
      vm._watcher = this;
    }
    vm._watchers.push(this);
    // options
    if (options) {
      this.deep = !!options.deep;
      this.user = !!options.user;
      this.lazy = !!options.lazy;
      this.sync = !!options.sync;
      this.before = options.before;
    } else {
      this.deep = this.user = this.lazy = this.sync = false;
    }
    this.cb = cb;
    this.id = ++uid$2; // uid for batching
    this.active = true;
    this.dirty = this.lazy; // for lazy watchers
    this.deps = [];
    this.newDeps = [];
    this.depIds = new _Set();
    this.newDepIds = new _Set();
    this.expression = expOrFn.toString();
    // parse expression for getter
    if (typeof expOrFn === 'function') {
      this.getter = expOrFn;
    } else {
      this.getter = parsePath(expOrFn);
      if (!this.getter) {
        this.getter = noop;
        warn(
          "Failed watching path: \"" + expOrFn + "\" " +
          'Watcher only accepts simple dot-delimited paths. ' +
          'For full control, use a function instead.',
          vm
        );
      }
    }
    this.value = this.lazy
      ? undefined
      : this.get();
  };

  /**
   * Evaluate the getter, and re-collect dependencies.
   */
  Watcher.prototype.get = function get () {
    pushTarget(this);
    var value;
    var vm = this.vm;
    try {
      value = this.getter.call(vm, vm);
    } catch (e) {
      if (this.user) {
        handleError(e, vm, ("getter for watcher \"" + (this.expression) + "\""));
      } else {
        throw e
      }
    } finally {
      // "touch" every property so they are all tracked as
      // dependencies for deep watching
      if (this.deep) {
        traverse(value);
      }
      popTarget();
      this.cleanupDeps();
    }
    return value
  };

  /**
   * Add a dependency to this directive.
   */
  Watcher.prototype.addDep = function addDep (dep) {
    var id = dep.id;
    if (!this.newDepIds.has(id)) {
      this.newDepIds.add(id);
      this.newDeps.push(dep);
      if (!this.depIds.has(id)) {
        dep.addSub(this);
      }
    }
  };

  /**
   * Clean up for dependency collection.
   */
  Watcher.prototype.cleanupDeps = function cleanupDeps () {
    var i = this.deps.length;
    while (i--) {
      var dep = this.deps[i];
      if (!this.newDepIds.has(dep.id)) {
        dep.removeSub(this);
      }
    }
    var tmp = this.depIds;
    this.depIds = this.newDepIds;
    this.newDepIds = tmp;
    this.newDepIds.clear();
    tmp = this.deps;
    this.deps = this.newDeps;
    this.newDeps = tmp;
    this.newDeps.length = 0;
  };

  /**
   * Subscriber interface.
   * Will be called when a dependency changes.
   */
  Watcher.prototype.update = function update () {
    /* istanbul ignore else */
    if (this.lazy) {
      this.dirty = true;
    } else if (this.sync) {
      this.run();
    } else {
      queueWatcher(this);
    }
  };

  /**
   * Scheduler job interface.
   * Will be called by the scheduler.
   */
  Watcher.prototype.run = function run () {
    if (this.active) {
      var value = this.get();
      if (
        value !== this.value ||
        // Deep watchers and watchers on Object/Arrays should fire even
        // when the value is the same, because the value may
        // have mutated.
        isObject(value) ||
        this.deep
      ) {
        // set new value
        var oldValue = this.value;
        this.value = value;
        if (this.user) {
          try {
            this.cb.call(this.vm, value, oldValue);
          } catch (e) {
            handleError(e, this.vm, ("callback for watcher \"" + (this.expression) + "\""));
          }
        } else {
          this.cb.call(this.vm, value, oldValue);
        }
      }
    }
  };

  /**
   * Evaluate the value of the watcher.
   * This only gets called for lazy watchers.
   */
  Watcher.prototype.evaluate = function evaluate () {
    this.value = this.get();
    this.dirty = false;
  };

  /**
   * Depend on all deps collected by this watcher.
   */
  Watcher.prototype.depend = function depend () {
    var i = this.deps.length;
    while (i--) {
      this.deps[i].depend();
    }
  };

  /**
   * Remove self from all dependencies' subscriber list.
   */
  Watcher.prototype.teardown = function teardown () {
    if (this.active) {
      // remove self from vm's watcher list
      // this is a somewhat expensive operation so we skip it
      // if the vm is being destroyed.
      if (!this.vm._isBeingDestroyed) {
        remove(this.vm._watchers, this);
      }
      var i = this.deps.length;
      while (i--) {
        this.deps[i].removeSub(this);
      }
      this.active = false;
    }
  };

  /*  */

  var sharedPropertyDefinition = {
    enumerable: true,
    configurable: true,
    get: noop,
    set: noop
  };

  function proxy (target, sourceKey, key) {
    sharedPropertyDefinition.get = function proxyGetter () {
      return this[sourceKey][key]
    };
    sharedPropertyDefinition.set = function proxySetter (val) {
      this[sourceKey][key] = val;
    };
    Object.defineProperty(target, key, sharedPropertyDefinition);
  }

  function initState (vm) {
    vm._watchers = [];
    var opts = vm.$options;
    if (opts.props) { initProps(vm, opts.props); }
    if (opts.methods) { initMethods(vm, opts.methods); }
    if (opts.data) {
      initData(vm);
    } else {
      observe(vm._data = {}, true /* asRootData */);
    }
    if (opts.computed) { initComputed(vm, opts.computed); }
    if (opts.watch && opts.watch !== nativeWatch) {
      initWatch(vm, opts.watch);
    }
  }

  function initProps (vm, propsOptions) {
    var propsData = vm.$options.propsData || {};
    var props = vm._props = {};
    // cache prop keys so that future props updates can iterate using Array
    // instead of dynamic object key enumeration.
    var keys = vm.$options._propKeys = [];
    var isRoot = !vm.$parent;
    // root instance props should be converted
    if (!isRoot) {
      toggleObserving(false);
    }
    var loop = function ( key ) {
      keys.push(key);
      var value = validateProp(key, propsOptions, propsData, vm);
      /* istanbul ignore else */
      {
        var hyphenatedKey = hyphenate(key);
        if (isReservedAttribute(hyphenatedKey) ||
            config.isReservedAttr(hyphenatedKey)) {
          warn(
            ("\"" + hyphenatedKey + "\" is a reserved attribute and cannot be used as component prop."),
            vm
          );
        }
        defineReactive$$1(props, key, value, function () {
          if (!isRoot && !isUpdatingChildComponent) {
            warn(
              "Avoid mutating a prop directly since the value will be " +
              "overwritten whenever the parent component re-renders. " +
              "Instead, use a data or computed property based on the prop's " +
              "value. Prop being mutated: \"" + key + "\"",
              vm
            );
          }
        });
      }
      // static props are already proxied on the component's prototype
      // during Vue.extend(). We only need to proxy props defined at
      // instantiation here.
      if (!(key in vm)) {
        proxy(vm, "_props", key);
      }
    };

    for (var key in propsOptions) loop( key );
    toggleObserving(true);
  }

  function initData (vm) {
    var data = vm.$options.data;
    data = vm._data = typeof data === 'function'
      ? getData(data, vm)
      : data || {};
    if (!isPlainObject(data)) {
      data = {};
      warn(
        'data functions should return an object:\n' +
        'https://vuejs.org/v2/guide/components.html#data-Must-Be-a-Function',
        vm
      );
    }
    // proxy data on instance
    var keys = Object.keys(data);
    var props = vm.$options.props;
    var methods = vm.$options.methods;
    var i = keys.length;
    while (i--) {
      var key = keys[i];
      {
        if (methods && hasOwn(methods, key)) {
          warn(
            ("Method \"" + key + "\" has already been defined as a data property."),
            vm
          );
        }
      }
      if (props && hasOwn(props, key)) {
        warn(
          "The data property \"" + key + "\" is already declared as a prop. " +
          "Use prop default value instead.",
          vm
        );
      } else if (!isReserved(key)) {
        proxy(vm, "_data", key);
      }
    }
    // observe data
    observe(data, true /* asRootData */);
  }

  function getData (data, vm) {
    // #7573 disable dep collection when invoking data getters
    pushTarget();
    try {
      return data.call(vm, vm)
    } catch (e) {
      handleError(e, vm, "data()");
      return {}
    } finally {
      popTarget();
    }
  }

  var computedWatcherOptions = { lazy: true };

  function initComputed (vm, computed) {
    // $flow-disable-line
    var watchers = vm._computedWatchers = Object.create(null);
    // computed properties are just getters during SSR
    var isSSR = isServerRendering();

    for (var key in computed) {
      var userDef = computed[key];
      var getter = typeof userDef === 'function' ? userDef : userDef.get;
      if (getter == null) {
        warn(
          ("Getter is missing for computed property \"" + key + "\"."),
          vm
        );
      }

      if (!isSSR) {
        // create internal watcher for the computed property.
        watchers[key] = new Watcher(
          vm,
          getter || noop,
          noop,
          computedWatcherOptions
        );
      }

      // component-defined computed properties are already defined on the
      // component prototype. We only need to define computed properties defined
      // at instantiation here.
      if (!(key in vm)) {
        defineComputed(vm, key, userDef);
      } else {
        if (key in vm.$data) {
          warn(("The computed property \"" + key + "\" is already defined in data."), vm);
        } else if (vm.$options.props && key in vm.$options.props) {
          warn(("The computed property \"" + key + "\" is already defined as a prop."), vm);
        }
      }
    }
  }

  function defineComputed (
    target,
    key,
    userDef
  ) {
    var shouldCache = !isServerRendering();
    if (typeof userDef === 'function') {
      sharedPropertyDefinition.get = shouldCache
        ? createComputedGetter(key)
        : createGetterInvoker(userDef);
      sharedPropertyDefinition.set = noop;
    } else {
      sharedPropertyDefinition.get = userDef.get
        ? shouldCache && userDef.cache !== false
          ? createComputedGetter(key)
          : createGetterInvoker(userDef.get)
        : noop;
      sharedPropertyDefinition.set = userDef.set || noop;
    }
    if (sharedPropertyDefinition.set === noop) {
      sharedPropertyDefinition.set = function () {
        warn(
          ("Computed property \"" + key + "\" was assigned to but it has no setter."),
          this
        );
      };
    }
    Object.defineProperty(target, key, sharedPropertyDefinition);
  }

  function createComputedGetter (key) {
    return function computedGetter () {
      var watcher = this._computedWatchers && this._computedWatchers[key];
      if (watcher) {
        if (watcher.dirty) {
          watcher.evaluate();
        }
        if (Dep.target) {
          watcher.depend();
        }
        return watcher.value
      }
    }
  }

  function createGetterInvoker(fn) {
    return function computedGetter () {
      return fn.call(this, this)
    }
  }

  function initMethods (vm, methods) {
    var props = vm.$options.props;
    for (var key in methods) {
      {
        if (typeof methods[key] !== 'function') {
          warn(
            "Method \"" + key + "\" has type \"" + (typeof methods[key]) + "\" in the component definition. " +
            "Did you reference the function correctly?",
            vm
          );
        }
        if (props && hasOwn(props, key)) {
          warn(
            ("Method \"" + key + "\" has already been defined as a prop."),
            vm
          );
        }
        if ((key in vm) && isReserved(key)) {
          warn(
            "Method \"" + key + "\" conflicts with an existing Vue instance method. " +
            "Avoid defining component methods that start with _ or $."
          );
        }
      }
      vm[key] = typeof methods[key] !== 'function' ? noop : bind(methods[key], vm);
    }
  }

  function initWatch (vm, watch) {
    for (var key in watch) {
      var handler = watch[key];
      if (Array.isArray(handler)) {
        for (var i = 0; i < handler.length; i++) {
          createWatcher(vm, key, handler[i]);
        }
      } else {
        createWatcher(vm, key, handler);
      }
    }
  }

  function createWatcher (
    vm,
    expOrFn,
    handler,
    options
  ) {
    if (isPlainObject(handler)) {
      options = handler;
      handler = handler.handler;
    }
    if (typeof handler === 'string') {
      handler = vm[handler];
    }
    return vm.$watch(expOrFn, handler, options)
  }

  function stateMixin (Vue) {
    // flow somehow has problems with directly declared definition object
    // when using Object.defineProperty, so we have to procedurally build up
    // the object here.
    var dataDef = {};
    dataDef.get = function () { return this._data };
    var propsDef = {};
    propsDef.get = function () { return this._props };
    {
      dataDef.set = function () {
        warn(
          'Avoid replacing instance root $data. ' +
          'Use nested data properties instead.',
          this
        );
      };
      propsDef.set = function () {
        warn("$props is readonly.", this);
      };
    }
    Object.defineProperty(Vue.prototype, '$data', dataDef);
    Object.defineProperty(Vue.prototype, '$props', propsDef);

    Vue.prototype.$set = set;
    Vue.prototype.$delete = del;

    Vue.prototype.$watch = function (
      expOrFn,
      cb,
      options
    ) {
      var vm = this;
      if (isPlainObject(cb)) {
        return createWatcher(vm, expOrFn, cb, options)
      }
      options = options || {};
      options.user = true;
      var watcher = new Watcher(vm, expOrFn, cb, options);
      if (options.immediate) {
        try {
          cb.call(vm, watcher.value);
        } catch (error) {
          handleError(error, vm, ("callback for immediate watcher \"" + (watcher.expression) + "\""));
        }
      }
      return function unwatchFn () {
        watcher.teardown();
      }
    };
  }

  /*  */

  var uid$3 = 0;

  function initMixin (Vue) {
    Vue.prototype._init = function (options) {
      var vm = this;
      // a uid
      vm._uid = uid$3++;

      var startTag, endTag;
      /* istanbul ignore if */
      if (config.performance && mark) {
        startTag = "vue-perf-start:" + (vm._uid);
        endTag = "vue-perf-end:" + (vm._uid);
        mark(startTag);
      }

      // a flag to avoid this being observed
      vm._isVue = true;
      // merge options
      if (options && options._isComponent) {
        // optimize internal component instantiation
        // since dynamic options merging is pretty slow, and none of the
        // internal component options needs special treatment.
        initInternalComponent(vm, options);
      } else {
        vm.$options = mergeOptions(
          resolveConstructorOptions(vm.constructor),
          options || {},
          vm
        );
      }
      /* istanbul ignore else */
      {
        initProxy(vm);
      }
      // expose real self
      vm._self = vm;
      initLifecycle(vm);
      initEvents(vm);
      initRender(vm);
      callHook(vm, 'beforeCreate');
      initInjections(vm); // resolve injections before data/props
      initState(vm);
      initProvide(vm); // resolve provide after data/props
      callHook(vm, 'created');

      /* istanbul ignore if */
      if (config.performance && mark) {
        vm._name = formatComponentName(vm, false);
        mark(endTag);
        measure(("vue " + (vm._name) + " init"), startTag, endTag);
      }

      if (vm.$options.el) {
        vm.$mount(vm.$options.el);
      }
    };
  }

  function initInternalComponent (vm, options) {
    var opts = vm.$options = Object.create(vm.constructor.options);
    // doing this because it's faster than dynamic enumeration.
    var parentVnode = options._parentVnode;
    opts.parent = options.parent;
    opts._parentVnode = parentVnode;

    var vnodeComponentOptions = parentVnode.componentOptions;
    opts.propsData = vnodeComponentOptions.propsData;
    opts._parentListeners = vnodeComponentOptions.listeners;
    opts._renderChildren = vnodeComponentOptions.children;
    opts._componentTag = vnodeComponentOptions.tag;

    if (options.render) {
      opts.render = options.render;
      opts.staticRenderFns = options.staticRenderFns;
    }
  }

  function resolveConstructorOptions (Ctor) {
    var options = Ctor.options;
    if (Ctor.super) {
      var superOptions = resolveConstructorOptions(Ctor.super);
      var cachedSuperOptions = Ctor.superOptions;
      if (superOptions !== cachedSuperOptions) {
        // super option changed,
        // need to resolve new options.
        Ctor.superOptions = superOptions;
        // check if there are any late-modified/attached options (#4976)
        var modifiedOptions = resolveModifiedOptions(Ctor);
        // update base extend options
        if (modifiedOptions) {
          extend(Ctor.extendOptions, modifiedOptions);
        }
        options = Ctor.options = mergeOptions(superOptions, Ctor.extendOptions);
        if (options.name) {
          options.components[options.name] = Ctor;
        }
      }
    }
    return options
  }

  function resolveModifiedOptions (Ctor) {
    var modified;
    var latest = Ctor.options;
    var sealed = Ctor.sealedOptions;
    for (var key in latest) {
      if (latest[key] !== sealed[key]) {
        if (!modified) { modified = {}; }
        modified[key] = latest[key];
      }
    }
    return modified
  }

  function Vue (options) {
    if (!(this instanceof Vue)
    ) {
      warn('Vue is a constructor and should be called with the `new` keyword');
    }
    this._init(options);
  }

  initMixin(Vue);
  stateMixin(Vue);
  eventsMixin(Vue);
  lifecycleMixin(Vue);
  renderMixin(Vue);

  /*  */

  function initUse (Vue) {
    Vue.use = function (plugin) {
      var installedPlugins = (this._installedPlugins || (this._installedPlugins = []));
      if (installedPlugins.indexOf(plugin) > -1) {
        return this
      }

      // additional parameters
      var args = toArray(arguments, 1);
      args.unshift(this);
      if (typeof plugin.install === 'function') {
        plugin.install.apply(plugin, args);
      } else if (typeof plugin === 'function') {
        plugin.apply(null, args);
      }
      installedPlugins.push(plugin);
      return this
    };
  }

  /*  */

  function initMixin$1 (Vue) {
    Vue.mixin = function (mixin) {
      this.options = mergeOptions(this.options, mixin);
      return this
    };
  }

  /*  */

  function initExtend (Vue) {
    /**
     * Each instance constructor, including Vue, has a unique
     * cid. This enables us to create wrapped "child
     * constructors" for prototypal inheritance and cache them.
     */
    Vue.cid = 0;
    var cid = 1;

    /**
     * Class inheritance
     */
    Vue.extend = function (extendOptions) {
      extendOptions = extendOptions || {};
      var Super = this;
      var SuperId = Super.cid;
      var cachedCtors = extendOptions._Ctor || (extendOptions._Ctor = {});
      if (cachedCtors[SuperId]) {
        return cachedCtors[SuperId]
      }

      var name = extendOptions.name || Super.options.name;
      if (name) {
        validateComponentName(name);
      }

      var Sub = function VueComponent (options) {
        this._init(options);
      };
      Sub.prototype = Object.create(Super.prototype);
      Sub.prototype.constructor = Sub;
      Sub.cid = cid++;
      Sub.options = mergeOptions(
        Super.options,
        extendOptions
      );
      Sub['super'] = Super;

      // For props and computed properties, we define the proxy getters on
      // the Vue instances at extension time, on the extended prototype. This
      // avoids Object.defineProperty calls for each instance created.
      if (Sub.options.props) {
        initProps$1(Sub);
      }
      if (Sub.options.computed) {
        initComputed$1(Sub);
      }

      // allow further extension/mixin/plugin usage
      Sub.extend = Super.extend;
      Sub.mixin = Super.mixin;
      Sub.use = Super.use;

      // create asset registers, so extended classes
      // can have their private assets too.
      ASSET_TYPES.forEach(function (type) {
        Sub[type] = Super[type];
      });
      // enable recursive self-lookup
      if (name) {
        Sub.options.components[name] = Sub;
      }

      // keep a reference to the super options at extension time.
      // later at instantiation we can check if Super's options have
      // been updated.
      Sub.superOptions = Super.options;
      Sub.extendOptions = extendOptions;
      Sub.sealedOptions = extend({}, Sub.options);

      // cache constructor
      cachedCtors[SuperId] = Sub;
      return Sub
    };
  }

  function initProps$1 (Comp) {
    var props = Comp.options.props;
    for (var key in props) {
      proxy(Comp.prototype, "_props", key);
    }
  }

  function initComputed$1 (Comp) {
    var computed = Comp.options.computed;
    for (var key in computed) {
      defineComputed(Comp.prototype, key, computed[key]);
    }
  }

  /*  */

  function initAssetRegisters (Vue) {
    /**
     * Create asset registration methods.
     */
    ASSET_TYPES.forEach(function (type) {
      Vue[type] = function (
        id,
        definition
      ) {
        if (!definition) {
          return this.options[type + 's'][id]
        } else {
          /* istanbul ignore if */
          if (type === 'component') {
            validateComponentName(id);
          }
          if (type === 'component' && isPlainObject(definition)) {
            definition.name = definition.name || id;
            definition = this.options._base.extend(definition);
          }
          if (type === 'directive' && typeof definition === 'function') {
            definition = { bind: definition, update: definition };
          }
          this.options[type + 's'][id] = definition;
          return definition
        }
      };
    });
  }

  /*  */



  function getComponentName (opts) {
    return opts && (opts.Ctor.options.name || opts.tag)
  }

  function matches (pattern, name) {
    if (Array.isArray(pattern)) {
      return pattern.indexOf(name) > -1
    } else if (typeof pattern === 'string') {
      return pattern.split(',').indexOf(name) > -1
    } else if (isRegExp(pattern)) {
      return pattern.test(name)
    }
    /* istanbul ignore next */
    return false
  }

  function pruneCache (keepAliveInstance, filter) {
    var cache = keepAliveInstance.cache;
    var keys = keepAliveInstance.keys;
    var _vnode = keepAliveInstance._vnode;
    for (var key in cache) {
      var cachedNode = cache[key];
      if (cachedNode) {
        var name = getComponentName(cachedNode.componentOptions);
        if (name && !filter(name)) {
          pruneCacheEntry(cache, key, keys, _vnode);
        }
      }
    }
  }

  function pruneCacheEntry (
    cache,
    key,
    keys,
    current
  ) {
    var cached$$1 = cache[key];
    if (cached$$1 && (!current || cached$$1.tag !== current.tag)) {
      cached$$1.componentInstance.$destroy();
    }
    cache[key] = null;
    remove(keys, key);
  }

  var patternTypes = [String, RegExp, Array];

  var KeepAlive = {
    name: 'keep-alive',
    abstract: true,

    props: {
      include: patternTypes,
      exclude: patternTypes,
      max: [String, Number]
    },

    created: function created () {
      this.cache = Object.create(null);
      this.keys = [];
    },

    destroyed: function destroyed () {
      for (var key in this.cache) {
        pruneCacheEntry(this.cache, key, this.keys);
      }
    },

    mounted: function mounted () {
      var this$1 = this;

      this.$watch('include', function (val) {
        pruneCache(this$1, function (name) { return matches(val, name); });
      });
      this.$watch('exclude', function (val) {
        pruneCache(this$1, function (name) { return !matches(val, name); });
      });
    },

    render: function render () {
      var slot = this.$slots.default;
      var vnode = getFirstComponentChild(slot);
      var componentOptions = vnode && vnode.componentOptions;
      if (componentOptions) {
        // check pattern
        var name = getComponentName(componentOptions);
        var ref = this;
        var include = ref.include;
        var exclude = ref.exclude;
        if (
          // not included
          (include && (!name || !matches(include, name))) ||
          // excluded
          (exclude && name && matches(exclude, name))
        ) {
          return vnode
        }

        var ref$1 = this;
        var cache = ref$1.cache;
        var keys = ref$1.keys;
        var key = vnode.key == null
          // same constructor may get registered as different local components
          // so cid alone is not enough (#3269)
          ? componentOptions.Ctor.cid + (componentOptions.tag ? ("::" + (componentOptions.tag)) : '')
          : vnode.key;
        if (cache[key]) {
          vnode.componentInstance = cache[key].componentInstance;
          // make current key freshest
          remove(keys, key);
          keys.push(key);
        } else {
          cache[key] = vnode;
          keys.push(key);
          // prune oldest entry
          if (this.max && keys.length > parseInt(this.max)) {
            pruneCacheEntry(cache, keys[0], keys, this._vnode);
          }
        }

        vnode.data.keepAlive = true;
      }
      return vnode || (slot && slot[0])
    }
  };

  var builtInComponents = {
    KeepAlive: KeepAlive
  };

  /*  */

  function initGlobalAPI (Vue) {
    // config
    var configDef = {};
    configDef.get = function () { return config; };
    {
      configDef.set = function () {
        warn(
          'Do not replace the Vue.config object, set individual fields instead.'
        );
      };
    }
    Object.defineProperty(Vue, 'config', configDef);

    // exposed util methods.
    // NOTE: these are not considered part of the public API - avoid relying on
    // them unless you are aware of the risk.
    Vue.util = {
      warn: warn,
      extend: extend,
      mergeOptions: mergeOptions,
      defineReactive: defineReactive$$1
    };

    Vue.set = set;
    Vue.delete = del;
    Vue.nextTick = nextTick;

    // 2.6 explicit observable API
    Vue.observable = function (obj) {
      observe(obj);
      return obj
    };

    Vue.options = Object.create(null);
    ASSET_TYPES.forEach(function (type) {
      Vue.options[type + 's'] = Object.create(null);
    });

    // this is used to identify the "base" constructor to extend all plain-object
    // components with in Weex's multi-instance scenarios.
    Vue.options._base = Vue;

    extend(Vue.options.components, builtInComponents);

    initUse(Vue);
    initMixin$1(Vue);
    initExtend(Vue);
    initAssetRegisters(Vue);
  }

  initGlobalAPI(Vue);

  Object.defineProperty(Vue.prototype, '$isServer', {
    get: isServerRendering
  });

  Object.defineProperty(Vue.prototype, '$ssrContext', {
    get: function get () {
      /* istanbul ignore next */
      return this.$vnode && this.$vnode.ssrContext
    }
  });

  // expose FunctionalRenderContext for ssr runtime helper installation
  Object.defineProperty(Vue, 'FunctionalRenderContext', {
    value: FunctionalRenderContext
  });

  Vue.version = '2.6.11';

  /*  */

  // these are reserved for web because they are directly compiled away
  // during template compilation
  var isReservedAttr = makeMap('style,class');

  // attributes that should be using props for binding
  var acceptValue = makeMap('input,textarea,option,select,progress');
  var mustUseProp = function (tag, type, attr) {
    return (
      (attr === 'value' && acceptValue(tag)) && type !== 'button' ||
      (attr === 'selected' && tag === 'option') ||
      (attr === 'checked' && tag === 'input') ||
      (attr === 'muted' && tag === 'video')
    )
  };

  var isEnumeratedAttr = makeMap('contenteditable,draggable,spellcheck');

  var isValidContentEditableValue = makeMap('events,caret,typing,plaintext-only');

  var convertEnumeratedValue = function (key, value) {
    return isFalsyAttrValue(value) || value === 'false'
      ? 'false'
      // allow arbitrary string value for contenteditable
      : key === 'contenteditable' && isValidContentEditableValue(value)
        ? value
        : 'true'
  };

  var isBooleanAttr = makeMap(
    'allowfullscreen,async,autofocus,autoplay,checked,compact,controls,declare,' +
    'default,defaultchecked,defaultmuted,defaultselected,defer,disabled,' +
    'enabled,formnovalidate,hidden,indeterminate,inert,ismap,itemscope,loop,multiple,' +
    'muted,nohref,noresize,noshade,novalidate,nowrap,open,pauseonexit,readonly,' +
    'required,reversed,scoped,seamless,selected,sortable,translate,' +
    'truespeed,typemustmatch,visible'
  );

  var xlinkNS = 'http://www.w3.org/1999/xlink';

  var isXlink = function (name) {
    return name.charAt(5) === ':' && name.slice(0, 5) === 'xlink'
  };

  var getXlinkProp = function (name) {
    return isXlink(name) ? name.slice(6, name.length) : ''
  };

  var isFalsyAttrValue = function (val) {
    return val == null || val === false
  };

  /*  */

  function genClassForVnode (vnode) {
    var data = vnode.data;
    var parentNode = vnode;
    var childNode = vnode;
    while (isDef(childNode.componentInstance)) {
      childNode = childNode.componentInstance._vnode;
      if (childNode && childNode.data) {
        data = mergeClassData(childNode.data, data);
      }
    }
    while (isDef(parentNode = parentNode.parent)) {
      if (parentNode && parentNode.data) {
        data = mergeClassData(data, parentNode.data);
      }
    }
    return renderClass(data.staticClass, data.class)
  }

  function mergeClassData (child, parent) {
    return {
      staticClass: concat(child.staticClass, parent.staticClass),
      class: isDef(child.class)
        ? [child.class, parent.class]
        : parent.class
    }
  }

  function renderClass (
    staticClass,
    dynamicClass
  ) {
    if (isDef(staticClass) || isDef(dynamicClass)) {
      return concat(staticClass, stringifyClass(dynamicClass))
    }
    /* istanbul ignore next */
    return ''
  }

  function concat (a, b) {
    return a ? b ? (a + ' ' + b) : a : (b || '')
  }

  function stringifyClass (value) {
    if (Array.isArray(value)) {
      return stringifyArray(value)
    }
    if (isObject(value)) {
      return stringifyObject(value)
    }
    if (typeof value === 'string') {
      return value
    }
    /* istanbul ignore next */
    return ''
  }

  function stringifyArray (value) {
    var res = '';
    var stringified;
    for (var i = 0, l = value.length; i < l; i++) {
      if (isDef(stringified = stringifyClass(value[i])) && stringified !== '') {
        if (res) { res += ' '; }
        res += stringified;
      }
    }
    return res
  }

  function stringifyObject (value) {
    var res = '';
    for (var key in value) {
      if (value[key]) {
        if (res) { res += ' '; }
        res += key;
      }
    }
    return res
  }

  /*  */

  var namespaceMap = {
    svg: 'http://www.w3.org/2000/svg',
    math: 'http://www.w3.org/1998/Math/MathML'
  };

  var isHTMLTag = makeMap(
    'html,body,base,head,link,meta,style,title,' +
    'address,article,aside,footer,header,h1,h2,h3,h4,h5,h6,hgroup,nav,section,' +
    'div,dd,dl,dt,figcaption,figure,picture,hr,img,li,main,ol,p,pre,ul,' +
    'a,b,abbr,bdi,bdo,br,cite,code,data,dfn,em,i,kbd,mark,q,rp,rt,rtc,ruby,' +
    's,samp,small,span,strong,sub,sup,time,u,var,wbr,area,audio,map,track,video,' +
    'embed,object,param,source,canvas,script,noscript,del,ins,' +
    'caption,col,colgroup,table,thead,tbody,td,th,tr,' +
    'button,datalist,fieldset,form,input,label,legend,meter,optgroup,option,' +
    'output,progress,select,textarea,' +
    'details,dialog,menu,menuitem,summary,' +
    'content,element,shadow,template,blockquote,iframe,tfoot'
  );

  // this map is intentionally selective, only covering SVG elements that may
  // contain child elements.
  var isSVG = makeMap(
    'svg,animate,circle,clippath,cursor,defs,desc,ellipse,filter,font-face,' +
    'foreignObject,g,glyph,image,line,marker,mask,missing-glyph,path,pattern,' +
    'polygon,polyline,rect,switch,symbol,text,textpath,tspan,use,view',
    true
  );

  var isReservedTag = function (tag) {
    return isHTMLTag(tag) || isSVG(tag)
  };

  function getTagNamespace (tag) {
    if (isSVG(tag)) {
      return 'svg'
    }
    // basic support for MathML
    // note it doesn't support other MathML elements being component roots
    if (tag === 'math') {
      return 'math'
    }
  }

  var unknownElementCache = Object.create(null);
  function isUnknownElement (tag) {
    /* istanbul ignore if */
    if (!inBrowser) {
      return true
    }
    if (isReservedTag(tag)) {
      return false
    }
    tag = tag.toLowerCase();
    /* istanbul ignore if */
    if (unknownElementCache[tag] != null) {
      return unknownElementCache[tag]
    }
    var el = document.createElement(tag);
    if (tag.indexOf('-') > -1) {
      // http://stackoverflow.com/a/28210364/1070244
      return (unknownElementCache[tag] = (
        el.constructor === window.HTMLUnknownElement ||
        el.constructor === window.HTMLElement
      ))
    } else {
      return (unknownElementCache[tag] = /HTMLUnknownElement/.test(el.toString()))
    }
  }

  var isTextInputType = makeMap('text,number,password,search,email,tel,url');

  /*  */

  /**
   * Query an element selector if it's not an element already.
   */
  function query (el) {
    if (typeof el === 'string') {
      var selected = document.querySelector(el);
      if (!selected) {
        warn(
          'Cannot find element: ' + el
        );
        return document.createElement('div')
      }
      return selected
    } else {
      return el
    }
  }

  /*  */

  function createElement$1 (tagName, vnode) {
    var elm = document.createElement(tagName);
    if (tagName !== 'select') {
      return elm
    }
    // false or null will remove the attribute but undefined will not
    if (vnode.data && vnode.data.attrs && vnode.data.attrs.multiple !== undefined) {
      elm.setAttribute('multiple', 'multiple');
    }
    return elm
  }

  function createElementNS (namespace, tagName) {
    return document.createElementNS(namespaceMap[namespace], tagName)
  }

  function createTextNode (text) {
    return document.createTextNode(text)
  }

  function createComment (text) {
    return document.createComment(text)
  }

  function insertBefore (parentNode, newNode, referenceNode) {
    parentNode.insertBefore(newNode, referenceNode);
  }

  function removeChild (node, child) {
    node.removeChild(child);
  }

  function appendChild (node, child) {
    node.appendChild(child);
  }

  function parentNode (node) {
    return node.parentNode
  }

  function nextSibling (node) {
    return node.nextSibling
  }

  function tagName (node) {
    return node.tagName
  }

  function setTextContent (node, text) {
    node.textContent = text;
  }

  function setStyleScope (node, scopeId) {
    node.setAttribute(scopeId, '');
  }

  var nodeOps = /*#__PURE__*/Object.freeze({
    createElement: createElement$1,
    createElementNS: createElementNS,
    createTextNode: createTextNode,
    createComment: createComment,
    insertBefore: insertBefore,
    removeChild: removeChild,
    appendChild: appendChild,
    parentNode: parentNode,
    nextSibling: nextSibling,
    tagName: tagName,
    setTextContent: setTextContent,
    setStyleScope: setStyleScope
  });

  /*  */

  var ref = {
    create: function create (_, vnode) {
      registerRef(vnode);
    },
    update: function update (oldVnode, vnode) {
      if (oldVnode.data.ref !== vnode.data.ref) {
        registerRef(oldVnode, true);
        registerRef(vnode);
      }
    },
    destroy: function destroy (vnode) {
      registerRef(vnode, true);
    }
  };

  function registerRef (vnode, isRemoval) {
    var key = vnode.data.ref;
    if (!isDef(key)) { return }

    var vm = vnode.context;
    var ref = vnode.componentInstance || vnode.elm;
    var refs = vm.$refs;
    if (isRemoval) {
      if (Array.isArray(refs[key])) {
        remove(refs[key], ref);
      } else if (refs[key] === ref) {
        refs[key] = undefined;
      }
    } else {
      if (vnode.data.refInFor) {
        if (!Array.isArray(refs[key])) {
          refs[key] = [ref];
        } else if (refs[key].indexOf(ref) < 0) {
          // $flow-disable-line
          refs[key].push(ref);
        }
      } else {
        refs[key] = ref;
      }
    }
  }

  /**
   * Virtual DOM patching algorithm based on Snabbdom by
   * Simon Friis Vindum (@paldepind)
   * Licensed under the MIT License
   * https://github.com/paldepind/snabbdom/blob/master/LICENSE
   *
   * modified by Evan You (@yyx990803)
   *
   * Not type-checking this because this file is perf-critical and the cost
   * of making flow understand it is not worth it.
   */

  var emptyNode = new VNode('', {}, []);

  var hooks = ['create', 'activate', 'update', 'remove', 'destroy'];

  function sameVnode (a, b) {
    return (
      a.key === b.key && (
        (
          a.tag === b.tag &&
          a.isComment === b.isComment &&
          isDef(a.data) === isDef(b.data) &&
          sameInputType(a, b)
        ) || (
          isTrue(a.isAsyncPlaceholder) &&
          a.asyncFactory === b.asyncFactory &&
          isUndef(b.asyncFactory.error)
        )
      )
    )
  }

  function sameInputType (a, b) {
    if (a.tag !== 'input') { return true }
    var i;
    var typeA = isDef(i = a.data) && isDef(i = i.attrs) && i.type;
    var typeB = isDef(i = b.data) && isDef(i = i.attrs) && i.type;
    return typeA === typeB || isTextInputType(typeA) && isTextInputType(typeB)
  }

  function createKeyToOldIdx (children, beginIdx, endIdx) {
    var i, key;
    var map = {};
    for (i = beginIdx; i <= endIdx; ++i) {
      key = children[i].key;
      if (isDef(key)) { map[key] = i; }
    }
    return map
  }

  function createPatchFunction (backend) {
    var i, j;
    var cbs = {};

    var modules = backend.modules;
    var nodeOps = backend.nodeOps;

    for (i = 0; i < hooks.length; ++i) {
      cbs[hooks[i]] = [];
      for (j = 0; j < modules.length; ++j) {
        if (isDef(modules[j][hooks[i]])) {
          cbs[hooks[i]].push(modules[j][hooks[i]]);
        }
      }
    }

    function emptyNodeAt (elm) {
      return new VNode(nodeOps.tagName(elm).toLowerCase(), {}, [], undefined, elm)
    }

    function createRmCb (childElm, listeners) {
      function remove$$1 () {
        if (--remove$$1.listeners === 0) {
          removeNode(childElm);
        }
      }
      remove$$1.listeners = listeners;
      return remove$$1
    }

    function removeNode (el) {
      var parent = nodeOps.parentNode(el);
      // element may have already been removed due to v-html / v-text
      if (isDef(parent)) {
        nodeOps.removeChild(parent, el);
      }
    }

    function isUnknownElement$$1 (vnode, inVPre) {
      return (
        !inVPre &&
        !vnode.ns &&
        !(
          config.ignoredElements.length &&
          config.ignoredElements.some(function (ignore) {
            return isRegExp(ignore)
              ? ignore.test(vnode.tag)
              : ignore === vnode.tag
          })
        ) &&
        config.isUnknownElement(vnode.tag)
      )
    }

    var creatingElmInVPre = 0;

    function createElm (
      vnode,
      insertedVnodeQueue,
      parentElm,
      refElm,
      nested,
      ownerArray,
      index
    ) {
      if (isDef(vnode.elm) && isDef(ownerArray)) {
        // This vnode was used in a previous render!
        // now it's used as a new node, overwriting its elm would cause
        // potential patch errors down the road when it's used as an insertion
        // reference node. Instead, we clone the node on-demand before creating
        // associated DOM element for it.
        vnode = ownerArray[index] = cloneVNode(vnode);
      }

      vnode.isRootInsert = !nested; // for transition enter check
      if (createComponent(vnode, insertedVnodeQueue, parentElm, refElm)) {
        return
      }

      var data = vnode.data;
      var children = vnode.children;
      var tag = vnode.tag;
      if (isDef(tag)) {
        {
          if (data && data.pre) {
            creatingElmInVPre++;
          }
          if (isUnknownElement$$1(vnode, creatingElmInVPre)) {
            warn(
              'Unknown custom element: <' + tag + '> - did you ' +
              'register the component correctly? For recursive components, ' +
              'make sure to provide the "name" option.',
              vnode.context
            );
          }
        }

        vnode.elm = vnode.ns
          ? nodeOps.createElementNS(vnode.ns, tag)
          : nodeOps.createElement(tag, vnode);
        setScope(vnode);

        /* istanbul ignore if */
        {
          createChildren(vnode, children, insertedVnodeQueue);
          if (isDef(data)) {
            invokeCreateHooks(vnode, insertedVnodeQueue);
          }
          insert(parentElm, vnode.elm, refElm);
        }

        if (data && data.pre) {
          creatingElmInVPre--;
        }
      } else if (isTrue(vnode.isComment)) {
        vnode.elm = nodeOps.createComment(vnode.text);
        insert(parentElm, vnode.elm, refElm);
      } else {
        vnode.elm = nodeOps.createTextNode(vnode.text);
        insert(parentElm, vnode.elm, refElm);
      }
    }

    function createComponent (vnode, insertedVnodeQueue, parentElm, refElm) {
      var i = vnode.data;
      if (isDef(i)) {
        var isReactivated = isDef(vnode.componentInstance) && i.keepAlive;
        if (isDef(i = i.hook) && isDef(i = i.init)) {
          i(vnode, false /* hydrating */);
        }
        // after calling the init hook, if the vnode is a child component
        // it should've created a child instance and mounted it. the child
        // component also has set the placeholder vnode's elm.
        // in that case we can just return the element and be done.
        if (isDef(vnode.componentInstance)) {
          initComponent(vnode, insertedVnodeQueue);
          insert(parentElm, vnode.elm, refElm);
          if (isTrue(isReactivated)) {
            reactivateComponent(vnode, insertedVnodeQueue, parentElm, refElm);
          }
          return true
        }
      }
    }

    function initComponent (vnode, insertedVnodeQueue) {
      if (isDef(vnode.data.pendingInsert)) {
        insertedVnodeQueue.push.apply(insertedVnodeQueue, vnode.data.pendingInsert);
        vnode.data.pendingInsert = null;
      }
      vnode.elm = vnode.componentInstance.$el;
      if (isPatchable(vnode)) {
        invokeCreateHooks(vnode, insertedVnodeQueue);
        setScope(vnode);
      } else {
        // empty component root.
        // skip all element-related modules except for ref (#3455)
        registerRef(vnode);
        // make sure to invoke the insert hook
        insertedVnodeQueue.push(vnode);
      }
    }

    function reactivateComponent (vnode, insertedVnodeQueue, parentElm, refElm) {
      var i;
      // hack for #4339: a reactivated component with inner transition
      // does not trigger because the inner node's created hooks are not called
      // again. It's not ideal to involve module-specific logic in here but
      // there doesn't seem to be a better way to do it.
      var innerNode = vnode;
      while (innerNode.componentInstance) {
        innerNode = innerNode.componentInstance._vnode;
        if (isDef(i = innerNode.data) && isDef(i = i.transition)) {
          for (i = 0; i < cbs.activate.length; ++i) {
            cbs.activate[i](emptyNode, innerNode);
          }
          insertedVnodeQueue.push(innerNode);
          break
        }
      }
      // unlike a newly created component,
      // a reactivated keep-alive component doesn't insert itself
      insert(parentElm, vnode.elm, refElm);
    }

    function insert (parent, elm, ref$$1) {
      if (isDef(parent)) {
        if (isDef(ref$$1)) {
          if (nodeOps.parentNode(ref$$1) === parent) {
            nodeOps.insertBefore(parent, elm, ref$$1);
          }
        } else {
          nodeOps.appendChild(parent, elm);
        }
      }
    }

    function createChildren (vnode, children, insertedVnodeQueue) {
      if (Array.isArray(children)) {
        {
          checkDuplicateKeys(children);
        }
        for (var i = 0; i < children.length; ++i) {
          createElm(children[i], insertedVnodeQueue, vnode.elm, null, true, children, i);
        }
      } else if (isPrimitive(vnode.text)) {
        nodeOps.appendChild(vnode.elm, nodeOps.createTextNode(String(vnode.text)));
      }
    }

    function isPatchable (vnode) {
      while (vnode.componentInstance) {
        vnode = vnode.componentInstance._vnode;
      }
      return isDef(vnode.tag)
    }

    function invokeCreateHooks (vnode, insertedVnodeQueue) {
      for (var i$1 = 0; i$1 < cbs.create.length; ++i$1) {
        cbs.create[i$1](emptyNode, vnode);
      }
      i = vnode.data.hook; // Reuse variable
      if (isDef(i)) {
        if (isDef(i.create)) { i.create(emptyNode, vnode); }
        if (isDef(i.insert)) { insertedVnodeQueue.push(vnode); }
      }
    }

    // set scope id attribute for scoped CSS.
    // this is implemented as a special case to avoid the overhead
    // of going through the normal attribute patching process.
    function setScope (vnode) {
      var i;
      if (isDef(i = vnode.fnScopeId)) {
        nodeOps.setStyleScope(vnode.elm, i);
      } else {
        var ancestor = vnode;
        while (ancestor) {
          if (isDef(i = ancestor.context) && isDef(i = i.$options._scopeId)) {
            nodeOps.setStyleScope(vnode.elm, i);
          }
          ancestor = ancestor.parent;
        }
      }
      // for slot content they should also get the scopeId from the host instance.
      if (isDef(i = activeInstance) &&
        i !== vnode.context &&
        i !== vnode.fnContext &&
        isDef(i = i.$options._scopeId)
      ) {
        nodeOps.setStyleScope(vnode.elm, i);
      }
    }

    function addVnodes (parentElm, refElm, vnodes, startIdx, endIdx, insertedVnodeQueue) {
      for (; startIdx <= endIdx; ++startIdx) {
        createElm(vnodes[startIdx], insertedVnodeQueue, parentElm, refElm, false, vnodes, startIdx);
      }
    }

    function invokeDestroyHook (vnode) {
      var i, j;
      var data = vnode.data;
      if (isDef(data)) {
        if (isDef(i = data.hook) && isDef(i = i.destroy)) { i(vnode); }
        for (i = 0; i < cbs.destroy.length; ++i) { cbs.destroy[i](vnode); }
      }
      if (isDef(i = vnode.children)) {
        for (j = 0; j < vnode.children.length; ++j) {
          invokeDestroyHook(vnode.children[j]);
        }
      }
    }

    function removeVnodes (vnodes, startIdx, endIdx) {
      for (; startIdx <= endIdx; ++startIdx) {
        var ch = vnodes[startIdx];
        if (isDef(ch)) {
          if (isDef(ch.tag)) {
            removeAndInvokeRemoveHook(ch);
            invokeDestroyHook(ch);
          } else { // Text node
            removeNode(ch.elm);
          }
        }
      }
    }

    function removeAndInvokeRemoveHook (vnode, rm) {
      if (isDef(rm) || isDef(vnode.data)) {
        var i;
        var listeners = cbs.remove.length + 1;
        if (isDef(rm)) {
          // we have a recursively passed down rm callback
          // increase the listeners count
          rm.listeners += listeners;
        } else {
          // directly removing
          rm = createRmCb(vnode.elm, listeners);
        }
        // recursively invoke hooks on child component root node
        if (isDef(i = vnode.componentInstance) && isDef(i = i._vnode) && isDef(i.data)) {
          removeAndInvokeRemoveHook(i, rm);
        }
        for (i = 0; i < cbs.remove.length; ++i) {
          cbs.remove[i](vnode, rm);
        }
        if (isDef(i = vnode.data.hook) && isDef(i = i.remove)) {
          i(vnode, rm);
        } else {
          rm();
        }
      } else {
        removeNode(vnode.elm);
      }
    }

    function updateChildren (parentElm, oldCh, newCh, insertedVnodeQueue, removeOnly) {
      var oldStartIdx = 0;
      var newStartIdx = 0;
      var oldEndIdx = oldCh.length - 1;
      var oldStartVnode = oldCh[0];
      var oldEndVnode = oldCh[oldEndIdx];
      var newEndIdx = newCh.length - 1;
      var newStartVnode = newCh[0];
      var newEndVnode = newCh[newEndIdx];
      var oldKeyToIdx, idxInOld, vnodeToMove, refElm;

      // removeOnly is a special flag used only by <transition-group>
      // to ensure removed elements stay in correct relative positions
      // during leaving transitions
      var canMove = !removeOnly;

      {
        checkDuplicateKeys(newCh);
      }

      while (oldStartIdx <= oldEndIdx && newStartIdx <= newEndIdx) {
        if (isUndef(oldStartVnode)) {
          oldStartVnode = oldCh[++oldStartIdx]; // Vnode has been moved left
        } else if (isUndef(oldEndVnode)) {
          oldEndVnode = oldCh[--oldEndIdx];
        } else if (sameVnode(oldStartVnode, newStartVnode)) {
          patchVnode(oldStartVnode, newStartVnode, insertedVnodeQueue, newCh, newStartIdx);
          oldStartVnode = oldCh[++oldStartIdx];
          newStartVnode = newCh[++newStartIdx];
        } else if (sameVnode(oldEndVnode, newEndVnode)) {
          patchVnode(oldEndVnode, newEndVnode, insertedVnodeQueue, newCh, newEndIdx);
          oldEndVnode = oldCh[--oldEndIdx];
          newEndVnode = newCh[--newEndIdx];
        } else if (sameVnode(oldStartVnode, newEndVnode)) { // Vnode moved right
          patchVnode(oldStartVnode, newEndVnode, insertedVnodeQueue, newCh, newEndIdx);
          canMove && nodeOps.insertBefore(parentElm, oldStartVnode.elm, nodeOps.nextSibling(oldEndVnode.elm));
          oldStartVnode = oldCh[++oldStartIdx];
          newEndVnode = newCh[--newEndIdx];
        } else if (sameVnode(oldEndVnode, newStartVnode)) { // Vnode moved left
          patchVnode(oldEndVnode, newStartVnode, insertedVnodeQueue, newCh, newStartIdx);
          canMove && nodeOps.insertBefore(parentElm, oldEndVnode.elm, oldStartVnode.elm);
          oldEndVnode = oldCh[--oldEndIdx];
          newStartVnode = newCh[++newStartIdx];
        } else {
          if (isUndef(oldKeyToIdx)) { oldKeyToIdx = createKeyToOldIdx(oldCh, oldStartIdx, oldEndIdx); }
          idxInOld = isDef(newStartVnode.key)
            ? oldKeyToIdx[newStartVnode.key]
            : findIdxInOld(newStartVnode, oldCh, oldStartIdx, oldEndIdx);
          if (isUndef(idxInOld)) { // New element
            createElm(newStartVnode, insertedVnodeQueue, parentElm, oldStartVnode.elm, false, newCh, newStartIdx);
          } else {
            vnodeToMove = oldCh[idxInOld];
            if (sameVnode(vnodeToMove, newStartVnode)) {
              patchVnode(vnodeToMove, newStartVnode, insertedVnodeQueue, newCh, newStartIdx);
              oldCh[idxInOld] = undefined;
              canMove && nodeOps.insertBefore(parentElm, vnodeToMove.elm, oldStartVnode.elm);
            } else {
              // same key but different element. treat as new element
              createElm(newStartVnode, insertedVnodeQueue, parentElm, oldStartVnode.elm, false, newCh, newStartIdx);
            }
          }
          newStartVnode = newCh[++newStartIdx];
        }
      }
      if (oldStartIdx > oldEndIdx) {
        refElm = isUndef(newCh[newEndIdx + 1]) ? null : newCh[newEndIdx + 1].elm;
        addVnodes(parentElm, refElm, newCh, newStartIdx, newEndIdx, insertedVnodeQueue);
      } else if (newStartIdx > newEndIdx) {
        removeVnodes(oldCh, oldStartIdx, oldEndIdx);
      }
    }

    function checkDuplicateKeys (children) {
      var seenKeys = {};
      for (var i = 0; i < children.length; i++) {
        var vnode = children[i];
        var key = vnode.key;
        if (isDef(key)) {
          if (seenKeys[key]) {
            warn(
              ("Duplicate keys detected: '" + key + "'. This may cause an update error."),
              vnode.context
            );
          } else {
            seenKeys[key] = true;
          }
        }
      }
    }

    function findIdxInOld (node, oldCh, start, end) {
      for (var i = start; i < end; i++) {
        var c = oldCh[i];
        if (isDef(c) && sameVnode(node, c)) { return i }
      }
    }

    function patchVnode (
      oldVnode,
      vnode,
      insertedVnodeQueue,
      ownerArray,
      index,
      removeOnly
    ) {
      if (oldVnode === vnode) {
        return
      }

      if (isDef(vnode.elm) && isDef(ownerArray)) {
        // clone reused vnode
        vnode = ownerArray[index] = cloneVNode(vnode);
      }

      var elm = vnode.elm = oldVnode.elm;

      if (isTrue(oldVnode.isAsyncPlaceholder)) {
        if (isDef(vnode.asyncFactory.resolved)) {
          hydrate(oldVnode.elm, vnode, insertedVnodeQueue);
        } else {
          vnode.isAsyncPlaceholder = true;
        }
        return
      }

      // reuse element for static trees.
      // note we only do this if the vnode is cloned -
      // if the new node is not cloned it means the render functions have been
      // reset by the hot-reload-api and we need to do a proper re-render.
      if (isTrue(vnode.isStatic) &&
        isTrue(oldVnode.isStatic) &&
        vnode.key === oldVnode.key &&
        (isTrue(vnode.isCloned) || isTrue(vnode.isOnce))
      ) {
        vnode.componentInstance = oldVnode.componentInstance;
        return
      }

      var i;
      var data = vnode.data;
      if (isDef(data) && isDef(i = data.hook) && isDef(i = i.prepatch)) {
        i(oldVnode, vnode);
      }

      var oldCh = oldVnode.children;
      var ch = vnode.children;
      if (isDef(data) && isPatchable(vnode)) {
        for (i = 0; i < cbs.update.length; ++i) { cbs.update[i](oldVnode, vnode); }
        if (isDef(i = data.hook) && isDef(i = i.update)) { i(oldVnode, vnode); }
      }
      if (isUndef(vnode.text)) {
        if (isDef(oldCh) && isDef(ch)) {
          if (oldCh !== ch) { updateChildren(elm, oldCh, ch, insertedVnodeQueue, removeOnly); }
        } else if (isDef(ch)) {
          {
            checkDuplicateKeys(ch);
          }
          if (isDef(oldVnode.text)) { nodeOps.setTextContent(elm, ''); }
          addVnodes(elm, null, ch, 0, ch.length - 1, insertedVnodeQueue);
        } else if (isDef(oldCh)) {
          removeVnodes(oldCh, 0, oldCh.length - 1);
        } else if (isDef(oldVnode.text)) {
          nodeOps.setTextContent(elm, '');
        }
      } else if (oldVnode.text !== vnode.text) {
        nodeOps.setTextContent(elm, vnode.text);
      }
      if (isDef(data)) {
        if (isDef(i = data.hook) && isDef(i = i.postpatch)) { i(oldVnode, vnode); }
      }
    }

    function invokeInsertHook (vnode, queue, initial) {
      // delay insert hooks for component root nodes, invoke them after the
      // element is really inserted
      if (isTrue(initial) && isDef(vnode.parent)) {
        vnode.parent.data.pendingInsert = queue;
      } else {
        for (var i = 0; i < queue.length; ++i) {
          queue[i].data.hook.insert(queue[i]);
        }
      }
    }

    var hydrationBailed = false;
    // list of modules that can skip create hook during hydration because they
    // are already rendered on the client or has no need for initialization
    // Note: style is excluded because it relies on initial clone for future
    // deep updates (#7063).
    var isRenderedModule = makeMap('attrs,class,staticClass,staticStyle,key');

    // Note: this is a browser-only function so we can assume elms are DOM nodes.
    function hydrate (elm, vnode, insertedVnodeQueue, inVPre) {
      var i;
      var tag = vnode.tag;
      var data = vnode.data;
      var children = vnode.children;
      inVPre = inVPre || (data && data.pre);
      vnode.elm = elm;

      if (isTrue(vnode.isComment) && isDef(vnode.asyncFactory)) {
        vnode.isAsyncPlaceholder = true;
        return true
      }
      // assert node match
      {
        if (!assertNodeMatch(elm, vnode, inVPre)) {
          return false
        }
      }
      if (isDef(data)) {
        if (isDef(i = data.hook) && isDef(i = i.init)) { i(vnode, true /* hydrating */); }
        if (isDef(i = vnode.componentInstance)) {
          // child component. it should have hydrated its own tree.
          initComponent(vnode, insertedVnodeQueue);
          return true
        }
      }
      if (isDef(tag)) {
        if (isDef(children)) {
          // empty element, allow client to pick up and populate children
          if (!elm.hasChildNodes()) {
            createChildren(vnode, children, insertedVnodeQueue);
          } else {
            // v-html and domProps: innerHTML
            if (isDef(i = data) && isDef(i = i.domProps) && isDef(i = i.innerHTML)) {
              if (i !== elm.innerHTML) {
                /* istanbul ignore if */
                if (typeof console !== 'undefined' &&
                  !hydrationBailed
                ) {
                  hydrationBailed = true;
                  console.warn('Parent: ', elm);
                  console.warn('server innerHTML: ', i);
                  console.warn('client innerHTML: ', elm.innerHTML);
                }
                return false
              }
            } else {
              // iterate and compare children lists
              var childrenMatch = true;
              var childNode = elm.firstChild;
              for (var i$1 = 0; i$1 < children.length; i$1++) {
                if (!childNode || !hydrate(childNode, children[i$1], insertedVnodeQueue, inVPre)) {
                  childrenMatch = false;
                  break
                }
                childNode = childNode.nextSibling;
              }
              // if childNode is not null, it means the actual childNodes list is
              // longer than the virtual children list.
              if (!childrenMatch || childNode) {
                /* istanbul ignore if */
                if (typeof console !== 'undefined' &&
                  !hydrationBailed
                ) {
                  hydrationBailed = true;
                  console.warn('Parent: ', elm);
                  console.warn('Mismatching childNodes vs. VNodes: ', elm.childNodes, children);
                }
                return false
              }
            }
          }
        }
        if (isDef(data)) {
          var fullInvoke = false;
          for (var key in data) {
            if (!isRenderedModule(key)) {
              fullInvoke = true;
              invokeCreateHooks(vnode, insertedVnodeQueue);
              break
            }
          }
          if (!fullInvoke && data['class']) {
            // ensure collecting deps for deep class bindings for future updates
            traverse(data['class']);
          }
        }
      } else if (elm.data !== vnode.text) {
        elm.data = vnode.text;
      }
      return true
    }

    function assertNodeMatch (node, vnode, inVPre) {
      if (isDef(vnode.tag)) {
        return vnode.tag.indexOf('vue-component') === 0 || (
          !isUnknownElement$$1(vnode, inVPre) &&
          vnode.tag.toLowerCase() === (node.tagName && node.tagName.toLowerCase())
        )
      } else {
        return node.nodeType === (vnode.isComment ? 8 : 3)
      }
    }

    return function patch (oldVnode, vnode, hydrating, removeOnly) {
      if (isUndef(vnode)) {
        if (isDef(oldVnode)) { invokeDestroyHook(oldVnode); }
        return
      }

      var isInitialPatch = false;
      var insertedVnodeQueue = [];

      if (isUndef(oldVnode)) {
        // empty mount (likely as component), create new root element
        isInitialPatch = true;
        createElm(vnode, insertedVnodeQueue);
      } else {
        var isRealElement = isDef(oldVnode.nodeType);
        if (!isRealElement && sameVnode(oldVnode, vnode)) {
          // patch existing root node
          patchVnode(oldVnode, vnode, insertedVnodeQueue, null, null, removeOnly);
        } else {
          if (isRealElement) {
            // mounting to a real element
            // check if this is server-rendered content and if we can perform
            // a successful hydration.
            if (oldVnode.nodeType === 1 && oldVnode.hasAttribute(SSR_ATTR)) {
              oldVnode.removeAttribute(SSR_ATTR);
              hydrating = true;
            }
            if (isTrue(hydrating)) {
              if (hydrate(oldVnode, vnode, insertedVnodeQueue)) {
                invokeInsertHook(vnode, insertedVnodeQueue, true);
                return oldVnode
              } else {
                warn(
                  'The client-side rendered virtual DOM tree is not matching ' +
                  'server-rendered content. This is likely caused by incorrect ' +
                  'HTML markup, for example nesting block-level elements inside ' +
                  '<p>, or missing <tbody>. Bailing hydration and performing ' +
                  'full client-side render.'
                );
              }
            }
            // either not server-rendered, or hydration failed.
            // create an empty node and replace it
            oldVnode = emptyNodeAt(oldVnode);
          }

          // replacing existing element
          var oldElm = oldVnode.elm;
          var parentElm = nodeOps.parentNode(oldElm);

          // create new node
          createElm(
            vnode,
            insertedVnodeQueue,
            // extremely rare edge case: do not insert if old element is in a
            // leaving transition. Only happens when combining transition +
            // keep-alive + HOCs. (#4590)
            oldElm._leaveCb ? null : parentElm,
            nodeOps.nextSibling(oldElm)
          );

          // update parent placeholder node element, recursively
          if (isDef(vnode.parent)) {
            var ancestor = vnode.parent;
            var patchable = isPatchable(vnode);
            while (ancestor) {
              for (var i = 0; i < cbs.destroy.length; ++i) {
                cbs.destroy[i](ancestor);
              }
              ancestor.elm = vnode.elm;
              if (patchable) {
                for (var i$1 = 0; i$1 < cbs.create.length; ++i$1) {
                  cbs.create[i$1](emptyNode, ancestor);
                }
                // #6513
                // invoke insert hooks that may have been merged by create hooks.
                // e.g. for directives that uses the "inserted" hook.
                var insert = ancestor.data.hook.insert;
                if (insert.merged) {
                  // start at index 1 to avoid re-invoking component mounted hook
                  for (var i$2 = 1; i$2 < insert.fns.length; i$2++) {
                    insert.fns[i$2]();
                  }
                }
              } else {
                registerRef(ancestor);
              }
              ancestor = ancestor.parent;
            }
          }

          // destroy old node
          if (isDef(parentElm)) {
            removeVnodes([oldVnode], 0, 0);
          } else if (isDef(oldVnode.tag)) {
            invokeDestroyHook(oldVnode);
          }
        }
      }

      invokeInsertHook(vnode, insertedVnodeQueue, isInitialPatch);
      return vnode.elm
    }
  }

  /*  */

  var directives = {
    create: updateDirectives,
    update: updateDirectives,
    destroy: function unbindDirectives (vnode) {
      updateDirectives(vnode, emptyNode);
    }
  };

  function updateDirectives (oldVnode, vnode) {
    if (oldVnode.data.directives || vnode.data.directives) {
      _update(oldVnode, vnode);
    }
  }

  function _update (oldVnode, vnode) {
    var isCreate = oldVnode === emptyNode;
    var isDestroy = vnode === emptyNode;
    var oldDirs = normalizeDirectives$1(oldVnode.data.directives, oldVnode.context);
    var newDirs = normalizeDirectives$1(vnode.data.directives, vnode.context);

    var dirsWithInsert = [];
    var dirsWithPostpatch = [];

    var key, oldDir, dir;
    for (key in newDirs) {
      oldDir = oldDirs[key];
      dir = newDirs[key];
      if (!oldDir) {
        // new directive, bind
        callHook$1(dir, 'bind', vnode, oldVnode);
        if (dir.def && dir.def.inserted) {
          dirsWithInsert.push(dir);
        }
      } else {
        // existing directive, update
        dir.oldValue = oldDir.value;
        dir.oldArg = oldDir.arg;
        callHook$1(dir, 'update', vnode, oldVnode);
        if (dir.def && dir.def.componentUpdated) {
          dirsWithPostpatch.push(dir);
        }
      }
    }

    if (dirsWithInsert.length) {
      var callInsert = function () {
        for (var i = 0; i < dirsWithInsert.length; i++) {
          callHook$1(dirsWithInsert[i], 'inserted', vnode, oldVnode);
        }
      };
      if (isCreate) {
        mergeVNodeHook(vnode, 'insert', callInsert);
      } else {
        callInsert();
      }
    }

    if (dirsWithPostpatch.length) {
      mergeVNodeHook(vnode, 'postpatch', function () {
        for (var i = 0; i < dirsWithPostpatch.length; i++) {
          callHook$1(dirsWithPostpatch[i], 'componentUpdated', vnode, oldVnode);
        }
      });
    }

    if (!isCreate) {
      for (key in oldDirs) {
        if (!newDirs[key]) {
          // no longer present, unbind
          callHook$1(oldDirs[key], 'unbind', oldVnode, oldVnode, isDestroy);
        }
      }
    }
  }

  var emptyModifiers = Object.create(null);

  function normalizeDirectives$1 (
    dirs,
    vm
  ) {
    var res = Object.create(null);
    if (!dirs) {
      // $flow-disable-line
      return res
    }
    var i, dir;
    for (i = 0; i < dirs.length; i++) {
      dir = dirs[i];
      if (!dir.modifiers) {
        // $flow-disable-line
        dir.modifiers = emptyModifiers;
      }
      res[getRawDirName(dir)] = dir;
      dir.def = resolveAsset(vm.$options, 'directives', dir.name, true);
    }
    // $flow-disable-line
    return res
  }

  function getRawDirName (dir) {
    return dir.rawName || ((dir.name) + "." + (Object.keys(dir.modifiers || {}).join('.')))
  }

  function callHook$1 (dir, hook, vnode, oldVnode, isDestroy) {
    var fn = dir.def && dir.def[hook];
    if (fn) {
      try {
        fn(vnode.elm, dir, vnode, oldVnode, isDestroy);
      } catch (e) {
        handleError(e, vnode.context, ("directive " + (dir.name) + " " + hook + " hook"));
      }
    }
  }

  var baseModules = [
    ref,
    directives
  ];

  /*  */

  function updateAttrs (oldVnode, vnode) {
    var opts = vnode.componentOptions;
    if (isDef(opts) && opts.Ctor.options.inheritAttrs === false) {
      return
    }
    if (isUndef(oldVnode.data.attrs) && isUndef(vnode.data.attrs)) {
      return
    }
    var key, cur, old;
    var elm = vnode.elm;
    var oldAttrs = oldVnode.data.attrs || {};
    var attrs = vnode.data.attrs || {};
    // clone observed objects, as the user probably wants to mutate it
    if (isDef(attrs.__ob__)) {
      attrs = vnode.data.attrs = extend({}, attrs);
    }

    for (key in attrs) {
      cur = attrs[key];
      old = oldAttrs[key];
      if (old !== cur) {
        setAttr(elm, key, cur);
      }
    }
    // #4391: in IE9, setting type can reset value for input[type=radio]
    // #6666: IE/Edge forces progress value down to 1 before setting a max
    /* istanbul ignore if */
    if ((isIE || isEdge) && attrs.value !== oldAttrs.value) {
      setAttr(elm, 'value', attrs.value);
    }
    for (key in oldAttrs) {
      if (isUndef(attrs[key])) {
        if (isXlink(key)) {
          elm.removeAttributeNS(xlinkNS, getXlinkProp(key));
        } else if (!isEnumeratedAttr(key)) {
          elm.removeAttribute(key);
        }
      }
    }
  }

  function setAttr (el, key, value) {
    if (el.tagName.indexOf('-') > -1) {
      baseSetAttr(el, key, value);
    } else if (isBooleanAttr(key)) {
      // set attribute for blank value
      // e.g. <option disabled>Select one</option>
      if (isFalsyAttrValue(value)) {
        el.removeAttribute(key);
      } else {
        // technically allowfullscreen is a boolean attribute for <iframe>,
        // but Flash expects a value of "true" when used on <embed> tag
        value = key === 'allowfullscreen' && el.tagName === 'EMBED'
          ? 'true'
          : key;
        el.setAttribute(key, value);
      }
    } else if (isEnumeratedAttr(key)) {
      el.setAttribute(key, convertEnumeratedValue(key, value));
    } else if (isXlink(key)) {
      if (isFalsyAttrValue(value)) {
        el.removeAttributeNS(xlinkNS, getXlinkProp(key));
      } else {
        el.setAttributeNS(xlinkNS, key, value);
      }
    } else {
      baseSetAttr(el, key, value);
    }
  }

  function baseSetAttr (el, key, value) {
    if (isFalsyAttrValue(value)) {
      el.removeAttribute(key);
    } else {
      // #7138: IE10 & 11 fires input event when setting placeholder on
      // <textarea>... block the first input event and remove the blocker
      // immediately.
      /* istanbul ignore if */
      if (
        isIE && !isIE9 &&
        el.tagName === 'TEXTAREA' &&
        key === 'placeholder' && value !== '' && !el.__ieph
      ) {
        var blocker = function (e) {
          e.stopImmediatePropagation();
          el.removeEventListener('input', blocker);
        };
        el.addEventListener('input', blocker);
        // $flow-disable-line
        el.__ieph = true; /* IE placeholder patched */
      }
      el.setAttribute(key, value);
    }
  }

  var attrs = {
    create: updateAttrs,
    update: updateAttrs
  };

  /*  */

  function updateClass (oldVnode, vnode) {
    var el = vnode.elm;
    var data = vnode.data;
    var oldData = oldVnode.data;
    if (
      isUndef(data.staticClass) &&
      isUndef(data.class) && (
        isUndef(oldData) || (
          isUndef(oldData.staticClass) &&
          isUndef(oldData.class)
        )
      )
    ) {
      return
    }

    var cls = genClassForVnode(vnode);

    // handle transition classes
    var transitionClass = el._transitionClasses;
    if (isDef(transitionClass)) {
      cls = concat(cls, stringifyClass(transitionClass));
    }

    // set the class
    if (cls !== el._prevClass) {
      el.setAttribute('class', cls);
      el._prevClass = cls;
    }
  }

  var klass = {
    create: updateClass,
    update: updateClass
  };

  /*  */

  /*  */

  /*  */

  /*  */

  // in some cases, the event used has to be determined at runtime
  // so we used some reserved tokens during compile.
  var RANGE_TOKEN = '__r';
  var CHECKBOX_RADIO_TOKEN = '__c';

  /*  */

  // normalize v-model event tokens that can only be determined at runtime.
  // it's important to place the event as the first in the array because
  // the whole point is ensuring the v-model callback gets called before
  // user-attached handlers.
  function normalizeEvents (on) {
    /* istanbul ignore if */
    if (isDef(on[RANGE_TOKEN])) {
      // IE input[type=range] only supports `change` event
      var event = isIE ? 'change' : 'input';
      on[event] = [].concat(on[RANGE_TOKEN], on[event] || []);
      delete on[RANGE_TOKEN];
    }
    // This was originally intended to fix #4521 but no longer necessary
    // after 2.5. Keeping it for backwards compat with generated code from < 2.4
    /* istanbul ignore if */
    if (isDef(on[CHECKBOX_RADIO_TOKEN])) {
      on.change = [].concat(on[CHECKBOX_RADIO_TOKEN], on.change || []);
      delete on[CHECKBOX_RADIO_TOKEN];
    }
  }

  var target$1;

  function createOnceHandler$1 (event, handler, capture) {
    var _target = target$1; // save current target element in closure
    return function onceHandler () {
      var res = handler.apply(null, arguments);
      if (res !== null) {
        remove$2(event, onceHandler, capture, _target);
      }
    }
  }

  // #9446: Firefox <= 53 (in particular, ESR 52) has incorrect Event.timeStamp
  // implementation and does not fire microtasks in between event propagation, so
  // safe to exclude.
  var useMicrotaskFix = isUsingMicroTask && !(isFF && Number(isFF[1]) <= 53);

  function add$1 (
    name,
    handler,
    capture,
    passive
  ) {
    // async edge case #6566: inner click event triggers patch, event handler
    // attached to outer element during patch, and triggered again. This
    // happens because browsers fire microtask ticks between event propagation.
    // the solution is simple: we save the timestamp when a handler is attached,
    // and the handler would only fire if the event passed to it was fired
    // AFTER it was attached.
    if (useMicrotaskFix) {
      var attachedTimestamp = currentFlushTimestamp;
      var original = handler;
      handler = original._wrapper = function (e) {
        if (
          // no bubbling, should always fire.
          // this is just a safety net in case event.timeStamp is unreliable in
          // certain weird environments...
          e.target === e.currentTarget ||
          // event is fired after handler attachment
          e.timeStamp >= attachedTimestamp ||
          // bail for environments that have buggy event.timeStamp implementations
          // #9462 iOS 9 bug: event.timeStamp is 0 after history.pushState
          // #9681 QtWebEngine event.timeStamp is negative value
          e.timeStamp <= 0 ||
          // #9448 bail if event is fired in another document in a multi-page
          // electron/nw.js app, since event.timeStamp will be using a different
          // starting reference
          e.target.ownerDocument !== document
        ) {
          return original.apply(this, arguments)
        }
      };
    }
    target$1.addEventListener(
      name,
      handler,
      supportsPassive
        ? { capture: capture, passive: passive }
        : capture
    );
  }

  function remove$2 (
    name,
    handler,
    capture,
    _target
  ) {
    (_target || target$1).removeEventListener(
      name,
      handler._wrapper || handler,
      capture
    );
  }

  function updateDOMListeners (oldVnode, vnode) {
    if (isUndef(oldVnode.data.on) && isUndef(vnode.data.on)) {
      return
    }
    var on = vnode.data.on || {};
    var oldOn = oldVnode.data.on || {};
    target$1 = vnode.elm;
    normalizeEvents(on);
    updateListeners(on, oldOn, add$1, remove$2, createOnceHandler$1, vnode.context);
    target$1 = undefined;
  }

  var events = {
    create: updateDOMListeners,
    update: updateDOMListeners
  };

  /*  */

  var svgContainer;

  function updateDOMProps (oldVnode, vnode) {
    if (isUndef(oldVnode.data.domProps) && isUndef(vnode.data.domProps)) {
      return
    }
    var key, cur;
    var elm = vnode.elm;
    var oldProps = oldVnode.data.domProps || {};
    var props = vnode.data.domProps || {};
    // clone observed objects, as the user probably wants to mutate it
    if (isDef(props.__ob__)) {
      props = vnode.data.domProps = extend({}, props);
    }

    for (key in oldProps) {
      if (!(key in props)) {
        elm[key] = '';
      }
    }

    for (key in props) {
      cur = props[key];
      // ignore children if the node has textContent or innerHTML,
      // as these will throw away existing DOM nodes and cause removal errors
      // on subsequent patches (#3360)
      if (key === 'textContent' || key === 'innerHTML') {
        if (vnode.children) { vnode.children.length = 0; }
        if (cur === oldProps[key]) { continue }
        // #6601 work around Chrome version <= 55 bug where single textNode
        // replaced by innerHTML/textContent retains its parentNode property
        if (elm.childNodes.length === 1) {
          elm.removeChild(elm.childNodes[0]);
        }
      }

      if (key === 'value' && elm.tagName !== 'PROGRESS') {
        // store value as _value as well since
        // non-string values will be stringified
        elm._value = cur;
        // avoid resetting cursor position when value is the same
        var strCur = isUndef(cur) ? '' : String(cur);
        if (shouldUpdateValue(elm, strCur)) {
          elm.value = strCur;
        }
      } else if (key === 'innerHTML' && isSVG(elm.tagName) && isUndef(elm.innerHTML)) {
        // IE doesn't support innerHTML for SVG elements
        svgContainer = svgContainer || document.createElement('div');
        svgContainer.innerHTML = "<svg>" + cur + "</svg>";
        var svg = svgContainer.firstChild;
        while (elm.firstChild) {
          elm.removeChild(elm.firstChild);
        }
        while (svg.firstChild) {
          elm.appendChild(svg.firstChild);
        }
      } else if (
        // skip the update if old and new VDOM state is the same.
        // `value` is handled separately because the DOM value may be temporarily
        // out of sync with VDOM state due to focus, composition and modifiers.
        // This  #4521 by skipping the unnecesarry `checked` update.
        cur !== oldProps[key]
      ) {
        // some property updates can throw
        // e.g. `value` on <progress> w/ non-finite value
        try {
          elm[key] = cur;
        } catch (e) {}
      }
    }
  }

  // check platforms/web/util/attrs.js acceptValue


  function shouldUpdateValue (elm, checkVal) {
    return (!elm.composing && (
      elm.tagName === 'OPTION' ||
      isNotInFocusAndDirty(elm, checkVal) ||
      isDirtyWithModifiers(elm, checkVal)
    ))
  }

  function isNotInFocusAndDirty (elm, checkVal) {
    // return true when textbox (.number and .trim) loses focus and its value is
    // not equal to the updated value
    var notInFocus = true;
    // #6157
    // work around IE bug when accessing document.activeElement in an iframe
    try { notInFocus = document.activeElement !== elm; } catch (e) {}
    return notInFocus && elm.value !== checkVal
  }

  function isDirtyWithModifiers (elm, newVal) {
    var value = elm.value;
    var modifiers = elm._vModifiers; // injected by v-model runtime
    if (isDef(modifiers)) {
      if (modifiers.number) {
        return toNumber(value) !== toNumber(newVal)
      }
      if (modifiers.trim) {
        return value.trim() !== newVal.trim()
      }
    }
    return value !== newVal
  }

  var domProps = {
    create: updateDOMProps,
    update: updateDOMProps
  };

  /*  */

  var parseStyleText = cached(function (cssText) {
    var res = {};
    var listDelimiter = /;(?![^(]*\))/g;
    var propertyDelimiter = /:(.+)/;
    cssText.split(listDelimiter).forEach(function (item) {
      if (item) {
        var tmp = item.split(propertyDelimiter);
        tmp.length > 1 && (res[tmp[0].trim()] = tmp[1].trim());
      }
    });
    return res
  });

  // merge static and dynamic style data on the same vnode
  function normalizeStyleData (data) {
    var style = normalizeStyleBinding(data.style);
    // static style is pre-processed into an object during compilation
    // and is always a fresh object, so it's safe to merge into it
    return data.staticStyle
      ? extend(data.staticStyle, style)
      : style
  }

  // normalize possible array / string values into Object
  function normalizeStyleBinding (bindingStyle) {
    if (Array.isArray(bindingStyle)) {
      return toObject(bindingStyle)
    }
    if (typeof bindingStyle === 'string') {
      return parseStyleText(bindingStyle)
    }
    return bindingStyle
  }

  /**
   * parent component style should be after child's
   * so that parent component's style could override it
   */
  function getStyle (vnode, checkChild) {
    var res = {};
    var styleData;

    if (checkChild) {
      var childNode = vnode;
      while (childNode.componentInstance) {
        childNode = childNode.componentInstance._vnode;
        if (
          childNode && childNode.data &&
          (styleData = normalizeStyleData(childNode.data))
        ) {
          extend(res, styleData);
        }
      }
    }

    if ((styleData = normalizeStyleData(vnode.data))) {
      extend(res, styleData);
    }

    var parentNode = vnode;
    while ((parentNode = parentNode.parent)) {
      if (parentNode.data && (styleData = normalizeStyleData(parentNode.data))) {
        extend(res, styleData);
      }
    }
    return res
  }

  /*  */

  var cssVarRE = /^--/;
  var importantRE = /\s*!important$/;
  var setProp = function (el, name, val) {
    /* istanbul ignore if */
    if (cssVarRE.test(name)) {
      el.style.setProperty(name, val);
    } else if (importantRE.test(val)) {
      el.style.setProperty(hyphenate(name), val.replace(importantRE, ''), 'important');
    } else {
      var normalizedName = normalize(name);
      if (Array.isArray(val)) {
        // Support values array created by autoprefixer, e.g.
        // {display: ["-webkit-box", "-ms-flexbox", "flex"]}
        // Set them one by one, and the browser will only set those it can recognize
        for (var i = 0, len = val.length; i < len; i++) {
          el.style[normalizedName] = val[i];
        }
      } else {
        el.style[normalizedName] = val;
      }
    }
  };

  var vendorNames = ['Webkit', 'Moz', 'ms'];

  var emptyStyle;
  var normalize = cached(function (prop) {
    emptyStyle = emptyStyle || document.createElement('div').style;
    prop = camelize(prop);
    if (prop !== 'filter' && (prop in emptyStyle)) {
      return prop
    }
    var capName = prop.charAt(0).toUpperCase() + prop.slice(1);
    for (var i = 0; i < vendorNames.length; i++) {
      var name = vendorNames[i] + capName;
      if (name in emptyStyle) {
        return name
      }
    }
  });

  function updateStyle (oldVnode, vnode) {
    var data = vnode.data;
    var oldData = oldVnode.data;

    if (isUndef(data.staticStyle) && isUndef(data.style) &&
      isUndef(oldData.staticStyle) && isUndef(oldData.style)
    ) {
      return
    }

    var cur, name;
    var el = vnode.elm;
    var oldStaticStyle = oldData.staticStyle;
    var oldStyleBinding = oldData.normalizedStyle || oldData.style || {};

    // if static style exists, stylebinding already merged into it when doing normalizeStyleData
    var oldStyle = oldStaticStyle || oldStyleBinding;

    var style = normalizeStyleBinding(vnode.data.style) || {};

    // store normalized style under a different key for next diff
    // make sure to clone it if it's reactive, since the user likely wants
    // to mutate it.
    vnode.data.normalizedStyle = isDef(style.__ob__)
      ? extend({}, style)
      : style;

    var newStyle = getStyle(vnode, true);

    for (name in oldStyle) {
      if (isUndef(newStyle[name])) {
        setProp(el, name, '');
      }
    }
    for (name in newStyle) {
      cur = newStyle[name];
      if (cur !== oldStyle[name]) {
        // ie9 setting to null has no effect, must use empty string
        setProp(el, name, cur == null ? '' : cur);
      }
    }
  }

  var style = {
    create: updateStyle,
    update: updateStyle
  };

  /*  */

  var whitespaceRE = /\s+/;

  /**
   * Add class with compatibility for SVG since classList is not supported on
   * SVG elements in IE
   */
  function addClass (el, cls) {
    /* istanbul ignore if */
    if (!cls || !(cls = cls.trim())) {
      return
    }

    /* istanbul ignore else */
    if (el.classList) {
      if (cls.indexOf(' ') > -1) {
        cls.split(whitespaceRE).forEach(function (c) { return el.classList.add(c); });
      } else {
        el.classList.add(cls);
      }
    } else {
      var cur = " " + (el.getAttribute('class') || '') + " ";
      if (cur.indexOf(' ' + cls + ' ') < 0) {
        el.setAttribute('class', (cur + cls).trim());
      }
    }
  }

  /**
   * Remove class with compatibility for SVG since classList is not supported on
   * SVG elements in IE
   */
  function removeClass (el, cls) {
    /* istanbul ignore if */
    if (!cls || !(cls = cls.trim())) {
      return
    }

    /* istanbul ignore else */
    if (el.classList) {
      if (cls.indexOf(' ') > -1) {
        cls.split(whitespaceRE).forEach(function (c) { return el.classList.remove(c); });
      } else {
        el.classList.remove(cls);
      }
      if (!el.classList.length) {
        el.removeAttribute('class');
      }
    } else {
      var cur = " " + (el.getAttribute('class') || '') + " ";
      var tar = ' ' + cls + ' ';
      while (cur.indexOf(tar) >= 0) {
        cur = cur.replace(tar, ' ');
      }
      cur = cur.trim();
      if (cur) {
        el.setAttribute('class', cur);
      } else {
        el.removeAttribute('class');
      }
    }
  }

  /*  */

  function resolveTransition (def$$1) {
    if (!def$$1) {
      return
    }
    /* istanbul ignore else */
    if (typeof def$$1 === 'object') {
      var res = {};
      if (def$$1.css !== false) {
        extend(res, autoCssTransition(def$$1.name || 'v'));
      }
      extend(res, def$$1);
      return res
    } else if (typeof def$$1 === 'string') {
      return autoCssTransition(def$$1)
    }
  }

  var autoCssTransition = cached(function (name) {
    return {
      enterClass: (name + "-enter"),
      enterToClass: (name + "-enter-to"),
      enterActiveClass: (name + "-enter-active"),
      leaveClass: (name + "-leave"),
      leaveToClass: (name + "-leave-to"),
      leaveActiveClass: (name + "-leave-active")
    }
  });

  var hasTransition = inBrowser && !isIE9;
  var TRANSITION = 'transition';
  var ANIMATION = 'animation';

  // Transition property/event sniffing
  var transitionProp = 'transition';
  var transitionEndEvent = 'transitionend';
  var animationProp = 'animation';
  var animationEndEvent = 'animationend';
  if (hasTransition) {
    /* istanbul ignore if */
    if (window.ontransitionend === undefined &&
      window.onwebkittransitionend !== undefined
    ) {
      transitionProp = 'WebkitTransition';
      transitionEndEvent = 'webkitTransitionEnd';
    }
    if (window.onanimationend === undefined &&
      window.onwebkitanimationend !== undefined
    ) {
      animationProp = 'WebkitAnimation';
      animationEndEvent = 'webkitAnimationEnd';
    }
  }

  // binding to window is necessary to make hot reload work in IE in strict mode
  var raf = inBrowser
    ? window.requestAnimationFrame
      ? window.requestAnimationFrame.bind(window)
      : setTimeout
    : /* istanbul ignore next */ function (fn) { return fn(); };

  function nextFrame (fn) {
    raf(function () {
      raf(fn);
    });
  }

  function addTransitionClass (el, cls) {
    var transitionClasses = el._transitionClasses || (el._transitionClasses = []);
    if (transitionClasses.indexOf(cls) < 0) {
      transitionClasses.push(cls);
      addClass(el, cls);
    }
  }

  function removeTransitionClass (el, cls) {
    if (el._transitionClasses) {
      remove(el._transitionClasses, cls);
    }
    removeClass(el, cls);
  }

  function whenTransitionEnds (
    el,
    expectedType,
    cb
  ) {
    var ref = getTransitionInfo(el, expectedType);
    var type = ref.type;
    var timeout = ref.timeout;
    var propCount = ref.propCount;
    if (!type) { return cb() }
    var event = type === TRANSITION ? transitionEndEvent : animationEndEvent;
    var ended = 0;
    var end = function () {
      el.removeEventListener(event, onEnd);
      cb();
    };
    var onEnd = function (e) {
      if (e.target === el) {
        if (++ended >= propCount) {
          end();
        }
      }
    };
    setTimeout(function () {
      if (ended < propCount) {
        end();
      }
    }, timeout + 1);
    el.addEventListener(event, onEnd);
  }

  var transformRE = /\b(transform|all)(,|$)/;

  function getTransitionInfo (el, expectedType) {
    var styles = window.getComputedStyle(el);
    // JSDOM may return undefined for transition properties
    var transitionDelays = (styles[transitionProp + 'Delay'] || '').split(', ');
    var transitionDurations = (styles[transitionProp + 'Duration'] || '').split(', ');
    var transitionTimeout = getTimeout(transitionDelays, transitionDurations);
    var animationDelays = (styles[animationProp + 'Delay'] || '').split(', ');
    var animationDurations = (styles[animationProp + 'Duration'] || '').split(', ');
    var animationTimeout = getTimeout(animationDelays, animationDurations);

    var type;
    var timeout = 0;
    var propCount = 0;
    /* istanbul ignore if */
    if (expectedType === TRANSITION) {
      if (transitionTimeout > 0) {
        type = TRANSITION;
        timeout = transitionTimeout;
        propCount = transitionDurations.length;
      }
    } else if (expectedType === ANIMATION) {
      if (animationTimeout > 0) {
        type = ANIMATION;
        timeout = animationTimeout;
        propCount = animationDurations.length;
      }
    } else {
      timeout = Math.max(transitionTimeout, animationTimeout);
      type = timeout > 0
        ? transitionTimeout > animationTimeout
          ? TRANSITION
          : ANIMATION
        : null;
      propCount = type
        ? type === TRANSITION
          ? transitionDurations.length
          : animationDurations.length
        : 0;
    }
    var hasTransform =
      type === TRANSITION &&
      transformRE.test(styles[transitionProp + 'Property']);
    return {
      type: type,
      timeout: timeout,
      propCount: propCount,
      hasTransform: hasTransform
    }
  }

  function getTimeout (delays, durations) {
    /* istanbul ignore next */
    while (delays.length < durations.length) {
      delays = delays.concat(delays);
    }

    return Math.max.apply(null, durations.map(function (d, i) {
      return toMs(d) + toMs(delays[i])
    }))
  }

  // Old versions of Chromium (below 61.0.3163.100) formats floating pointer numbers
  // in a locale-dependent way, using a comma instead of a dot.
  // If comma is not replaced with a dot, the input will be rounded down (i.e. acting
  // as a floor function) causing unexpected behaviors
  function toMs (s) {
    return Number(s.slice(0, -1).replace(',', '.')) * 1000
  }

  /*  */

  function enter (vnode, toggleDisplay) {
    var el = vnode.elm;

    // call leave callback now
    if (isDef(el._leaveCb)) {
      el._leaveCb.cancelled = true;
      el._leaveCb();
    }

    var data = resolveTransition(vnode.data.transition);
    if (isUndef(data)) {
      return
    }

    /* istanbul ignore if */
    if (isDef(el._enterCb) || el.nodeType !== 1) {
      return
    }

    var css = data.css;
    var type = data.type;
    var enterClass = data.enterClass;
    var enterToClass = data.enterToClass;
    var enterActiveClass = data.enterActiveClass;
    var appearClass = data.appearClass;
    var appearToClass = data.appearToClass;
    var appearActiveClass = data.appearActiveClass;
    var beforeEnter = data.beforeEnter;
    var enter = data.enter;
    var afterEnter = data.afterEnter;
    var enterCancelled = data.enterCancelled;
    var beforeAppear = data.beforeAppear;
    var appear = data.appear;
    var afterAppear = data.afterAppear;
    var appearCancelled = data.appearCancelled;
    var duration = data.duration;

    // activeInstance will always be the <transition> component managing this
    // transition. One edge case to check is when the <transition> is placed
    // as the root node of a child component. In that case we need to check
    // <transition>'s parent for appear check.
    var context = activeInstance;
    var transitionNode = activeInstance.$vnode;
    while (transitionNode && transitionNode.parent) {
      context = transitionNode.context;
      transitionNode = transitionNode.parent;
    }

    var isAppear = !context._isMounted || !vnode.isRootInsert;

    if (isAppear && !appear && appear !== '') {
      return
    }

    var startClass = isAppear && appearClass
      ? appearClass
      : enterClass;
    var activeClass = isAppear && appearActiveClass
      ? appearActiveClass
      : enterActiveClass;
    var toClass = isAppear && appearToClass
      ? appearToClass
      : enterToClass;

    var beforeEnterHook = isAppear
      ? (beforeAppear || beforeEnter)
      : beforeEnter;
    var enterHook = isAppear
      ? (typeof appear === 'function' ? appear : enter)
      : enter;
    var afterEnterHook = isAppear
      ? (afterAppear || afterEnter)
      : afterEnter;
    var enterCancelledHook = isAppear
      ? (appearCancelled || enterCancelled)
      : enterCancelled;

    var explicitEnterDuration = toNumber(
      isObject(duration)
        ? duration.enter
        : duration
    );

    if (explicitEnterDuration != null) {
      checkDuration(explicitEnterDuration, 'enter', vnode);
    }

    var expectsCSS = css !== false && !isIE9;
    var userWantsControl = getHookArgumentsLength(enterHook);

    var cb = el._enterCb = once(function () {
      if (expectsCSS) {
        removeTransitionClass(el, toClass);
        removeTransitionClass(el, activeClass);
      }
      if (cb.cancelled) {
        if (expectsCSS) {
          removeTransitionClass(el, startClass);
        }
        enterCancelledHook && enterCancelledHook(el);
      } else {
        afterEnterHook && afterEnterHook(el);
      }
      el._enterCb = null;
    });

    if (!vnode.data.show) {
      // remove pending leave element on enter by injecting an insert hook
      mergeVNodeHook(vnode, 'insert', function () {
        var parent = el.parentNode;
        var pendingNode = parent && parent._pending && parent._pending[vnode.key];
        if (pendingNode &&
          pendingNode.tag === vnode.tag &&
          pendingNode.elm._leaveCb
        ) {
          pendingNode.elm._leaveCb();
        }
        enterHook && enterHook(el, cb);
      });
    }

    // start enter transition
    beforeEnterHook && beforeEnterHook(el);
    if (expectsCSS) {
      addTransitionClass(el, startClass);
      addTransitionClass(el, activeClass);
      nextFrame(function () {
        removeTransitionClass(el, startClass);
        if (!cb.cancelled) {
          addTransitionClass(el, toClass);
          if (!userWantsControl) {
            if (isValidDuration(explicitEnterDuration)) {
              setTimeout(cb, explicitEnterDuration);
            } else {
              whenTransitionEnds(el, type, cb);
            }
          }
        }
      });
    }

    if (vnode.data.show) {
      toggleDisplay && toggleDisplay();
      enterHook && enterHook(el, cb);
    }

    if (!expectsCSS && !userWantsControl) {
      cb();
    }
  }

  function leave (vnode, rm) {
    var el = vnode.elm;

    // call enter callback now
    if (isDef(el._enterCb)) {
      el._enterCb.cancelled = true;
      el._enterCb();
    }

    var data = resolveTransition(vnode.data.transition);
    if (isUndef(data) || el.nodeType !== 1) {
      return rm()
    }

    /* istanbul ignore if */
    if (isDef(el._leaveCb)) {
      return
    }

    var css = data.css;
    var type = data.type;
    var leaveClass = data.leaveClass;
    var leaveToClass = data.leaveToClass;
    var leaveActiveClass = data.leaveActiveClass;
    var beforeLeave = data.beforeLeave;
    var leave = data.leave;
    var afterLeave = data.afterLeave;
    var leaveCancelled = data.leaveCancelled;
    var delayLeave = data.delayLeave;
    var duration = data.duration;

    var expectsCSS = css !== false && !isIE9;
    var userWantsControl = getHookArgumentsLength(leave);

    var explicitLeaveDuration = toNumber(
      isObject(duration)
        ? duration.leave
        : duration
    );

    if (isDef(explicitLeaveDuration)) {
      checkDuration(explicitLeaveDuration, 'leave', vnode);
    }

    var cb = el._leaveCb = once(function () {
      if (el.parentNode && el.parentNode._pending) {
        el.parentNode._pending[vnode.key] = null;
      }
      if (expectsCSS) {
        removeTransitionClass(el, leaveToClass);
        removeTransitionClass(el, leaveActiveClass);
      }
      if (cb.cancelled) {
        if (expectsCSS) {
          removeTransitionClass(el, leaveClass);
        }
        leaveCancelled && leaveCancelled(el);
      } else {
        rm();
        afterLeave && afterLeave(el);
      }
      el._leaveCb = null;
    });

    if (delayLeave) {
      delayLeave(performLeave);
    } else {
      performLeave();
    }

    function performLeave () {
      // the delayed leave may have already been cancelled
      if (cb.cancelled) {
        return
      }
      // record leaving element
      if (!vnode.data.show && el.parentNode) {
        (el.parentNode._pending || (el.parentNode._pending = {}))[(vnode.key)] = vnode;
      }
      beforeLeave && beforeLeave(el);
      if (expectsCSS) {
        addTransitionClass(el, leaveClass);
        addTransitionClass(el, leaveActiveClass);
        nextFrame(function () {
          removeTransitionClass(el, leaveClass);
          if (!cb.cancelled) {
            addTransitionClass(el, leaveToClass);
            if (!userWantsControl) {
              if (isValidDuration(explicitLeaveDuration)) {
                setTimeout(cb, explicitLeaveDuration);
              } else {
                whenTransitionEnds(el, type, cb);
              }
            }
          }
        });
      }
      leave && leave(el, cb);
      if (!expectsCSS && !userWantsControl) {
        cb();
      }
    }
  }

  // only used in dev mode
  function checkDuration (val, name, vnode) {
    if (typeof val !== 'number') {
      warn(
        "<transition> explicit " + name + " duration is not a valid number - " +
        "got " + (JSON.stringify(val)) + ".",
        vnode.context
      );
    } else if (isNaN(val)) {
      warn(
        "<transition> explicit " + name + " duration is NaN - " +
        'the duration expression might be incorrect.',
        vnode.context
      );
    }
  }

  function isValidDuration (val) {
    return typeof val === 'number' && !isNaN(val)
  }

  /**
   * Normalize a transition hook's argument length. The hook may be:
   * - a merged hook (invoker) with the original in .fns
   * - a wrapped component method (check ._length)
   * - a plain function (.length)
   */
  function getHookArgumentsLength (fn) {
    if (isUndef(fn)) {
      return false
    }
    var invokerFns = fn.fns;
    if (isDef(invokerFns)) {
      // invoker
      return getHookArgumentsLength(
        Array.isArray(invokerFns)
          ? invokerFns[0]
          : invokerFns
      )
    } else {
      return (fn._length || fn.length) > 1
    }
  }

  function _enter (_, vnode) {
    if (vnode.data.show !== true) {
      enter(vnode);
    }
  }

  var transition = inBrowser ? {
    create: _enter,
    activate: _enter,
    remove: function remove$$1 (vnode, rm) {
      /* istanbul ignore else */
      if (vnode.data.show !== true) {
        leave(vnode, rm);
      } else {
        rm();
      }
    }
  } : {};

  var platformModules = [
    attrs,
    klass,
    events,
    domProps,
    style,
    transition
  ];

  /*  */

  // the directive module should be applied last, after all
  // built-in modules have been applied.
  var modules = platformModules.concat(baseModules);

  var patch = createPatchFunction({ nodeOps: nodeOps, modules: modules });

  /**
   * Not type checking this file because flow doesn't like attaching
   * properties to Elements.
   */

  /* istanbul ignore if */
  if (isIE9) {
    // http://www.matts411.com/post/internet-explorer-9-oninput/
    document.addEventListener('selectionchange', function () {
      var el = document.activeElement;
      if (el && el.vmodel) {
        trigger(el, 'input');
      }
    });
  }

  var directive = {
    inserted: function inserted (el, binding, vnode, oldVnode) {
      if (vnode.tag === 'select') {
        // #6903
        if (oldVnode.elm && !oldVnode.elm._vOptions) {
          mergeVNodeHook(vnode, 'postpatch', function () {
            directive.componentUpdated(el, binding, vnode);
          });
        } else {
          setSelected(el, binding, vnode.context);
        }
        el._vOptions = [].map.call(el.options, getValue);
      } else if (vnode.tag === 'textarea' || isTextInputType(el.type)) {
        el._vModifiers = binding.modifiers;
        if (!binding.modifiers.lazy) {
          el.addEventListener('compositionstart', onCompositionStart);
          el.addEventListener('compositionend', onCompositionEnd);
          // Safari < 10.2 & UIWebView doesn't fire compositionend when
          // switching focus before confirming composition choice
          // this also fixes the issue where some browsers e.g. iOS Chrome
          // fires "change" instead of "input" on autocomplete.
          el.addEventListener('change', onCompositionEnd);
          /* istanbul ignore if */
          if (isIE9) {
            el.vmodel = true;
          }
        }
      }
    },

    componentUpdated: function componentUpdated (el, binding, vnode) {
      if (vnode.tag === 'select') {
        setSelected(el, binding, vnode.context);
        // in case the options rendered by v-for have changed,
        // it's possible that the value is out-of-sync with the rendered options.
        // detect such cases and filter out values that no longer has a matching
        // option in the DOM.
        var prevOptions = el._vOptions;
        var curOptions = el._vOptions = [].map.call(el.options, getValue);
        if (curOptions.some(function (o, i) { return !looseEqual(o, prevOptions[i]); })) {
          // trigger change event if
          // no matching option found for at least one value
          var needReset = el.multiple
            ? binding.value.some(function (v) { return hasNoMatchingOption(v, curOptions); })
            : binding.value !== binding.oldValue && hasNoMatchingOption(binding.value, curOptions);
          if (needReset) {
            trigger(el, 'change');
          }
        }
      }
    }
  };

  function setSelected (el, binding, vm) {
    actuallySetSelected(el, binding, vm);
    /* istanbul ignore if */
    if (isIE || isEdge) {
      setTimeout(function () {
        actuallySetSelected(el, binding, vm);
      }, 0);
    }
  }

  function actuallySetSelected (el, binding, vm) {
    var value = binding.value;
    var isMultiple = el.multiple;
    if (isMultiple && !Array.isArray(value)) {
      warn(
        "<select multiple v-model=\"" + (binding.expression) + "\"> " +
        "expects an Array value for its binding, but got " + (Object.prototype.toString.call(value).slice(8, -1)),
        vm
      );
      return
    }
    var selected, option;
    for (var i = 0, l = el.options.length; i < l; i++) {
      option = el.options[i];
      if (isMultiple) {
        selected = looseIndexOf(value, getValue(option)) > -1;
        if (option.selected !== selected) {
          option.selected = selected;
        }
      } else {
        if (looseEqual(getValue(option), value)) {
          if (el.selectedIndex !== i) {
            el.selectedIndex = i;
          }
          return
        }
      }
    }
    if (!isMultiple) {
      el.selectedIndex = -1;
    }
  }

  function hasNoMatchingOption (value, options) {
    return options.every(function (o) { return !looseEqual(o, value); })
  }

  function getValue (option) {
    return '_value' in option
      ? option._value
      : option.value
  }

  function onCompositionStart (e) {
    e.target.composing = true;
  }

  function onCompositionEnd (e) {
    // prevent triggering an input event for no reason
    if (!e.target.composing) { return }
    e.target.composing = false;
    trigger(e.target, 'input');
  }

  function trigger (el, type) {
    var e = document.createEvent('HTMLEvents');
    e.initEvent(type, true, true);
    el.dispatchEvent(e);
  }

  /*  */

  // recursively search for possible transition defined inside the component root
  function locateNode (vnode) {
    return vnode.componentInstance && (!vnode.data || !vnode.data.transition)
      ? locateNode(vnode.componentInstance._vnode)
      : vnode
  }

  var show = {
    bind: function bind (el, ref, vnode) {
      var value = ref.value;

      vnode = locateNode(vnode);
      var transition$$1 = vnode.data && vnode.data.transition;
      var originalDisplay = el.__vOriginalDisplay =
        el.style.display === 'none' ? '' : el.style.display;
      if (value && transition$$1) {
        vnode.data.show = true;
        enter(vnode, function () {
          el.style.display = originalDisplay;
        });
      } else {
        el.style.display = value ? originalDisplay : 'none';
      }
    },

    update: function update (el, ref, vnode) {
      var value = ref.value;
      var oldValue = ref.oldValue;

      /* istanbul ignore if */
      if (!value === !oldValue) { return }
      vnode = locateNode(vnode);
      var transition$$1 = vnode.data && vnode.data.transition;
      if (transition$$1) {
        vnode.data.show = true;
        if (value) {
          enter(vnode, function () {
            el.style.display = el.__vOriginalDisplay;
          });
        } else {
          leave(vnode, function () {
            el.style.display = 'none';
          });
        }
      } else {
        el.style.display = value ? el.__vOriginalDisplay : 'none';
      }
    },

    unbind: function unbind (
      el,
      binding,
      vnode,
      oldVnode,
      isDestroy
    ) {
      if (!isDestroy) {
        el.style.display = el.__vOriginalDisplay;
      }
    }
  };

  var platformDirectives = {
    model: directive,
    show: show
  };

  /*  */

  var transitionProps = {
    name: String,
    appear: Boolean,
    css: Boolean,
    mode: String,
    type: String,
    enterClass: String,
    leaveClass: String,
    enterToClass: String,
    leaveToClass: String,
    enterActiveClass: String,
    leaveActiveClass: String,
    appearClass: String,
    appearActiveClass: String,
    appearToClass: String,
    duration: [Number, String, Object]
  };

  // in case the child is also an abstract component, e.g. <keep-alive>
  // we want to recursively retrieve the real component to be rendered
  function getRealChild (vnode) {
    var compOptions = vnode && vnode.componentOptions;
    if (compOptions && compOptions.Ctor.options.abstract) {
      return getRealChild(getFirstComponentChild(compOptions.children))
    } else {
      return vnode
    }
  }

  function extractTransitionData (comp) {
    var data = {};
    var options = comp.$options;
    // props
    for (var key in options.propsData) {
      data[key] = comp[key];
    }
    // events.
    // extract listeners and pass them directly to the transition methods
    var listeners = options._parentListeners;
    for (var key$1 in listeners) {
      data[camelize(key$1)] = listeners[key$1];
    }
    return data
  }

  function placeholder (h, rawChild) {
    if (/\d-keep-alive$/.test(rawChild.tag)) {
      return h('keep-alive', {
        props: rawChild.componentOptions.propsData
      })
    }
  }

  function hasParentTransition (vnode) {
    while ((vnode = vnode.parent)) {
      if (vnode.data.transition) {
        return true
      }
    }
  }

  function isSameChild (child, oldChild) {
    return oldChild.key === child.key && oldChild.tag === child.tag
  }

  var isNotTextNode = function (c) { return c.tag || isAsyncPlaceholder(c); };

  var isVShowDirective = function (d) { return d.name === 'show'; };

  var Transition = {
    name: 'transition',
    props: transitionProps,
    abstract: true,

    render: function render (h) {
      var this$1 = this;

      var children = this.$slots.default;
      if (!children) {
        return
      }

      // filter out text nodes (possible whitespaces)
      children = children.filter(isNotTextNode);
      /* istanbul ignore if */
      if (!children.length) {
        return
      }

      // warn multiple elements
      if (children.length > 1) {
        warn(
          '<transition> can only be used on a single element. Use ' +
          '<transition-group> for lists.',
          this.$parent
        );
      }

      var mode = this.mode;

      // warn invalid mode
      if (mode && mode !== 'in-out' && mode !== 'out-in'
      ) {
        warn(
          'invalid <transition> mode: ' + mode,
          this.$parent
        );
      }

      var rawChild = children[0];

      // if this is a component root node and the component's
      // parent container node also has transition, skip.
      if (hasParentTransition(this.$vnode)) {
        return rawChild
      }

      // apply transition data to child
      // use getRealChild() to ignore abstract components e.g. keep-alive
      var child = getRealChild(rawChild);
      /* istanbul ignore if */
      if (!child) {
        return rawChild
      }

      if (this._leaving) {
        return placeholder(h, rawChild)
      }

      // ensure a key that is unique to the vnode type and to this transition
      // component instance. This key will be used to remove pending leaving nodes
      // during entering.
      var id = "__transition-" + (this._uid) + "-";
      child.key = child.key == null
        ? child.isComment
          ? id + 'comment'
          : id + child.tag
        : isPrimitive(child.key)
          ? (String(child.key).indexOf(id) === 0 ? child.key : id + child.key)
          : child.key;

      var data = (child.data || (child.data = {})).transition = extractTransitionData(this);
      var oldRawChild = this._vnode;
      var oldChild = getRealChild(oldRawChild);

      // mark v-show
      // so that the transition module can hand over the control to the directive
      if (child.data.directives && child.data.directives.some(isVShowDirective)) {
        child.data.show = true;
      }

      if (
        oldChild &&
        oldChild.data &&
        !isSameChild(child, oldChild) &&
        !isAsyncPlaceholder(oldChild) &&
        // #6687 component root is a comment node
        !(oldChild.componentInstance && oldChild.componentInstance._vnode.isComment)
      ) {
        // replace old child transition data with fresh one
        // important for dynamic transitions!
        var oldData = oldChild.data.transition = extend({}, data);
        // handle transition mode
        if (mode === 'out-in') {
          // return placeholder node and queue update when leave finishes
          this._leaving = true;
          mergeVNodeHook(oldData, 'afterLeave', function () {
            this$1._leaving = false;
            this$1.$forceUpdate();
          });
          return placeholder(h, rawChild)
        } else if (mode === 'in-out') {
          if (isAsyncPlaceholder(child)) {
            return oldRawChild
          }
          var delayedLeave;
          var performLeave = function () { delayedLeave(); };
          mergeVNodeHook(data, 'afterEnter', performLeave);
          mergeVNodeHook(data, 'enterCancelled', performLeave);
          mergeVNodeHook(oldData, 'delayLeave', function (leave) { delayedLeave = leave; });
        }
      }

      return rawChild
    }
  };

  /*  */

  var props = extend({
    tag: String,
    moveClass: String
  }, transitionProps);

  delete props.mode;

  var TransitionGroup = {
    props: props,

    beforeMount: function beforeMount () {
      var this$1 = this;

      var update = this._update;
      this._update = function (vnode, hydrating) {
        var restoreActiveInstance = setActiveInstance(this$1);
        // force removing pass
        this$1.__patch__(
          this$1._vnode,
          this$1.kept,
          false, // hydrating
          true // removeOnly (!important, avoids unnecessary moves)
        );
        this$1._vnode = this$1.kept;
        restoreActiveInstance();
        update.call(this$1, vnode, hydrating);
      };
    },

    render: function render (h) {
      var tag = this.tag || this.$vnode.data.tag || 'span';
      var map = Object.create(null);
      var prevChildren = this.prevChildren = this.children;
      var rawChildren = this.$slots.default || [];
      var children = this.children = [];
      var transitionData = extractTransitionData(this);

      for (var i = 0; i < rawChildren.length; i++) {
        var c = rawChildren[i];
        if (c.tag) {
          if (c.key != null && String(c.key).indexOf('__vlist') !== 0) {
            children.push(c);
            map[c.key] = c
            ;(c.data || (c.data = {})).transition = transitionData;
          } else {
            var opts = c.componentOptions;
            var name = opts ? (opts.Ctor.options.name || opts.tag || '') : c.tag;
            warn(("<transition-group> children must be keyed: <" + name + ">"));
          }
        }
      }

      if (prevChildren) {
        var kept = [];
        var removed = [];
        for (var i$1 = 0; i$1 < prevChildren.length; i$1++) {
          var c$1 = prevChildren[i$1];
          c$1.data.transition = transitionData;
          c$1.data.pos = c$1.elm.getBoundingClientRect();
          if (map[c$1.key]) {
            kept.push(c$1);
          } else {
            removed.push(c$1);
          }
        }
        this.kept = h(tag, null, kept);
        this.removed = removed;
      }

      return h(tag, null, children)
    },

    updated: function updated () {
      var children = this.prevChildren;
      var moveClass = this.moveClass || ((this.name || 'v') + '-move');
      if (!children.length || !this.hasMove(children[0].elm, moveClass)) {
        return
      }

      // we divide the work into three loops to avoid mixing DOM reads and writes
      // in each iteration - which helps prevent layout thrashing.
      children.forEach(callPendingCbs);
      children.forEach(recordPosition);
      children.forEach(applyTranslation);

      // force reflow to put everything in position
      // assign to this to avoid being removed in tree-shaking
      // $flow-disable-line
      this._reflow = document.body.offsetHeight;

      children.forEach(function (c) {
        if (c.data.moved) {
          var el = c.elm;
          var s = el.style;
          addTransitionClass(el, moveClass);
          s.transform = s.WebkitTransform = s.transitionDuration = '';
          el.addEventListener(transitionEndEvent, el._moveCb = function cb (e) {
            if (e && e.target !== el) {
              return
            }
            if (!e || /transform$/.test(e.propertyName)) {
              el.removeEventListener(transitionEndEvent, cb);
              el._moveCb = null;
              removeTransitionClass(el, moveClass);
            }
          });
        }
      });
    },

    methods: {
      hasMove: function hasMove (el, moveClass) {
        /* istanbul ignore if */
        if (!hasTransition) {
          return false
        }
        /* istanbul ignore if */
        if (this._hasMove) {
          return this._hasMove
        }
        // Detect whether an element with the move class applied has
        // CSS transitions. Since the element may be inside an entering
        // transition at this very moment, we make a clone of it and remove
        // all other transition classes applied to ensure only the move class
        // is applied.
        var clone = el.cloneNode();
        if (el._transitionClasses) {
          el._transitionClasses.forEach(function (cls) { removeClass(clone, cls); });
        }
        addClass(clone, moveClass);
        clone.style.display = 'none';
        this.$el.appendChild(clone);
        var info = getTransitionInfo(clone);
        this.$el.removeChild(clone);
        return (this._hasMove = info.hasTransform)
      }
    }
  };

  function callPendingCbs (c) {
    /* istanbul ignore if */
    if (c.elm._moveCb) {
      c.elm._moveCb();
    }
    /* istanbul ignore if */
    if (c.elm._enterCb) {
      c.elm._enterCb();
    }
  }

  function recordPosition (c) {
    c.data.newPos = c.elm.getBoundingClientRect();
  }

  function applyTranslation (c) {
    var oldPos = c.data.pos;
    var newPos = c.data.newPos;
    var dx = oldPos.left - newPos.left;
    var dy = oldPos.top - newPos.top;
    if (dx || dy) {
      c.data.moved = true;
      var s = c.elm.style;
      s.transform = s.WebkitTransform = "translate(" + dx + "px," + dy + "px)";
      s.transitionDuration = '0s';
    }
  }

  var platformComponents = {
    Transition: Transition,
    TransitionGroup: TransitionGroup
  };

  /*  */

  // install platform specific utils
  Vue.config.mustUseProp = mustUseProp;
  Vue.config.isReservedTag = isReservedTag;
  Vue.config.isReservedAttr = isReservedAttr;
  Vue.config.getTagNamespace = getTagNamespace;
  Vue.config.isUnknownElement = isUnknownElement;

  // install platform runtime directives & components
  extend(Vue.options.directives, platformDirectives);
  extend(Vue.options.components, platformComponents);

  // install platform patch function
  Vue.prototype.__patch__ = inBrowser ? patch : noop;

  // public mount method
  Vue.prototype.$mount = function (
    el,
    hydrating
  ) {
    el = el && inBrowser ? query(el) : undefined;
    return mountComponent(this, el, hydrating)
  };

  // devtools global hook
  /* istanbul ignore next */
  if (inBrowser) {
    setTimeout(function () {
      if (config.devtools) {
        if (devtools) {
          devtools.emit('init', Vue);
        } else {
          console[console.info ? 'info' : 'log'](
            'Download the Vue Devtools extension for a better development experience:\n' +
            'https://github.com/vuejs/vue-devtools'
          );
        }
      }
      if (config.productionTip !== false &&
        typeof console !== 'undefined'
      ) {
        console[console.info ? 'info' : 'log'](
          "You are running Vue in development mode.\n" +
          "Make sure to turn on production mode when deploying for production.\n" +
          "See more tips at https://vuejs.org/guide/deployment.html"
        );
      }
    }, 0);
  }

  var alarmFill = Vue.extend({
    name: 'AlarmFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-alarm-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.5 2.5A.5.5 0 018 2h4a.5.5 0 010 1h-1v1.07a7.002 7.002 0 013.537 12.26l.817.816a.5.5 0 01-.708.708l-.924-.925A6.967 6.967 0 0110 18a6.967 6.967 0 01-3.722-1.07l-.924.924a.5.5 0 01-.708-.708l.817-.816A7.002 7.002 0 019 4.07V3H8a.5.5 0 01-.5-.5zM2.86 7.387A2.5 2.5 0 116.387 3.86 8.035 8.035 0 002.86 7.387zM15.5 3c-.753 0-1.429.333-1.887.86a8.035 8.035 0 013.527 3.527A2.5 2.5 0 0015.5 3zm-5 4a.5.5 0 00-1 0v3.882l-1.447 2.894a.5.5 0 10.894.448l1.5-3A.5.5 0 0010.5 11V7z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var alarm = Vue.extend({
    name: 'AlarmIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-alarm",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 17a6 6 0 100-12 6 6 0 000 12zm0 1a7 7 0 100-14 7 7 0 000 14z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 6.5a.5.5 0 01.5.5v4a.5.5 0 01-.053.224l-1.5 3a.5.5 0 11-.894-.448L9.5 10.882V7a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M2.86 7.387A2.5 2.5 0 116.387 3.86 8.035 8.035 0 002.86 7.387zM13.613 3.86a2.5 2.5 0 113.527 3.527 8.035 8.035 0 00-3.527-3.527z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.646 16.146a.5.5 0 01.708 0l1 1a.5.5 0 01-.708.708l-1-1a.5.5 0 010-.708zm-7.292 0a.5.5 0 00-.708 0l-1 1a.5.5 0 00.708.708l1-1a.5.5 0 000-.708zM7.5 2.5A.5.5 0 018 2h4a.5.5 0 010 1H8a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9 3h2v2H9V3z"
        }
      })]);
    }
  });

  var alertCircleFill = Vue.extend({
    name: 'AlertCircleFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-alert-circle-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M18 10a8 8 0 11-16 0 8 8 0 0116 0zm-8.998 3a1 1 0 112 0 1 1 0 01-2 0zM10 6a.905.905 0 00-.9.995l.35 3.507a.553.553 0 001.1 0l.35-3.507A.905.905 0 0010 6z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var alertCircle = Vue.extend({
    name: 'AlertCircleIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-alert-circle",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 17a7 7 0 100-14 7 7 0 000 14zm0 1a8 8 0 100-16 8 8 0 000 16z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9.002 13a1 1 0 112 0 1 1 0 01-2 0zM9.1 6.995a.905.905 0 111.8 0l-.35 3.507a.553.553 0 01-1.1 0L9.1 6.995z"
        }
      })]);
    }
  });

  var alertOctagonFill = Vue.extend({
    name: 'AlertOctagonFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-alert-octagon-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.107 2a.5.5 0 01.353.146l4.394 4.394a.5.5 0 01.146.353v6.214a.5.5 0 01-.146.353l-4.394 4.394a.5.5 0 01-.353.146H6.893a.5.5 0 01-.353-.146L2.146 13.46A.5.5 0 012 13.107V6.893a.5.5 0 01.146-.353L6.54 2.146A.5.5 0 016.893 2h6.214zM9.002 13a1 1 0 112 0 1 1 0 01-2 0zM10 6a.905.905 0 00-.9.995l.35 3.507a.553.553 0 001.1 0l.35-3.507A.905.905 0 0010 6z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var alertOctagon = Vue.extend({
    name: 'AlertOctagonIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-alert-octagon",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.54 2.146A.5.5 0 016.893 2h6.214a.5.5 0 01.353.146l4.394 4.394a.5.5 0 01.146.353v6.214a.5.5 0 01-.146.353l-4.394 4.394a.5.5 0 01-.353.146H6.893a.5.5 0 01-.353-.146L2.146 13.46A.5.5 0 012 13.107V6.893a.5.5 0 01.146-.353L6.54 2.146zM7.1 3L3 7.1v5.8L7.1 17h5.8l4.1-4.1V7.1L12.9 3H7.1z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "2",
          height: "2",
          x: "9.002",
          y: "12",
          rx: "1"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9.1 6.995a.905.905 0 111.8 0l-.35 3.507a.553.553 0 01-1.1 0L9.1 6.995z"
        }
      })]);
    }
  });

  var alertSquareFill = Vue.extend({
    name: 'AlertSquareFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-alert-square-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2 4a2 2 0 012-2h12a2 2 0 012 2v12a2 2 0 01-2 2H4a2 2 0 01-2-2V4zm7.002 9a1 1 0 112 0 1 1 0 01-2 0zM10 6a.905.905 0 00-.9.995l.35 3.507a.553.553 0 001.1 0l.35-3.507A.905.905 0 0010 6z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var alertSquare = Vue.extend({
    name: 'AlertSquareIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-alert-square",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 3H4a1 1 0 00-1 1v12a1 1 0 001 1h12a1 1 0 001-1V4a1 1 0 00-1-1zM4 2a2 2 0 00-2 2v12a2 2 0 002 2h12a2 2 0 002-2V4a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "2",
          height: "2",
          x: "9.002",
          y: "12",
          rx: "1"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9.1 6.995a.905.905 0 111.8 0l-.35 3.507a.553.553 0 01-1.1 0L9.1 6.995z"
        }
      })]);
    }
  });

  var alertTriangleFill = Vue.extend({
    name: 'AlertTriangleFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-alert-triangle-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.022 3.566a1.13 1.13 0 011.96 0l6.857 11.667c.457.778-.092 1.767-.98 1.767H3.144c-.889 0-1.437-.99-.98-1.767L9.022 3.566zM9.002 14a1 1 0 112 0 1 1 0 01-2 0zM10 7a.905.905 0 00-.9.995l.35 3.507a.553.553 0 001.1 0l.35-3.507A.905.905 0 0010 7z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var alertTriangle = Vue.extend({
    name: 'AlertTriangleIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-alert-triangle",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.938 4.016a.146.146 0 00-.054.057L3.027 15.74a.176.176 0 00-.002.183c.016.03.037.05.054.06.015.01.034.017.066.017h13.713a.12.12 0 00.066-.017.163.163 0 00.055-.06.176.176 0 00-.003-.183L10.12 4.073a.146.146 0 00-.054-.057.13.13 0 00-.063-.016.13.13 0 00-.064.016zm1.043-.45a1.13 1.13 0 00-1.96 0L2.166 15.233c-.457.778.091 1.767.98 1.767h13.713c.889 0 1.438-.99.98-1.767L10.982 3.566z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "2",
          height: "2",
          x: "9.002",
          y: "13",
          rx: "1"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9.1 7.995a.905.905 0 111.8 0l-.35 3.507a.553.553 0 01-1.1 0L9.1 7.995z"
        }
      })]);
    }
  });

  var archiveFill = Vue.extend({
    name: 'ArchiveFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-archive-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14.643 17C15.979 17 17 15.845 17 14.5V7H3v7.5C3 15.845 4.021 17 5.357 17h9.286zM8 9a.5.5 0 000 1h4a.5.5 0 000-1H8zM3 3a1 1 0 00-1 1v1.5a1 1 0 001 1h14a1 1 0 001-1V4a1 1 0 00-1-1H3z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var archive = Vue.extend({
    name: 'ArchiveIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-archive",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 7v7.5c0 .864.642 1.5 1.357 1.5h9.286c.715 0 1.357-.636 1.357-1.5V7h1v7.5c0 1.345-1.021 2.5-2.357 2.5H5.357C4.021 17 3 15.845 3 14.5V7h1z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.5 9.5A.5.5 0 018 9h4a.5.5 0 010 1H8a.5.5 0 01-.5-.5zM17 4H3v2h14V4zM3 3a1 1 0 00-1 1v2a1 1 0 001 1h14a1 1 0 001-1V4a1 1 0 00-1-1H3z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowBarBottom = Vue.extend({
    name: 'ArrowBarBottomIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-bar-bottom",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.354 12.146a.5.5 0 010 .708l-3 3a.5.5 0 01-.708 0l-3-3a.5.5 0 01.708-.708L10 14.793l2.646-2.647a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 8a.5.5 0 01.5.5V15a.5.5 0 01-1 0V8.5A.5.5 0 0110 8zM4 5.75a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowBarLeft = Vue.extend({
    name: 'ArrowBarLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-bar-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.854 6.646a.5.5 0 00-.708 0l-3 3a.5.5 0 000 .708l3 3a.5.5 0 00.708-.708L5.207 10l2.647-2.646a.5.5 0 000-.708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12 10a.5.5 0 00-.5-.5H5a.5.5 0 000 1h6.5a.5.5 0 00.5-.5zm2.5 6a.5.5 0 01-.5-.5v-11a.5.5 0 011 0v11a.5.5 0 01-.5.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowBarRight = Vue.extend({
    name: 'ArrowBarRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-bar-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.146 6.646a.5.5 0 01.708 0l3 3a.5.5 0 010 .708l-3 3a.5.5 0 01-.708-.708L14.793 10l-2.647-2.646a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8 10a.5.5 0 01.5-.5H15a.5.5 0 010 1H8.5A.5.5 0 018 10zm-2.5 6a.5.5 0 01-.5-.5v-11a.5.5 0 011 0v11a.5.5 0 01-.5.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowBarUp = Vue.extend({
    name: 'ArrowBarUpIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-bar-up",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.354 7.854a.5.5 0 000-.708l-3-3a.5.5 0 00-.708 0l-3 3a.5.5 0 10.708.708L10 5.207l2.646 2.647a.5.5 0 00.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 12a.5.5 0 00.5-.5V5a.5.5 0 00-1 0v6.5a.5.5 0 00.5.5zm-6 2.75a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowClockwise = Vue.extend({
    name: 'ArrowClockwiseIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-clockwise",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 4.5a5.5 5.5 0 105.5 5.5.5.5 0 011 0 6.5 6.5 0 11-3.25-5.63l-.5.865A5.472 5.472 0 0010 4.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.646 1.646a.5.5 0 01.708 0l2.5 2.5a.5.5 0 010 .708l-2.5 2.5a.5.5 0 01-.708-.708L12.793 4.5l-2.147-2.146a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowCounterclockwise = Vue.extend({
    name: 'ArrowCounterclockwiseIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-counterclockwise",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 4.5A5.5 5.5 0 114.5 10a.5.5 0 00-1 0 6.5 6.5 0 103.25-5.63l.5.865A5.472 5.472 0 0110 4.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.354 1.646a.5.5 0 00-.708 0l-2.5 2.5a.5.5 0 000 .708l2.5 2.5a.5.5 0 10.708-.708L7.207 4.5l2.147-2.146a.5.5 0 000-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowDownLeft = Vue.extend({
    name: 'ArrowDownLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-down-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5 9.5a.5.5 0 01.5.5v4.5H10a.5.5 0 010 1H5a.5.5 0 01-.5-.5v-5a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14.354 5.646a.5.5 0 010 .708l-9 9a.5.5 0 01-.708-.708l9-9a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowDownRight = Vue.extend({
    name: 'ArrowDownRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-down-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14 9.5a.5.5 0 01.5.5v5a.5.5 0 01-.5.5H9a.5.5 0 010-1h4.5V10a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.646 5.646a.5.5 0 01.708 0l9 9a.5.5 0 01-.708.708l-9-9a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowDownShort = Vue.extend({
    name: 'ArrowDownShortIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-down-short",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.646 9.646a.5.5 0 01.708 0L10 12.293l2.646-2.647a.5.5 0 01.708.708l-3 3a.5.5 0 01-.708 0l-3-3a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 6.5a.5.5 0 01.5.5v5a.5.5 0 01-1 0V7a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowDown = Vue.extend({
    name: 'ArrowDownIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-down",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.646 11.646a.5.5 0 01.708 0L10 14.293l2.646-2.647a.5.5 0 01.708.708l-3 3a.5.5 0 01-.708 0l-3-3a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 4.5a.5.5 0 01.5.5v9a.5.5 0 01-1 0V5a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowLeftRight = Vue.extend({
    name: 'ArrowLeftRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-left-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.146 9.646a.5.5 0 01.708 0l3 3a.5.5 0 010 .708l-3 3a.5.5 0 01-.708-.708L14.793 13l-2.647-2.646a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 13a.5.5 0 01.5-.5H15a.5.5 0 010 1H4.5A.5.5 0 014 13zm3.854-9.354a.5.5 0 010 .708L5.207 7l2.647 2.646a.5.5 0 01-.708.708l-3-3a.5.5 0 010-.708l3-3a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.5 7a.5.5 0 01.5-.5h10.5a.5.5 0 010 1H5a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowLeftShort = Vue.extend({
    name: 'ArrowLeftShortIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-left-short",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.854 6.646a.5.5 0 010 .708L7.207 10l2.647 2.646a.5.5 0 01-.708.708l-3-3a.5.5 0 010-.708l3-3a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.5 10a.5.5 0 01.5-.5h6.5a.5.5 0 010 1H7a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowLeft = Vue.extend({
    name: 'ArrowLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.854 6.646a.5.5 0 010 .708L5.207 10l2.647 2.646a.5.5 0 01-.708.708l-3-3a.5.5 0 010-.708l3-3a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.5 10a.5.5 0 01.5-.5h10.5a.5.5 0 010 1H5a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowRepeat = Vue.extend({
    name: 'ArrowRepeatIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-repeat",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 9.5a.5.5 0 00-.5.5 6.5 6.5 0 0012.13 3.25.5.5 0 00-.866-.5A5.5 5.5 0 014.5 10a.5.5 0 00-.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.354 9.146a.5.5 0 00-.708 0l-2 2a.5.5 0 00.708.708L4 10.207l1.646 1.647a.5.5 0 00.708-.708l-2-2zM15.947 10.5a.5.5 0 00.5-.5 6.5 6.5 0 00-12.13-3.25.5.5 0 10.866.5A5.5 5.5 0 0115.448 10a.5.5 0 00.5.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M18.354 8.146a.5.5 0 00-.708 0L16 9.793l-1.646-1.647a.5.5 0 00-.708.708l2 2a.5.5 0 00.708 0l2-2a.5.5 0 000-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowRightShort = Vue.extend({
    name: 'ArrowRightShortIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-right-short",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.146 6.646a.5.5 0 01.708 0l3 3a.5.5 0 010 .708l-3 3a.5.5 0 01-.708-.708L12.793 10l-2.647-2.646a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6 10a.5.5 0 01.5-.5H13a.5.5 0 010 1H6.5A.5.5 0 016 10z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowRight = Vue.extend({
    name: 'ArrowRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.146 6.646a.5.5 0 01.708 0l3 3a.5.5 0 010 .708l-3 3a.5.5 0 01-.708-.708L14.793 10l-2.647-2.646a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 10a.5.5 0 01.5-.5H15a.5.5 0 010 1H4.5A.5.5 0 014 10z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowUpDown = Vue.extend({
    name: 'ArrowUpDownIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-up-down",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13 5.5a.5.5 0 01.5.5v9a.5.5 0 01-1 0V6a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.646 4.646a.5.5 0 01.708 0l3 3a.5.5 0 01-.708.708L13 5.707l-2.646 2.647a.5.5 0 01-.708-.708l3-3zm-9 7a.5.5 0 01.708 0L7 14.293l2.646-2.647a.5.5 0 01.708.708l-3 3a.5.5 0 01-.708 0l-3-3a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7 4.5a.5.5 0 01.5.5v9a.5.5 0 01-1 0V5a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowUpLeft = Vue.extend({
    name: 'ArrowUpLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-up-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.5 6a.5.5 0 01.5-.5h5a.5.5 0 010 1H5.5V11a.5.5 0 01-1 0V6z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.646 5.646a.5.5 0 01.708 0l9 9a.5.5 0 01-.708.708l-9-9a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowUpRight = Vue.extend({
    name: 'ArrowUpRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-up-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.5 6a.5.5 0 01.5-.5h5a.5.5 0 01.5.5v5a.5.5 0 01-1 0V6.5H9a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14.354 5.646a.5.5 0 010 .708l-9 9a.5.5 0 01-.708-.708l9-9a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowUpShort = Vue.extend({
    name: 'ArrowUpShortIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-up-short",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 7.5a.5.5 0 01.5.5v5a.5.5 0 01-1 0V8a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.646 6.646a.5.5 0 01.708 0l3 3a.5.5 0 01-.708.708L10 7.707l-2.646 2.647a.5.5 0 01-.708-.708l3-3z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowUp = Vue.extend({
    name: 'ArrowUpIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrow-up",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 5.5a.5.5 0 01.5.5v9a.5.5 0 01-1 0V6a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.646 4.646a.5.5 0 01.708 0l3 3a.5.5 0 01-.708.708L10 5.707 7.354 8.354a.5.5 0 11-.708-.708l3-3z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowsAngleContract = Vue.extend({
    name: 'ArrowsAngleContractIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrows-angle-contract",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.5 4.036a.5.5 0 01.5.5v3.5h3.5a.5.5 0 010 1h-4a.5.5 0 01-.5-.5v-4a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16.354 3.646a.5.5 0 010 .708l-4.5 4.5a.5.5 0 01-.708-.708l4.5-4.5a.5.5 0 01.708 0zm-7.5 7.5a.5.5 0 010 .708l-4.5 4.5a.5.5 0 01-.708-.708l4.5-4.5a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.036 11.5a.5.5 0 01.5-.5h4a.5.5 0 01.5.5v4a.5.5 0 01-1 0V12h-3.5a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowsAngleExpand = Vue.extend({
    name: 'ArrowsAngleExpandIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrows-angle-expand",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 11.5a.5.5 0 01.5.5v3.5H8a.5.5 0 010 1H4a.5.5 0 01-.5-.5v-4a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.854 11.11a.5.5 0 010 .708l-4.5 4.5a.5.5 0 11-.708-.707l4.5-4.5a.5.5 0 01.708 0zm7.464-7.464a.5.5 0 010 .708l-4.5 4.5a.5.5 0 11-.707-.708l4.5-4.5a.5.5 0 01.707 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.5 4a.5.5 0 01.5-.5h4a.5.5 0 01.5.5v4a.5.5 0 01-1 0V4.5H12a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowsCollapse = Vue.extend({
    name: 'ArrowsCollapseIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrows-collapse",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 10a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11A.5.5 0 014 10zm6-7a.5.5 0 01.5.5V8a.5.5 0 01-1 0V3.5A.5.5 0 0110 3z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.354 5.646a.5.5 0 010 .708l-2 2a.5.5 0 01-.708 0l-2-2a.5.5 0 11.708-.708L10 7.293l1.646-1.647a.5.5 0 01.708 0zM10 17a.5.5 0 00.5-.5V12a.5.5 0 00-1 0v4.5a.5.5 0 00.5.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.354 14.354a.5.5 0 000-.708l-2-2a.5.5 0 00-.708 0l-2 2a.5.5 0 00.708.708L10 12.707l1.646 1.647a.5.5 0 00.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowsExpand = Vue.extend({
    name: 'ArrowsExpandIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrows-expand",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 10a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11A.5.5 0 014 10zm6-1.5a.5.5 0 00.5-.5V3.5a.5.5 0 00-1 0V8a.5.5 0 00.5.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.354 5.854a.5.5 0 000-.708l-2-2a.5.5 0 00-.708 0l-2 2a.5.5 0 10.708.708L10 4.207l1.646 1.647a.5.5 0 00.708 0zM10 11.5a.5.5 0 01.5.5v4.5a.5.5 0 01-1 0V12a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.354 14.146a.5.5 0 010 .708l-2 2a.5.5 0 01-.708 0l-2-2a.5.5 0 01.708-.708L10 15.793l1.646-1.647a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var arrowsFullscreen = Vue.extend({
    name: 'ArrowsFullscreenIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-arrows-fullscreen",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 11.5a.5.5 0 01.5.5v3.5H8a.5.5 0 010 1H4a.5.5 0 01-.5-.5v-4a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.854 11.11a.5.5 0 010 .708l-4.5 4.5a.5.5 0 11-.708-.707l4.5-4.5a.5.5 0 01.708 0zm7.464-7.464a.5.5 0 010 .708l-4.5 4.5a.5.5 0 11-.707-.708l4.5-4.5a.5.5 0 01.707 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.5 4a.5.5 0 01.5-.5h4a.5.5 0 01.5.5v4a.5.5 0 01-1 0V4.5H12a.5.5 0 01-.5-.5zm4.5 7.5a.5.5 0 00-.5.5v3.5H12a.5.5 0 000 1h4a.5.5 0 00.5-.5v-4a.5.5 0 00-.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.146 11.11a.5.5 0 000 .708l4.5 4.5a.5.5 0 00.708-.707l-4.5-4.5a.5.5 0 00-.708 0zM3.682 3.646a.5.5 0 000 .708l4.5 4.5a.5.5 0 10.707-.708l-4.5-4.5a.5.5 0 00-.707 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.5 4a.5.5 0 00-.5-.5H4a.5.5 0 00-.5.5v4a.5.5 0 001 0V4.5H8a.5.5 0 00.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var at = Vue.extend({
    name: 'AtIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-at",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.106 9.222c0-2.967-2.249-5.032-5.482-5.032-3.35 0-5.646 2.318-5.646 5.702 0 3.493 2.235 5.708 5.762 5.708.862 0 1.689-.123 2.304-.335v-.862c-.43.199-1.354.328-2.29.328-2.926 0-4.813-1.88-4.813-4.798 0-2.844 1.921-4.881 4.594-4.881 2.735 0 4.608 1.688 4.608 4.156 0 1.682-.554 2.769-1.416 2.769-.492 0-.772-.28-.772-.76V7.206h-1.032v.834h-.11c-.266-.595-.881-.964-1.6-.964-1.4 0-2.378 1.162-2.378 2.823 0 1.737.957 2.906 2.379 2.906.8 0 1.415-.39 1.709-1.087h.11c.081.67.703 1.148 1.503 1.148 1.572 0 2.57-1.415 2.57-3.643zm-7.177.704c0-1.197.54-1.907 1.456-1.907.93 0 1.524.738 1.524 1.907s-.601 1.914-1.538 1.914c-.895 0-1.442-.725-1.442-1.914z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var award = Vue.extend({
    name: 'AwardIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-award",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M10 2l1.669.864 1.858.282.842 1.68 1.337 1.32L15.4 8l.306 1.854-1.337 1.32-.842 1.68-1.858.282L10 14l-1.669-.864-1.858-.282-.842-1.68-1.337-1.32L4.6 8l-.306-1.854 1.337-1.32.842-1.68 1.858-.282L10 2z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M6 13.794V18l4-1 4 1v-4.206l-2.018.306L10 15.126 8.018 14.1 6 13.794z"
        }
      })]);
    }
  });

  var backspaceFill = Vue.extend({
    name: 'BackspaceFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-backspace-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17.683 5a2 2 0 00-2-2h-7.08a2 2 0 00-1.519.698L2.241 9.35a1 1 0 000 1.302l4.843 5.65A2 2 0 008.603 17h7.08a2 2 0 002-2V5zM7.829 7.854a.5.5 0 11.707-.708l2.147 2.147 2.146-2.147a.5.5 0 11.707.708L11.39 10l2.146 2.146a.5.5 0 01-.707.708l-2.146-2.147-2.147 2.147a.5.5 0 01-.707-.708L9.976 10 7.829 7.854z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var backspaceReverseFill = Vue.extend({
    name: 'BackspaceReverseFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-backspace-reverse-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2 5a2 2 0 012-2h7.08a2 2 0 011.519.698l4.843 5.651a1 1 0 010 1.302L12.6 16.3a2 2 0 01-1.52.7H4a2 2 0 01-2-2V5zm9.854 2.854a.5.5 0 00-.708-.708L9 9.293 6.854 7.146a.5.5 0 10-.708.708L8.293 10l-2.147 2.146a.5.5 0 00.708.708L9 10.707l2.146 2.147a.5.5 0 00.708-.708L9.707 10l2.147-2.146z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var backspaceReverse = Vue.extend({
    name: 'BackspaceReverseIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-backspace-reverse",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.08 4H4a1 1 0 00-1 1v10a1 1 0 001 1h7.08a1 1 0 00.76-.35L16.682 10l-4.844-5.65A1 1 0 0011.08 4zM4 3a2 2 0 00-2 2v10a2 2 0 002 2h7.08a2 2 0 001.519-.698l4.843-5.651a1 1 0 000-1.302L12.6 3.7a2 2 0 00-1.52-.7H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.854 7.146a.5.5 0 010 .708l-5 5a.5.5 0 01-.708-.708l5-5a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.146 7.146a.5.5 0 000 .708l5 5a.5.5 0 00.708-.708l-5-5a.5.5 0 00-.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var backspace = Vue.extend({
    name: 'BackspaceIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-backspace",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.603 4h7.08a1 1 0 011 1v10a1 1 0 01-1 1h-7.08a1 1 0 01-.76-.35L3 10l4.844-5.65A1 1 0 018.603 4zm7.08-1a2 2 0 012 2v10a2 2 0 01-2 2h-7.08a2 2 0 01-1.519-.698L2.241 10.65a1 1 0 010-1.302L7.084 3.7A2 2 0 018.603 3h7.08z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.83 7.146a.5.5 0 000 .708l5 5a.5.5 0 00.707-.708l-5-5a.5.5 0 00-.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.537 7.146a.5.5 0 010 .708l-5 5a.5.5 0 01-.708-.708l5-5a.5.5 0 01.707 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var barChartFill = Vue.extend({
    name: 'BarChartFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-bar-chart-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("rect", {
        attrs: {
          width: "4",
          height: "5",
          x: "3",
          y: "12",
          rx: "1"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "4",
          height: "9",
          x: "8",
          y: "8",
          rx: "1"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "4",
          height: "14",
          x: "13",
          y: "3",
          rx: "1"
        }
      })]);
    }
  });

  var barChart = Vue.extend({
    name: 'BarChartIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-bar-chart",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6 13H4v3h2v-3zm5-4H9v7h2V9zm5-5h-2v12h2V4zm-2-1a1 1 0 00-1 1v12a1 1 0 001 1h2a1 1 0 001-1V4a1 1 0 00-1-1h-2zM8 9a1 1 0 011-1h2a1 1 0 011 1v7a1 1 0 01-1 1H9a1 1 0 01-1-1V9zm-5 4a1 1 0 011-1h2a1 1 0 011 1v3a1 1 0 01-1 1H4a1 1 0 01-1-1v-3z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var batteryCharging = Vue.extend({
    name: 'BatteryChargingIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-battery-charging",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M16.5 11.5a1.5 1.5 0 000-3v3z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.585 4.568a.5.5 0 01.226.579l-1.134 3.686h1.99a.5.5 0 01.364.843l-5.334 5.667a.5.5 0 01-.842-.49l1.135-3.686H6a.5.5 0 01-.364-.843l5.333-5.667a.5.5 0 01.616-.09z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.332 6H4a2 2 0 00-2 2v4a2 2 0 002 2h2.072l.307-1H4a1 1 0 01-1-1V8a1 1 0 011-1h3.391l.941-1zM6.45 8H4v4h1.313a1.5 1.5 0 01-.405-2.361L6.45 8zm.976 5l-.308 1H8.96l.21-.224h.001l.73-.776H8.53l-.085.09.028-.09H7.426zm1.354-1H7.733l.257-.833H6a.5.5 0 01-.364-.843l.793-.843L7.823 8h1.373l-2.039 2.167h1.51a.492.492 0 01.166.028.5.5 0 01.312.619L8.78 12zm.69 0h1.373l1.395-1.482.793-.842a.5.5 0 00-.364-.843h-1.99L10.933 8H9.887l-.166.54-.199.646a.5.5 0 00.478.647h1.51L9.47 12zm.725-5h1.046l.308-1H9.706l-.942 1h1.374l.085-.09-.028.09zm2.4-1l-.308 1H14a1 1 0 011 1v4a1 1 0 01-1 1h-2.724l-.942 1H14a2 2 0 002-2V8a2 2 0 00-2-2h-1.405zm-.378 6H14v-1.98a1.499 1.499 0 01-.241.341L12.217 12zM14 8.646V8h-.646a1.5 1.5 0 01.646.646z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var batteryFull = Vue.extend({
    name: 'BatteryFullIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-battery-full",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14 7H4a1 1 0 00-1 1v4a1 1 0 001 1h10a1 1 0 001-1V8a1 1 0 00-1-1zM4 6a2 2 0 00-2 2v4a2 2 0 002 2h10a2 2 0 002-2V8a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M4 8h10v4H4V8zm12.5 3.5a1.5 1.5 0 000-3v3z"
        }
      })]);
    }
  });

  var battery = Vue.extend({
    name: 'BatteryIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-battery",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14 7H4a1 1 0 00-1 1v4a1 1 0 001 1h10a1 1 0 001-1V8a1 1 0 00-1-1zM4 6a2 2 0 00-2 2v4a2 2 0 002 2h10a2 2 0 002-2V8a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M16.5 11.5a1.5 1.5 0 000-3v3z"
        }
      })]);
    }
  });

  var bellFill = Vue.extend({
    name: 'BellFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-bell-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M10 18a2 2 0 002-2H8a2 2 0 002 2zm.995-14.901a1 1 0 10-1.99 0A5.002 5.002 0 005 8c0 1.098-.5 6-2 7h14c-1.5-1-2-5.902-2-7 0-2.42-1.72-4.44-4.005-4.901z"
        }
      })]);
    }
  });

  var bell = Vue.extend({
    name: 'BellIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-bell",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M10 18a2 2 0 002-2H8a2 2 0 002 2z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 3.918l-.797.161A4.002 4.002 0 006 8c0 .628-.134 2.197-.459 3.742-.16.767-.376 1.566-.663 2.258h10.244c-.287-.692-.502-1.49-.663-2.258C14.134 10.197 14 8.628 14 8a4.002 4.002 0 00-3.203-3.92L10 3.917zM16.22 14c.223.447.482.801.78 1H3c.299-.199.557-.553.78-1C4.68 12.2 5 8.88 5 8c0-2.42 1.72-4.44 4.005-4.901a1 1 0 111.99 0A5.002 5.002 0 0115 8c0 .88.32 4.2 1.22 6z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var blockquoteLeft = Vue.extend({
    name: 'BlockquoteLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-blockquote-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 5.5a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm5 3a.5.5 0 01.5-.5h6a.5.5 0 010 1h-6a.5.5 0 01-.5-.5zm0 3a.5.5 0 01.5-.5h6a.5.5 0 010 1h-6a.5.5 0 01-.5-.5zm-5 3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M5.734 8.352a6.586 6.586 0 00-.445.275 1.94 1.94 0 00-.346.299 1.38 1.38 0 00-.252.369c-.058.129-.1.295-.123.498h.282c.242 0 .431.06.568.182.14.117.21.29.21.521a.697.697 0 01-.187.463c-.12.14-.289.21-.503.21-.336 0-.577-.109-.721-.327-.145-.223-.217-.514-.217-.873 0-.254.055-.485.164-.692.11-.21.242-.398.399-.562.16-.168.33-.31.51-.428.179-.117.33-.213.45-.287l.211.352zm2.168 0a6.588 6.588 0 00-.445.275 1.94 1.94 0 00-.346.299c-.113.12-.199.246-.257.375a1.75 1.75 0 00-.118.492h.282c.242 0 .431.06.568.182.14.117.21.29.21.521a.697.697 0 01-.187.463c-.12.14-.289.21-.504.21-.335 0-.576-.109-.72-.327-.145-.223-.217-.514-.217-.873 0-.254.055-.485.164-.692.11-.21.242-.398.398-.562.16-.168.33-.31.51-.428.18-.117.33-.213.451-.287l.211.352z"
        }
      })]);
    }
  });

  var blockquoteRight = Vue.extend({
    name: 'BlockquoteRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-blockquote-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 5.5a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm0 3a.5.5 0 01.5-.5h6a.5.5 0 010 1h-6a.5.5 0 01-.5-.5zm0 3a.5.5 0 01.5-.5h6a.5.5 0 010 1h-6a.5.5 0 01-.5-.5zm0 3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M14.168 8.352c.184.105.332.197.445.275.114.074.229.174.346.299.11.117.193.24.252.369s.1.295.123.498h-.281c-.243 0-.432.06-.569.182-.14.117-.21.29-.21.521 0 .164.062.319.187.463.121.14.289.21.504.21.336 0 .576-.109.72-.327.145-.223.217-.514.217-.873 0-.254-.054-.485-.164-.692a2.436 2.436 0 00-.398-.562c-.16-.168-.33-.31-.51-.428-.18-.117-.33-.213-.451-.287l-.211.352zm-2.168 0c.184.105.332.197.445.275.114.074.229.174.346.299.113.12.2.246.258.375.055.125.094.289.117.492h-.281c-.242 0-.432.06-.569.182-.14.117-.21.29-.21.521 0 .164.062.319.187.463.121.14.289.21.504.21.336 0 .576-.109.72-.327.145-.223.217-.514.217-.873 0-.254-.054-.485-.164-.692a2.438 2.438 0 00-.398-.562c-.16-.168-.33-.31-.51-.428-.18-.117-.33-.213-.451-.287L12 8.352z"
        }
      })]);
    }
  });

  var bookHalfFill = Vue.extend({
    name: 'BookHalfFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-book-half-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.214 3.072c1.599-.32 3.702-.363 5.14 1.074a.5.5 0 01.146.354v11a.5.5 0 01-.854.354c-.843-.844-2.115-1.059-3.47-.92-1.344.14-2.66.617-3.452 1.013A.5.5 0 012 15.5v-11a.5.5 0 01.276-.447L2.5 4.5l-.224-.447.002-.001.004-.002.013-.006a5.116 5.116 0 01.22-.103 12.958 12.958 0 012.7-.869zM3 4.82v9.908c.846-.343 1.944-.672 3.074-.788 1.143-.118 2.387-.023 3.426.56V4.718c-1.063-.929-2.631-.956-4.09-.664A11.958 11.958 0 003 4.82z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14.786 3.072c-1.598-.32-3.702-.363-5.14 1.074A.5.5 0 009.5 4.5v11a.5.5 0 00.854.354c.844-.844 2.115-1.059 3.47-.92 1.344.14 2.66.617 3.452 1.013A.5.5 0 0018 15.5v-11a.5.5 0 00-.276-.447L17.5 4.5l.224-.447-.002-.001-.004-.002-.013-.006-.047-.023a12.582 12.582 0 00-.799-.34 12.96 12.96 0 00-2.073-.609z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var book = Vue.extend({
    name: 'BookIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-book",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.214 3.072c1.599-.32 3.702-.363 5.14 1.074a.5.5 0 01.146.354v11a.5.5 0 01-.854.354c-.843-.844-2.115-1.059-3.47-.92-1.344.14-2.66.617-3.452 1.013A.5.5 0 012 15.5v-11a.5.5 0 01.276-.447L2.5 4.5l-.224-.447.002-.001.004-.002.013-.006a5.116 5.116 0 01.22-.103 12.958 12.958 0 012.7-.869zM3 4.82v9.908c.846-.343 1.944-.672 3.074-.788 1.143-.118 2.387-.023 3.426.56V4.718c-1.063-.929-2.631-.956-4.09-.664A11.958 11.958 0 003 4.82z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14.786 3.072c-1.598-.32-3.702-.363-5.14 1.074A.5.5 0 009.5 4.5v11a.5.5 0 00.854.354c.844-.844 2.115-1.059 3.47-.92 1.344.14 2.66.617 3.452 1.013A.5.5 0 0018 15.5v-11a.5.5 0 00-.276-.447L17.5 4.5l.224-.447-.002-.001-.004-.002-.013-.006-.047-.023a12.582 12.582 0 00-.799-.34 12.96 12.96 0 00-2.073-.609zM17 4.82v9.908c-.846-.343-1.944-.672-3.074-.788-1.143-.118-2.386-.023-3.426.56V4.718c1.063-.929 2.631-.956 4.09-.664A11.956 11.956 0 0117 4.82z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var bookmarkFill = Vue.extend({
    name: 'BookmarkFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-bookmark-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5 5a2 2 0 012-2h6a2 2 0 012 2v12l-5-3-5 3V5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var bookmark = Vue.extend({
    name: 'BookmarkIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-bookmark",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 14l5 3V5a2 2 0 00-2-2H7a2 2 0 00-2 2v12l5-3zm-4 1.234l4-2.4 4 2.4V5a1 1 0 00-1-1H7a1 1 0 00-1 1v10.234z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var bootstrapFill = Vue.extend({
    name: 'BootstrapFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-bootstrap-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.002 2a4 4 0 00-4 4v8a4 4 0 004 4h8a4 4 0 004-4V6a4 4 0 00-4-4h-8zm1.06 12h3.475c1.804 0 2.888-.908 2.888-2.396 0-1.102-.761-1.916-1.904-2.034v-.1c.832-.14 1.482-.93 1.482-1.816 0-1.3-.955-2.11-2.543-2.11H7.063V14zm1.313-4.875V6.658h1.78c.974 0 1.542.457 1.542 1.237 0 .802-.604 1.23-1.764 1.23H8.375zm0 3.762h1.898c1.184 0 1.81-.48 1.81-1.377 0-.885-.65-1.348-1.886-1.348H8.375v2.725z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var bootstrapReboot = Vue.extend({
    name: 'BootstrapRebootIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-bootstrap-reboot",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.161 10a6.84 6.84 0 106.842-6.84.58.58 0 110-1.16 8 8 0 11-6.556 3.412l-.663-.577a.58.58 0 01.227-.997l2.52-.69a.58.58 0 01.728.633l-.332 2.592a.58.58 0 01-.956.364l-.643-.56A6.812 6.812 0 003.16 10zm5.228-.079V7.277h1.57c.881 0 1.416.499 1.416 1.32 0 .84-.505 1.324-1.386 1.324h-1.6zm0 3.75v-2.828h1.57l1.498 2.828h1.314l-1.646-3.006c.897-.3 1.427-1.106 1.427-2.1 0-1.37-.943-2.246-2.456-2.246H7.248v7.352h1.141z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var bootstrap = Vue.extend({
    name: 'BootstrapIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-bootstrap",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14 3H6a3 3 0 00-3 3v8a3 3 0 003 3h8a3 3 0 003-3V6a3 3 0 00-3-3zM6 2a4 4 0 00-4 4v8a4 4 0 004 4h8a4 4 0 004-4V6a4 4 0 00-4-4H6z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.537 14H7.062V5.545h3.398c1.588 0 2.543.809 2.543 2.11 0 .884-.65 1.675-1.482 1.816v.1c1.143.117 1.904.931 1.904 2.033 0 1.488-1.084 2.396-2.888 2.396zM8.375 6.658v2.467h1.558c1.16 0 1.764-.428 1.764-1.23 0-.78-.568-1.237-1.541-1.237H8.375zm1.898 6.229H8.375v-2.725h1.822c1.236 0 1.887.463 1.887 1.348 0 .896-.627 1.377-1.811 1.377z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var boxArrowBottomLeft = Vue.extend({
    name: 'BoxArrowBottomLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-box-arrow-bottom-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15 3.5A1.5 1.5 0 0116.5 5v8a1.5 1.5 0 01-1.5 1.5h-4a.5.5 0 010-1h4a.5.5 0 00.5-.5V5a.5.5 0 00-.5-.5H7a.5.5 0 00-.5.5v4a.5.5 0 01-1 0V5A1.5 1.5 0 017 3.5h8zm-11 7a.5.5 0 00-.5.5v5a.5.5 0 00.5.5h5a.5.5 0 000-1H4.5V11a.5.5 0 00-.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.646 16.354a.5.5 0 00.708 0l8-8a.5.5 0 00-.708-.708l-8 8a.5.5 0 000 .708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var boxArrowBottomRight = Vue.extend({
    name: 'BoxArrowBottomRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-box-arrow-bottom-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5 3.5A1.5 1.5 0 003.5 5v8A1.5 1.5 0 005 14.5h4a.5.5 0 000-1H5a.5.5 0 01-.5-.5V5a.5.5 0 01.5-.5h8a.5.5 0 01.5.5v4a.5.5 0 001 0V5A1.5 1.5 0 0013 3.5H5zm11 7a.5.5 0 01.5.5v5a.5.5 0 01-.5.5h-5a.5.5 0 010-1h4.5V11a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16.354 16.354a.5.5 0 01-.708 0l-8-8a.5.5 0 11.708-.708l8 8a.5.5 0 010 .708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var boxArrowDown = Vue.extend({
    name: 'BoxArrowDownIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-box-arrow-down",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.646 13.646a.5.5 0 01.708 0L10 16.293l2.646-2.647a.5.5 0 01.708.708l-3 3a.5.5 0 01-.708 0l-3-3a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 6.5a.5.5 0 01.5.5v9a.5.5 0 01-1 0V7a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.5 4A1.5 1.5 0 016 2.5h8A1.5 1.5 0 0115.5 4v7a1.5 1.5 0 01-1.5 1.5h-1.5a.5.5 0 010-1H14a.5.5 0 00.5-.5V4a.5.5 0 00-.5-.5H6a.5.5 0 00-.5.5v7a.5.5 0 00.5.5h1.5a.5.5 0 010 1H6A1.5 1.5 0 014.5 11V4z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var boxArrowLeft = Vue.extend({
    name: 'BoxArrowLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-box-arrow-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.354 13.354a.5.5 0 000-.708L3.707 10l2.647-2.646a.5.5 0 10-.708-.708l-3 3a.5.5 0 000 .708l3 3a.5.5 0 00.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.5 10a.5.5 0 00-.5-.5H4a.5.5 0 000 1h9a.5.5 0 00.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 15.5a1.5 1.5 0 001.5-1.5V6A1.5 1.5 0 0016 4.5H9A1.5 1.5 0 007.5 6v1.5a.5.5 0 001 0V6a.5.5 0 01.5-.5h7a.5.5 0 01.5.5v8a.5.5 0 01-.5.5H9a.5.5 0 01-.5-.5v-1.5a.5.5 0 00-1 0V14A1.5 1.5 0 009 15.5h7z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var boxArrowRight = Vue.extend({
    name: 'BoxArrowRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-box-arrow-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.646 13.354a.5.5 0 010-.708L16.293 10l-2.647-2.646a.5.5 0 01.708-.708l3 3a.5.5 0 010 .708l-3 3a.5.5 0 01-.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.5 10a.5.5 0 01.5-.5h9a.5.5 0 010 1H7a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 15.5A1.5 1.5 0 012.5 14V6A1.5 1.5 0 014 4.5h7A1.5 1.5 0 0112.5 6v1.5a.5.5 0 01-1 0V6a.5.5 0 00-.5-.5H4a.5.5 0 00-.5.5v8a.5.5 0 00.5.5h7a.5.5 0 00.5-.5v-1.5a.5.5 0 011 0V14a1.5 1.5 0 01-1.5 1.5H4z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var boxArrowUpLeft = Vue.extend({
    name: 'BoxArrowUpLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-box-arrow-up-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16.5 15a1.5 1.5 0 01-1.5 1.5H7A1.5 1.5 0 015.5 15v-4a.5.5 0 011 0v4a.5.5 0 00.5.5h8a.5.5 0 00.5-.5V7a.5.5 0 00-.5-.5h-4a.5.5 0 010-1h4A1.5 1.5 0 0116.5 7v8zm-7-11a.5.5 0 00-.5-.5H4a.5.5 0 00-.5.5v5a.5.5 0 001 0V4.5H9a.5.5 0 00.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.646 3.646a.5.5 0 000 .708l8 8a.5.5 0 00.708-.708l-8-8a.5.5 0 00-.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var boxArrowUpRight = Vue.extend({
    name: 'BoxArrowUpRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-box-arrow-up-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.5 15A1.5 1.5 0 005 16.5h8a1.5 1.5 0 001.5-1.5v-4a.5.5 0 00-1 0v4a.5.5 0 01-.5.5H5a.5.5 0 01-.5-.5V7a.5.5 0 01.5-.5h4a.5.5 0 000-1H5A1.5 1.5 0 003.5 7v8zm7-11a.5.5 0 01.5-.5h5a.5.5 0 01.5.5v5a.5.5 0 01-1 0V4.5H11a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16.354 3.646a.5.5 0 010 .708l-8 8a.5.5 0 01-.708-.708l8-8a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var boxArrowUp = Vue.extend({
    name: 'BoxArrowUpIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-box-arrow-up",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.646 6.354a.5.5 0 00.708 0L10 3.707l2.646 2.647a.5.5 0 00.708-.708l-3-3a.5.5 0 00-.708 0l-3 3a.5.5 0 000 .708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 13.5a.5.5 0 00.5-.5V4a.5.5 0 00-1 0v9a.5.5 0 00.5.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.5 16A1.5 1.5 0 006 17.5h8a1.5 1.5 0 001.5-1.5V9A1.5 1.5 0 0014 7.5h-1.5a.5.5 0 000 1H14a.5.5 0 01.5.5v7a.5.5 0 01-.5.5H6a.5.5 0 01-.5-.5V9a.5.5 0 01.5-.5h1.5a.5.5 0 000-1H6A1.5 1.5 0 004.5 9v7z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var braces = Vue.extend({
    name: 'BracesIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-braces",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M4.114 10.063V9.9c1.005-.102 1.497-.615 1.497-1.6V6.503c0-1.094.39-1.538 1.354-1.538h.273V4h-.376C5.25 4 4.49 4.759 4.49 6.352v1.524c0 1.094-.376 1.456-1.49 1.456v1.299c1.114 0 1.49.362 1.49 1.456v1.524c0 1.593.759 2.352 2.372 2.352h.376v-.964h-.273c-.964 0-1.354-.444-1.354-1.538v-1.798c0-.984-.492-1.497-1.497-1.6zM15.886 9.9v.163c-1.005.103-1.497.616-1.497 1.6v1.798c0 1.094-.39 1.538-1.354 1.538h-.273v.964h.376c1.613 0 2.372-.759 2.372-2.352v-1.524c0-1.094.376-1.456 1.49-1.456V9.332c-1.114 0-1.49-.362-1.49-1.456V6.352C15.51 4.759 14.75 4 13.138 4h-.376v.964h.273c.964 0 1.354.444 1.354 1.538V8.3c0 .984.492 1.497 1.497 1.6z"
        }
      })]);
    }
  });

  var brightnessFillHigh = Vue.extend({
    name: 'BrightnessFillHighIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-brightness-fill-high",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("circle", {
        attrs: {
          cx: "10",
          cy: "10",
          r: "4"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 2a.5.5 0 01.5.5v2a.5.5 0 01-1 0v-2A.5.5 0 0110 2zm0 13a.5.5 0 01.5.5v2a.5.5 0 01-1 0v-2a.5.5 0 01.5-.5zm8-5a.5.5 0 01-.5.5h-2a.5.5 0 010-1h2a.5.5 0 01.5.5zM5 10a.5.5 0 01-.5.5h-2a.5.5 0 010-1h2a.5.5 0 01.5.5zm10.657-5.657a.5.5 0 010 .707l-1.414 1.414a.5.5 0 01-.707-.707l1.414-1.414a.5.5 0 01.707 0zm-9.193 9.193a.5.5 0 010 .707L5.05 15.657a.5.5 0 01-.707-.707l1.414-1.414a.5.5 0 01.707 0zm9.193 2.121a.5.5 0 01-.707 0l-1.414-1.414a.5.5 0 01.707-.707l1.414 1.414a.5.5 0 010 .707zM6.464 6.464a.5.5 0 01-.707 0L4.343 5.05a.5.5 0 01.707-.707l1.414 1.414a.5.5 0 010 .707z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var brightnessFillLow = Vue.extend({
    name: 'BrightnessFillLowIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-brightness-fill-low",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("circle", {
        attrs: {
          cx: "10",
          cy: "10",
          r: "4"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "10",
          cy: "4.5",
          r: ".5"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "10",
          cy: "15.5",
          r: ".5"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "15.5",
          cy: "10",
          r: ".5",
          transform: "rotate(90 15.5 10)"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "4.5",
          cy: "10",
          r: ".5",
          transform: "rotate(90 4.5 10)"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "13.889",
          cy: "6.111",
          r: ".5",
          transform: "rotate(45 13.89 6.11)"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "6.111",
          cy: "13.889",
          r: ".5",
          transform: "rotate(45 6.11 13.89)"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "13.889",
          cy: "13.889",
          r: ".5",
          transform: "rotate(135 13.89 13.89)"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "6.111",
          cy: "6.111",
          r: ".5",
          transform: "rotate(135 6.11 6.11)"
        }
      })]);
    }
  });

  var brightnessHigh = Vue.extend({
    name: 'BrightnessHighIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-brightness-high",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 13a3 3 0 100-6 3 3 0 000 6zm0 1a4 4 0 100-8 4 4 0 000 8zm0-12a.5.5 0 01.5.5v2a.5.5 0 01-1 0v-2A.5.5 0 0110 2zm0 13a.5.5 0 01.5.5v2a.5.5 0 01-1 0v-2a.5.5 0 01.5-.5zm8-5a.5.5 0 01-.5.5h-2a.5.5 0 010-1h2a.5.5 0 01.5.5zM5 10a.5.5 0 01-.5.5h-2a.5.5 0 010-1h2a.5.5 0 01.5.5zm10.657-5.657a.5.5 0 010 .707l-1.414 1.414a.5.5 0 11-.707-.707l1.414-1.414a.5.5 0 01.707 0zm-9.193 9.193a.5.5 0 010 .707L5.05 15.657a.5.5 0 01-.707-.707l1.414-1.414a.5.5 0 01.707 0zm9.193 2.121a.5.5 0 01-.707 0l-1.414-1.414a.5.5 0 01.707-.707l1.414 1.414a.5.5 0 010 .707zM6.464 6.464a.5.5 0 01-.707 0L4.343 5.05a.5.5 0 01.707-.707l1.414 1.414a.5.5 0 010 .707z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var brightnessLow = Vue.extend({
    name: 'BrightnessLowIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-brightness-low",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 13a3 3 0 100-6 3 3 0 000 6zm0 1a4 4 0 100-8 4 4 0 000 8z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "10",
          cy: "4.5",
          r: ".5"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "10",
          cy: "15.5",
          r: ".5"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "15.5",
          cy: "10",
          r: ".5",
          transform: "rotate(90 15.5 10)"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "4.5",
          cy: "10",
          r: ".5",
          transform: "rotate(90 4.5 10)"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "13.889",
          cy: "6.111",
          r: ".5",
          transform: "rotate(45 13.89 6.11)"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "6.111",
          cy: "13.889",
          r: ".5",
          transform: "rotate(45 6.11 13.89)"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "13.889",
          cy: "13.889",
          r: ".5",
          transform: "rotate(135 13.89 13.89)"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "6.111",
          cy: "6.111",
          r: ".5",
          transform: "rotate(135 6.11 6.11)"
        }
      })]);
    }
  });

  var brush = Vue.extend({
    name: 'BrushIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-brush",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M17.213 3.018a.572.572 0 01.755.05.57.57 0 01.058.746c-.941 1.268-3.982 5.293-6.426 7.736a12.89 12.89 0 01-1.952 1.596c-.508.339-1.167.234-1.599-.197-.416-.416-.53-1.047-.213-1.543.347-.542.888-1.273 1.643-1.977 2.521-2.35 6.476-5.44 7.734-6.411z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9 14a2 2 0 01-2 2c-1 0-2 0-3.5-.5s.5-1 1-1.5 1.395-2 2.5-2a2 2 0 012 2z"
        }
      })]);
    }
  });

  var bucketFill = Vue.extend({
    name: 'BucketFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-bucket-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 3.5A4.5 4.5 0 005.5 8h-1a5.5 5.5 0 1111 0h-1A4.5 4.5 0 0010 3.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.61 7.687A.5.5 0 014 7.5h12a.5.5 0 01.488.608l-1.826 8.217a1.5 1.5 0 01-1.464 1.175H6.802a1.5 1.5 0 01-1.464-1.175L3.512 8.108a.5.5 0 01.098-.42z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var bucket = Vue.extend({
    name: 'BucketIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-bucket",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 3.5A4.5 4.5 0 005.5 8h-1a5.5 5.5 0 1111 0h-1A4.5 4.5 0 0010 3.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.61 7.687A.5.5 0 014 7.5h12a.5.5 0 01.488.608l-1.826 8.217a1.5 1.5 0 01-1.464 1.175H6.802a1.5 1.5 0 01-1.464-1.175L3.512 8.108a.5.5 0 01.098-.42zm1.013.813l1.691 7.608a.5.5 0 00.488.392h6.396a.5.5 0 00.488-.392l1.69-7.608H4.624z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var building = Vue.extend({
    name: 'BuildingIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-building",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17.285 2.089a.5.5 0 01.215.411v15a.5.5 0 01-.5.5h-3a.5.5 0 01-.5-.5V16h-1v1.5a.5.5 0 01-.5.5H3a.5.5 0 01-.5-.5v-6a.5.5 0 01.418-.493l5.582-.93V5.5a.5.5 0 01.324-.468l8-3a.5.5 0 01.46.057zM9.5 5.846V10.5a.5.5 0 01-.418.493l-5.582.93V17h8v-1.5a.5.5 0 01.5-.5h2a.5.5 0 01.5.5V17h2V3.221l-7 2.625z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.5 17.5v-7h1v7h-1z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M4.5 13h1v1h-1v-1zm2 0h1v1h-1v-1zm-2 2h1v1h-1v-1zm2 0h1v1h-1v-1zm6-10h1v1h-1V5zm2 0h1v1h-1V5zm-4 2h1v1h-1V7zm2 0h1v1h-1V7zm2 0h1v1h-1V7zm-2 2h1v1h-1V9zm2 0h1v1h-1V9zm-4 0h1v1h-1V9zm0 2h1v1h-1v-1zm2 0h1v1h-1v-1zm2 0h1v1h-1v-1zm-4 2h1v1h-1v-1zm2 0h1v1h-1v-1zm2 0h1v1h-1v-1z"
        }
      })]);
    }
  });

  var bullseye = Vue.extend({
    name: 'BullseyeIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-bullseye",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 17a7 7 0 100-14 7 7 0 000 14zm0 1a8 8 0 100-16 8 8 0 000 16z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 15a5 5 0 100-10 5 5 0 000 10zm0 1a6 6 0 100-12 6 6 0 000 12z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 13a3 3 0 100-6 3 3 0 000 6zm0 1a4 4 0 100-8 4 4 0 000 8z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M11.5 10a1.5 1.5 0 11-3 0 1.5 1.5 0 013 0z"
        }
      })]);
    }
  });

  var calendarFill = Vue.extend({
    name: 'CalendarFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-calendar-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M2 4a2 2 0 012-2h12a2 2 0 012 2H2z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2 5h16v11a2 2 0 01-2 2H4a2 2 0 01-2-2V5zm6.5 4a1 1 0 100-2 1 1 0 000 2zm4-1a1 1 0 11-2 0 1 1 0 012 0zm2 1a1 1 0 100-2 1 1 0 000 2zm-8 2a1 1 0 11-2 0 1 1 0 012 0zm2 1a1 1 0 100-2 1 1 0 000 2zm4-1a1 1 0 11-2 0 1 1 0 012 0zm2 1a1 1 0 100-2 1 1 0 000 2zm-8 2a1 1 0 11-2 0 1 1 0 012 0zm2 1a1 1 0 100-2 1 1 0 000 2zm4-1a1 1 0 11-2 0 1 1 0 012 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var calendar = Vue.extend({
    name: 'CalendarIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-calendar",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 2H4a2 2 0 00-2 2v12a2 2 0 002 2h12a2 2 0 002-2V4a2 2 0 00-2-2zM3 5.857C3 5.384 3.448 5 4 5h12c.552 0 1 .384 1 .857v10.286c0 .473-.448.857-1 .857H4c-.552 0-1-.384-1-.857V5.857z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.5 9a1 1 0 100-2 1 1 0 000 2zm3 0a1 1 0 100-2 1 1 0 000 2zm3 0a1 1 0 100-2 1 1 0 000 2zm-9 3a1 1 0 100-2 1 1 0 000 2zm3 0a1 1 0 100-2 1 1 0 000 2zm3 0a1 1 0 100-2 1 1 0 000 2zm3 0a1 1 0 100-2 1 1 0 000 2zm-9 3a1 1 0 100-2 1 1 0 000 2zm3 0a1 1 0 100-2 1 1 0 000 2zm3 0a1 1 0 100-2 1 1 0 000 2z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var cameraVideoFill = Vue.extend({
    name: 'CameraVideoFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-camera-video-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M4.667 5h6.666C12.253 5 13 5.746 13 6.667v6.666c0 .92-.746 1.667-1.667 1.667H4.667C3.747 15 3 14.254 3 13.333V6.667C3 5.747 3.746 5 4.667 5z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9.404 10.697l6.363 3.692c.54.313 1.233-.066 1.233-.697V6.308c0-.63-.692-1.01-1.233-.696L9.404 9.304a.802.802 0 000 1.393z"
        }
      })]);
    }
  });

  var cameraVideo = Vue.extend({
    name: 'CameraVideoIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-camera-video",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.667 5.5c-.645 0-1.167.522-1.167 1.167v6.666c0 .645.522 1.167 1.167 1.167h6.666c.645 0 1.167-.522 1.167-1.167V6.667c0-.645-.522-1.167-1.167-1.167H4.667zM2.5 6.667C2.5 5.47 3.47 4.5 4.667 4.5h6.666c1.197 0 2.167.97 2.167 2.167v6.666c0 1.197-.97 2.167-2.167 2.167H4.667A2.167 2.167 0 012.5 13.333V6.667z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.25 7.65l2.768-1.605a.318.318 0 01.482.263v7.384c0 .228-.26.393-.482.264l-2.767-1.605-.502.865 2.767 1.605c.859.498 1.984-.095 1.984-1.129V6.308c0-1.033-1.125-1.626-1.984-1.128L12.75 6.785l.502.865z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var camera = Vue.extend({
    name: 'CameraIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-camera",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M11 7c-1.657 0-4 1.343-4 3a4 4 0 014-4v1z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16.333 5h-2.015A5.97 5.97 0 0011 4a5.972 5.972 0 00-3.318 1H3.667C2.747 5 2 5.746 2 6.667v6.666C2 14.253 2.746 15 3.667 15h4.015c.95.632 2.091 1 3.318 1a5.973 5.973 0 003.318-1h2.015c.92 0 1.667-.746 1.667-1.667V6.667C18 5.747 17.254 5 16.333 5zM3.5 7a.5.5 0 100-1 .5.5 0 000 1zm7.5 8a5 5 0 100-10 5 5 0 000 10z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M4 5a1 1 0 011-1h1a1 1 0 010 2H5a1 1 0 01-1-1z"
        }
      })]);
    }
  });

  var capslockFill = Vue.extend({
    name: 'CapslockFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-capslock-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.27 3.047a1 1 0 011.46 0l6.345 6.77c.6.638.146 1.683-.73 1.683H13.5v1a1 1 0 01-1 1h-5a1 1 0 01-1-1v-1H3.654c-.875 0-1.328-1.045-.73-1.684L9.27 3.047zM6.5 15.5a1 1 0 011-1h5a1 1 0 011 1v1a1 1 0 01-1 1h-5a1 1 0 01-1-1v-1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var capslock = Vue.extend({
    name: 'CapslockIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-capslock",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.27 3.047a1 1 0 011.46 0l6.345 6.77c.6.638.146 1.683-.73 1.683H13.5v1a1 1 0 01-1 1h-5a1 1 0 01-1-1v-1H3.654c-.875 0-1.328-1.045-.73-1.684L9.27 3.047zm7.076 7.453L10 3.731 3.654 10.5H6.5a1 1 0 011 1v1h5v-1a1 1 0 011-1h2.846zm-9.846 5a1 1 0 011-1h5a1 1 0 011 1v1a1 1 0 01-1 1h-5a1 1 0 01-1-1v-1zm6 0h-5v1h5v-1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var chatFill = Vue.extend({
    name: 'ChatFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-chat-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.5 14s.65.65.117 1.948A4.821 4.821 0 017 17c.449 0 .887-.067 1.307-.181C10.692 16.169 12.5 14 12.5 14a5.5 5.5 0 100-11h-5a5.5 5.5 0 100 11z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var chat = Vue.extend({
    name: 'ChatIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-chat",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.207 13.293L7.5 14a5.5 5.5 0 110-11h5a5.5 5.5 0 110 11s-1.807 2.169-4.193 2.818C7.887 16.933 7.449 17 7 17c.291-.389.488-.74.617-1.052C8.149 14.649 7.5 14 7.5 14c.707-.707.708-.707.708-.706h.001l.002.003.004.004.01.01a1.184 1.184 0 01.074.084c.039.047.085.108.134.183.097.15.206.36.284.626.114.386.154.855.047 1.394.717-.313 1.37-.765 1.895-1.201a10.266 10.266 0 001.013-.969l.05-.056.01-.012m0 0A1 1 0 0112.5 13a4.5 4.5 0 100-9h-5a4.5 4.5 0 000 9 1 1 0 01.707.293",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var checkBox = Vue.extend({
    name: 'CheckBoxIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-check-box",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17.354 4.646a.5.5 0 010 .708l-7 7a.5.5 0 01-.708 0l-3-3a.5.5 0 11.708-.708L10 11.293l6.646-6.647a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.5 15A1.5 1.5 0 005 16.5h10a1.5 1.5 0 001.5-1.5v-5a.5.5 0 00-1 0v5a.5.5 0 01-.5.5H5a.5.5 0 01-.5-.5V5a.5.5 0 01.5-.5h8a.5.5 0 000-1H5A1.5 1.5 0 003.5 5v10z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var checkCircle = Vue.extend({
    name: 'CheckCircleIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-check-circle",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17.354 4.646a.5.5 0 010 .708l-7 7a.5.5 0 01-.708 0l-3-3a.5.5 0 11.708-.708L10 11.293l6.646-6.647a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 4.5a5.5 5.5 0 105.5 5.5.5.5 0 011 0 6.5 6.5 0 11-3.25-5.63.5.5 0 11-.5.865A5.472 5.472 0 0010 4.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var check = Vue.extend({
    name: 'CheckIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-check",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.854 5.646a.5.5 0 010 .708l-7 7a.5.5 0 01-.708 0l-3.5-3.5a.5.5 0 11.708-.708L8.5 12.293l6.646-6.647a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var chevronCompactDown = Vue.extend({
    name: 'ChevronCompactDownIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-chevron-compact-down",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.553 8.776a.5.5 0 01.67-.223L10 11.44l5.776-2.888a.5.5 0 11.448.894l-6 3a.5.5 0 01-.448 0l-6-3a.5.5 0 01-.223-.67z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var chevronCompactLeft = Vue.extend({
    name: 'ChevronCompactLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-chevron-compact-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.224 3.553a.5.5 0 01.223.67L8.56 10l2.888 5.776a.5.5 0 11-.894.448l-3-6a.5.5 0 010-.448l3-6a.5.5 0 01.67-.223z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var chevronCompactRight = Vue.extend({
    name: 'ChevronCompactRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-chevron-compact-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.776 3.553a.5.5 0 01.671.223l3 6a.5.5 0 010 .448l-3 6a.5.5 0 11-.894-.448L11.44 10 8.553 4.224a.5.5 0 01.223-.671z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var chevronCompactUp = Vue.extend({
    name: 'ChevronCompactUpIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-chevron-compact-up",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.776 7.553a.5.5 0 01.448 0l6 3a.5.5 0 11-.448.894L10 8.56l-5.776 2.888a.5.5 0 11-.448-.894l6-3z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var chevronDown = Vue.extend({
    name: 'ChevronDownIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-chevron-down",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.646 6.646a.5.5 0 01.708 0L10 12.293l5.646-5.647a.5.5 0 01.708.708l-6 6a.5.5 0 01-.708 0l-6-6a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var chevronLeft = Vue.extend({
    name: 'ChevronLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-chevron-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.354 3.646a.5.5 0 010 .708L7.707 10l5.647 5.646a.5.5 0 01-.708.708l-6-6a.5.5 0 010-.708l6-6a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var chevronRight = Vue.extend({
    name: 'ChevronRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-chevron-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.646 3.646a.5.5 0 01.708 0l6 6a.5.5 0 010 .708l-6 6a.5.5 0 01-.708-.708L12.293 10 6.646 4.354a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var chevronUp = Vue.extend({
    name: 'ChevronUpIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-chevron-up",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.646 6.646a.5.5 0 01.708 0l6 6a.5.5 0 01-.708.708L10 7.707l-5.646 5.647a.5.5 0 01-.708-.708l6-6z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var circleFill = Vue.extend({
    name: 'CircleFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-circle-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("circle", {
        attrs: {
          cx: "10",
          cy: "10",
          r: "8"
        }
      })]);
    }
  });

  var circleHalf = Vue.extend({
    name: 'CircleHalfIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-circle-half",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 17V3a7 7 0 000 14zm0 1a8 8 0 100-16 8 8 0 000 16z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var circleSlash = Vue.extend({
    name: 'CircleSlashIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-circle-slash",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 1.5a8.5 8.5 0 100 17 8.5 8.5 0 000-17zM5.071 4.347a7.5 7.5 0 0110.582 10.582L5.071 4.347zm-.724.724a7.5 7.5 0 0010.582 10.582L4.347 5.071z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var circle = Vue.extend({
    name: 'CircleIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-circle",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 17a7 7 0 100-14 7 7 0 000 14zm0 1a8 8 0 100-16 8 8 0 000 16z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var clockFill = Vue.extend({
    name: 'ClockFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-clock-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M18 10a8 8 0 11-16 0 8 8 0 0116 0zM5.5 9.5h4v-5a.5.5 0 011 0V10a.5.5 0 01-.5.5H5.5a.5.5 0 010-1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var clock = Vue.extend({
    name: 'ClockIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-clock",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 17a7 7 0 100-14 7 7 0 000 14zm8-7a8 8 0 11-16 0 8 8 0 0116 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 4a.5.5 0 01.5.5V10a.5.5 0 01-.5.5H5.5a.5.5 0 010-1h4v-5A.5.5 0 0110 4z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var cloudDownload = Vue.extend({
    name: 'CloudDownloadIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-cloud-download",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M6.887 7.2l-.964-.165A2.5 2.5 0 105.5 12H8v1H5.5a3.5 3.5 0 11.59-6.95 5.002 5.002 0 119.804 1.98A2.501 2.501 0 0115.5 13H12v-1h3.5a1.5 1.5 0 00.237-2.981L14.7 8.854l.216-1.028a4 4 0 10-7.843-1.587l-.185.96z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7 14.5a.5.5 0 01.707 0L10 16.793l2.293-2.293a.5.5 0 11.707.707l-2.646 2.647a.5.5 0 01-.708 0L7 15.207a.5.5 0 010-.707z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 8a.5.5 0 01.5.5v8a.5.5 0 01-1 0v-8A.5.5 0 0110 8z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var cloudFill = Vue.extend({
    name: 'CloudFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-cloud-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.5 15a3.5 3.5 0 11.59-6.95 5.002 5.002 0 119.804 1.98A2.5 2.5 0 0115.5 15h-10z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var cloudUpload = Vue.extend({
    name: 'CloudUploadIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-cloud-upload",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M6.887 8.2l-.964-.165A2.5 2.5 0 105.5 13H8v1H5.5a3.5 3.5 0 11.59-6.95 5.002 5.002 0 119.804 1.98A2.501 2.501 0 0115.5 14H12v-1h3.5a1.5 1.5 0 00.237-2.982L14.7 9.854l.216-1.028a4 4 0 10-7.843-1.587l-.185.96z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7 10.854a.5.5 0 00.707 0L10 8.56l2.293 2.293a.5.5 0 00.707-.707L10.354 7.5a.5.5 0 00-.708 0L7 10.146a.5.5 0 000 .708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 8a.5.5 0 01.5.5v8a.5.5 0 01-1 0v-8A.5.5 0 0110 8z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var cloud = Vue.extend({
    name: 'CloudIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-cloud",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.887 9.2l-.964-.165A2.5 2.5 0 105.5 14h10a1.5 1.5 0 00.237-2.982l-1.038-.164.216-1.028a4 4 0 10-7.843-1.587l-.185.96zm9.084.341a5 5 0 00-9.88-1.492A3.5 3.5 0 105.5 15h9.999a2.5 2.5 0 00.394-4.968c.033-.16.06-.324.077-.49z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var codeSlash = Vue.extend({
    name: 'CodeSlashIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-code-slash",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.854 6.146a.5.5 0 010 .708L3.707 10l3.147 3.146a.5.5 0 01-.708.708l-3.5-3.5a.5.5 0 010-.708l3.5-3.5a.5.5 0 01.708 0zm6.292 0a.5.5 0 000 .708L16.293 10l-3.147 3.146a.5.5 0 00.708.708l3.5-3.5a.5.5 0 000-.708l-3.5-3.5a.5.5 0 00-.708 0zm-.999-3.124a.5.5 0 01.33.625l-4 13a.5.5 0 11-.955-.294l4-13a.5.5 0 01.625-.33z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var code = Vue.extend({
    name: 'CodeIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-code",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.854 6.146a.5.5 0 010 .708L4.707 10l3.147 3.146a.5.5 0 01-.708.708l-3.5-3.5a.5.5 0 010-.708l3.5-3.5a.5.5 0 01.708 0zm4.292 0a.5.5 0 000 .708L15.293 10l-3.147 3.146a.5.5 0 00.708.708l3.5-3.5a.5.5 0 000-.708l-3.5-3.5a.5.5 0 00-.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var columnsGutters = Vue.extend({
    name: 'ColumnsGuttersIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-columns-gutters",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8 3H3v3h5V3zM3 2a1 1 0 00-1 1v3a1 1 0 001 1h5a1 1 0 001-1V3a1 1 0 00-1-1H3zm14 12h-5v3h5v-3zm-5-1a1 1 0 00-1 1v3a1 1 0 001 1h5a1 1 0 001-1v-3a1 1 0 00-1-1h-5zm-4-3H3v7h5v-7zM3 9a1 1 0 00-1 1v7a1 1 0 001 1h5a1 1 0 001-1v-7a1 1 0 00-1-1H3zm14-6h-5v7h5V3zm-5-1a1 1 0 00-1 1v7a1 1 0 001 1h5a1 1 0 001-1V3a1 1 0 00-1-1h-5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var columns = Vue.extend({
    name: 'ColumnsIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-columns",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17 4H3v12h14V4zM3 3a1 1 0 00-1 1v12a1 1 0 001 1h14a1 1 0 001-1V4a1 1 0 00-1-1H3z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.5 16V4h1v12h-1zm0-8H3V7h6.5v1zm7.5 5h-6.5v-1H17v1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var command = Vue.extend({
    name: 'CommandIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-command",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 5.5A1.5 1.5 0 005.5 7H7V5.5a1.5 1.5 0 10-3 0zM8 8V5.5A2.5 2.5 0 105.5 8H8zm8-2.5A1.5 1.5 0 0114.5 7H13V5.5a1.5 1.5 0 013 0zM12 8V5.5A2.5 2.5 0 1114.5 8H12zm-8 6.5A1.5 1.5 0 015.5 13H7v1.5a1.5 1.5 0 01-3 0zM8 12v2.5A2.5 2.5 0 115.5 12H8zm8 2.5a1.5 1.5 0 00-1.5-1.5H13v1.5a1.5 1.5 0 003 0zM12 12v2.5a2.5 2.5 0 102.5-2.5H12z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12 8H8v4h4V8zM7 7v6h6V7H7z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var compass = Vue.extend({
    name: 'CompassIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-compass",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 17.016a6.5 6.5 0 100-13 6.5 6.5 0 000 13zm0 1a7.5 7.5 0 100-15 7.5 7.5 0 000 15z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "4",
          height: "2",
          x: "8",
          y: "2",
          rx: "1"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M8.94 9.44l4.95-2.83-2.83 4.95-4.95 2.83 2.83-4.95z"
        }
      })]);
    }
  });

  var coneStriped = Vue.extend({
    name: 'ConeStripedIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-cone-striped",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.879 13.015a.5.5 0 01.242 0l6 1.5a.5.5 0 01.037.96l-6 2a.499.499 0 01-.316 0l-6-2a.5.5 0 01.037-.96l6-1.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M13.885 14.538l-.72-2.877c-.862.212-1.964.339-3.165.339s-2.303-.127-3.165-.339l-.72 2.877c-.073.292-.002.6.256.756.49.295 1.545.706 3.629.706s3.14-.411 3.63-.706c.257-.155.328-.464.255-.756zM11.97 6.88l.953 3.811C12.159 10.878 11.14 11 10 11c-1.14 0-2.159-.122-2.923-.309L8.03 6.88C8.635 6.957 9.3 7 10 7s1.365-.043 1.97-.12zm-.245-.978L10.97 2.88c-.252-1.01-1.688-1.01-1.94 0L8.275 5.9C8.8 5.965 9.382 6 10 6c.618 0 1.2-.036 1.725-.098z"
        }
      })]);
    }
  });

  var cone = Vue.extend({
    name: 'ConeIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-cone",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M9.03 3.88c.252-1.01 1.688-1.01 1.94 0L14 16H6L9.03 3.88z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.5 16a.5.5 0 01.5-.5h12a.5.5 0 010 1H4a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var controller = Vue.extend({
    name: 'ControllerIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-controller",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.119 4.693c.904.19 1.75.495 2.235.98.407.408.779 1.05 1.094 1.772.32.733.599 1.591.805 2.466.206.875.34 1.78.364 2.606.024.815-.059 1.602-.328 2.21a1.42 1.42 0 01-1.445.83c-.636-.067-1.115-.394-1.513-.773a11.307 11.307 0 01-.739-.809c-.126-.147-.25-.291-.368-.422-.728-.804-1.597-1.527-3.224-1.527-1.627 0-2.496.723-3.224 1.527-.119.131-.242.275-.368.422-.243.283-.494.576-.739.81-.398.378-.877.705-1.513.772a1.42 1.42 0 01-1.445-.83c-.27-.608-.352-1.395-.329-2.21.024-.826.16-1.73.365-2.606.206-.875.486-1.733.805-2.466.315-.722.687-1.364 1.094-1.772.486-.485 1.331-.79 2.235-.98.932-.196 2.03-.292 3.119-.292 1.089 0 2.187.096 3.119.292zm-6.032.979c-.877.185-1.469.443-1.733.708-.276.276-.587.783-.885 1.465a13.748 13.748 0 00-.748 2.295 12.35 12.35 0 00-.339 2.406c-.022.755.062 1.368.243 1.776a.42.42 0 00.426.24c.327-.034.61-.199.929-.502.212-.202.4-.423.615-.674.133-.156.276-.323.44-.505.826-.912 1.943-1.854 3.965-1.854s3.139.942 3.965 1.854c.164.182.307.35.44.505.214.25.403.472.615.674.318.303.601.468.929.503a.42.42 0 00.426-.241c.18-.408.265-1.02.243-1.776a12.353 12.353 0 00-.339-2.406 13.753 13.753 0 00-.748-2.295c-.298-.682-.61-1.19-.885-1.465-.264-.265-.856-.523-1.733-.708-.85-.179-1.877-.27-2.913-.27-1.036 0-2.063.091-2.913.27z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M13.5 8.026a.5.5 0 11-1 0 .5.5 0 011 0zm-1 1a.5.5 0 11-1 0 .5.5 0 011 0zm2 0a.5.5 0 11-1 0 .5.5 0 011 0zm-1 1.001a.5.5 0 11-1 0 .5.5 0 011 0zm-7-2.501h1v3h-1v-3z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M5.5 8.526h3v1h-3v-1zM5.051 5.26a.5.5 0 01.354-.613l1.932-.518a.5.5 0 01.258.966l-1.932.518a.5.5 0 01-.612-.354zm9.976 0a.5.5 0 00-.353-.613l-1.932-.518a.5.5 0 10-.259.966l1.932.518a.5.5 0 00.612-.354z"
        }
      })]);
    }
  });

  var creditCard = Vue.extend({
    name: 'CreditCardIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-credit-card",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 5H4a1 1 0 00-1 1v8a1 1 0 001 1h12a1 1 0 001-1V6a1 1 0 00-1-1zM4 4a2 2 0 00-2 2v8a2 2 0 002 2h12a2 2 0 002-2V6a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "3",
          height: "3",
          x: "4",
          y: "11",
          rx: "1"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M3 7h14v2H3z"
        }
      })]);
    }
  });

  var cursorFill = Vue.extend({
    name: 'CursorFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-cursor-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16.081 4.182a.5.5 0 01.104.557l-5.657 12.727a.5.5 0 01-.917-.006L7.57 12.694l-4.766-2.042a.5.5 0 01-.006-.917L15.525 4.08a.5.5 0 01.556.103z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var cursor = Vue.extend({
    name: 'CursorIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-cursor",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16.081 4.182a.5.5 0 01.104.557l-5.657 12.727a.5.5 0 01-.917-.006L7.57 12.694l-4.766-2.042a.5.5 0 01-.006-.917L15.525 4.08a.5.5 0 01.556.103zM4.25 10.184l3.897 1.67a.5.5 0 01.262.263l1.67 3.897L14.743 5.52 4.25 10.184z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var dash = Vue.extend({
    name: 'DashIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-dash",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.5 10a.5.5 0 01.5-.5h8a.5.5 0 010 1H6a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var diamondHalf = Vue.extend({
    name: 'DiamondHalfIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-diamond-half",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.94 2.354a1.5 1.5 0 012.12 0l6.586 6.585a1.5 1.5 0 010 2.122l-6.585 6.585a1.5 1.5 0 01-2.122 0l-6.585-6.585a1.5 1.5 0 010-2.122l6.585-6.585zm1.06.56a.498.498 0 00-.354.147L3.061 9.646a.5.5 0 000 .707l6.585 6.586a.499.499 0 00.354.147V2.914z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var diamond = Vue.extend({
    name: 'DiamondIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-diamond",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.1 2.7a.5.5 0 01.4-.2h9a.5.5 0 01.4.2l2.976 3.974c.149.185.156.45.01.644L10.4 17.3a.5.5 0 01-.8 0l-7.5-10a.5.5 0 010-.6l3-4zm11.386 3.785l-1.806-2.41-.776 2.413 2.582-.003zm-3.633.004l.961-2.989H6.186l.963 2.995 5.704-.006zM7.47 7.495l5.062-.005L10 15.366 7.47 7.495zm-1.371-.999l-.78-2.422-1.818 2.425 2.598-.003zM3.499 7.5l2.92-.003 2.193 6.82L3.5 7.5zm7.889 6.817l2.194-6.828 2.929-.003-5.123 6.831z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var displayFill = Vue.extend({
    name: 'DisplayFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-display-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M7.75 15.5c.167-.333.25-.833.25-1.5h4c0 .667.083 1.167.25 1.5H13a.5.5 0 010 1H7a.5.5 0 010-1h.75z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.991 5H4c-.325 0-.502.078-.602.145a.758.758 0 00-.254.302A1.46 1.46 0 003 6.01V12c0 .325.078.502.145.602.07.105.17.188.302.254a1.464 1.464 0 00.538.143L4.01 13H16c.325 0 .502-.078.602-.145a.758.758 0 00.254-.302 1.464 1.464 0 00.143-.538L17 11.99V6c0-.325-.078-.502-.145-.602a.757.757 0 00-.302-.254A1.46 1.46 0 0015.99 5zM16 4H4C2 4 2 6 2 6v6c0 2 2 2 2 2h12c2 0 2-2 2-2V6c0-2-2-2-2-2z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M4 6h12v6H4z"
        }
      })]);
    }
  });

  var display = Vue.extend({
    name: 'DisplayIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-display",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M7.75 15.5c.167-.333.25-.833.25-1.5h4c0 .667.083 1.167.25 1.5H13a.5.5 0 010 1H7a.5.5 0 010-1h.75z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.991 5H4c-.325 0-.502.078-.602.145a.758.758 0 00-.254.302A1.46 1.46 0 003 6.01V12c0 .325.078.502.145.602.07.105.17.188.302.254a1.464 1.464 0 00.538.143L4.01 13H16c.325 0 .502-.078.602-.145a.758.758 0 00.254-.302 1.464 1.464 0 00.143-.538L17 11.99V6c0-.325-.078-.502-.145-.602a.757.757 0 00-.302-.254A1.46 1.46 0 0015.99 5zM16 4H4C2 4 2 6 2 6v6c0 2 2 2 2 2h12c2 0 2-2 2-2V6c0-2-2-2-2-2z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var documentCode = Vue.extend({
    name: 'DocumentCodeIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-document-code",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6 3h8a2 2 0 012 2v10a2 2 0 01-2 2H6a2 2 0 01-2-2V5a2 2 0 012-2zm0 1a1 1 0 00-1 1v10a1 1 0 001 1h8a1 1 0 001-1V5a1 1 0 00-1-1H6z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.646 7.646a.5.5 0 01.708 0l2 2a.5.5 0 010 .708l-2 2a.5.5 0 01-.708-.708L12.293 10l-1.647-1.646a.5.5 0 010-.708zm-1.292 0a.5.5 0 00-.708 0l-2 2a.5.5 0 000 .708l2 2a.5.5 0 00.708-.708L7.707 10l1.647-1.646a.5.5 0 000-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var documentDiff = Vue.extend({
    name: 'DocumentDiffIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-document-diff",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6 3h8a2 2 0 012 2v10a2 2 0 01-2 2H6a2 2 0 01-2-2V5a2 2 0 012-2zm0 1a1 1 0 00-1 1v10a1 1 0 001 1h8a1 1 0 001-1V5a1 1 0 00-1-1H6z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.5 13a.5.5 0 01.5-.5h4a.5.5 0 010 1H8a.5.5 0 01-.5-.5zM10 6.5a.5.5 0 01.5.5v4a.5.5 0 01-1 0V7a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.5 9a.5.5 0 01.5-.5h4a.5.5 0 010 1H8a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var documentRichtext = Vue.extend({
    name: 'DocumentRichtextIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-document-richtext",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6 3h8a2 2 0 012 2v10a2 2 0 01-2 2H6a2 2 0 01-2-2V5a2 2 0 012-2zm0 1a1 1 0 00-1 1v10a1 1 0 001 1h8a1 1 0 001-1V5a1 1 0 00-1-1H6z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.5 14a.5.5 0 01.5-.5h3a.5.5 0 010 1H7a.5.5 0 01-.5-.5zm0-2a.5.5 0 01.5-.5h6a.5.5 0 010 1H7a.5.5 0 01-.5-.5zm1.639-3.958l1.33.886 1.854-1.855a.25.25 0 01.289-.047L13.5 8v1.75a.5.5 0 01-.5.5H7a.5.5 0 01-.5-.5v-.5s1.54-1.274 1.639-1.208zM8.25 7a.75.75 0 100-1.5.75.75 0 000 1.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var documentSpreadsheet = Vue.extend({
    name: 'DocumentSpreadsheetIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-document-spreadsheet",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6 3h8a2 2 0 012 2v10a2 2 0 01-2 2H6a2 2 0 01-2-2V5a2 2 0 012-2zm0 1a1 1 0 00-1 1v10a1 1 0 001 1h8a1 1 0 001-1V5a1 1 0 00-1-1H6z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15 8H5V7h10v1zm0 3H5v-1h10v1zm0 3H5v-1h10v1z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7 16V8h1v8H7zm4 0V8h1v8h-1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var documentText = Vue.extend({
    name: 'DocumentTextIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-document-text",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6 3h8a2 2 0 012 2v10a2 2 0 01-2 2H6a2 2 0 01-2-2V5a2 2 0 012-2zm0 1a1 1 0 00-1 1v10a1 1 0 001 1h8a1 1 0 001-1V5a1 1 0 00-1-1H6z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.5 14a.5.5 0 01.5-.5h3a.5.5 0 010 1H7a.5.5 0 01-.5-.5zm0-2a.5.5 0 01.5-.5h6a.5.5 0 010 1H7a.5.5 0 01-.5-.5zm0-2a.5.5 0 01.5-.5h6a.5.5 0 010 1H7a.5.5 0 01-.5-.5zm0-2a.5.5 0 01.5-.5h6a.5.5 0 010 1H7a.5.5 0 01-.5-.5zm0-2a.5.5 0 01.5-.5h6a.5.5 0 010 1H7a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var document$1 = Vue.extend({
    name: 'DocumentIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-document",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6 3h8a2 2 0 012 2v10a2 2 0 01-2 2H6a2 2 0 01-2-2V5a2 2 0 012-2zm0 1a1 1 0 00-1 1v10a1 1 0 001 1h8a1 1 0 001-1V5a1 1 0 00-1-1H6z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var documentsAlt = Vue.extend({
    name: 'DocumentsAltIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-documents-alt",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5 3h8a2 2 0 012 2v10a2 2 0 01-2 2H5a2 2 0 01-2-2V5a2 2 0 012-2zm0 1a1 1 0 00-1 1v10a1 1 0 001 1h8a1 1 0 001-1V5a1 1 0 00-1-1H5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M15 6V5a2 2 0 012 2v6a2 2 0 01-2 2v-1a1 1 0 001-1V7a1 1 0 00-1-1z"
        }
      })]);
    }
  });

  var documents = Vue.extend({
    name: 'DocumentsIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-documents",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5 4h8a2 2 0 012 2v10a2 2 0 01-2 2H5a2 2 0 01-2-2V6a2 2 0 012-2zm0 1a1 1 0 00-1 1v10a1 1 0 001 1h8a1 1 0 001-1V6a1 1 0 00-1-1H5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M7 2h8a2 2 0 012 2v10a2 2 0 01-2 2v-1a1 1 0 001-1V4a1 1 0 00-1-1H7a1 1 0 00-1 1H5a2 2 0 012-2z"
        }
      })]);
    }
  });

  var dot = Vue.extend({
    name: 'DotIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-dot",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 11.5a1.5 1.5 0 100-3 1.5 1.5 0 000 3z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var download = Vue.extend({
    name: 'DownloadIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-download",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.5 10a.5.5 0 01.5.5V14a1 1 0 001 1h12a1 1 0 001-1v-3.5a.5.5 0 011 0V14a2 2 0 01-2 2H4a2 2 0 01-2-2v-3.5a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7 9.5a.5.5 0 01.707 0L10 11.793 12.293 9.5a.5.5 0 01.707.707l-2.646 2.647a.5.5 0 01-.708 0L7 10.207A.5.5 0 017 9.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 3a.5.5 0 01.5.5v8a.5.5 0 01-1 0v-8A.5.5 0 0110 3z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var eggFried = Vue.extend({
    name: 'EggFriedIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-egg-fried",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.665 8.113a1 1 0 01-.667-.977L15 7a4 4 0 00-6.483-3.136 1 1 0 01-.8.2 4 4 0 00-3.693 6.61 1 1 0 01.2 1 4 4 0 006.67 4.087 1 1 0 011.262-.152 2.5 2.5 0 003.715-2.905 1 1 0 01.341-1.113 2.001 2.001 0 00-.547-3.478zM16 7c0 .057 0 .113-.003.17a3.001 3.001 0 01.822 5.216 3.5 3.5 0 01-5.201 4.065 5 5 0 01-8.336-5.109A5 5 0 017.896 3.08 5 5 0 0116 7z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "10",
          cy: "10",
          r: "3"
        }
      })]);
    }
  });

  var ejectFill = Vue.extend({
    name: 'EjectFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-eject-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.27 3.047a1 1 0 011.46 0l6.345 6.77c.6.638.146 1.683-.73 1.683H3.656c-.876 0-1.33-1.045-.73-1.684L9.27 3.047zM2.5 13.5a1 1 0 011-1h13a1 1 0 011 1v1a1 1 0 01-1 1h-13a1 1 0 01-1-1v-1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var eject = Vue.extend({
    name: 'EjectIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-eject",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.27 3.047a1 1 0 011.46 0l6.345 6.77c.6.638.146 1.683-.73 1.683H3.656c-.876 0-1.33-1.045-.73-1.684L9.27 3.047zm7.076 7.453L10 3.731 3.654 10.5h12.692zM2.5 13.5a1 1 0 011-1h13a1 1 0 011 1v1a1 1 0 01-1 1h-13a1 1 0 01-1-1v-1zm14 0h-13v1h13v-1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var envelopeFill = Vue.extend({
    name: 'EnvelopeFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-envelope-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M2.05 5.555L10 10.414l7.95-4.859A2 2 0 0016 4H4a2 2 0 00-1.95 1.555zM18 6.697l-5.875 3.59L18 13.743V6.697zm-.168 8.108l-6.675-3.926-1.157.707-1.157-.707-6.675 3.926A2 2 0 004 16h12a2 2 0 001.832-1.195zM2 13.743l5.875-3.456L2 6.697v7.046z"
        }
      })]);
    }
  });

  var envelopeOpenFill = Vue.extend({
    name: 'EnvelopeOpenFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-envelope-open-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.941 2.435a2 2 0 00-1.882 0l-6 3.2A2 2 0 002 7.4v.125l8 4.889 8-4.889V7.4a2 2 0 00-1.059-1.765l-6-3.2zM18 8.697l-5.875 3.59L18 15.743V8.697zm-.168 8.108l-6.586-3.874-.088-.052-.897.548-.261.159-.26-.16-.897-.547-.09.052-6.585 3.874A2 2 0 004 18h12a2 2 0 001.832-1.195zM2 15.743l5.875-3.456L2 8.697v7.046z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var envelopeOpen = Vue.extend({
    name: 'EnvelopeOpenIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-envelope-open",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.243 8.929l.514-.858L10 12.417l7.243-4.346.514.858L10 13.583 2.243 8.93z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.184 12.68l-6.432 3.752-.504-.864 6.432-3.752.504.864zm1.632 0l6.432 3.752.504-.864-6.432-3.752-.504.864z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.47 3.318a1 1 0 00-.94 0l-6 3.2A1 1 0 003 7.4V16a1 1 0 001 1h12a1 1 0 001-1V7.4a1 1 0 00-.53-.882l-6-3.2zm-1.41-.883a2 2 0 011.882 0l6 3.2A2 2 0 0118 7.4V16a2 2 0 01-2 2H4a2 2 0 01-2-2V7.4a2 2 0 011.059-1.765l6-3.2z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var envelope = Vue.extend({
    name: 'EnvelopeIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-envelope",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 5H4a1 1 0 00-1 1v8a1 1 0 001 1h12a1 1 0 001-1V6a1 1 0 00-1-1zM4 4a2 2 0 00-2 2v8a2 2 0 002 2h12a2 2 0 002-2V6a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.071 6.243a.5.5 0 01.686-.172L10 10.417l7.243-4.346a.5.5 0 11.514.858L10 11.583 2.243 6.93a.5.5 0 01-.172-.686z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var eyeFill = Vue.extend({
    name: 'EyeFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-eye-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M12.5 10a2.5 2.5 0 11-5 0 2.5 2.5 0 015 0z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2 10s3-5.5 8-5.5 8 5.5 8 5.5-3 5.5-8 5.5S2 10 2 10zm8 3.5a3.5 3.5 0 100-7 3.5 3.5 0 000 7z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var eyeSlashFill = Vue.extend({
    name: 'EyeSlashFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-eye-slash-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M12.79 14.912l-1.614-1.615a3.5 3.5 0 01-4.474-4.474l-2.06-2.06C2.938 8.278 2 10 2 10s3 5.5 8 5.5a7.027 7.027 0 002.79-.588zM7.21 5.088A7.028 7.028 0 0110 4.5c5 0 8 5.5 8 5.5s-.939 1.72-2.641 3.238l-2.062-2.062a3.5 3.5 0 00-4.474-4.474L7.21 5.088z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M7.525 9.646a2.5 2.5 0 002.829 2.829l-2.83-2.829zm4.95.708l-2.829-2.83a2.5 2.5 0 012.829 2.829z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.646 16.354l-12-12 .708-.708 12 12-.708.707z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var eyeSlash = Vue.extend({
    name: 'EyeSlashIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-eye-slash",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M15.359 13.238C17.06 11.72 18 10 18 10s-3-5.5-8-5.5a7.028 7.028 0 00-2.79.588l.77.771A5.944 5.944 0 0110 5.5c2.12 0 3.879 1.168 5.168 2.457A13.134 13.134 0 0116.828 10c-.058.087-.122.183-.195.288a13.14 13.14 0 01-1.465 1.755c-.165.165-.337.328-.517.486l.708.709z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M13.297 11.176a3.5 3.5 0 00-4.474-4.474l.823.823a2.5 2.5 0 012.829 2.829l.822.822zm-2.943 1.299l.822.822a3.5 3.5 0 01-4.474-4.474l.823.823a2.5 2.5 0 002.829 2.829z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M5.35 7.47c-.18.16-.353.322-.518.487A13.134 13.134 0 003.172 10l.195.288c.335.48.83 1.12 1.465 1.755C6.121 13.332 7.881 14.5 10 14.5c.716 0 1.39-.133 2.02-.36l.77.772A7.027 7.027 0 0110 15.5c-5 0-8-5.5-8-5.5s.939-1.721 2.641-3.238l.708.709z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.646 16.354l-12-12 .708-.708 12 12-.708.707z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var eye = Vue.extend({
    name: 'EyeIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-eye",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M18 10s-3-5.5-8-5.5S2 10 2 10s3 5.5 8 5.5 8-5.5 8-5.5zM3.173 10a13.133 13.133 0 001.66 2.043C6.12 13.332 7.88 14.5 10 14.5c2.12 0 3.879-1.168 5.168-2.457A13.133 13.133 0 0016.828 10a13.133 13.133 0 00-1.66-2.043C13.879 6.668 12.119 5.5 10 5.5c-2.12 0-3.879 1.168-5.168 2.457A13.133 13.133 0 003.172 10z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 7.5a2.5 2.5 0 100 5 2.5 2.5 0 000-5zM6.5 10a3.5 3.5 0 117 0 3.5 3.5 0 01-7 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var filter = Vue.extend({
    name: 'FilterIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-filter",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.5 13a.5.5 0 01.5-.5h4a.5.5 0 010 1H8a.5.5 0 01-.5-.5zm-2-3a.5.5 0 01.5-.5h8a.5.5 0 010 1H6a.5.5 0 01-.5-.5zm-2-3a.5.5 0 01.5-.5h12a.5.5 0 010 1H4a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var flagFill = Vue.extend({
    name: 'FlagFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-flag-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.5 3a.5.5 0 01.5.5v13a.5.5 0 01-1 0v-13a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.762 4.558C6.735 3.909 7.348 3.5 8.5 3.5c.653 0 1.139.325 1.495.562l.032.022c.392.26.646.416.973.416.168 0 .356-.042.587-.126.187-.068.376-.153.593-.25.058-.027.117-.053.18-.08.57-.255 1.278-.544 2.14-.544a.5.5 0 01.5.5v6a.5.5 0 01-.5.5c-.638 0-1.18.21-1.734.457l-.159.07c-.22.1-.453.205-.678.287A2.718 2.718 0 0111 11.5c-.653 0-1.139-.325-1.495-.563l-.032-.021c-.391-.26-.646-.416-.973-.416-.833 0-1.218.246-2.223.916A.5.5 0 015.5 11V5a.5.5 0 01.223-.416l.04-.026z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var flag = Vue.extend({
    name: 'FlagIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-flag",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.5 3a.5.5 0 01.5.5v13a.5.5 0 01-1 0v-13a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.762 4.558C6.735 3.909 7.348 3.5 8.5 3.5c.653 0 1.139.325 1.495.562l.032.022c.392.26.646.416.973.416.168 0 .356-.042.587-.126.187-.068.376-.153.593-.25.058-.027.117-.053.18-.08.57-.255 1.278-.544 2.14-.544a.5.5 0 01.5.5v6a.5.5 0 01-.5.5c-.638 0-1.18.21-1.734.457l-.159.07c-.22.1-.453.205-.678.287A2.718 2.718 0 0111 11.5c-.653 0-1.139-.325-1.495-.563l-.032-.021c-.391-.26-.646-.416-.973-.416-.833 0-1.218.246-2.223.916a.5.5 0 11-.515-.858C6.735 9.909 7.348 9.5 8.5 9.5c.653 0 1.139.325 1.495.563l.032.021c.392.26.646.416.973.416.168 0 .356-.042.587-.126.187-.068.376-.153.593-.25.058-.027.117-.053.18-.08.456-.204 1-.43 1.64-.512V4.543c-.433.074-.83.234-1.234.414l-.159.07c-.22.1-.453.205-.678.287A2.72 2.72 0 0111 5.5c-.653 0-1.139-.325-1.495-.562l-.032-.022c-.391-.26-.646-.416-.973-.416-.833 0-1.218.246-2.223.916a.5.5 0 01-.554-.832l.04-.026z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var folderFill = Vue.extend({
    name: 'FolderFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-folder-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.828 5h3.982a2 2 0 011.992 2.181l-.637 7A2 2 0 0115.174 16H4.826a2 2 0 01-1.991-1.819l-.637-7a1.99 1.99 0 01.342-1.31L2.5 5a2 2 0 012-2h3.672a2 2 0 011.414.586l.828.828A2 2 0 0011.828 5zm-8.322.12C3.72 5.042 3.95 5 4.19 5h5.396l-.707-.707A1 1 0 008.172 4H4.5a1 1 0 00-1 .981l.006.139z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var folderSymlinkFill = Vue.extend({
    name: 'FolderSymlinkFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-folder-symlink-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.81 5h-3.982a2 2 0 01-1.414-.586l-.828-.828A2 2 0 008.172 3H4.5a2 2 0 00-2 2l.04.87a1.99 1.99 0 00-.342 1.311l.637 7A2 2 0 004.826 16h10.348a2 2 0 001.991-1.819l.637-7A2 2 0 0015.81 5zM4.19 5c-.24 0-.47.042-.684.12L3.5 4.98a1 1 0 011-.98h3.672a1 1 0 01.707.293L9.586 5H4.19zm9.608 5.271l-3.182 1.97c-.27.166-.616-.036-.616-.372V11.1s-2.571-.3-4 2.4c.571-4.8 3.143-4.8 4-4.8v-.769c0-.336.346-.538.616-.371l3.182 1.969c.27.166.27.576 0 .742z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var folderSymlink = Vue.extend({
    name: 'FolderSymlinkIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-folder-symlink",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M11.828 6a3 3 0 01-2.12-.879l-.83-.828A1 1 0 008.173 4H4.5a1 1 0 00-1 .981L3.546 6h-1L2.5 5a2 2 0 012-2h3.672a2 2 0 011.414.586l.828.828A2 2 0 0011.828 5v1z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.81 6H4.19a1 1 0 00-.996 1.09l.637 7a1 1 0 00.995.91h10.348a1 1 0 00.995-.91l.637-7A1 1 0 0015.81 6zM4.19 5a2 2 0 00-1.992 2.181l.637 7A2 2 0 004.826 16h10.348a2 2 0 001.991-1.819l.637-7A2 2 0 0015.81 5H4.19z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M10.616 12.24l3.182-1.969a.442.442 0 000-.742l-3.182-1.97c-.27-.166-.616.036-.616.372V8.7c-.857 0-3.429 0-4 4.8 1.429-2.7 4-2.4 4-2.4v.769c0 .336.346.538.616.371z"
        }
      })]);
    }
  });

  var folder = Vue.extend({
    name: 'FolderIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-folder",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M11.828 6a3 3 0 01-2.12-.879l-.83-.828A1 1 0 008.173 4H4.5a1 1 0 00-1 .981L3.546 6h-1L2.5 5a2 2 0 012-2h3.672a2 2 0 011.414.586l.828.828A2 2 0 0011.828 5v1z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.81 6H4.19a1 1 0 00-.996 1.09l.637 7a1 1 0 00.995.91h10.348a1 1 0 00.995-.91l.637-7A1 1 0 0015.81 6zM4.19 5a2 2 0 00-1.992 2.181l.637 7A2 2 0 004.826 16h10.348a2 2 0 001.991-1.819l.637-7A2 2 0 0015.81 5H4.19z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var fonts = Vue.extend({
    name: 'FontsIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-fonts",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M14.258 5H5.747l-.082 2.46h.479c.26-1.544.758-1.783 2.693-1.845l.424-.013v7.827c0 .663-.144.82-1.3.923v.52h4.082v-.52c-1.162-.103-1.306-.26-1.306-.923V5.602l.43.013c1.935.062 2.434.301 2.694 1.846h.479L14.258 5z"
        }
      })]);
    }
  });

  var forwardFill = Vue.extend({
    name: 'ForwardFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-forward-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M11.77 14.11l4.012-2.953a.647.647 0 000-1.114L11.771 7.09a.644.644 0 00-.971.557V8.65H4v3.9h6.8v1.003c0 .505.545.808.97.557z"
        }
      })]);
    }
  });

  var forward = Vue.extend({
    name: 'ForwardIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-forward",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.502 7.513a.144.144 0 00-.202.134V8.65a.5.5 0 01-.5.5H4.5v2.9h6.3a.5.5 0 01.5.5v1.003c0 .108.11.176.202.134l3.984-2.933a.522.522 0 01.042-.028.147.147 0 000-.252.523.523 0 01-.042-.028l-3.984-2.933zm-1.202.134a1.144 1.144 0 011.767-.96l3.994 2.94a1.147 1.147 0 010 1.946l-3.994 2.94a1.144 1.144 0 01-1.767-.96v-.503H4a.5.5 0 01-.5-.5v-3.9a.5.5 0 01.5-.5h6.3v-.503z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var gearFill = Vue.extend({
    name: 'GearFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-gear-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.405 3.05c-.413-1.4-2.397-1.4-2.81 0l-.1.34a1.464 1.464 0 01-2.105.872l-.31-.17c-1.283-.698-2.686.705-1.987 1.987l.169.311c.446.82.023 1.841-.872 2.105l-.34.1c-1.4.413-1.4 2.397 0 2.81l.34.1a1.464 1.464 0 01.872 2.105l-.17.31c-.698 1.283.705 2.686 1.987 1.987l.311-.169a1.464 1.464 0 012.105.872l.1.34c.413 1.4 2.397 1.4 2.81 0l.1-.34a1.464 1.464 0 012.105-.872l.31.17c1.283.698 2.686-.705 1.987-1.987l-.169-.311a1.464 1.464 0 01.872-2.105l.34-.1c1.4-.413 1.4-2.397 0-2.81l-.34-.1a1.464 1.464 0 01-.872-2.105l.17-.31c.698-1.283-.705-2.686-1.987-1.987l-.311.169a1.464 1.464 0 01-2.105-.872l-.1-.34zM10 12.93a2.929 2.929 0 100-5.858 2.929 2.929 0 000 5.858z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var gearWideConnected = Vue.extend({
    name: 'GearWideConnectedIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-gear-wide-connected",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.932 2.724c-.243-.97-1.621-.97-1.864 0l-.072.286a.96.96 0 01-1.622.434l-.205-.211c-.695-.72-1.889-.03-1.614.932l.08.283a.96.96 0 01-1.187 1.188l-.283-.081c-.962-.275-1.651.919-.932 1.614l.211.205a.96.96 0 01-.434 1.622l-.286.072c-.97.243-.97 1.621 0 1.864l.286.072a.96.96 0 01.434 1.622l-.211.205c-.72.695-.03 1.889.932 1.614l.283-.08a.96.96 0 011.188 1.187l-.081.283c-.275.962.919 1.651 1.614.932l.205-.211a.96.96 0 011.622.434l.072.286c.243.97 1.621.97 1.864 0l.072-.286a.96.96 0 011.622-.434l.205.211c.695.72 1.889.03 1.614-.932l-.08-.283a.96.96 0 011.187-1.188l.283.081c.962.275 1.651-.919.932-1.614l-.211-.205a.96.96 0 01.434-1.622l.286-.072c.97-.243.97-1.621 0-1.864l-.286-.072a.96.96 0 01-.434-1.622l.211-.205c.72-.695.03-1.889-.932-1.614l-.283.08a.96.96 0 01-1.188-1.187l.081-.283c.275-.962-.919-1.651-1.614-.932l-.205.211a.96.96 0 01-1.622-.434l-.072-.286zM10 15a5 5 0 100-10 5 5 0 000 10z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.375 10L6.6 6.3l.8-.6 2.85 3.8H15v1h-4.75L7.4 14.3l-.8-.6L9.375 10z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var gearWide = Vue.extend({
    name: 'GearWideIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-gear-wide",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.932 2.724c-.243-.97-1.621-.97-1.864 0l-.072.286a.96.96 0 01-1.622.434l-.205-.211c-.695-.72-1.889-.03-1.614.932l.08.283a.96.96 0 01-1.187 1.188l-.283-.081c-.962-.275-1.651.919-.932 1.614l.211.205a.96.96 0 01-.434 1.622l-.286.072c-.97.243-.97 1.621 0 1.864l.286.072a.96.96 0 01.434 1.622l-.211.205c-.72.695-.03 1.889.932 1.614l.283-.08a.96.96 0 011.188 1.187l-.081.283c-.275.962.919 1.651 1.614.932l.205-.211a.96.96 0 011.622.434l.072.286c.243.97 1.621.97 1.864 0l.072-.286a.96.96 0 011.622-.434l.205.211c.695.72 1.889.03 1.614-.932l-.08-.283a.96.96 0 011.187-1.188l.283.081c.962.275 1.651-.919.932-1.614l-.211-.205a.96.96 0 01.434-1.622l.286-.072c.97-.243.97-1.621 0-1.864l-.286-.072a.96.96 0 01-.434-1.622l.211-.205c.72-.695.03-1.889-.932-1.614l-.283.08a.96.96 0 01-1.188-1.187l.081-.283c.275-.962-.919-1.651-1.614-.932l-.205.211a.96.96 0 01-1.622-.434l-.072-.286zM10 15a5 5 0 100-10 5 5 0 000 10z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var gear = Vue.extend({
    name: 'GearIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-gear",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.837 3.626c-.246-.835-1.428-.835-1.674 0l-.094.319A1.873 1.873 0 016.377 5.06l-.292-.16c-.764-.415-1.6.42-1.184 1.185l.159.292a1.873 1.873 0 01-1.115 2.692l-.319.094c-.835.246-.835 1.428 0 1.674l.319.094a1.873 1.873 0 011.115 2.693l-.16.291c-.415.764.42 1.6 1.185 1.184l.292-.159a1.873 1.873 0 012.692 1.115l.094.319c.246.835 1.428.835 1.674 0l.094-.319a1.873 1.873 0 012.693-1.115l.291.16c.764.415 1.6-.42 1.184-1.185l-.159-.291a1.873 1.873 0 011.115-2.693l.319-.094c.835-.246.835-1.428 0-1.674l-.319-.094a1.873 1.873 0 01-1.115-2.692l.16-.292c.415-.764-.42-1.6-1.185-1.184l-.291.159a1.873 1.873 0 01-2.693-1.115l-.094-.319zm-2.633-.283c.527-1.79 3.064-1.79 3.592 0l.094.319a.873.873 0 001.255.52l.292-.16c1.64-.892 3.434.901 2.54 2.541l-.159.292a.873.873 0 00.52 1.255l.319.094c1.79.527 1.79 3.064 0 3.592l-.319.094a.873.873 0 00-.52 1.255l.16.292c.893 1.64-.902 3.434-2.541 2.54l-.292-.159a.873.873 0 00-1.255.52l-.094.319c-.527 1.79-3.065 1.79-3.592 0l-.094-.319a.873.873 0 00-1.255-.52l-.292.16c-1.64.893-3.433-.902-2.54-2.541l.159-.292a.873.873 0 00-.52-1.255l-.319-.094c-1.79-.527-1.79-3.065 0-3.592l.319-.094a.873.873 0 00.52-1.255l-.16-.292c-.892-1.64.901-3.433 2.541-2.54l.292.159a.873.873 0 001.255-.52l.094-.319z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 7.754a2.246 2.246 0 100 4.492 2.246 2.246 0 000-4.492zM6.754 10a3.246 3.246 0 116.492 0 3.246 3.246 0 01-6.492 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var geo = Vue.extend({
    name: 'GeoIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-geo",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M13 6a3 3 0 11-6 0 3 3 0 016 0z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9.5 6h1v9a.5.5 0 01-1 0V6z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.489 14.095a.5.5 0 01-.383.594c-.565.123-1.003.292-1.286.472-.302.192-.32.321-.32.339 0 .013.005.085.146.21.14.124.372.26.701.383.655.245 1.593.407 2.653.407s1.998-.162 2.653-.407c.329-.124.56-.259.701-.383.14-.125.146-.197.146-.21 0-.018-.018-.147-.32-.339-.283-.18-.721-.35-1.286-.472a.5.5 0 11.212-.977c.63.137 1.193.34 1.61.606.4.253.784.645.784 1.182 0 .402-.219.724-.483.958-.264.235-.618.423-1.013.57-.793.298-1.855.472-3.004.472s-2.21-.174-3.004-.471c-.395-.148-.749-.337-1.013-.571-.264-.234-.483-.556-.483-.958 0-.537.384-.929.783-1.182.418-.266.98-.47 1.611-.606a.5.5 0 01.595.383z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var graphDown = Vue.extend({
    name: 'GraphDownIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-graph-down",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M2 2h1v16H2V2zm1 15h15v1H3v-1z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16.39 11.041l-4.349-5.436L9 8.646 5.354 5l-.708.707L9 10.061l2.959-2.959 3.65 4.564.781-.625z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12 11.854a.5.5 0 00.5.5h4a.5.5 0 00.5-.5v-4a.5.5 0 00-1 0v3.5h-3.5a.5.5 0 00-.5.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var graphUp = Vue.extend({
    name: 'GraphUpIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-graph-up",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M2 2h1v16H2V2zm1 15h15v1H3v-1z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16.39 6.312l-4.349 5.437L9 8.707l-3.646 3.647-.708-.708L9 7.293l2.959 2.958 3.65-4.563.781.624z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12 5.5a.5.5 0 01.5-.5h4a.5.5 0 01.5.5v4a.5.5 0 01-1 0V6h-3.5a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var gridFill = Vue.extend({
    name: 'GridFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-grid-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("rect", {
        attrs: {
          width: "6",
          height: "6",
          x: "3.5",
          y: "10.5",
          rx: "1"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "6",
          height: "6",
          x: "10.5",
          y: "10.5",
          rx: "1"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "6",
          height: "6",
          x: "10.5",
          y: "3.5",
          rx: "1"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "6",
          height: "6",
          x: "3.5",
          y: "3.5",
          rx: "1"
        }
      })]);
    }
  });

  var grid = Vue.extend({
    name: 'GridIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-grid",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.5 4.5a1 1 0 00-1-1h-4a1 1 0 00-1 1v4a1 1 0 001 1h4a1 1 0 001-1v-4zm-1 7h-4v4h4v-4zm7 0h-4v4h4v-4zm0-7h-4v4h4v-4zm-7 0h-4v4h4v-4zm2 0a1 1 0 011-1h4a1 1 0 011 1v4a1 1 0 01-1 1h-4a1 1 0 01-1-1v-4zm-6 6a1 1 0 00-1 1v4a1 1 0 001 1h4a1 1 0 001-1v-4a1 1 0 00-1-1h-4zm7 0a1 1 0 00-1 1v4a1 1 0 001 1h4a1 1 0 001-1v-4a1 1 0 00-1-1h-4z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var hammer = Vue.extend({
    name: 'HammerIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-hammer",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M11.812 3.952a.5.5 0 01-.312.89c-1.671 0-2.852.596-3.616 1.185L6.857 7.073V8.21a.5.5 0 01-.146.354L5.426 9.853a.5.5 0 01-.709 0L2.146 7.274a.5.5 0 010-.706l1.286-1.29a.5.5 0 01.354-.146H4.84c1.664-1.904 3.375-2.27 4.716-2.091a5.008 5.008 0 012.076.782l.18.129z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.012 5.5a.5.5 0 01.359.165l9.146 8.646A.5.5 0 0117.5 15L16 16.5a.5.5 0 01-.756-.056L6.598 7.297a.5.5 0 01.048-.65l1-1a.5.5 0 01.366-.147z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var hash = Vue.extend({
    name: 'HashIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-hash",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M10.39 14.648a1.32 1.32 0 00-.015.18c0 .305.21.508.5.508.266 0 .492-.172.555-.477l.554-2.703h1.204c.421 0 .617-.234.617-.547 0-.312-.188-.53-.617-.53h-.985l.516-2.524h1.265c.43 0 .618-.227.618-.547 0-.313-.188-.524-.618-.524h-1.046l.476-2.304c.008-.04.016-.117.016-.164a.51.51 0 00-.516-.516.54.54 0 00-.539.43l-.523 2.554H9.617l.477-2.304c.008-.04.015-.117.015-.164a.512.512 0 00-.523-.516.539.539 0 00-.531.43L8.53 7.484H7.414c-.43 0-.617.22-.617.532 0 .312.187.539.617.539h.906l-.515 2.523H6.609c-.421 0-.609.219-.609.531 0 .313.188.547.61.547h.976l-.516 2.492c-.008.04-.015.125-.015.18 0 .305.21.508.5.508.265 0 .492-.172.554-.477l.555-2.703h2.242l-.515 2.492zm-1-6.109h2.266l-.515 2.563H8.859l.532-2.563z"
        }
      })]);
    }
  });

  var heartFill = Vue.extend({
    name: 'HeartFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-heart-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 3.314C14.438-1.248 25.534 6.735 10 17-5.534 6.736 5.562-1.248 10 3.314z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var heart = Vue.extend({
    name: 'HeartIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-heart",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 4.748l-.717-.737C7.6 2.281 4.514 2.878 3.4 5.053c-.523 1.023-.641 2.5.314 4.385.92 1.815 2.834 3.989 6.286 6.357 3.452-2.368 5.365-4.542 6.286-6.357.955-1.886.837-3.362.314-4.385-1.114-2.175-4.2-2.773-5.883-1.043L10 4.748zM10 17C-5.333 6.868 5.279-1.04 9.824 3.143c.06.055.119.112.176.171a3.12 3.12 0 01.176-.17C14.72-1.042 25.333 6.867 10 17z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var houseFill = Vue.extend({
    name: 'HouseFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-house-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M8.5 12.995V16.5a.5.5 0 01-.5.5H4a.5.5 0 01-.5-.5v-7a.5.5 0 01.146-.354l6-6a.5.5 0 01.708 0l6 6a.5.5 0 01.146.354v7a.5.5 0 01-.5.5h-4a.5.5 0 01-.5-.5V13c0-.25-.25-.5-.5-.5H9c-.25 0-.5.25-.5.495z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15 4.5V8l-2-2V4.5a.5.5 0 01.5-.5h1a.5.5 0 01.5.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var house = Vue.extend({
    name: 'HouseIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-house",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.646 3.146a.5.5 0 01.708 0l6 6a.5.5 0 01.146.354v7a.5.5 0 01-.5.5h-4.5a.5.5 0 01-.5-.5v-4H9v4a.5.5 0 01-.5.5H4a.5.5 0 01-.5-.5v-7a.5.5 0 01.146-.354l6-6zM4.5 9.707V16H8v-4a.5.5 0 01.5-.5h3a.5.5 0 01.5.5v4h3.5V9.707l-5.5-5.5-5.5 5.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15 4.5V8l-2-2V4.5a.5.5 0 01.5-.5h1a.5.5 0 01.5.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var imageAlt = Vue.extend({
    name: 'ImageAltIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-image-alt",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M12.648 8.646a.5.5 0 01.577-.093l4.777 3.947V17a1 1 0 01-1 1h-14a1 1 0 01-1-1v-2l3.646-4.354a.5.5 0 01.63-.062l2.66 2.773 3.71-4.71z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.5 7a2.5 2.5 0 100-5 2.5 2.5 0 000 5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var imageFill = Vue.extend({
    name: 'ImageFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-image-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.002 5a2 2 0 012-2h12a2 2 0 012 2v10a2 2 0 01-2 2h-12a2 2 0 01-2-2V5zm1 9l2.646-2.354a.5.5 0 01.63-.062l2.66 1.773 3.71-3.71a.5.5 0 01.577-.094l3.777 1.947V15a1 1 0 01-1 1h-12a1 1 0 01-1-1v-1zm5-6.5a1.5 1.5 0 11-3 0 1.5 1.5 0 013 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var image = Vue.extend({
    name: 'ImageIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-image",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16.002 4h-12a1 1 0 00-1 1v10a1 1 0 001 1h12a1 1 0 001-1V5a1 1 0 00-1-1zm-12-1a2 2 0 00-2 2v10a2 2 0 002 2h12a2 2 0 002-2V5a2 2 0 00-2-2h-12z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M12.648 9.646a.5.5 0 01.577-.093l3.777 1.947V16h-14v-2l2.646-2.354a.5.5 0 01.63-.062l2.66 1.773 3.71-3.71z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.502 9a1.5 1.5 0 100-3 1.5 1.5 0 000 3z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var images = Vue.extend({
    name: 'ImagesIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-images",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14.002 6h-10a1 1 0 00-1 1v8a1 1 0 001 1h10a1 1 0 001-1V7a1 1 0 00-1-1zm-10-1a2 2 0 00-2 2v8a2 2 0 002 2h10a2 2 0 002-2V7a2 2 0 00-2-2h-10z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M12.648 10.646a.5.5 0 01.577-.093l1.777 1.947V16h-12v-1l2.646-2.354a.5.5 0 01.63-.062l2.66 1.773 3.71-3.71z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.502 11a1.5 1.5 0 100-3 1.5 1.5 0 000 3zM6 4h10a1 1 0 011 1v8a1 1 0 01-1 1v1a2 2 0 002-2V5a2 2 0 00-2-2H6a2 2 0 00-2 2h1a1 1 0 011-1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var inboxFill = Vue.extend({
    name: 'InboxFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-inbox-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.81 6.063A1.5 1.5 0 016.98 5.5h6.04a1.5 1.5 0 011.17.563l3.7 4.625a.5.5 0 11-.78.624l-3.7-4.624a.5.5 0 00-.39-.188H6.98a.5.5 0 00-.39.188l-3.7 4.624a.5.5 0 11-.78-.624l3.7-4.625z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.125 10.67a.5.5 0 01.375-.17h5a.5.5 0 01.5.5c0 .828.625 2 2 2s2-1.172 2-2a.5.5 0 01.5-.5h5a.5.5 0 01.496.562l-.39 3.124a1.5 1.5 0 01-1.489 1.314H3.883a1.5 1.5 0 01-1.489-1.314l-.39-3.124a.5.5 0 01.121-.393z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var inbox = Vue.extend({
    name: 'InboxIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-inbox",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.81 6.063A1.5 1.5 0 016.98 5.5h6.04a1.5 1.5 0 011.17.563l3.7 4.625a.5.5 0 11-.78.624l-3.7-4.624a.5.5 0 00-.39-.188H6.98a.5.5 0 00-.39.188l-3.7 4.624a.5.5 0 11-.78-.624l3.7-4.625z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.125 10.67a.5.5 0 01.375-.17H8a.5.5 0 01.5.5 1.5 1.5 0 003 0 .5.5 0 01.5-.5h5.5a.5.5 0 01.496.562l-.39 3.124a1.5 1.5 0 01-1.489 1.314H3.883a1.5 1.5 0 01-1.489-1.314l-.39-3.124a.5.5 0 01.121-.393zm.941.83l.32 2.562a.5.5 0 00.497.438h12.234a.5.5 0 00.496-.438l.32-2.562H12.45a2.5 2.5 0 01-4.9 0H3.066z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var inboxesFill = Vue.extend({
    name: 'InboxesFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-inboxes-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.125 13.17A.5.5 0 012.5 13H8a.5.5 0 01.5.5 1.5 1.5 0 003 0 .5.5 0 01.5-.5h5.5a.5.5 0 01.496.562l-.39 3.124A1.5 1.5 0 0116.117 18H3.883a1.5 1.5 0 01-1.489-1.314l-.39-3.124a.5.5 0 01.121-.393zM5.81 2.563A1.5 1.5 0 016.98 2h6.04a1.5 1.5 0 011.17.563l3.7 4.625a.5.5 0 11-.78.624l-3.7-4.624A.5.5 0 0013.02 3H6.98a.5.5 0 00-.39.188l-3.7 4.624a.5.5 0 11-.78-.624l3.7-4.625z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.125 7.17A.5.5 0 012.5 7H8a.5.5 0 01.5.5 1.5 1.5 0 003 0A.5.5 0 0112 7h5.5a.5.5 0 01.496.562l-.39 3.124A1.5 1.5 0 0116.117 12H3.883a1.5 1.5 0 01-1.489-1.314l-.39-3.124a.5.5 0 01.121-.393z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var inboxes = Vue.extend({
    name: 'InboxesIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-inboxes",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.125 13.17A.5.5 0 012.5 13H8a.5.5 0 01.5.5 1.5 1.5 0 003 0 .5.5 0 01.5-.5h5.5a.5.5 0 01.496.562l-.39 3.124A1.5 1.5 0 0116.117 18H3.883a1.5 1.5 0 01-1.489-1.314l-.39-3.124a.5.5 0 01.121-.393zm.941.83l.32 2.562a.5.5 0 00.497.438h12.234a.5.5 0 00.496-.438l.32-2.562H12.45a2.5 2.5 0 01-4.9 0H3.066zM5.81 2.563A1.5 1.5 0 016.98 2h6.04a1.5 1.5 0 011.17.563l3.7 4.625a.5.5 0 11-.78.624l-3.7-4.624A.5.5 0 0013.02 3H6.98a.5.5 0 00-.39.188l-3.7 4.624a.5.5 0 11-.78-.624l3.7-4.625z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.125 7.17A.5.5 0 012.5 7H8a.5.5 0 01.5.5 1.5 1.5 0 003 0A.5.5 0 0112 7h5.5a.5.5 0 01.496.562l-.39 3.124A1.5 1.5 0 0116.117 12H3.883a1.5 1.5 0 01-1.489-1.314l-.39-3.124a.5.5 0 01.121-.393zm.941.83l.32 2.562a.5.5 0 00.497.438h12.234a.5.5 0 00.496-.438L16.933 8H12.45a2.5 2.5 0 01-4.9 0H3.066z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var infoFill = Vue.extend({
    name: 'InfoFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-info-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 18a8 8 0 100-16 8 8 0 000 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533l1.002-4.705zM10 7.5a1 1 0 100-2 1 1 0 000 2z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var infoSquareFill = Vue.extend({
    name: 'InfoSquareFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-info-square-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2 4a2 2 0 012-2h12a2 2 0 012 2v12a2 2 0 01-2 2H4a2 2 0 01-2-2V4zm8.93 4.588l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533l1.002-4.705zM10 7.5a1 1 0 100-2 1 1 0 000 2z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var infoSquare = Vue.extend({
    name: 'InfoSquareIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-info-square",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 3H4a1 1 0 00-1 1v12a1 1 0 001 1h12a1 1 0 001-1V4a1 1 0 00-1-1zM4 2a2 2 0 00-2 2v12a2 2 0 002 2h12a2 2 0 002-2V4a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M10.93 8.588l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533l1.002-4.705z"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "10",
          cy: "6.5",
          r: "1"
        }
      })]);
    }
  });

  var info = Vue.extend({
    name: 'InfoIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-info",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 17a7 7 0 100-14 7 7 0 000 14zm8-7a8 8 0 11-16 0 8 8 0 0116 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M10.93 8.588l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533l1.002-4.705z"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "10",
          cy: "6.5",
          r: "1"
        }
      })]);
    }
  });

  var justifyLeft = Vue.extend({
    name: 'JustifyLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-justify-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 14.5a.5.5 0 01.5-.5h7a.5.5 0 010 1h-7a.5.5 0 01-.5-.5zm0-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm0-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm0-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var justifyRight = Vue.extend({
    name: 'JustifyRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-justify-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8 14.5a.5.5 0 01.5-.5h7a.5.5 0 010 1h-7a.5.5 0 01-.5-.5zm-4-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm0-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm0-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var justify = Vue.extend({
    name: 'JustifyIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-justify",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 14.5a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm0-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm0-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm0-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var kanbanFill = Vue.extend({
    name: 'KanbanFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-kanban-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.5 2a2 2 0 00-2 2v12a2 2 0 002 2h11a2 2 0 002-2V4a2 2 0 00-2-2h-11zm5 2a1 1 0 00-1 1v3a1 1 0 001 1h1a1 1 0 001-1V5a1 1 0 00-1-1h-1zm-5 1a1 1 0 011-1h1a1 1 0 011 1v7a1 1 0 01-1 1h-1a1 1 0 01-1-1V5zm9-1a1 1 0 00-1 1v10a1 1 0 001 1h1a1 1 0 001-1V5a1 1 0 00-1-1h-1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var kanban = Vue.extend({
    name: 'KanbanIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-kanban",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.5 3h-11a1 1 0 00-1 1v12a1 1 0 001 1h11a1 1 0 001-1V4a1 1 0 00-1-1zm-11-1a2 2 0 00-2 2v12a2 2 0 002 2h11a2 2 0 002-2V4a2 2 0 00-2-2h-11z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "3",
          height: "5",
          x: "8.5",
          y: "4",
          rx: "1"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "3",
          height: "9",
          x: "4.5",
          y: "4",
          rx: "1"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "3",
          height: "12",
          x: "12.5",
          y: "4",
          rx: "1"
        }
      })]);
    }
  });

  var laptop = Vue.extend({
    name: 'LaptopIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-laptop",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.5 4h-11a.5.5 0 00-.5.5v7a.5.5 0 00.5.5h11a.5.5 0 00.5-.5v-7a.5.5 0 00-.5-.5zm-11-1A1.5 1.5 0 003 4.5v7A1.5 1.5 0 004.5 13h11a1.5 1.5 0 001.5-1.5v-7A1.5 1.5 0 0015.5 3h-11z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M2.81 13.758A1 1 0 013.78 13h12.44a1 1 0 01.97.758l.5 2A1 1 0 0116.72 17H3.28a1 1 0 01-.97-1.242l.5-2z"
        }
      })]);
    }
  });

  var layoutSidebarReverse = Vue.extend({
    name: 'LayoutSidebarReverseIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-layout-sidebar-reverse",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 4H4a1 1 0 00-1 1v10a1 1 0 001 1h12a1 1 0 001-1V5a1 1 0 00-1-1zM4 3a2 2 0 00-2 2v10a2 2 0 002 2h12a2 2 0 002-2V5a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13 16V4h1v12h-1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var layoutSidebar = Vue.extend({
    name: 'LayoutSidebarIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-layout-sidebar",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 4H4a1 1 0 00-1 1v10a1 1 0 001 1h12a1 1 0 001-1V5a1 1 0 00-1-1zM4 3a2 2 0 00-2 2v10a2 2 0 002 2h12a2 2 0 002-2V5a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6 16V4h1v12H6z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var layoutSplit = Vue.extend({
    name: 'LayoutSplitIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-layout-split",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M1.5 5A2.5 2.5 0 014 2.5h12A2.5 2.5 0 0118.5 5v10a2.5 2.5 0 01-2.5 2.5H4A2.5 2.5 0 011.5 15V5zM4 3.5A1.5 1.5 0 002.5 5v10A1.5 1.5 0 004 16.5h12a1.5 1.5 0 001.5-1.5V5A1.5 1.5 0 0016 3.5h-5.5v13h-1v-13H4z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var listCheck = Vue.extend({
    name: 'ListCheckIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-list-check",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7 13.5a.5.5 0 01.5-.5h9a.5.5 0 010 1h-9a.5.5 0 01-.5-.5zm0-4a.5.5 0 01.5-.5h9a.5.5 0 010 1h-9a.5.5 0 01-.5-.5zm0-4a.5.5 0 01.5-.5h9a.5.5 0 010 1h-9a.5.5 0 01-.5-.5zM5.854 4.146a.5.5 0 010 .708l-1.5 1.5a.5.5 0 01-.708 0l-.5-.5a.5.5 0 11.708-.708L4 5.293l1.146-1.147a.5.5 0 01.708 0zm0 4a.5.5 0 010 .708l-1.5 1.5a.5.5 0 01-.708 0l-.5-.5a.5.5 0 11.708-.708L4 9.293l1.146-1.147a.5.5 0 01.708 0zm0 4a.5.5 0 010 .708l-1.5 1.5a.5.5 0 01-.708 0l-.5-.5a.5.5 0 01.708-.708l.146.147 1.146-1.147a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var listOl = Vue.extend({
    name: 'ListOlIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-list-ol",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7 13.5a.5.5 0 01.5-.5h9a.5.5 0 010 1h-9a.5.5 0 01-.5-.5zm0-4a.5.5 0 01.5-.5h9a.5.5 0 010 1h-9a.5.5 0 01-.5-.5zm0-4a.5.5 0 01.5-.5h9a.5.5 0 010 1h-9a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M3.713 13.865v-.474H4c.217 0 .363-.137.363-.317 0-.185-.158-.31-.361-.31-.223 0-.367.152-.373.31h-.59c.016-.467.373-.787.986-.787.588-.002.954.291.957.703a.595.595 0 01-.492.594v.033a.615.615 0 01.569.631c.003.533-.502.8-1.051.8-.656 0-1-.37-1.008-.794h.582c.008.178.186.306.422.309.254 0 .424-.145.422-.35-.002-.195-.155-.348-.414-.348h-.3zm-.004-4.699h-.604v-.035c0-.408.295-.844.958-.844.583 0 .96.326.96.756 0 .389-.257.617-.476.848l-.537.572v.03h1.054V11H3.143v-.395l.957-.99c.138-.142.293-.304.293-.508 0-.18-.147-.32-.342-.32a.33.33 0 00-.342.338v.041zM4.564 7h-.635V4.924h-.031l-.598.42v-.567l.629-.443h.635V7z"
        }
      })]);
    }
  });

  var listTask = Vue.extend({
    name: 'ListTaskIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-list-task",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 4.5a.5.5 0 00-.5.5v1a.5.5 0 00.5.5h1a.5.5 0 00.5-.5V5a.5.5 0 00-.5-.5H4zM5 5H4v1h1V5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M7 5.5a.5.5 0 01.5-.5h9a.5.5 0 010 1h-9a.5.5 0 01-.5-.5zM7.5 9a.5.5 0 000 1h9a.5.5 0 000-1h-9zm0 4a.5.5 0 000 1h9a.5.5 0 000-1h-9z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.5 9a.5.5 0 01.5-.5h1a.5.5 0 01.5.5v1a.5.5 0 01-.5.5H4a.5.5 0 01-.5-.5V9zM4 9h1v1H4V9zm0 3.5a.5.5 0 00-.5.5v1a.5.5 0 00.5.5h1a.5.5 0 00.5-.5v-1a.5.5 0 00-.5-.5H4zm1 .5H4v1h1v-1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var listUl = Vue.extend({
    name: 'ListUlIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-list-ul",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7 13.5a.5.5 0 01.5-.5h9a.5.5 0 010 1h-9a.5.5 0 01-.5-.5zm0-4a.5.5 0 01.5-.5h9a.5.5 0 010 1h-9a.5.5 0 01-.5-.5zm0-4a.5.5 0 01.5-.5h9a.5.5 0 010 1h-9a.5.5 0 01-.5-.5zm-3 1a1 1 0 100-2 1 1 0 000 2zm0 4a1 1 0 100-2 1 1 0 000 2zm0 4a1 1 0 100-2 1 1 0 000 2z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var list = Vue.extend({
    name: 'ListIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-list",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.5 13.5A.5.5 0 015 13h10a.5.5 0 010 1H5a.5.5 0 01-.5-.5zm0-4A.5.5 0 015 9h10a.5.5 0 010 1H5a.5.5 0 01-.5-.5zm0-4A.5.5 0 015 5h10a.5.5 0 010 1H5a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var lockFill = Vue.extend({
    name: 'LockFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-lock-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("rect", {
        attrs: {
          width: "11",
          height: "9",
          x: "4.5",
          y: "8",
          rx: "2"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.5 5a3.5 3.5 0 117 0v3h-1V5a2.5 2.5 0 00-5 0v3h-1V5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var lock = Vue.extend({
    name: 'LockIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-lock",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.655 9H6.333c-.264 0-.398.068-.471.121a.73.73 0 00-.224.296 1.626 1.626 0 00-.138.59V15c0 .342.076.531.14.635.064.106.151.18.256.237a1.122 1.122 0 00.436.127l.013.001h7.322c.264 0 .398-.068.471-.121a.73.73 0 00.224-.296 1.627 1.627 0 00.138-.59V10c0-.342-.076-.531-.14-.635a.658.658 0 00-.255-.237 1.123 1.123 0 00-.45-.128zm.012-1H6.333C4.5 8 4.5 10 4.5 10v5c0 2 1.833 2 1.833 2h7.334c1.833 0 1.833-2 1.833-2v-5c0-2-1.833-2-1.833-2zM6.5 5a3.5 3.5 0 117 0v3h-1V5a2.5 2.5 0 00-5 0v3h-1V5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var map = Vue.extend({
    name: 'MapIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-map",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17.817 2.613A.5.5 0 0118 3v13a.5.5 0 01-.402.49l-5 1a.502.502 0 01-.196 0L7.5 16.51l-4.902.98A.5.5 0 012 17V4a.5.5 0 01.402-.49l5-1a.5.5 0 01.196 0l4.902.98 4.902-.98a.5.5 0 01.415.103zM12 4.41l-4-.8v11.98l4 .8V4.41zm1 11.98l4-.8V3.61l-4 .8v11.98zm-6-.8V3.61l-4 .8v11.98l4-.8z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var mic = Vue.extend({
    name: 'MicIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-mic",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M7 5a3 3 0 016 0v5a3 3 0 11-6 0V5z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.5 8.5A.5.5 0 016 9v1a4 4 0 008 0V9a.5.5 0 011 0v1a5 5 0 01-4.5 4.975V17h3a.5.5 0 010 1h-7a.5.5 0 010-1h3v-2.025A5 5 0 015 10V9a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var moon = Vue.extend({
    name: 'MoonIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-moon",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M17.293 13.293A8 8 0 016.707 2.707a8.002 8.002 0 1010.586 10.586z"
        }
      })]);
    }
  });

  var musicPlayerFill = Vue.extend({
    name: 'MusicPlayerFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-music-player-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 4a2 2 0 012-2h8a2 2 0 012 2v12a2 2 0 01-2 2H6a2 2 0 01-2-2V4zm2 1a1 1 0 011-1h6a1 1 0 011 1v2.5a1 1 0 01-1 1H7a1 1 0 01-1-1V5zm7 8a3 3 0 11-6 0 3 3 0 016 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "10",
          cy: "13",
          r: "1"
        }
      })]);
    }
  });

  var musicPlayer = Vue.extend({
    name: 'MusicPlayerIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-music-player",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14 3H6a1 1 0 00-1 1v12a1 1 0 001 1h8a1 1 0 001-1V4a1 1 0 00-1-1zM6 2a2 2 0 00-2 2v12a2 2 0 002 2h8a2 2 0 002-2V4a2 2 0 00-2-2H6z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13 5H7v3h6V5zM7 4a1 1 0 00-1 1v3a1 1 0 001 1h6a1 1 0 001-1V5a1 1 0 00-1-1H7zm3 11a2 2 0 100-4 2 2 0 000 4zm3-2a3 3 0 11-6 0 3 3 0 016 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("circle", {
        attrs: {
          cx: "10",
          cy: "13",
          r: "1"
        }
      })]);
    }
  });

  var option = Vue.extend({
    name: 'OptionIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-option",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.5 4.5A.5.5 0 013 4h4a.5.5 0 01.439.26L13.297 15H17a.5.5 0 010 1h-4a.5.5 0 01-.439-.26L6.703 5H3a.5.5 0 01-.5-.5zm10 0A.5.5 0 0113 4h4a.5.5 0 010 1h-4a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var outlet = Vue.extend({
    name: 'OutletIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-outlet",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.34 4.994c.275-.338.68-.494 1.074-.494h7.172c.393 0 .798.156 1.074.494.578.708 1.84 2.534 1.84 5.006 0 2.472-1.262 4.297-1.84 5.006-.276.338-.68.494-1.074.494H6.414c-.394 0-.799-.156-1.074-.494C4.762 14.297 3.5 12.472 3.5 10c0-2.472 1.262-4.298 1.84-5.006zm1.074.506a.376.376 0 00-.299.126C5.599 6.259 4.5 7.863 4.5 10c0 2.137 1.099 3.74 1.615 4.374.06.073.163.126.3.126h7.17c.137 0 .24-.053.3-.126.516-.633 1.615-2.237 1.615-4.374 0-2.137-1.099-3.74-1.615-4.374a.376.376 0 00-.3-.126h-7.17z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8 7.5a.5.5 0 01.5.5v1.5a.5.5 0 01-1 0V8a.5.5 0 01.5-.5zm4 0a.5.5 0 01.5.5v1.5a.5.5 0 01-1 0V8a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9 12v1h2v-1a1 1 0 10-2 0z"
        }
      })]);
    }
  });

  var pauseFill = Vue.extend({
    name: 'PauseFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-pause-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M7.5 5.5A1.5 1.5 0 019 7v6a1.5 1.5 0 01-3 0V7a1.5 1.5 0 011.5-1.5zm5 0A1.5 1.5 0 0114 7v6a1.5 1.5 0 01-3 0V7a1.5 1.5 0 011.5-1.5z"
        }
      })]);
    }
  });

  var pause = Vue.extend({
    name: 'PauseIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-pause",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8 5.5a.5.5 0 01.5.5v8a.5.5 0 01-1 0V6a.5.5 0 01.5-.5zm4 0a.5.5 0 01.5.5v8a.5.5 0 01-1 0V6a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var pen = Vue.extend({
    name: 'PenIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-pen",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.707 15.707a1 1 0 01-.39.242l-3 1a1 1 0 01-1.266-1.265l1-3a1 1 0 01.242-.391L12.086 4.5a2 2 0 012.828 0l.586.586a2 2 0 010 2.828l-7.793 7.793zM5 13l7.793-7.793a1 1 0 011.414 0l.586.586a1 1 0 010 1.414L7 15l-3 1 1-3z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.854 4.56a.5.5 0 00-.708 0L7.854 7.855a.5.5 0 11-.708-.708l3.293-3.292a1.5 1.5 0 012.122 0l.293.292a.5.5 0 11-.708.708l-.292-.293z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M15.293 3.207a1 1 0 011.414 0l.03.03a1 1 0 01.03 1.383L15.5 6 14 4.5l1.293-1.293z"
        }
      })]);
    }
  });

  var pencil = Vue.extend({
    name: 'PencilIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-pencil",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.293 3.293a1 1 0 011.414 0l2 2a1 1 0 010 1.414l-9 9a1 1 0 01-.39.242l-3 1a1 1 0 01-1.266-1.265l1-3a1 1 0 01.242-.391l9-9zM14 4l2 2-9 9-3 1 1-3 9-9z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14.146 8.354l-2.5-2.5.708-.708 2.5 2.5-.708.708zM5 12v.5a.5.5 0 00.5.5H6v.5a.5.5 0 00.5.5H7v.5a.5.5 0 00.5.5H8v-1.5a.5.5 0 00-.5-.5H7v-.5a.5.5 0 00-.5-.5H5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var peopleFill = Vue.extend({
    name: 'PeopleFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-people-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9 16s-1 0-1-1 1-4 5-4 5 3 5 4-1 1-1 1H9zm4-6a3 3 0 100-6 3 3 0 000 6zm-5.784 6A2.238 2.238 0 017 15c0-1.355.68-2.75 1.936-3.72A6.325 6.325 0 007 11c-4 0-5 3-5 4s1 1 1 1h4.216zM6.5 10a2.5 2.5 0 100-5 2.5 2.5 0 000 5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var people = Vue.extend({
    name: 'PeopleIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-people",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17 16s1 0 1-1-1-4-5-4-5 3-5 4 1 1 1 1h8zm-7.995-.944v-.002zM9.022 15h7.956a.274.274 0 00.014-.002l.008-.002c-.002-.264-.167-1.03-.76-1.72C15.688 12.629 14.718 12 13 12c-1.717 0-2.687.63-3.24 1.276-.593.69-.759 1.457-.76 1.72a1.05 1.05 0 00.022.004zm7.973.056v-.002zM13 9a2 2 0 100-4 2 2 0 000 4zm3-2a3 3 0 11-6 0 3 3 0 016 0zm-7.064 4.28a5.873 5.873 0 00-1.23-.247A7.334 7.334 0 007 11c-4 0-5 3-5 4 0 .667.333 1 1 1h4.216A2.238 2.238 0 017 15c0-1.01.377-2.042 1.09-2.904.243-.294.526-.569.846-.816zM6.92 12c-1.668.02-2.615.64-3.16 1.276C3.163 13.97 3 14.739 3 15h3c0-1.045.323-2.086.92-3zM3.5 7.5a3 3 0 116 0 3 3 0 01-6 0zm3-2a2 2 0 100 4 2 2 0 000-4z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var personFill = Vue.extend({
    name: 'PersonFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-person-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5 16s-1 0-1-1 1-4 6-4 6 3 6 4-1 1-1 1H5zm5-6a3 3 0 100-6 3 3 0 000 6z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var person = Vue.extend({
    name: 'PersonIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-person",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15 16s1 0 1-1-1-4-6-4-6 3-6 4 1 1 1 1h10zm-9.995-.944v-.002zM5.022 15h9.956a.274.274 0 00.014-.002l.008-.002c-.001-.246-.154-.986-.832-1.664C13.516 12.68 12.289 12 10 12c-2.29 0-3.516.68-4.168 1.332-.678.678-.83 1.418-.832 1.664a1.05 1.05 0 00.022.004zm9.974.056v-.002zM10 9a2 2 0 100-4 2 2 0 000 4zm3-2a3 3 0 11-6 0 3 3 0 016 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var phoneLandscape = Vue.extend({
    name: 'PhoneLandscapeIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-phone-landscape",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.5 6.5v6a1 1 0 001 1h11a1 1 0 001-1v-6a1 1 0 00-1-1h-11a1 1 0 00-1 1zm-1 6a2 2 0 002 2h11a2 2 0 002-2v-6a2 2 0 00-2-2h-11a2 2 0 00-2 2v6z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.5 9.5a1 1 0 10-2 0 1 1 0 002 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var phone = Vue.extend({
    name: 'PhoneIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-phone",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13 3H7a1 1 0 00-1 1v11a1 1 0 001 1h6a1 1 0 001-1V4a1 1 0 00-1-1zM7 2a2 2 0 00-2 2v11a2 2 0 002 2h6a2 2 0 002-2V4a2 2 0 00-2-2H7z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 15a1 1 0 100-2 1 1 0 000 2z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var pieChartFill = Vue.extend({
    name: 'PieChartFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-pie-chart-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M17.985 10.5h-7.778l-5.5 5.5a8 8 0 0013.277-5.5zM4 15.292A8 8 0 019.5 2.015v7.778l-5.5 5.5zm6.5-13.277V9.5h7.485A8.001 8.001 0 0010.5 2.015z"
        }
      })]);
    }
  });

  var pieChart = Vue.extend({
    name: 'PieChartIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-pie-chart",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 17a7 7 0 100-14 7 7 0 000 14zm0 1a8 8 0 100-16 8 8 0 000 16z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.5 9.793V3h1v6.5H17v1h-6.793l-4.853 4.854-.708-.708L9.5 9.793z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var playFill = Vue.extend({
    name: 'PlayFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-play-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M13.596 10.697l-6.363 3.692c-.54.313-1.233-.066-1.233-.697V6.308c0-.63.692-1.01 1.233-.696l6.363 3.692a.802.802 0 010 1.393z"
        }
      })]);
    }
  });

  var play = Vue.extend({
    name: 'PlayIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-play",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.804 10L7 6.633v6.734L12.804 10zm.792-.696a.802.802 0 010 1.392l-6.363 3.692C6.713 14.69 6 14.345 6 13.692V6.308c0-.653.713-.998 1.233-.696l6.363 3.692z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var plug = Vue.extend({
    name: 'PlugIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-plug",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M6 7h8v3a4 4 0 01-8 0V7z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8 3.5a.5.5 0 01.5.5v3a.5.5 0 01-1 0V4a.5.5 0 01.5-.5zm4 0a.5.5 0 01.5.5v3a.5.5 0 01-1 0V4a.5.5 0 01.5-.5zM9.115 15.651c.256-.511.385-1.408.385-2.651h1c0 1.257-.121 2.36-.49 3.099-.191.381-.47.707-.87.877-.401.17-.845.15-1.298-.002-.961-.32-1.534-.175-1.851.046-.33.23-.491.615-.491.98h-1c0-.635.278-1.353.918-1.8.653-.456 1.58-.561 2.74-.174.297.099.478.078.592.03.115-.05.244-.161.365-.405z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var plus = Vue.extend({
    name: 'PlusIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-plus",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 5.5a.5.5 0 01.5.5v4a.5.5 0 01-.5.5H6a.5.5 0 010-1h3.5V6a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.5 10a.5.5 0 01.5-.5h4a.5.5 0 010 1h-3.5V14a.5.5 0 01-1 0v-4z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var power = Vue.extend({
    name: 'PowerIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-power",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.578 6.437a5 5 0 104.922.044l.5-.865a6 6 0 11-5.908-.053l.486.874z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.5 10V3h1v7h-1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var questionFill = Vue.extend({
    name: 'QuestionFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-question-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M18 10a8 8 0 11-16 0 8 8 0 0116 0zM8.57 8.033H7.25C7.22 6.147 8.68 5.5 10.006 5.5c1.397 0 2.673.73 2.673 2.24 0 1.08-.635 1.594-1.244 2.057-.737.559-1.01.768-1.01 1.486v.355H9.117l-.007-.463c-.038-.927.495-1.498 1.168-1.987.59-.444.965-.736.965-1.371 0-.825-.628-1.168-1.314-1.168-.901 0-1.358.603-1.358 1.384zm1.251 6.443c-.584 0-1.009-.394-1.009-.927 0-.552.425-.94 1.01-.94.609 0 1.028.388 1.028.94 0 .533-.42.927-1.029.927z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var questionSquareFill = Vue.extend({
    name: 'QuestionSquareFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-question-square-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 2a2 2 0 00-2 2v12a2 2 0 002 2h12a2 2 0 002-2V4a2 2 0 00-2-2H4zm4.57 6.033H7.25C7.22 6.147 8.68 5.5 10.006 5.5c1.397 0 2.673.73 2.673 2.24 0 1.08-.635 1.594-1.244 2.057-.737.559-1.01.768-1.01 1.486v.355H9.117l-.007-.463c-.038-.927.495-1.498 1.168-1.987.59-.444.965-.736.965-1.371 0-.825-.628-1.168-1.314-1.168-.901 0-1.358.603-1.358 1.384zm1.251 6.443c-.584 0-1.009-.394-1.009-.927 0-.552.425-.94 1.01-.94.609 0 1.028.388 1.028.94 0 .533-.42.927-1.029.927z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var questionSquare = Vue.extend({
    name: 'QuestionSquareIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-question-square",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 3H4a1 1 0 00-1 1v12a1 1 0 001 1h12a1 1 0 001-1V4a1 1 0 00-1-1zM4 2a2 2 0 00-2 2v12a2 2 0 002 2h12a2 2 0 002-2V4a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M7.25 8.033h1.32c0-.781.458-1.384 1.36-1.384.685 0 1.313.343 1.313 1.168 0 .635-.374.927-.965 1.371-.673.489-1.206 1.06-1.168 1.987l.007.463h1.307v-.355c0-.718.273-.927 1.01-1.486.609-.463 1.244-.977 1.244-2.056 0-1.511-1.276-2.241-2.673-2.241-1.326 0-2.786.647-2.754 2.533zm1.562 5.516c0 .533.425.927 1.01.927.609 0 1.028-.394 1.028-.927 0-.552-.42-.94-1.029-.94-.584 0-1.009.388-1.009.94z"
        }
      })]);
    }
  });

  var question = Vue.extend({
    name: 'QuestionIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-question",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 17a7 7 0 100-14 7 7 0 000 14zm8-7a8 8 0 11-16 0 8 8 0 0116 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M7.25 8.033h1.32c0-.781.458-1.384 1.36-1.384.685 0 1.313.343 1.313 1.168 0 .635-.374.927-.965 1.371-.673.489-1.206 1.06-1.168 1.987l.007.463h1.307v-.355c0-.718.273-.927 1.01-1.486.609-.463 1.244-.977 1.244-2.056 0-1.511-1.276-2.241-2.673-2.241-1.326 0-2.786.647-2.754 2.533zm1.562 5.516c0 .533.425.927 1.01.927.609 0 1.028-.394 1.028-.927 0-.552-.42-.94-1.029-.94-.584 0-1.009.388-1.009.94z"
        }
      })]);
    }
  });

  var replyAllFill = Vue.extend({
    name: 'ReplyAllFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-reply-all-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M10.079 13.9l4.568-3.281a.719.719 0 000-1.238L10.079 6.1A.716.716 0 009 6.719V8c-1.5 0-6 0-7 8 2.5-4.5 7-4 7-4v1.281c0 .56.606.898 1.079.62z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.868 6.293a.5.5 0 01.7-.106l3.993 2.94a1.147 1.147 0 010 1.946l-3.994 2.94a.5.5 0 11-.593-.805l4.012-2.954a.523.523 0 01.042-.028.147.147 0 000-.252.512.512 0 01-.042-.028l-4.012-2.954a.5.5 0 01-.106-.699z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var replyAll = Vue.extend({
    name: 'ReplyAllIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-reply-all",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.002 7.013a.144.144 0 00-.202.134V8.3a.5.5 0 01-.5.5c-.667 0-2.013.005-3.3.822-.984.624-1.99 1.76-2.595 3.876 1.02-.983 2.185-1.516 3.205-1.799a8.745 8.745 0 011.921-.306 7.47 7.47 0 01.798.008h.013l.005.001h.001L9.3 11.9l.05-.498a.5.5 0 01.45.498v1.153c0 .108.11.176.202.134l3.984-2.933a.522.522 0 01.042-.028.147.147 0 000-.252.51.51 0 01-.042-.028l-3.984-2.933zM8.8 12.386a7.745 7.745 0 00-1.923.277c-1.326.368-2.896 1.201-3.94 3.08a.5.5 0 01-.933-.305c.464-3.71 1.886-5.662 3.46-6.66 1.245-.79 2.527-.942 3.336-.971v-.66a1.144 1.144 0 011.767-.96l3.994 2.94a1.147 1.147 0 010 1.946l-3.994 2.94a1.144 1.144 0 01-1.767-.96v-.667z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.868 6.293a.5.5 0 01.7-.106l3.993 2.94a1.147 1.147 0 010 1.946l-3.994 2.94a.5.5 0 11-.593-.805l4.012-2.954a.523.523 0 01.042-.028.147.147 0 000-.252.512.512 0 01-.042-.028l-4.012-2.954a.5.5 0 01-.106-.699z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var replyFill = Vue.extend({
    name: 'ReplyFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-reply-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M11.079 13.9l4.568-3.281a.719.719 0 000-1.238L11.079 6.1A.716.716 0 0010 6.719V8c-1.5 0-6 0-7 8 2.5-4.5 7-4 7-4v1.281c0 .56.606.898 1.079.62z"
        }
      })]);
    }
  });

  var reply = Vue.extend({
    name: 'ReplyIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-reply",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.502 7.013a.144.144 0 00-.202.134V8.3a.5.5 0 01-.5.5c-.667 0-2.013.005-3.3.822-.984.624-1.99 1.76-2.595 3.876 1.02-.983 2.185-1.516 3.205-1.799a8.745 8.745 0 011.921-.306 7.468 7.468 0 01.798.008h.013l.005.001h.001l-.048.498.05-.498a.5.5 0 01.45.498v1.153c0 .108.11.176.202.134l3.984-2.933a.522.522 0 01.042-.028.147.147 0 000-.252.51.51 0 01-.042-.028l-3.984-2.933zM10.3 12.386a7.745 7.745 0 00-1.923.277c-1.326.368-2.896 1.201-3.94 3.08a.5.5 0 01-.933-.305c.464-3.71 1.886-5.662 3.46-6.66 1.245-.79 2.527-.942 3.336-.971v-.66a1.144 1.144 0 011.767-.96l3.994 2.94a1.147 1.147 0 010 1.946l-3.994 2.94a1.144 1.144 0 01-1.767-.96v-.667z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var screwdriver = Vue.extend({
    name: 'ScrewdriverIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-screwdriver",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2 3l1-1 3.081 2.2a1 1 0 01.419.815v.07a1 1 0 00.293.708L12.5 11.5l.914-.305a1 1 0 011.023.242l3.356 3.356a1 1 0 010 1.414l-1.586 1.586a1 1 0 01-1.414 0l-3.356-3.356a1 1 0 01-.242-1.023l.305-.914-5.707-5.707a1 1 0 00-.707-.293h-.071a1 1 0 01-.814-.419L2 3zm11.354 9.646a.5.5 0 00-.708.708l3 3a.5.5 0 00.708-.708l-3-3z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var search = Vue.extend({
    name: 'SearchIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-search",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.442 12.442a1 1 0 011.415 0l3.85 3.85a1 1 0 01-1.414 1.415l-3.85-3.85a1 1 0 010-1.415z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.5 14a5.5 5.5 0 100-11 5.5 5.5 0 000 11zM15 8.5a6.5 6.5 0 11-13 0 6.5 6.5 0 0113 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var shieldFill = Vue.extend({
    name: 'ShieldFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-shield-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.187 3.025C8.23 2.749 9.337 2.5 10 2.5c.662 0 1.77.249 2.813.525 1.066.282 2.14.614 2.772.815.528.168.926.623 1.003 1.184.573 4.197-.756 7.307-2.367 9.365a11.192 11.192 0 01-2.418 2.3 6.942 6.942 0 01-1.007.586c-.27.124-.558.225-.796.225s-.527-.101-.796-.225a6.908 6.908 0 01-1.007-.586 11.192 11.192 0 01-2.418-2.3c-1.611-2.058-2.94-5.168-2.367-9.365A1.454 1.454 0 014.415 3.84a61.113 61.113 0 012.772-.815z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var shieldLockFill = Vue.extend({
    name: 'ShieldLockFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-shield-lock-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.187 3.025C8.23 2.749 9.337 2.5 10 2.5c.662 0 1.77.249 2.813.525a61.1 61.1 0 012.772.815c.527.168.926.623 1.003 1.184.573 4.197-.756 7.307-2.368 9.365a11.19 11.19 0 01-2.417 2.3 6.942 6.942 0 01-1.007.586c-.27.124-.558.225-.796.225s-.527-.101-.796-.225a6.908 6.908 0 01-1.007-.586 11.192 11.192 0 01-2.418-2.3c-1.611-2.058-2.94-5.168-2.367-9.365A1.454 1.454 0 014.415 3.84a61.105 61.105 0 012.772-.815zm3.328 6.884a1.5 1.5 0 10-1.06-.011.5.5 0 00-.044.136l-.333 2a.5.5 0 00.493.582h.835a.5.5 0 00.493-.585l-.347-2a.501.501 0 00-.037-.122z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var shieldLock = Vue.extend({
    name: 'ShieldLockIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-shield-lock",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.443 3.991a60.17 60.17 0 00-2.725.802.454.454 0 00-.315.366C3.87 9.056 5.1 11.9 6.567 13.773c.736.94 1.533 1.636 2.197 2.093.333.228.626.394.857.5.116.053.21.089.282.11A.73.73 0 0010 16.5a.774.774 0 00.097-.023c.072-.022.166-.058.282-.111.23-.106.524-.272.857-.5a10.198 10.198 0 002.197-2.093C14.9 11.9 16.13 9.056 15.597 5.159a.454.454 0 00-.315-.366c-.626-.2-1.682-.526-2.725-.802C11.491 3.71 10.51 3.5 10 3.5c-.51 0-1.49.21-2.557.491zm-.256-.966C8.23 2.749 9.337 2.5 10 2.5c.662 0 1.77.249 2.813.525 1.066.282 2.14.614 2.772.815.528.168.926.623 1.003 1.184.573 4.197-.756 7.307-2.367 9.365a11.192 11.192 0 01-2.418 2.3 6.942 6.942 0 01-1.007.586c-.27.124-.558.225-.796.225s-.527-.101-.796-.225a6.908 6.908 0 01-1.007-.586 11.192 11.192 0 01-2.418-2.3c-1.611-2.058-2.94-5.168-2.367-9.365A1.454 1.454 0 014.415 3.84a61.113 61.113 0 012.772-.815z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M11.5 8.5a1.5 1.5 0 11-3 0 1.5 1.5 0 013 0z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9.41 10.034a.5.5 0 01.494-.417h.156a.5.5 0 01.492.414l.347 2a.5.5 0 01-.493.585h-.835a.5.5 0 01-.493-.582l.333-2z"
        }
      })]);
    }
  });

  var shieldShaded = Vue.extend({
    name: 'ShieldShadedIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-shield-shaded",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.443 3.991a60.17 60.17 0 00-2.725.802.454.454 0 00-.315.366C3.87 9.056 5.1 11.9 6.567 13.773c.736.94 1.533 1.636 2.197 2.093.333.228.626.394.857.5.116.053.21.089.282.11A.73.73 0 0010 16.5a.774.774 0 00.097-.023c.072-.022.166-.058.282-.111a5.94 5.94 0 00.857-.5 10.198 10.198 0 002.197-2.093C14.9 11.9 16.13 9.056 15.597 5.159a.454.454 0 00-.315-.366c-.626-.2-1.682-.526-2.725-.802C11.491 3.71 10.51 3.5 10 3.5c-.51 0-1.49.21-2.557.491zm-.256-.966C8.23 2.749 9.337 2.5 10 2.5c.662 0 1.77.249 2.813.525 1.066.282 2.14.614 2.772.815.528.168.926.623 1.003 1.184.573 4.197-.756 7.307-2.367 9.365a11.192 11.192 0 01-2.418 2.3 6.942 6.942 0 01-1.007.586c-.27.124-.558.225-.796.225s-.526-.101-.796-.225a6.908 6.908 0 01-1.007-.586 11.192 11.192 0 01-2.418-2.3c-1.611-2.058-2.94-5.168-2.367-9.365A1.454 1.454 0 014.415 3.84a61.105 61.105 0 012.772-.815z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M10 4.25c.909 0 3.188.685 4.254 1.022a.94.94 0 01.656.773c.814 6.424-4.13 9.452-4.91 9.452V4.25z"
        }
      })]);
    }
  });

  var shield = Vue.extend({
    name: 'ShieldIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-shield",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.443 3.991a60.17 60.17 0 00-2.725.802.454.454 0 00-.315.366C3.87 9.056 5.1 11.9 6.567 13.773c.736.94 1.533 1.636 2.197 2.093.333.228.626.394.857.5.116.053.21.089.282.11A.73.73 0 0010 16.5a.774.774 0 00.097-.023c.072-.022.166-.058.282-.111.23-.106.524-.272.857-.5a10.198 10.198 0 002.197-2.093C14.9 11.9 16.13 9.056 15.597 5.159a.454.454 0 00-.315-.366c-.626-.2-1.682-.526-2.725-.802C11.491 3.71 10.51 3.5 10 3.5c-.51 0-1.49.21-2.557.491zm-.256-.966C8.23 2.749 9.337 2.5 10 2.5c.662 0 1.77.249 2.813.525 1.066.282 2.14.614 2.772.815.528.168.926.623 1.003 1.184.573 4.197-.756 7.307-2.367 9.365a11.192 11.192 0 01-2.418 2.3 6.942 6.942 0 01-1.007.586c-.27.124-.558.225-.796.225s-.527-.101-.796-.225a6.908 6.908 0 01-1.007-.586 11.192 11.192 0 01-2.418-2.3c-1.611-2.058-2.94-5.168-2.367-9.365A1.454 1.454 0 014.415 3.84a61.113 61.113 0 012.772-.815z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var shiftFill = Vue.extend({
    name: 'ShiftFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-shift-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.27 4.047a1 1 0 011.46 0l6.345 6.769c.6.639.146 1.684-.73 1.684H13.5v3a1 1 0 01-1 1h-5a1 1 0 01-1-1v-3H3.654c-.875 0-1.328-1.045-.73-1.684L9.27 4.047z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var shift = Vue.extend({
    name: 'ShiftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-shift",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.27 4.047a1 1 0 011.46 0l6.345 6.77c.6.638.146 1.683-.73 1.683H13.5v3a1 1 0 01-1 1h-5a1 1 0 01-1-1v-3H3.654c-.875 0-1.328-1.045-.73-1.684L9.27 4.047zm7.076 7.453L10 4.731 3.654 11.5H6.5a1 1 0 011 1v3h5v-3a1 1 0 011-1h2.846z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var skipBackwardFill = Vue.extend({
    name: 'SkipBackwardFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-skip-backward-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          stroke: "#000",
          "stroke-linecap": "round",
          d: "M14 6v8"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M13.596 10.697l-6.363 3.692c-.54.313-1.233-.066-1.233-.697V6.308c0-.63.692-1.01 1.233-.696l6.363 3.692a.802.802 0 010 1.393z"
        }
      })]);
    }
  });

  var skipBackward = Vue.extend({
    name: 'SkipBackwardIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-skip-backward",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.5 5.5A.5.5 0 013 6v3.248l6.267-3.636c.52-.302 1.233.043 1.233.696v2.94l6.267-3.636c.52-.302 1.233.043 1.233.696v7.384c0 .653-.713.998-1.233.696L10.5 10.752v2.94c0 .653-.713.998-1.233.696L3 10.752V14a.5.5 0 01-1 0V6a.5.5 0 01.5-.5zm7 1.133L3.696 10 9.5 13.367V6.633zm7.5 0L11.196 10 17 13.367V6.633z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var skipEndFill = Vue.extend({
    name: 'SkipEndFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-skip-end-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          stroke: "#000",
          "stroke-linecap": "round",
          d: "M2.5 6v8"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M2.904 10.697l6.363 3.692c.54.313 1.233-.066 1.233-.697V6.308c0-.63-.693-1.01-1.233-.696L2.904 9.304a.802.802 0 000 1.393z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M10.404 10.697l6.363 3.692c.54.313 1.233-.066 1.233-.697V6.308c0-.63-.692-1.01-1.233-.696l-6.363 3.692a.802.802 0 000 1.393z"
        }
      })]);
    }
  });

  var skipEnd = Vue.extend({
    name: 'SkipEndIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-skip-end",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14 5.5a.5.5 0 01.5.5v8a.5.5 0 01-1 0V6a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.804 10L7 6.633v6.734L12.804 10zm.792-.696a.802.802 0 010 1.392l-6.363 3.692C6.713 14.69 6 14.345 6 13.692V6.308c0-.653.713-.998 1.233-.696l6.363 3.692z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var skipForwardFill = Vue.extend({
    name: 'SkipForwardFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-skip-forward-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          stroke: "#000",
          "stroke-linecap": "round",
          d: "M17.5 6v8"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9.596 10.697l-6.363 3.692c-.54.313-1.233-.066-1.233-.697V6.308c0-.63.692-1.01 1.233-.696l6.363 3.692a.802.802 0 010 1.393z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M17.096 10.697l-6.363 3.692c-.54.313-1.233-.066-1.233-.697V6.308c0-.63.693-1.01 1.233-.696l6.363 3.692a.802.802 0 010 1.393z"
        }
      })]);
    }
  });

  var skipForward = Vue.extend({
    name: 'SkipForwardIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-skip-forward",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17.5 5.5a.5.5 0 01.5.5v8a.5.5 0 01-1 0v-3.248l-6.267 3.636c-.52.302-1.233-.043-1.233-.696v-2.94l-6.267 3.636C2.713 14.69 2 14.345 2 13.692V6.308c0-.653.713-.998 1.233-.696L9.5 9.248v-2.94c0-.653.713-.998 1.233-.696L17 9.248V6a.5.5 0 01.5-.5zM3 6.633v6.734L8.804 10 3 6.633zm7.5 0v6.734L16.304 10 10.5 6.633z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var skipStartFill = Vue.extend({
    name: 'SkipStartFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-skip-start-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          stroke: "#000",
          "stroke-linecap": "round",
          d: "M6.5 6v8"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M6.903 10.697l6.364 3.692c.54.313 1.232-.066 1.232-.697V6.308c0-.63-.692-1.01-1.232-.696L6.903 9.304a.802.802 0 000 1.393z"
        }
      })]);
    }
  });

  var skipStart = Vue.extend({
    name: 'SkipStartIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-skip-start",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.5 5.5A.5.5 0 006 6v8a.5.5 0 001 0V6a.5.5 0 00-.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.696 10L13.5 6.633v6.734L7.696 10zm-.792-.696a.802.802 0 000 1.392l6.363 3.692c.52.302 1.233-.043 1.233-.696V6.308c0-.653-.713-.998-1.233-.696L6.904 9.304z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var speaker = Vue.extend({
    name: 'SpeakerIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-speaker",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M11 6a1 1 0 11-2 0 1 1 0 012 0zm-2.5 6.5a1.5 1.5 0 113 0 1.5 1.5 0 01-3 0z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6 2a2 2 0 00-2 2v12a2 2 0 002 2h8a2 2 0 002-2V4a2 2 0 00-2-2H6zm6 4a2 2 0 11-4 0 2 2 0 014 0zm-2 3a3.5 3.5 0 100 7 3.5 3.5 0 000-7z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var squareFill = Vue.extend({
    name: 'SquareFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-square-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("rect", {
        attrs: {
          width: "16",
          height: "16",
          x: "2",
          y: "2",
          rx: "2"
        }
      })]);
    }
  });

  var squareHalf = Vue.extend({
    name: 'SquareHalfIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-square-half",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 3H4a1 1 0 00-1 1v12a1 1 0 001 1h6V3zM4 2a2 2 0 00-2 2v12a2 2 0 002 2h12a2 2 0 002-2V4a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var square = Vue.extend({
    name: 'SquareIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-square",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 3H4a1 1 0 00-1 1v12a1 1 0 001 1h12a1 1 0 001-1V4a1 1 0 00-1-1zM4 2a2 2 0 00-2 2v12a2 2 0 002 2h12a2 2 0 002-2V4a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var starFill = Vue.extend({
    name: 'StarFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-star-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M5.612 17.443c-.386.198-.824-.149-.746-.592l.83-4.73-3.522-3.356c-.33-.314-.16-.888.282-.95l4.898-.696 2.184-4.327c.197-.39.73-.39.927 0l2.184 4.327 4.898.696c.441.062.612.636.283.95l-3.523 3.356.83 4.73c.078.443-.36.79-.746.592L10 15.187l-4.389 2.256z"
        }
      })]);
    }
  });

  var starHalf = Vue.extend({
    name: 'StarHalfIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-star-half",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.354 7.119l2.184-4.327A.516.516 0 0110 2.5c.183 0 .366.097.465.292l2.184 4.327 4.898.696A.537.537 0 0118 8.32a.55.55 0 01-.17.445l-3.523 3.356.83 4.73c.078.443-.36.79-.746.592L10 15.187l-4.389 2.256a.52.52 0 01-.146.05c-.341.06-.668-.254-.6-.642l.83-4.73-3.522-3.356a.55.55 0 01-.172-.403.59.59 0 01.084-.302.513.513 0 01.37-.245l4.898-.696zM10 14.027c.08 0 .16.018.232.056l3.686 1.894-.694-3.957a.564.564 0 01.163-.505l2.907-2.77-4.053-.576a.525.525 0 01-.393-.288l-1.847-3.658-.001.003v9.8z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var star = Vue.extend({
    name: 'StarIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-star",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.866 16.85c-.078.444.36.791.746.593l4.39-2.256 4.389 2.256c.386.198.824-.149.746-.592l-.83-4.73 3.523-3.356c.329-.314.158-.888-.283-.95l-4.898-.696-2.184-4.327a.513.513 0 00-.927 0L7.354 7.12l-4.898.696c-.441.062-.612.636-.282.95l3.522 3.356-.83 4.73zm4.905-2.767l-3.686 1.894.694-3.957a.565.565 0 00-.163-.505L3.71 8.745l4.052-.576a.525.525 0 00.393-.288l1.847-3.658 1.846 3.658c.08.157.226.264.393.288l4.053.575-2.907 2.77a.564.564 0 00-.163.506l.694 3.957-3.686-1.894a.503.503 0 00-.461 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var stopFill = Vue.extend({
    name: 'StopFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-stop-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M7 5.5h6A1.5 1.5 0 0114.5 7v6a1.5 1.5 0 01-1.5 1.5H7A1.5 1.5 0 015.5 13V7A1.5 1.5 0 017 5.5z"
        }
      })]);
    }
  });

  var stop = Vue.extend({
    name: 'StopIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-stop",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.5 7A1.5 1.5 0 017 5.5h6A1.5 1.5 0 0114.5 7v6a1.5 1.5 0 01-1.5 1.5H7A1.5 1.5 0 015.5 13V7zM7 6.5a.5.5 0 00-.5.5v6a.5.5 0 00.5.5h6a.5.5 0 00.5-.5V7a.5.5 0 00-.5-.5H7z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var stopwatchFill = Vue.extend({
    name: 'StopwatchFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-stopwatch-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.5 2.5A.5.5 0 018 2h4a.5.5 0 010 1h-1v1.07A7.002 7.002 0 0110 18 7 7 0 019 4.07V3H8a.5.5 0 01-.5-.5zm3 4.5a.5.5 0 00-1 0v3.5h-3a.5.5 0 000 1H10a.5.5 0 00.5-.5V7z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var stopwatch = Vue.extend({
    name: 'StopwatchIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-stopwatch",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 17a6 6 0 100-12 6 6 0 000 12zm0 1a7 7 0 100-14 7 7 0 000 14z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 6.5a.5.5 0 01.5.5v4a.5.5 0 01-.5.5H6.5a.5.5 0 010-1h3V7a.5.5 0 01.5-.5zm-2.5-4A.5.5 0 018 2h4a.5.5 0 010 1H8a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9 3h2v2H9V3z"
        }
      })]);
    }
  });

  var sun = Vue.extend({
    name: 'SunIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-sun",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M5.5 10a4.5 4.5 0 119 0 4.5 4.5 0 01-9 0z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.202 2.28a.25.25 0 00-.404 0l-.91 1.255a.25.25 0 01-.334.067L7.232 2.79a.25.25 0 00-.374.155l-.36 1.508a.25.25 0 01-.282.189l-1.532-.244a.25.25 0 00-.286.286l.244 1.532a.25.25 0 01-.189.282l-1.508.36a.25.25 0 00-.155.374l.812 1.322a.25.25 0 01-.067.333l-1.256.91a.25.25 0 000 .405l1.256.91a.25.25 0 01.067.334l-.812 1.322a.25.25 0 00.155.374l1.508.36a.25.25 0 01.19.282l-.245 1.532a.25.25 0 00.286.286l1.532-.244a.25.25 0 01.282.189l.36 1.508a.25.25 0 00.374.155l1.322-.812a.25.25 0 01.333.067l.91 1.256a.25.25 0 00.405 0l.91-1.256a.25.25 0 01.334-.067l1.322.812a.25.25 0 00.374-.155l.36-1.508a.25.25 0 01.282-.19l1.532.245a.25.25 0 00.286-.286l-.244-1.532a.25.25 0 01.189-.282l1.508-.36a.25.25 0 00.155-.374l-.812-1.322a.25.25 0 01.067-.333l1.256-.91a.25.25 0 000-.405l-1.256-.91a.25.25 0 01-.067-.334l.812-1.322a.25.25 0 00-.155-.374l-1.508-.36a.25.25 0 01-.19-.282l.245-1.532a.25.25 0 00-.286-.286l-1.532.244a.25.25 0 01-.282-.189l-.36-1.509a.25.25 0 00-.374-.154l-1.322.812a.25.25 0 01-.333-.067l-.91-1.256zM10 4.5a5.5 5.5 0 100 11 5.5 5.5 0 000-11z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var table = Vue.extend({
    name: 'TableIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-table",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 3H4a1 1 0 00-1 1v12a1 1 0 001 1h12a1 1 0 001-1V4a1 1 0 00-1-1zM4 2a2 2 0 00-2 2v12a2 2 0 002 2h12a2 2 0 002-2V4a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17 6H3V5h14v1z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7 17.5v-14h1v14H7zm5 0v-14h1v14h-1z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17 10H3V9h14v1zm0 4H3v-1h14v1z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M2 4a2 2 0 012-2h12a2 2 0 012 2v2H2V4z"
        }
      })]);
    }
  });

  var tabletLandscape = Vue.extend({
    name: 'TabletLandscapeIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-tablet-landscape",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.5 6v8a1 1 0 001 1h11a1 1 0 001-1V6a1 1 0 00-1-1h-11a1 1 0 00-1 1zm-1 8a2 2 0 002 2h11a2 2 0 002-2V6a2 2 0 00-2-2h-11a2 2 0 00-2 2v8z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M15.5 10a1 1 0 10-2 0 1 1 0 002 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var tablet = Vue.extend({
    name: 'TabletIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-tablet",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14 3.5H6a1 1 0 00-1 1v11a1 1 0 001 1h8a1 1 0 001-1v-11a1 1 0 00-1-1zm-8-1a2 2 0 00-2 2v11a2 2 0 002 2h8a2 2 0 002-2v-11a2 2 0 00-2-2H6z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 15.5a1 1 0 100-2 1 1 0 000 2z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var tagFill = Vue.extend({
    name: 'TagFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-tag-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 3a1 1 0 00-1 1v4.586a1 1 0 00.293.707l7 7a1 1 0 001.414 0l4.586-4.586a1 1 0 000-1.414l-7-7A1 1 0 008.586 3H4zm4 3.5a1.5 1.5 0 11-3 0 1.5 1.5 0 013 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var tag = Vue.extend({
    name: 'TagIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-tag",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.5 4A1.5 1.5 0 014 2.5h4.586a1.5 1.5 0 011.06.44l7 7a1.5 1.5 0 010 2.12l-4.585 4.586a1.5 1.5 0 01-2.122 0l-7-7a1.5 1.5 0 01-.439-1.06V4zM4 3.5a.5.5 0 00-.5.5v4.586a.5.5 0 00.146.353l7 7a.5.5 0 00.708 0l4.585-4.585a.5.5 0 000-.708l-7-7a.5.5 0 00-.353-.146H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.5 6.5a2 2 0 114 0 2 2 0 01-4 0zm2-1a1 1 0 100 2 1 1 0 000-2z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var terminalFill = Vue.extend({
    name: 'TerminalFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-terminal-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2 5a2 2 0 012-2h12a2 2 0 012 2v10a2 2 0 01-2 2H4a2 2 0 01-2-2V5zm9.5 5.5h-3a.5.5 0 000 1h3a.5.5 0 000-1zm-6.354-.354L6.793 8.5 5.146 6.854a.5.5 0 11.708-.708l2 2a.5.5 0 010 .708l-2 2a.5.5 0 01-.708-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var terminal = Vue.extend({
    name: 'TerminalIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-terminal",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 4H4a1 1 0 00-1 1v10a1 1 0 001 1h12a1 1 0 001-1V5a1 1 0 00-1-1zM4 3a2 2 0 00-2 2v10a2 2 0 002 2h12a2 2 0 002-2V5a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8 11a.5.5 0 01.5-.5h3a.5.5 0 010 1h-3A.5.5 0 018 11zM5.146 6.146a.5.5 0 01.708 0l2 2a.5.5 0 010 .708l-2 2a.5.5 0 01-.708-.708L6.793 8.5 5.146 6.854a.5.5 0 010-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var textCenter = Vue.extend({
    name: 'TextCenterIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-text-center",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6 14.5a.5.5 0 01.5-.5h7a.5.5 0 010 1h-7a.5.5 0 01-.5-.5zm-2-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm2-3a.5.5 0 01.5-.5h7a.5.5 0 010 1h-7a.5.5 0 01-.5-.5zm-2-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var textIndentLeft = Vue.extend({
    name: 'TextIndentLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-text-indent-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 5.5a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm.646 2.146a.5.5 0 01.708 0l2 2a.5.5 0 010 .708l-2 2a.5.5 0 01-.708-.708L6.293 10 4.646 8.354a.5.5 0 010-.708zM9 8.5a.5.5 0 01.5-.5h6a.5.5 0 010 1h-6a.5.5 0 01-.5-.5zm0 3a.5.5 0 01.5-.5h6a.5.5 0 010 1h-6a.5.5 0 01-.5-.5zm-5 3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var textIndentRight = Vue.extend({
    name: 'TextIndentRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-text-indent-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 5.5a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm10.646 2.146a.5.5 0 01.708.708L13.707 10l1.647 1.646a.5.5 0 01-.708.708l-2-2a.5.5 0 010-.708l2-2zM4 8.5a.5.5 0 01.5-.5h6a.5.5 0 010 1h-6a.5.5 0 01-.5-.5zm0 3a.5.5 0 01.5-.5h6a.5.5 0 010 1h-6a.5.5 0 01-.5-.5zm0 3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var textLeft = Vue.extend({
    name: 'TextLeftIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-text-left",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 14.5a.5.5 0 01.5-.5h7a.5.5 0 010 1h-7a.5.5 0 01-.5-.5zm0-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5zm0-3a.5.5 0 01.5-.5h7a.5.5 0 010 1h-7a.5.5 0 01-.5-.5zm0-3a.5.5 0 01.5-.5h11a.5.5 0 010 1h-11a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var textRight = Vue.extend({
    name: 'TextRightIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-text-right",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          stroke: "#000",
          "stroke-linecap": "round",
          d: "M8.5 14.5h7m-11-3h11m-7-3h7m-11-3h11"
        }
      })]);
    }
  });

  var threeDotsVertical = Vue.extend({
    name: 'ThreeDotsVerticalIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-three-dots-vertical",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.5 15a1.5 1.5 0 11-3 0 1.5 1.5 0 013 0zm0-5a1.5 1.5 0 11-3 0 1.5 1.5 0 013 0zm0-5a1.5 1.5 0 11-3 0 1.5 1.5 0 013 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var threeDots = Vue.extend({
    name: 'ThreeDotsIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-three-dots",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5 11.5a1.5 1.5 0 110-3 1.5 1.5 0 010 3zm5 0a1.5 1.5 0 110-3 1.5 1.5 0 010 3zm5 0a1.5 1.5 0 110-3 1.5 1.5 0 010 3z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var toggleOff = Vue.extend({
    name: 'ToggleOffIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-toggle-off",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13 6a4 4 0 010 8h-3a4.992 4.992 0 002-4 4.992 4.992 0 00-2-4h3zm-6 8a4 4 0 110-8 4 4 0 010 8zm-5-4a5 5 0 005 5h6a5 5 0 000-10H7a5 5 0 00-5 5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var toggleOn = Vue.extend({
    name: 'ToggleOnIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-toggle-on",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7 5a5 5 0 000 10h6a5 5 0 000-10H7zm6 9a4 4 0 100-8 4 4 0 000 8z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var toggles = Vue.extend({
    name: 'TogglesIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-toggles",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.5 3h-7a2.5 2.5 0 000 5h7a2.5 2.5 0 000-5zm-7-1a3.5 3.5 0 100 7h7a3.5 3.5 0 100-7h-7zm0 9a3.5 3.5 0 100 7h7a3.5 3.5 0 100-7h-7zm7 6a2.5 2.5 0 100-5 2.5 2.5 0 000 5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 5.5a3.5 3.5 0 11-7 0 3.5 3.5 0 017 0zM6.5 8a2.5 2.5 0 100-5 2.5 2.5 0 000 5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var tools = Vue.extend({
    name: 'ToolsIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-tools",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2 3l1-1 3.081 2.2a1 1 0 01.419.815v.07a1 1 0 00.293.708L12.5 11.5l.914-.305a1 1 0 011.023.242l3.356 3.356a1 1 0 010 1.414l-1.586 1.586a1 1 0 01-1.414 0l-3.356-3.356a1 1 0 01-.242-1.023l.305-.914-5.707-5.707a1 1 0 00-.707-.293h-.071a1 1 0 01-.814-.419L2 3zm11.354 9.646a.5.5 0 00-.708.708l3 3a.5.5 0 00.708-.708l-3-3z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17.898 4.223a3.003 3.003 0 01-3.679 3.674L7.878 14.15a3 3 0 11-2.027-2.027l6.252-6.341a3 3 0 013.675-3.68l-2.142 2.142L14 6l1.757.364 2.141-2.141zm-13.37 9.019L5 13l.471.242.529.026.287.445.445.287.026.529L7 15l-.242.471-.026.529-.445.287-.287.445-.529.026L5 17l-.471-.242L4 16.732l-.287-.445L3.268 16l-.026-.529L3 15l.242-.471.026-.529.445-.287.287-.445.529-.026z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var trashFill = Vue.extend({
    name: 'TrashFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-trash-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.5 3a1 1 0 00-1 1v1a1 1 0 001 1H5v9a2 2 0 002 2h6a2 2 0 002-2V6h.5a1 1 0 001-1V4a1 1 0 00-1-1H12a1 1 0 00-1-1H9a1 1 0 00-1 1H4.5zm3 4a.5.5 0 01.5.5v7a.5.5 0 01-1 0v-7a.5.5 0 01.5-.5zM10 7a.5.5 0 01.5.5v7a.5.5 0 01-1 0v-7A.5.5 0 0110 7zm3 .5a.5.5 0 00-1 0v7a.5.5 0 001 0v-7z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var trash = Vue.extend({
    name: 'TrashIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-trash",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M7.5 7.5A.5.5 0 018 8v6a.5.5 0 01-1 0V8a.5.5 0 01.5-.5zm2.5 0a.5.5 0 01.5.5v6a.5.5 0 01-1 0V8a.5.5 0 01.5-.5zm3 .5a.5.5 0 00-1 0v6a.5.5 0 001 0V8z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16.5 5a1 1 0 01-1 1H15v9a2 2 0 01-2 2H7a2 2 0 01-2-2V6h-.5a1 1 0 01-1-1V4a1 1 0 011-1H8a1 1 0 011-1h2a1 1 0 011 1h3.5a1 1 0 011 1v1zM6.118 6L6 6.059V15a1 1 0 001 1h6a1 1 0 001-1V6.059L13.882 6H6.118zM4.5 5V4h11v1h-11z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var triangleFill = Vue.extend({
    name: 'TriangleFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-triangle-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.022 3.566a1.13 1.13 0 011.96 0l6.857 11.667c.457.778-.092 1.767-.98 1.767H3.144c-.889 0-1.437-.99-.98-1.767L9.022 3.566z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var triangleHalf = Vue.extend({
    name: 'TriangleHalfIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-triangle-half",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.938 4.016a.146.146 0 00-.054.057L3.027 15.74a.176.176 0 00-.002.183c.016.03.037.05.054.06.015.01.034.017.066.017l6.857-.017V4a.13.13 0 00-.064.016zm1.043-.45a1.13 1.13 0 00-1.96 0L2.166 15.233c-.457.778.091 1.767.98 1.767h13.713c.889 0 1.438-.99.98-1.767L10.982 3.566z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var triangle = Vue.extend({
    name: 'TriangleIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-triangle",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.938 4.016a.146.146 0 00-.054.057L3.027 15.74a.176.176 0 00-.002.183c.016.03.037.05.054.06.015.01.034.017.066.017h13.713a.12.12 0 00.066-.017.163.163 0 00.055-.06.176.176 0 00-.003-.183L10.12 4.073a.146.146 0 00-.054-.057.13.13 0 00-.063-.016.13.13 0 00-.064.016zm1.043-.45a1.13 1.13 0 00-1.96 0L2.166 15.233c-.457.778.091 1.767.98 1.767h13.713c.889 0 1.438-.99.98-1.767L10.982 3.566z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var trophy = Vue.extend({
    name: 'TrophyIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-trophy",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M5 3h10c-.495 3.467-.5 10-5 10S5.495 6.467 5 3zm0 15a1 1 0 011-1h8a1 1 0 011 1H5zm2-1a1 1 0 011-1h4a1 1 0 011 1H7z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14.5 5a2 2 0 100 4 2 2 0 000-4zm-3 2a3 3 0 116 0 3 3 0 01-6 0zm-6-2a2 2 0 100 4 2 2 0 000-4zm-3 2a3 3 0 116 0 3 3 0 01-6 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M9 12h2v4H9v-4z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M12 13c0 .552-.895 1-2 1s-2-.448-2-1 .895-1 2-1 2 .448 2 1z"
        }
      })]);
    }
  });

  var tvFill = Vue.extend({
    name: 'TvFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-tv-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4.5 15.5A.5.5 0 015 15h10a.5.5 0 010 1H5a.5.5 0 01-.5-.5zM4 4h12s2 0 2 2v6s0 2-2 2H4s-2 0-2-2V6s0-2 2-2z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var tv = Vue.extend({
    name: 'TvIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-tv",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.216 6H1.5v-.04l.005-.083a2.957 2.957 0 01.298-1.102c.154-.309.394-.633.763-.88C2.94 3.648 3.413 3.5 4 3.5h12.039l.083.005a2.958 2.958 0 011.102.298c.309.154.633.394.88.763.248.373.396.847.396 1.434v6H18h.5v.039l-.005.083a2.957 2.957 0 01-.298 1.102 2.257 2.257 0 01-.763.88c-.373.248-.847.396-1.434.396H3.961l-.083-.005a2.956 2.956 0 01-1.102-.298 2.254 2.254 0 01-.88-.763C1.648 13.06 1.5 12.588 1.5 12V6h.716zm.284.002v-.008l.003-.044a1.959 1.959 0 01.195-.726c.095-.191.23-.367.423-.495.19-.127.466-.229.879-.229h12.006l.044.003a1.958 1.958 0 01.726.195c.191.095.367.23.495.423.127.19.229.466.229.879v6.006l-.003.044a1.959 1.959 0 01-.195.726c-.095.191-.23.367-.423.495-.19.127-.466.229-.879.229H3.994l-.044-.003a1.96 1.96 0 01-.726-.195 1.256 1.256 0 01-.495-.423c-.127-.19-.229-.466-.229-.879V6.002zM4.003 13.5zm.497 2A.5.5 0 015 15h10a.5.5 0 010 1H5a.5.5 0 01-.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var typeBold = Vue.extend({
    name: 'TypeBoldIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-type-bold",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M10.21 15c2.106 0 3.412-1.087 3.412-2.823 0-1.306-.984-2.283-2.324-2.386v-.055a2.176 2.176 0 001.852-2.14c0-1.51-1.162-2.46-3.014-2.46H5.843V15h4.368zM7.908 6.674h1.696c.963 0 1.517.451 1.517 1.244 0 .834-.629 1.32-1.73 1.32H7.908V6.673zm0 6.788v-2.864h1.73c1.216 0 1.88.492 1.88 1.415 0 .943-.643 1.449-1.832 1.449H7.907z"
        }
      })]);
    }
  });

  var typeH1 = Vue.extend({
    name: 'TypeH1Icon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-type-h1",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M10.637 15V5.669H9.379V9.62H4.758V5.67H3.5V15h1.258v-4.273h4.62V15h1.259zm5.329 0V5.669h-1.244L12.5 7.316v1.265l2.16-1.565h.062V15h1.244z"
        }
      })]);
    }
  });

  var typeH2 = Vue.extend({
    name: 'TypeH2Icon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-type-h2",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M9.638 15V5.669H8.38V9.62H3.759V5.67H2.5V15h1.258v-4.273h4.62V15h1.259zm3.022-6.733v-.048c0-.889.63-1.668 1.716-1.668.957 0 1.675.608 1.675 1.572 0 .855-.554 1.504-1.067 2.085l-3.513 3.999V15H17.5v-1.094h-4.245v-.075l2.481-2.844c.875-.998 1.586-1.784 1.586-2.953 0-1.463-1.155-2.556-2.919-2.556-1.941 0-2.966 1.326-2.966 2.74v.049h1.223z"
        }
      })]);
    }
  });

  var typeH3 = Vue.extend({
    name: 'TypeH3Icon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-type-h3",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M9.637 15V5.669H8.379V9.62H3.758V5.67H2.5V15h1.258v-4.273h4.62V15h1.259zm3.625-4.273h1.018c1.142 0 1.935.67 1.949 1.675.013 1.005-.78 1.737-2.01 1.73-1.08-.007-1.853-.588-1.935-1.32h-1.176c.069 1.327 1.224 2.386 3.083 2.386 1.935 0 3.343-1.155 3.309-2.789-.027-1.51-1.251-2.16-2.037-2.249v-.068c.704-.123 1.764-.91 1.723-2.229-.035-1.353-1.176-2.4-2.954-2.385-1.873.006-2.857 1.162-2.898 2.358h1.196c.062-.69.711-1.299 1.696-1.299.998 0 1.695.622 1.695 1.525.007.922-.718 1.592-1.695 1.592h-.964v1.073z"
        }
      })]);
    }
  });

  var typeItalic = Vue.extend({
    name: 'TypeItalicIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-type-italic",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M9.991 13.674l1.538-7.219c.123-.595.246-.71 1.347-.807l.11-.52H9.211l-.11.52c1.06.096 1.128.212 1.005.807L8.57 13.674c-.123.595-.246.71-1.346.806l-.11.52h3.774l.11-.52c-1.06-.095-1.129-.211-1.006-.806z"
        }
      })]);
    }
  });

  var typeStrikethrough = Vue.extend({
    name: 'TypeStrikethroughIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-type-strikethrough",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M10.527 15.164c-2.153 0-3.589-1.107-3.705-2.81h1.23c.144 1.06 1.129 1.703 2.544 1.703 1.34 0 2.31-.705 2.31-1.675 0-.827-.547-1.374-1.914-1.675l-.946-.207h3.45c.468.437.675.994.675 1.697 0 1.826-1.436 2.967-3.644 2.967zM8.602 8.5H7.167a2.776 2.776 0 01-.099-.76c0-1.627 1.436-2.768 3.48-2.768 1.969 0 3.39 1.175 3.445 2.85h-1.23c-.11-1.08-.964-1.743-2.25-1.743-1.23 0-2.18.602-2.18 1.607 0 .31.083.581.27.814z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17 10.5H3v-1h14v1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var typeUnderline = Vue.extend({
    name: 'TypeUnderlineIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-type-underline",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M7.313 5.136h-1.23v6.405c0 2.105 1.47 3.623 3.917 3.623s3.917-1.518 3.917-3.623V5.136h-1.23v6.323c0 1.49-.978 2.57-2.687 2.57-1.709 0-2.687-1.08-2.687-2.57V5.136z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14.5 17h-9v-1h9v1z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var type = Vue.extend({
    name: 'TypeIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-type",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M4.244 15.081l.944-2.803H8.66l.944 2.803h1.257L7.54 5.75H6.322L3 15.081h1.244zm2.7-7.923l1.395 4.157h-2.83L6.91 7.158h.034zm9.146 7.027h.035v.896h1.128v-4.956c0-1.51-1.114-2.345-2.646-2.345-1.736 0-2.59.916-2.666 2.174h1.108c.068-.718.595-1.19 1.517-1.19.971 0 1.518.52 1.518 1.463v.732H14.19c-1.647.007-2.522.8-2.522 2.058 0 1.319.957 2.18 2.345 2.18 1.06 0 1.716-.43 2.078-1.011zm-1.763.035c-.752 0-1.456-.397-1.456-1.244 0-.65.424-1.115 1.408-1.115h1.805v.834c0 .896-.752 1.525-1.757 1.525z"
        }
      })]);
    }
  });

  var unlockFill = Vue.extend({
    name: 'UnlockFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-unlock-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M2.5 10a2 2 0 012-2h7a2 2 0 012 2v5a2 2 0 01-2 2h-7a2 2 0 01-2-2v-5z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.5 5a3.5 3.5 0 117 0v3h-1V5a2.5 2.5 0 00-5 0v3h-1V5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var unlock = Vue.extend({
    name: 'UnlockIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-unlock",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.655 9H4.333c-.264 0-.398.068-.471.121a.73.73 0 00-.224.296 1.626 1.626 0 00-.138.59V15c0 .342.076.531.14.635.064.106.151.18.256.237a1.122 1.122 0 00.436.127l.013.001h7.322c.264 0 .398-.068.471-.121a.73.73 0 00.224-.296 1.627 1.627 0 00.138-.59V10c0-.342-.076-.531-.14-.635a.658.658 0 00-.255-.237 1.123 1.123 0 00-.45-.128zm.012-1H4.333C2.5 8 2.5 10 2.5 10v5c0 2 1.833 2 1.833 2h7.334c1.833 0 1.833-2 1.833-2v-5c0-2-1.833-2-1.833-2zM10.5 5a3.5 3.5 0 117 0v3h-1V5a2.5 2.5 0 00-5 0v3h-1V5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var upload = Vue.extend({
    name: 'UploadIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-upload",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.5 10a.5.5 0 01.5.5V14a1 1 0 001 1h12a1 1 0 001-1v-3.5a.5.5 0 011 0V14a2 2 0 01-2 2H4a2 2 0 01-2-2v-3.5a.5.5 0 01.5-.5zM7 6.854a.5.5 0 00.707 0L10 4.56l2.293 2.293A.5.5 0 1013 6.146L10.354 3.5a.5.5 0 00-.708 0L7 6.146a.5.5 0 000 .708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 4a.5.5 0 01.5.5v8a.5.5 0 01-1 0v-8A.5.5 0 0110 4z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var volumeDownFill = Vue.extend({
    name: 'VolumeDownFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-volume-down-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.717 5.55A.5.5 0 0111 6v8a.5.5 0 01-.812.39L7.825 12.5H5.5A.5.5 0 015 12V8a.5.5 0 01.5-.5h2.325l2.363-1.89a.5.5 0 01.529-.06z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M12.707 13.182A4.486 4.486 0 0014.025 10a4.486 4.486 0 00-1.318-3.182L12 7.525A3.489 3.489 0 0113.025 10c0 .966-.392 1.841-1.025 2.475l.707.707z"
        }
      })]);
    }
  });

  var volumeDown = Vue.extend({
    name: 'VolumeDownIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-volume-down",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10.717 5.55A.5.5 0 0111 6v8a.5.5 0 01-.812.39L7.825 12.5H5.5A.5.5 0 015 12V8a.5.5 0 01.5-.5h2.325l2.363-1.89a.5.5 0 01.529-.06zM10 7.04L8.312 8.39A.5.5 0 018 8.5H6v3h2a.5.5 0 01.312.11L10 12.96V7.04z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M12.707 13.182A4.486 4.486 0 0014.025 10a4.486 4.486 0 00-1.318-3.182L12 7.525A3.489 3.489 0 0113.025 10c0 .966-.392 1.841-1.025 2.475l.707.707z"
        }
      })]);
    }
  });

  var volumeMuteFill = Vue.extend({
    name: 'VolumeMuteFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-volume-mute-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.717 5.55A.5.5 0 019 6v8a.5.5 0 01-.812.39L5.825 12.5H3.5A.5.5 0 013 12V8a.5.5 0 01.5-.5h2.325l2.363-1.89a.5.5 0 01.529-.06zm7.137 1.596a.5.5 0 010 .708l-4 4a.5.5 0 01-.708-.708l4-4a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.146 7.146a.5.5 0 000 .708l4 4a.5.5 0 00.708-.708l-4-4a.5.5 0 00-.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var volumeMute = Vue.extend({
    name: 'VolumeMuteIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-volume-mute",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.717 5.55A.5.5 0 019 6v8a.5.5 0 01-.812.39L5.825 12.5H3.5A.5.5 0 013 12V8a.5.5 0 01.5-.5h2.325l2.363-1.89a.5.5 0 01.529-.06zM8 7.04L6.312 8.39A.5.5 0 016 8.5H4v3h2a.5.5 0 01.312.11L8 12.96V7.04zm7.854.606a.5.5 0 010 .708l-4 4a.5.5 0 01-.708-.708l4-4a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M11.146 7.646a.5.5 0 000 .708l4 4a.5.5 0 00.708-.708l-4-4a.5.5 0 00-.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var volumeUpFill = Vue.extend({
    name: 'VolumeUpFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-volume-up-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          d: "M13.536 16.01a8.473 8.473 0 002.49-6.01 8.473 8.473 0 00-2.49-6.01l-.708.707A7.476 7.476 0 0115.025 10c0 2.071-.84 3.946-2.197 5.303l.708.707z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M12.121 14.596A6.48 6.48 0 0014.025 10a6.48 6.48 0 00-1.904-4.596l-.707.707A5.483 5.483 0 0113.025 10a5.483 5.483 0 01-1.61 3.89l.706.706z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M10.707 13.182A4.486 4.486 0 0012.025 10a4.486 4.486 0 00-1.318-3.182L10 7.525A3.489 3.489 0 0111.025 10c0 .966-.392 1.841-1.025 2.475l.707.707z"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.717 5.55A.5.5 0 019 6v8a.5.5 0 01-.812.39L5.825 12.5H3.5A.5.5 0 013 12V8a.5.5 0 01.5-.5h2.325l2.363-1.89a.5.5 0 01.529-.06z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var volumeUp = Vue.extend({
    name: 'VolumeUpIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-volume-up",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.717 5.55A.5.5 0 019 6v8a.5.5 0 01-.812.39L5.825 12.5H3.5A.5.5 0 013 12V8a.5.5 0 01.5-.5h2.325l2.363-1.89a.5.5 0 01.529-.06zM8 7.04L6.312 8.39A.5.5 0 016 8.5H4v3h2a.5.5 0 01.312.11L8 12.96V7.04z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M13.536 16.01a8.473 8.473 0 002.49-6.01 8.473 8.473 0 00-2.49-6.01l-.708.707A7.476 7.476 0 0115.025 10c0 2.071-.84 3.946-2.197 5.303l.708.707z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M12.121 14.596A6.48 6.48 0 0014.025 10a6.48 6.48 0 00-1.904-4.596l-.707.707A5.483 5.483 0 0113.025 10a5.483 5.483 0 01-1.61 3.89l.706.706z"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M10.707 13.182A4.486 4.486 0 0012.025 10a4.486 4.486 0 00-1.318-3.182L10 7.525A3.489 3.489 0 0111.025 10c0 .966-.392 1.841-1.025 2.475l.707.707z"
        }
      })]);
    }
  });

  var wallet = Vue.extend({
    name: 'WalletIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-wallet",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M3.5 5a.5.5 0 00-.5.5v2h5a.5.5 0 01.5.5c0 .253.08.644.306.958.207.288.557.542 1.194.542.637 0 .987-.254 1.194-.542.226-.314.306-.705.306-.958a.5.5 0 01.5-.5h5v-2a.5.5 0 00-.5-.5h-13zM17 8.5h-4.551a2.678 2.678 0 01-.443 1.042c-.393.546-1.043.958-2.006.958-.963 0-1.613-.412-2.006-.958A2.679 2.679 0 017.551 8.5H3v6a.5.5 0 00.5.5h13a.5.5 0 00.5-.5v-6zm-15-3A1.5 1.5 0 013.5 4h13A1.5 1.5 0 0118 5.5v9a1.5 1.5 0 01-1.5 1.5h-13A1.5 1.5 0 012 14.5v-9z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var watch = Vue.extend({
    name: 'WatchIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-watch",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6 16.333v-1.86A5.985 5.985 0 014 10c0-1.777.772-3.374 2-4.472V3.667C6 2.747 6.746 2 7.667 2h4.666C13.253 2 14 2.746 14 3.667v1.86A5.985 5.985 0 0116 10a5.985 5.985 0 01-2 4.472v1.861c0 .92-.746 1.667-1.667 1.667H7.667C6.747 18 6 17.254 6 16.333zM15 10a5 5 0 10-10 0 5 5 0 0010 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("rect", {
        attrs: {
          width: "1",
          height: "2",
          x: "15.5",
          y: "9",
          rx: ".5"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 6.5a.5.5 0 01.5.5v3a.5.5 0 01-.5.5H8a.5.5 0 010-1h1.5V7a.5.5 0 01.5-.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var wifi = Vue.extend({
    name: 'WifiIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-wifi",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M8.858 13.858A1.991 1.991 0 0110 13.5c.425 0 .818.132 1.142.358L10 15l-1.142-1.142z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.731 14.024l.269.269.269-.269a1.506 1.506 0 00-.538 0zm-1.159-.576A2.49 2.49 0 0110 13c.53 0 1.023.165 1.428.448a.5.5 0 01.068.763l-1.143 1.143a.5.5 0 01-.707 0L8.504 14.21a.5.5 0 01.354-.853v.5l-.286-.41zM10 11.5a4.478 4.478 0 00-2.7.9.5.5 0 01-.6-.8c.919-.69 2.062-1.1 3.3-1.1s2.381.41 3.3 1.1a.5.5 0 01-.6.8 4.478 4.478 0 00-2.7-.9zm0-3c-1.833 0-3.51.657-4.814 1.748a.5.5 0 11-.642-.766A8.468 8.468 0 0110 7.5c2.076 0 3.98.745 5.456 1.982a.5.5 0 01-.642.766A7.468 7.468 0 0010 8.5z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 5.5c-2.657 0-5.082.986-6.932 2.613a.5.5 0 11-.66-.75A11.458 11.458 0 0110 4.5c2.91 0 5.567 1.08 7.592 2.862a.5.5 0 11-.66.751A10.458 10.458 0 0010 5.5z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var window$1 = Vue.extend({
    name: 'WindowIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-window",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 4H4a1 1 0 00-1 1v10a1 1 0 001 1h12a1 1 0 001-1V5a1 1 0 00-1-1zM4 3a2 2 0 00-2 2v10a2 2 0 002 2h12a2 2 0 002-2V5a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M17 8H3V7h14v1z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          d: "M5 5.5a.5.5 0 11-1 0 .5.5 0 011 0zm1.5 0a.5.5 0 11-1 0 .5.5 0 011 0zm1.5 0a.5.5 0 11-1 0 .5.5 0 011 0z"
        }
      })]);
    }
  });

  var wrench = Vue.extend({
    name: 'WrenchIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-wrench",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M2.102 4.223A3.004 3.004 0 005 8c.27 0 .532-.036.78-.103l6.342 6.252A3.003 3.003 0 0015 18a3 3 0 10-.851-5.878L7.897 5.781A3.004 3.004 0 004.223 2.1l2.141 2.142L6 6l-1.757.364-2.141-2.141zm13.37 9.019L15 13l-.471.242-.529.026-.287.445-.445.287-.026.529L13 15l.242.471.026.529.445.287.287.445.529.026L15 17l.471-.242.529-.026.287-.445.445-.287.026-.529L17 15l-.242-.471-.026-.529-.445-.287-.287-.445-.529-.026z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var xCircleFill = Vue.extend({
    name: 'XCircleFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-x-circle-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 18a8 8 0 100-16 8 8 0 000 16zM7.354 6.646L10 9.293l2.646-2.647a.5.5 0 01.708.708L10.707 10l2.647 2.646a.5.5 0 01-.708.708L10 10.707l-2.646 2.647a.5.5 0 01-.708-.708L9.293 10 6.646 7.354a.5.5 0 11.708-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var xCircle = Vue.extend({
    name: 'XCircleIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-x-circle",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M10 17a7 7 0 100-14 7 7 0 000 14zm0 1a8 8 0 100-16 8 8 0 000 16z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M12.646 13.354l-6-6 .708-.708 6 6-.708.708z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M7.354 13.354l6-6-.708-.708-6 6 .708.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var xOctagonFill = Vue.extend({
    name: 'XOctagonFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-x-octagon-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M13.46 2.146A.5.5 0 0013.107 2H6.893a.5.5 0 00-.353.146L2.146 6.54A.5.5 0 002 6.893v6.214a.5.5 0 00.146.353l4.394 4.394a.5.5 0 00.353.146h6.214a.5.5 0 00.353-.146l4.394-4.394a.5.5 0 00.146-.353V6.893a.5.5 0 00-.146-.353L13.46 2.146zm-6.106 4.5L10 9.293l2.646-2.647a.5.5 0 01.708.708L10.707 10l2.647 2.646a.5.5 0 01-.708.708L10 10.707l-2.646 2.647a.5.5 0 01-.708-.708L9.293 10 6.646 7.354a.5.5 0 11.708-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var xOctagon = Vue.extend({
    name: 'XOctagonIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-x-octagon",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M6.54 2.146A.5.5 0 016.893 2h6.214a.5.5 0 01.353.146l4.394 4.394a.5.5 0 01.146.353v6.214a.5.5 0 01-.146.353l-4.394 4.394a.5.5 0 01-.353.146H6.893a.5.5 0 01-.353-.146L2.146 13.46A.5.5 0 012 13.107V6.893a.5.5 0 01.146-.353L6.54 2.146zM7.1 3L3 7.1v5.8L7.1 17h5.8l4.1-4.1V7.1L12.9 3H7.1z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.293 10L6.646 7.354l.708-.708L10 9.293l2.646-2.647.708.708L10.707 10l2.647 2.646-.707.708L10 10.707l-2.646 2.647-.708-.707L9.293 10z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var xSquareFill = Vue.extend({
    name: 'XSquareFillIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-x-square-fill",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M4 2a2 2 0 00-2 2v12a2 2 0 002 2h12a2 2 0 002-2V4a2 2 0 00-2-2H4zm3.354 4.646L10 9.293l2.646-2.647a.5.5 0 01.708.708L10.707 10l2.647 2.646a.5.5 0 01-.708.708L10 10.707l-2.646 2.647a.5.5 0 01-.708-.708L9.293 10 6.646 7.354a.5.5 0 11.708-.708z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var xSquare = Vue.extend({
    name: 'XSquareIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-x-square",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M16 3H4a1 1 0 00-1 1v12a1 1 0 001 1h12a1 1 0 001-1V4a1 1 0 00-1-1zM4 2a2 2 0 00-2 2v12a2 2 0 002 2h12a2 2 0 002-2V4a2 2 0 00-2-2H4z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M9.293 10L6.646 7.354l.708-.708L10 9.293l2.646-2.647.708.708L10.707 10l2.647 2.646-.708.708L10 10.707l-2.646 2.647-.708-.707L9.293 10z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  var x = Vue.extend({
    name: 'XIcon',
    props: {
      size: {
        type: [String, Number],
        default: 24
      }
    },
    functional: true,
    render: function render(h, ctx) {
      var size = "".concat(ctx.props.size, "px");
      var attrs = ctx.data.attrs || {};
      attrs.width = attrs.width || size;
      attrs.height = attrs.height || size;
      ctx.data.attrs = attrs; // @ts-ignore

      return h("svg", babelHelperVueJsxMergeProps([{
        "class": "bi bi-x",
        attrs: {
          width: "1em",
          height: "1em",
          viewBox: "0 0 20 20",
          fill: "currentColor",
          xmlns: "http://www.w3.org/2000/svg"
        }
      }, ctx.data]), ["  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M5.646 5.646a.5.5 0 000 .708l8 8a.5.5 0 00.708-.708l-8-8a.5.5 0 00-.708 0z",
          "clip-rule": "evenodd"
        }
      }), "  ", h("path", {
        attrs: {
          "fill-rule": "evenodd",
          d: "M14.354 5.646a.5.5 0 010 .708l-8 8a.5.5 0 01-.708-.708l8-8a.5.5 0 01.708 0z",
          "clip-rule": "evenodd"
        }
      })]);
    }
  });

  exports.AlarmFillIcon = alarmFill;
  exports.AlarmIcon = alarm;
  exports.AlertCircleFillIcon = alertCircleFill;
  exports.AlertCircleIcon = alertCircle;
  exports.AlertOctagonFillIcon = alertOctagonFill;
  exports.AlertOctagonIcon = alertOctagon;
  exports.AlertSquareFillIcon = alertSquareFill;
  exports.AlertSquareIcon = alertSquare;
  exports.AlertTriangleFillIcon = alertTriangleFill;
  exports.AlertTriangleIcon = alertTriangle;
  exports.ArchiveFillIcon = archiveFill;
  exports.ArchiveIcon = archive;
  exports.ArrowBarBottomIcon = arrowBarBottom;
  exports.ArrowBarLeftIcon = arrowBarLeft;
  exports.ArrowBarRightIcon = arrowBarRight;
  exports.ArrowBarUpIcon = arrowBarUp;
  exports.ArrowClockwiseIcon = arrowClockwise;
  exports.ArrowCounterclockwiseIcon = arrowCounterclockwise;
  exports.ArrowDownLeftIcon = arrowDownLeft;
  exports.ArrowDownRightIcon = arrowDownRight;
  exports.ArrowDownShortIcon = arrowDownShort;
  exports.ArrowDownIcon = arrowDown;
  exports.ArrowLeftRightIcon = arrowLeftRight;
  exports.ArrowLeftShortIcon = arrowLeftShort;
  exports.ArrowLeftIcon = arrowLeft;
  exports.ArrowRepeatIcon = arrowRepeat;
  exports.ArrowRightShortIcon = arrowRightShort;
  exports.ArrowRightIcon = arrowRight;
  exports.ArrowUpDownIcon = arrowUpDown;
  exports.ArrowUpLeftIcon = arrowUpLeft;
  exports.ArrowUpRightIcon = arrowUpRight;
  exports.ArrowUpShortIcon = arrowUpShort;
  exports.ArrowUpIcon = arrowUp;
  exports.ArrowsAngleContractIcon = arrowsAngleContract;
  exports.ArrowsAngleExpandIcon = arrowsAngleExpand;
  exports.ArrowsCollapseIcon = arrowsCollapse;
  exports.ArrowsExpandIcon = arrowsExpand;
  exports.ArrowsFullscreenIcon = arrowsFullscreen;
  exports.AtIcon = at;
  exports.AwardIcon = award;
  exports.BackspaceFillIcon = backspaceFill;
  exports.BackspaceReverseFillIcon = backspaceReverseFill;
  exports.BackspaceReverseIcon = backspaceReverse;
  exports.BackspaceIcon = backspace;
  exports.BarChartFillIcon = barChartFill;
  exports.BarChartIcon = barChart;
  exports.BatteryChargingIcon = batteryCharging;
  exports.BatteryFullIcon = batteryFull;
  exports.BatteryIcon = battery;
  exports.BellFillIcon = bellFill;
  exports.BellIcon = bell;
  exports.BlockquoteLeftIcon = blockquoteLeft;
  exports.BlockquoteRightIcon = blockquoteRight;
  exports.BookHalfFillIcon = bookHalfFill;
  exports.BookIcon = book;
  exports.BookmarkFillIcon = bookmarkFill;
  exports.BookmarkIcon = bookmark;
  exports.BootstrapFillIcon = bootstrapFill;
  exports.BootstrapRebootIcon = bootstrapReboot;
  exports.BootstrapIcon = bootstrap;
  exports.BoxArrowBottomLeftIcon = boxArrowBottomLeft;
  exports.BoxArrowBottomRightIcon = boxArrowBottomRight;
  exports.BoxArrowDownIcon = boxArrowDown;
  exports.BoxArrowLeftIcon = boxArrowLeft;
  exports.BoxArrowRightIcon = boxArrowRight;
  exports.BoxArrowUpLeftIcon = boxArrowUpLeft;
  exports.BoxArrowUpRightIcon = boxArrowUpRight;
  exports.BoxArrowUpIcon = boxArrowUp;
  exports.BracesIcon = braces;
  exports.BrightnessFillHighIcon = brightnessFillHigh;
  exports.BrightnessFillLowIcon = brightnessFillLow;
  exports.BrightnessHighIcon = brightnessHigh;
  exports.BrightnessLowIcon = brightnessLow;
  exports.BrushIcon = brush;
  exports.BucketFillIcon = bucketFill;
  exports.BucketIcon = bucket;
  exports.BuildingIcon = building;
  exports.BullseyeIcon = bullseye;
  exports.CalendarFillIcon = calendarFill;
  exports.CalendarIcon = calendar;
  exports.CameraVideoFillIcon = cameraVideoFill;
  exports.CameraVideoIcon = cameraVideo;
  exports.CameraIcon = camera;
  exports.CapslockFillIcon = capslockFill;
  exports.CapslockIcon = capslock;
  exports.ChatFillIcon = chatFill;
  exports.ChatIcon = chat;
  exports.CheckBoxIcon = checkBox;
  exports.CheckCircleIcon = checkCircle;
  exports.CheckIcon = check;
  exports.ChevronCompactDownIcon = chevronCompactDown;
  exports.ChevronCompactLeftIcon = chevronCompactLeft;
  exports.ChevronCompactRightIcon = chevronCompactRight;
  exports.ChevronCompactUpIcon = chevronCompactUp;
  exports.ChevronDownIcon = chevronDown;
  exports.ChevronLeftIcon = chevronLeft;
  exports.ChevronRightIcon = chevronRight;
  exports.ChevronUpIcon = chevronUp;
  exports.CircleFillIcon = circleFill;
  exports.CircleHalfIcon = circleHalf;
  exports.CircleSlashIcon = circleSlash;
  exports.CircleIcon = circle;
  exports.ClockFillIcon = clockFill;
  exports.ClockIcon = clock;
  exports.CloudDownloadIcon = cloudDownload;
  exports.CloudFillIcon = cloudFill;
  exports.CloudUploadIcon = cloudUpload;
  exports.CloudIcon = cloud;
  exports.CodeSlashIcon = codeSlash;
  exports.CodeIcon = code;
  exports.ColumnsGuttersIcon = columnsGutters;
  exports.ColumnsIcon = columns;
  exports.CommandIcon = command;
  exports.CompassIcon = compass;
  exports.ConeStripedIcon = coneStriped;
  exports.ConeIcon = cone;
  exports.ControllerIcon = controller;
  exports.CreditCardIcon = creditCard;
  exports.CursorFillIcon = cursorFill;
  exports.CursorIcon = cursor;
  exports.DashIcon = dash;
  exports.DiamondHalfIcon = diamondHalf;
  exports.DiamondIcon = diamond;
  exports.DisplayFillIcon = displayFill;
  exports.DisplayIcon = display;
  exports.DocumentCodeIcon = documentCode;
  exports.DocumentDiffIcon = documentDiff;
  exports.DocumentRichtextIcon = documentRichtext;
  exports.DocumentSpreadsheetIcon = documentSpreadsheet;
  exports.DocumentTextIcon = documentText;
  exports.DocumentIcon = document$1;
  exports.DocumentsAltIcon = documentsAlt;
  exports.DocumentsIcon = documents;
  exports.DotIcon = dot;
  exports.DownloadIcon = download;
  exports.EggFriedIcon = eggFried;
  exports.EjectFillIcon = ejectFill;
  exports.EjectIcon = eject;
  exports.EnvelopeFillIcon = envelopeFill;
  exports.EnvelopeOpenFillIcon = envelopeOpenFill;
  exports.EnvelopeOpenIcon = envelopeOpen;
  exports.EnvelopeIcon = envelope;
  exports.EyeFillIcon = eyeFill;
  exports.EyeSlashFillIcon = eyeSlashFill;
  exports.EyeSlashIcon = eyeSlash;
  exports.EyeIcon = eye;
  exports.FilterIcon = filter;
  exports.FlagFillIcon = flagFill;
  exports.FlagIcon = flag;
  exports.FolderFillIcon = folderFill;
  exports.FolderSymlinkFillIcon = folderSymlinkFill;
  exports.FolderSymlinkIcon = folderSymlink;
  exports.FolderIcon = folder;
  exports.FontsIcon = fonts;
  exports.ForwardFillIcon = forwardFill;
  exports.ForwardIcon = forward;
  exports.GearFillIcon = gearFill;
  exports.GearWideConnectedIcon = gearWideConnected;
  exports.GearWideIcon = gearWide;
  exports.GearIcon = gear;
  exports.GeoIcon = geo;
  exports.GraphDownIcon = graphDown;
  exports.GraphUpIcon = graphUp;
  exports.GridFillIcon = gridFill;
  exports.GridIcon = grid;
  exports.HammerIcon = hammer;
  exports.HashIcon = hash;
  exports.HeartFillIcon = heartFill;
  exports.HeartIcon = heart;
  exports.HouseFillIcon = houseFill;
  exports.HouseIcon = house;
  exports.ImageAltIcon = imageAlt;
  exports.ImageFillIcon = imageFill;
  exports.ImageIcon = image;
  exports.ImagesIcon = images;
  exports.InboxFillIcon = inboxFill;
  exports.InboxIcon = inbox;
  exports.InboxesFillIcon = inboxesFill;
  exports.InboxesIcon = inboxes;
  exports.InfoFillIcon = infoFill;
  exports.InfoSquareFillIcon = infoSquareFill;
  exports.InfoSquareIcon = infoSquare;
  exports.InfoIcon = info;
  exports.JustifyLeftIcon = justifyLeft;
  exports.JustifyRightIcon = justifyRight;
  exports.JustifyIcon = justify;
  exports.KanbanFillIcon = kanbanFill;
  exports.KanbanIcon = kanban;
  exports.LaptopIcon = laptop;
  exports.LayoutSidebarReverseIcon = layoutSidebarReverse;
  exports.LayoutSidebarIcon = layoutSidebar;
  exports.LayoutSplitIcon = layoutSplit;
  exports.ListCheckIcon = listCheck;
  exports.ListOlIcon = listOl;
  exports.ListTaskIcon = listTask;
  exports.ListUlIcon = listUl;
  exports.ListIcon = list;
  exports.LockFillIcon = lockFill;
  exports.LockIcon = lock;
  exports.MapIcon = map;
  exports.MicIcon = mic;
  exports.MoonIcon = moon;
  exports.MusicPlayerFillIcon = musicPlayerFill;
  exports.MusicPlayerIcon = musicPlayer;
  exports.OptionIcon = option;
  exports.OutletIcon = outlet;
  exports.PauseFillIcon = pauseFill;
  exports.PauseIcon = pause;
  exports.PenIcon = pen;
  exports.PencilIcon = pencil;
  exports.PeopleFillIcon = peopleFill;
  exports.PeopleIcon = people;
  exports.PersonFillIcon = personFill;
  exports.PersonIcon = person;
  exports.PhoneLandscapeIcon = phoneLandscape;
  exports.PhoneIcon = phone;
  exports.PieChartFillIcon = pieChartFill;
  exports.PieChartIcon = pieChart;
  exports.PlayFillIcon = playFill;
  exports.PlayIcon = play;
  exports.PlugIcon = plug;
  exports.PlusIcon = plus;
  exports.PowerIcon = power;
  exports.QuestionFillIcon = questionFill;
  exports.QuestionSquareFillIcon = questionSquareFill;
  exports.QuestionSquareIcon = questionSquare;
  exports.QuestionIcon = question;
  exports.ReplyAllFillIcon = replyAllFill;
  exports.ReplyAllIcon = replyAll;
  exports.ReplyFillIcon = replyFill;
  exports.ReplyIcon = reply;
  exports.ScrewdriverIcon = screwdriver;
  exports.SearchIcon = search;
  exports.ShieldFillIcon = shieldFill;
  exports.ShieldLockFillIcon = shieldLockFill;
  exports.ShieldLockIcon = shieldLock;
  exports.ShieldShadedIcon = shieldShaded;
  exports.ShieldIcon = shield;
  exports.ShiftFillIcon = shiftFill;
  exports.ShiftIcon = shift;
  exports.SkipBackwardFillIcon = skipBackwardFill;
  exports.SkipBackwardIcon = skipBackward;
  exports.SkipEndFillIcon = skipEndFill;
  exports.SkipEndIcon = skipEnd;
  exports.SkipForwardFillIcon = skipForwardFill;
  exports.SkipForwardIcon = skipForward;
  exports.SkipStartFillIcon = skipStartFill;
  exports.SkipStartIcon = skipStart;
  exports.SpeakerIcon = speaker;
  exports.SquareFillIcon = squareFill;
  exports.SquareHalfIcon = squareHalf;
  exports.SquareIcon = square;
  exports.StarFillIcon = starFill;
  exports.StarHalfIcon = starHalf;
  exports.StarIcon = star;
  exports.StopFillIcon = stopFill;
  exports.StopIcon = stop;
  exports.StopwatchFillIcon = stopwatchFill;
  exports.StopwatchIcon = stopwatch;
  exports.SunIcon = sun;
  exports.TableIcon = table;
  exports.TabletLandscapeIcon = tabletLandscape;
  exports.TabletIcon = tablet;
  exports.TagFillIcon = tagFill;
  exports.TagIcon = tag;
  exports.TerminalFillIcon = terminalFill;
  exports.TerminalIcon = terminal;
  exports.TextCenterIcon = textCenter;
  exports.TextIndentLeftIcon = textIndentLeft;
  exports.TextIndentRightIcon = textIndentRight;
  exports.TextLeftIcon = textLeft;
  exports.TextRightIcon = textRight;
  exports.ThreeDotsVerticalIcon = threeDotsVertical;
  exports.ThreeDotsIcon = threeDots;
  exports.ToggleOffIcon = toggleOff;
  exports.ToggleOnIcon = toggleOn;
  exports.TogglesIcon = toggles;
  exports.ToolsIcon = tools;
  exports.TrashFillIcon = trashFill;
  exports.TrashIcon = trash;
  exports.TriangleFillIcon = triangleFill;
  exports.TriangleHalfIcon = triangleHalf;
  exports.TriangleIcon = triangle;
  exports.TrophyIcon = trophy;
  exports.TvFillIcon = tvFill;
  exports.TvIcon = tv;
  exports.TypeBoldIcon = typeBold;
  exports.TypeH1Icon = typeH1;
  exports.TypeH2Icon = typeH2;
  exports.TypeH3Icon = typeH3;
  exports.TypeItalicIcon = typeItalic;
  exports.TypeStrikethroughIcon = typeStrikethrough;
  exports.TypeUnderlineIcon = typeUnderline;
  exports.TypeIcon = type;
  exports.UnlockFillIcon = unlockFill;
  exports.UnlockIcon = unlock;
  exports.UploadIcon = upload;
  exports.VolumeDownFillIcon = volumeDownFill;
  exports.VolumeDownIcon = volumeDown;
  exports.VolumeMuteFillIcon = volumeMuteFill;
  exports.VolumeMuteIcon = volumeMute;
  exports.VolumeUpFillIcon = volumeUpFill;
  exports.VolumeUpIcon = volumeUp;
  exports.WalletIcon = wallet;
  exports.WatchIcon = watch;
  exports.WifiIcon = wifi;
  exports.WindowIcon = window$1;
  exports.WrenchIcon = wrench;
  exports.XCircleFillIcon = xCircleFill;
  exports.XCircleIcon = xCircle;
  exports.XOctagonFillIcon = xOctagonFill;
  exports.XOctagonIcon = xOctagon;
  exports.XSquareFillIcon = xSquareFill;
  exports.XSquareIcon = xSquare;
  exports.XIcon = x;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
